@@ -1,10 +1,8 @@
 from __future__ import generators
-import user
 import config.base
 import config.package
 from sourceDatabase import SourceDB
 import os
-import string
 
 class Configure(config.package.Package):
   def __init__(self, framework):
@@ -127,7 +125,7 @@ class Configure(config.package.Package):
     foundLapack = 0
     self.f2c    = 0
     # allow a user-specified suffix to be appended to BLAS/LAPACK symbols
-    self.suffix = string.join(self.argDB.get('with-blaslapack-suffix', ''),'')
+    self.suffix = self.argDB.get('with-blaslapack-suffix', '')
     mangleFunc = self.compilers.fortranMangling
     foundBlas = self.checkBlas(blasLibrary, self.getOtherLibs(foundBlas, blasLibrary), mangleFunc,'dot')
     if foundBlas:
@@ -213,6 +211,7 @@ class Configure(config.package.Package):
       # If using 64 bit indices then look for 64 bit integer MKL
       if self.defaultIndexSize == 64: ILP64 = '_ilp64'
       else: ILP64 = '_lp64'
+      ILP64 = '_lp64'
 
       # Look for Multi-Threaded MKL for MKL_C/Pardiso
       useCPardiso=0
@@ -262,7 +261,8 @@ class Configure(config.package.Package):
         if not os.path.exists(os.path.join(dir,libdir)):
           self.logPrint('MKL Path not found.. skipping: '+os.path.join(dir,libdir))
         else:
-          yield ('User specified MKL11/12 Linux64', None, [os.path.join(dir,libdir,'libmkl_intel'+ILP64+'.a'),'mkl_sequential','mkl_core','pthread'])
+          yield ('User specified MKL11+ Linux64', None, [os.path.join(dir,libdir,'libmkl_intel'+ILP64+'.a'),'mkl_sequential','mkl_core','mkl_def','pthread'])
+          yield ('User specified MKL11+ Mac-64', None, [os.path.join(dir,libdir,'libmkl_intel'+ILP64+'.a'),'mkl_sequential','mkl_core','pthread'])
       # Older Linux MKL checks
       yield ('User specified MKL Linux lib dir', None, [os.path.join(dir, 'libmkl_lapack.a'), 'mkl', 'guide', 'pthread'])
       for libdir in ['32','64','em64t']:
@@ -468,8 +468,9 @@ class Configure(config.package.Package):
               break
 
           if not found:
-            raise RuntimeError('Unable to find MKL include directory. Please source the mklvars-script to set up the MKL environment.\n')
-        self.logPrint('MKL include path set to ' + str(self.include))
+            self.logPrint('Unable to find MKL include directory!')
+          else:
+            self.logPrint('MKL include path set to ' + str(self.include))
     self.logWrite(self.libraries.restoreLog())
     return
 
@@ -565,7 +566,7 @@ class Configure(config.package.Package):
         oldLibs  = self.compilers.LIBS
         self.compilers.LIBS = self.libraries.toString(lib)+' '+self.compilers.LIBS
       if self.checkRun(includes, body) and os.path.exists(filename):
-        f    = file(filename)
+        f    = open(filename)
         out  = f.read()
         f.close()
         os.remove(filename)
@@ -624,7 +625,7 @@ class Configure(config.package.Package):
                      } else {
                        sdotresult = '''+self.mangleBlasNoPrefix('sdot')+'''((const int*)&ione1,x1,(const int*)&ione1,x1,(const int*)&ione1);
                      }
-                  fprintf(output, "--known-sdot-returns-doubl=%d",sdotresult != 9);\n'''
+                  fprintf(output, "--known-sdot-returns-double=%d",sdotresult != 9);\n'''
     result = self.runTimeTest('known-sdot-returns-double',includes,body,self.dlib,nobatch=1)
     if result:
       self.log.write('Checking for sdot return double: result ' +str(result)+'\n')
