@@ -3,9 +3,9 @@ import config.package
 class Configure(config.package.Package):
   def __init__(self, framework):
     config.package.Package.__init__(self, framework)
-    self.gitcommit     = 'master'
+    self.gitcommit     = 'origin/featureChangedStructure'
     self.download      = ['git://https://github.com/SciCompKL/CoDiPack.git']
-    self.includes      = ['adjointInterface.hpp']
+    self.includes      = ['codi/adjointInterface.hpp']
     self.liblist       = []
     self.cxx           = 1
     self.requirescxx11 = 1
