@@ -2,7 +2,6 @@
 
 #!/usr/bin/env python
 from __future__ import generators
-import user
 import config.base
 import config.package
 import os
@@ -38,7 +37,6 @@ class Configure(config.package.Package):
     self.liblist          = liblist_mpich + liblist_lam + liblist_msmpi + liblist_other + liblist_single
     # defaults to --with-mpi=yes
     self.required         = 1
-    self.double           = 0
     self.complex          = 1
     self.isPOE            = 0
     self.usingMPIUni      = 0
@@ -48,6 +46,8 @@ class Configure(config.package.Package):
     self.commc2f          = 0
     self.needBatchMPI     = 1
     self.alternativedownload = 'mpich'
+    # support MPI-3 process shared memory
+    self.support_mpi3_shm = 0
     return
 
   def setupHelp(self, help):
@@ -56,7 +56,6 @@ class Configure(config.package.Package):
     help.addArgument('MPI', '-with-mpiexec=<prog>',                              nargs.Arg(None, None, 'The utility used to launch MPI jobs'))
     help.addArgument('MPI', '-with-mpi-compilers=<bool>',                        nargs.ArgBool(None, 1, 'Try to use the MPI compilers, e.g. mpicc'))
     help.addArgument('MPI', '-known-mpi-shared-libraries=<bool>',                nargs.ArgBool(None, None, 'Indicates the MPI libraries are shared (the usual test will be skipped)'))
-    help.addArgument('MPI', '-with-mpiuni-fortran-binding=<bool>',               nargs.ArgBool(None, 1, 'Build the MPIUni Fortran bindings'))
     return
 
   def setupDependencies(self, framework):
@@ -77,11 +76,14 @@ class Configure(config.package.Package):
     if self.mpich.found:
       yield (self.mpich.installDir)
       raise RuntimeError('--download-mpich libraries cannot be used')
+    if self.openmpi.found:
+      yield (self.openmpi.installDir)
+      raise RuntimeError('--download-openmpi libraries cannot be used')
 
     yield ''
     # Try configure package directories
     dirExp = re.compile(r'mpi(ch)?(-.*)?')
-    for packageDir in self.argDB['package-dirs']:
+    for packageDir in self.argDB['with-packages-search-path']:
       packageDir = os.path.abspath(packageDir)
       if not os.path.isdir(packageDir):
         raise RuntimeError('Invalid package directory: '+packageDir)
@@ -133,7 +135,7 @@ class Configure(config.package.Package):
     self.executeTest(self.configureMPIEXEC)
     try:
       self.shared = self.libraries.checkShared('#include <mpi.h>\n','MPI_Init','MPI_Initialized','MPI_Finalize',checkLink = self.checkPackageLink,libraries = self.lib, defaultArg = 'known-mpi-shared-libraries', executor = self.mpiexec)
-    except RuntimeError, e:
+    except RuntimeError as e:
       if self.argDB['with-shared-libraries']:
         raise RuntimeError('Shared libraries cannot be built using MPI provided.\nEither rebuild with --with-shared-libraries=0 or rebuild MPI with shared library support')
       self.logPrint('MPI libraries cannot be used with shared libraries')
@@ -174,6 +176,8 @@ class Configure(config.package.Package):
       if not self.getExecutable('/bin/false', path = [], useDefaultPath = 0, resultName = 'mpiexec',setMakeMacro=0):
         raise RuntimeError('Could not locate MPIEXEC - please specify --with-mpiexec option')
     self.mpiexec = self.mpiexec.replace(' -n 1','').replace(' ', '\\ ').replace('(', '\\(').replace(')', '\\)')
+    if (hasattr(self, 'ompi_major_version') and int(self.ompi_major_version) >= 3):
+      self.mpiexec = self.mpiexec + ' --oversubscribe'
     self.addMakeMacro('MPIEXEC', self.mpiexec)
     return
 
@@ -209,6 +213,36 @@ class Configure(config.package.Package):
     if self.checkLink('#include <mpi.h>\n', 'if (MPI_Reduce_local(0, 0, 0, MPI_INT, MPI_SUM));'):
       self.haveReduceLocal = 1
       self.addDefine('HAVE_MPI_REDUCE_LOCAL',1)
+    if self.checkLink('#include <mpi.h>\n', 'char version[MPI_MAX_LIBRARY_VERSION_STRING];int verlen;if (MPI_Get_library_version(version,&verlen));\n'):
+      self.addDefine('HAVE_MPI_GET_LIBRARY_VERSION', 1)
+    self.compilers.CPPFLAGS = oldFlags
+    self.compilers.LIBS = oldLibs
+    self.logWrite(self.framework.restoreLog())
+    return
+
+  def configureMPI3(self):
+    '''Check for functions added to the interface in MPI-3'''
+    oldFlags = self.compilers.CPPFLAGS
+    oldLibs  = self.compilers.LIBS
+    self.compilers.CPPFLAGS += ' '+self.headers.toString(self.include)
+    self.compilers.LIBS = self.libraries.toString(self.lib)+' '+self.compilers.LIBS
+    self.framework.saveLog()
+    if self.libraries.check(self.dlib, "MPI_Win_create"):
+      self.addDefine('HAVE_MPI_WIN_CREATE',1)
+      self.addDefine('HAVE_MPI_REPLACE',1) # MPI_REPLACE is strictly for use with the one-sided function MPI_Accumulate
+    if self.checkLink('#include <mpi.h>\n', 'MPI_Comm scomm; if (MPI_Comm_split_type(MPI_COMM_WORLD, MPI_COMM_TYPE_SHARED, 0, MPI_INFO_NULL, &scomm));\n'):
+      self.haveMPISharedComm = 1
+      self.addDefine('HAVE_MPI_SHARED_COMM', 1)
+    else: self.haveMPISharedComm = 0
+    if self.checkLink('#include <mpi.h>\n', 'MPI_Win win; if (MPI_Win_allocate_shared(100,10,MPI_INFO_NULL,MPI_COMM_WORLD, 0, &win));\n'):
+      self.addDefine('HAVE_MPI_WIN_ALLOCATE_SHARED', 1)
+    if self.checkLink('#include <mpi.h>\n', 'if (MPI_Win_shared_query(MPI_WIN_NULL,0,0,0,0));\n'):
+      self.addDefine('HAVE_MPI_WIN_SHARED_QUERY', 1)
+    if 'HAVE_MPI_WIN_CREATE' in self.defines and 'HAVE_MPI_WIN_ALLOCATE_SHARED' in self.defines and 'HAVE_MPI_WIN_SHARED_QUERY' in self.defines:
+      if (hasattr(self, 'mpich_numversion') and int(self.mpich_numversion) > 30004300) or not hasattr(self, 'mpich_numversion'):
+        self.addDefine('HAVE_MPI_WIN_CREATE_FEATURE',1)
+        self.addDefine('HAVE_MPI_PROCESS_SHARED_MEMORY',1)
+        self.support_mpi3_shm = 1
     self.compilers.CPPFLAGS = oldFlags
     self.compilers.LIBS = oldLibs
     self.logWrite(self.framework.restoreLog())
@@ -293,27 +327,19 @@ class Configure(config.package.Package):
     '''Setup MPIUNI, our uniprocessor version of MPI'''
     self.addDefine('HAVE_MPIUNI', 1)
     self.addMakeMacro('MPI_IS_MPIUNI', 1)
-    #
-    #  Even though MPI-Uni is not an external package (it is in PETSc source) we need to stick the
-    #  include path for its mpi.h and mpif.h so that external packages that are built with PETSc to
-    #  use MPI-Uni can find them.
-    self.include = [os.path.abspath(os.path.join('include', 'petsc','mpiuni'))]
     self.framework.packages.append(self)
-    self.mpiexec = '${PETSC_DIR}/bin/petsc-mpiexec.uni'
-    self.addMakeMacro('MPIEXEC','${PETSC_DIR}/bin/petsc-mpiexec.uni')
+    self.mpiexec = '${PETSC_DIR}/lib/petsc/bin/petsc-mpiexec.uni'
+    self.addMakeMacro('MPIEXEC','${PETSC_DIR}/lib/petsc/bin/petsc-mpiexec.uni')
     self.addDefine('HAVE_MPI_COMM_F2C', 1)
     self.addDefine('HAVE_MPI_COMM_C2F', 1)
     self.addDefine('HAVE_MPI_FINT', 1)
     self.addDefine('HAVE_MPI_IN_PLACE', 1)
+    self.addDefine('HAVE_MPI_TYPE_DUP', 1)
+    self.addDefine('HAVE_MPI_TYPE_GET_ENVELOPE', 1)
     self.framework.saveLog()
     self.framework.addDefine('MPI_Type_create_struct(count,lens,displs,types,newtype)', 'MPI_Type_struct((count),(lens),(displs),(types),(newtype))')
     self.framework.addDefine('MPI_Comm_create_errhandler(p_err_fun,p_errhandler)', 'MPI_Errhandler_create((p_err_fun),(p_errhandler))')
     self.framework.addDefine('MPI_Comm_set_errhandler(comm,p_errhandler)', 'MPI_Errhandler_set((comm),(p_errhandler))')
-    if hasattr(self.compilers, 'FC') and self.argDB['with-mpiuni-fortran-binding']:
-      self.usingMPIUniFortranBinding = 1
-      self.framework.addDefine('MPIUNI_FORTRAN_BINDING', 1)
-    else:
-      self.usingMPIUniFortranBinding = 0
     self.logWrite(self.framework.restoreLog())
     if self.getDefaultLanguage == 'C': self.addDefine('HAVE_MPI_C_DOUBLE_COMPLEX', 1)
     self.commf2c = 1
@@ -435,27 +461,38 @@ class Configure(config.package.Package):
     '''Determine if MPICH_NUMVERSION or OMPI_MAJOR_VERSION exist in mpi.h
        Used for consistency checking of MPI installation at compile time'''
     import re
-    mpich_test = '#include <mpi.h>\nint mpich_ver = MPICH_NUMVERSION;\n'
+    HASHLINESPACE = ' *(?:\n#.*\n *)*'
+    oldFlags = self.compilers.CPPFLAGS
+    self.compilers.CPPFLAGS += ' '+self.headers.toString(self.include)
+    for mpichpkg in ['i_mpi','mvapich2','mpich']:
+      MPICHPKG = mpichpkg.upper()
+      mpich_test = '#include <mpi.h>\nint mpich_ver = '+MPICHPKG+'_NUMVERSION;\n'
+      if self.checkCompile(mpich_test):
+        buf = self.outputPreprocess(mpich_test)
+        try:
+          mpich_numversion = re.compile('\nint mpich_ver ='+HASHLINESPACE+'([0-9]+)'+HASHLINESPACE+';').search(buf).group(1)
+          self.addDefine('HAVE_'+MPICHPKG+'_NUMVERSION',mpich_numversion)
+          if mpichpkg == 'mpich': self.mpich_numversion = mpich_numversion
+        except:
+          self.logPrint('Unable to parse '+MPICHPKG+' version from header. Probably a buggy preprocessor')
+        self.compilers.CPPFLAGS = oldFlags
+        return
     openmpi_test = '#include <mpi.h>\nint ompi_major = OMPI_MAJOR_VERSION;\nint ompi_minor = OMPI_MINOR_VERSION;\nint ompi_release = OMPI_RELEASE_VERSION;\n'
-    if self.checkCompile(mpich_test):
-      buf = self.outputPreprocess(mpich_test)
-      try:
-        mpich_numversion = re.compile('\nint mpich_ver = *([0-9]*) *;').search(buf).group(1)
-        self.addDefine('HAVE_MPICH_NUMVERSION',mpich_numversion)
-      except:
-        self.logPrint('Unable to parse MPICH version from header. Probably a buggy preprocessor')
-    elif self.checkCompile(openmpi_test):
+    if self.checkCompile(openmpi_test):
       buf = self.outputPreprocess(openmpi_test)
       ompi_major_version = ompi_minor_version = ompi_release_version = 'unknown'
       try:
-        ompi_major_version = re.compile('\nint ompi_major = *([0-9]*) *;').search(buf).group(1)
-        ompi_minor_version = re.compile('\nint ompi_minor = *([0-9]*) *;').search(buf).group(1)
-        ompi_release_version = re.compile('\nint ompi_release = *([0-9]*) *;').search(buf).group(1)
+        ompi_major_version = re.compile('\nint ompi_major ='+HASHLINESPACE+'([0-9]+)'+HASHLINESPACE+';').search(buf).group(1)
+        ompi_minor_version = re.compile('\nint ompi_minor ='+HASHLINESPACE+'([0-9]+)'+HASHLINESPACE+';').search(buf).group(1)
+        ompi_release_version = re.compile('\nint ompi_release ='+HASHLINESPACE+'([0-9]+)'+HASHLINESPACE+';').search(buf).group(1)
         self.addDefine('HAVE_OMPI_MAJOR_VERSION',ompi_major_version)
         self.addDefine('HAVE_OMPI_MINOR_VERSION',ompi_minor_version)
         self.addDefine('HAVE_OMPI_RELEASE_VERSION',ompi_release_version)
+        self.ompi_major_version = ompi_major_version
       except:
         self.logPrint('Unable to parse OpenMPI version from header. Probably a buggy preprocessor')
+    self.compilers.CPPFLAGS = oldFlags
+    return
 
   def findMPIInc(self):
     '''Find MPI include paths from "mpicc -show" and use with CUDAC_FLAGS'''
@@ -470,7 +507,7 @@ class Configure(config.package.Package):
     argIter = iter(output.split())
     try:
       while 1:
-        arg = argIter.next()
+        arg = next(argIter)
         self.logPrint( 'Checking arg '+arg, 4, 'compilers')
         m = re.match(r'^-I.*$', arg)
         if m:
@@ -489,7 +526,9 @@ class Configure(config.package.Package):
       self.argDB['with-'+self.package] = 1
     config.package.Package.configureLibrary(self)
     self.executeTest(self.configureConversion)
+    self.executeTest(self.checkMPICHorOpenMPI)
     self.executeTest(self.configureMPI2)
+    self.executeTest(self.configureMPI3) #depends on checkMPICHorOpenMPI for self.mpich_numversion
     self.executeTest(self.configureTypes)
     self.executeTest(self.configureMPITypes)
     self.executeTest(self.configureMissingPrototypes)
@@ -498,12 +537,8 @@ class Configure(config.package.Package):
     self.executeTest(self.FortranMPICheck)
     self.executeTest(self.configureIO)
     self.executeTest(self.findMPIInc)
-    self.executeTest(self.checkMPICHorOpenMPI)
     if self.libraries.check(self.dlib, "MPI_Alltoallw") and self.libraries.check(self.dlib, "MPI_Type_create_indexed_block"):
       self.addDefine('HAVE_MPI_ALLTOALLW',1)
-    if self.libraries.check(self.dlib, "MPI_Win_create"):
-      self.addDefine('HAVE_MPI_WIN_CREATE',1)
-      self.addDefine('HAVE_MPI_REPLACE',1) # MPI_REPLACE is strictly for use with the one-sided function MPI_Accumulate
     funcs = '''MPI_Comm_spawn MPI_Type_get_envelope MPI_Type_get_extent MPI_Type_dup MPI_Init_thread
       MPI_Iallreduce MPI_Ibarrier MPI_Finalized MPI_Exscan MPI_Reduce_scatter MPI_Reduce_scatter_block'''.split()
     found, missing = self.libraries.checkClassify(self.dlib, funcs)
