@@ -3,8 +3,9 @@ import config.package
 class Configure(config.package.Package):
   def __init__(self, framework):
     config.package.Package.__init__(self, framework)
-    self.gitcommit = 'v3.2p4'
-    self.download  = ['git://https://bitbucket.org/petsc/pkg-parms.git','http://ftp.mcs.anl.gov/pub/petsc/externalpackages/pARMS_3.2p4.tar.gz']
+    self.gitcommit = 'v3.2p5'
+    self.download  = ['git://https://bitbucket.org/petsc/pkg-parms.git','https://bitbucket.org/petsc/pkg-metis/get/'+self.gitcommit+'.tar.gz']
+    self.downloaddirnames  = ['petsc-pkg-parms','pARMS']
     self.functions = ['parms_PCCreate']
     self.includes  = ['parms.h']
     self.liblist   = [['libparms.a']]
