@@ -119,14 +119,14 @@ class Configure(config.package.Package):
     self.includes     = ['tetgen.h']
     self.liblist      = [['libtet.a']]
     self.cxx          = 1
-    self.needsMath    = 1
     return
 
   def setupDependencies(self, framework):
     config.package.Package.setupDependencies(self, framework)
     self.languages       = framework.require('PETSc.options.languages',   self)
     self.sharedLibraries = framework.require('PETSc.options.sharedLibraries', self)
-    self.deps            = []
+    self.mathlib         = framework.require('config.packages.mathlib',self)
+    self.deps            = [self.mathlib]
     return
 
   def Install(self):
@@ -191,7 +191,7 @@ class Configure(config.package.Package):
         output,err,ret = config.package.Package.executeShellCommand(self.installSudo+'mkdir -p '+os.path.join(self.installDir,'lib'), timeout=2500, log=self.log)
         output,err,ret = config.package.Package.executeShellCommand(self.installSudo+'mkdir -p '+os.path.join(self.installDir,'include'), timeout=2500, log=self.log)
         output1,err1,ret1  = config.package.Package.executeShellCommand('cd '+self.packageDir+' && make CXX="'+ self.setCompilers.getCompiler() + '" CXXFLAGS="' + cflags + '" PREDCXXFLAGS="' + predcflags + '" tetlib && mv tetgen_def.h tetgen.h && '+self.installSudo+'cp *.a ' + libDir + ' && rm *.a *.o', timeout=2500, log = self.log)
-      except RuntimeError, e:
+      except RuntimeError as e:
         raise RuntimeError('Error running make on TetGen: '+str(e))
       output2,err2,ret2  = config.package.Package.executeShellCommand(self.installSudo+'cp -f '+os.path.join(self.packageDir, 'tetgen.h')+' '+includeDir, timeout=5, log = self.log)
       self.postInstall(output1+err1+output2+err2,'make.inc')
