@@ -1004,7 +1004,7 @@ class Configure(config.base.Configure):
         output.find('not recognized') >= 0 or output.find('not recognised') >= 0 or
         output.find('unknown option') >= 0 or output.find('unknown flag') >= 0 or output.find('Unknown switch') >= 0 or
         output.find('ignoring option') >= 0 or output.find('ignored') >= 0 or
-        output.find('argument unused') >= 0 or
+        output.find('argument unused') >= 0 or output.find('not supported') >= 0 or
         # When checking for the existence of 'attribute'
         output.find('is unsupported and will be skipped') >= 0 or
         output.find('illegal option') >= 0 or output.find('Invalid option') >= 0 or
@@ -1040,6 +1040,7 @@ class Configure(config.base.Configure):
 
   def insertCompilerFlag(self, flag, compilerOnly):
     '''DANGEROUS: Put in the compiler flag without checking'''
+    if not flag: return
     flagsArg = self.getCompilerFlagsArg(compilerOnly)
     setattr(self, flagsArg, getattr(self, flagsArg)+' '+flag)
     self.log.write('Added '+self.language[-1]+' compiler flag '+flag+'\n')
@@ -1073,6 +1074,11 @@ class Configure(config.base.Configure):
     if not self.argDB['with-pic'] and not useSharedLibraries:
       self.logPrint("Skip checking PIC options on user request")
       return
+    if self.argDB['with-pic'] and not useSharedLibraries:
+      # this is a flaw in configure; it is a legitimate use case where PETSc is built with PIC flags but not shared libraries
+      # to fix it the capability to build shared libraries must be enabled in configure if --with-pic=true even if shared libraries are off and this
+      # test must use that capability instead of using the default shared library build in that case which is static libraries
+      raise RuntimeError("Cannot determine compiler PIC flags if shared libraries is turned off\nEither run using --with-shared-libraries or --with-pic=0 and supply the compiler PIC flag via CFLAGS, CXXXFLAGS, and FCFLAGS\n")
     languages = ['C']
     if hasattr(self, 'CXX'):
       languages.append('Cxx')
@@ -1089,7 +1095,10 @@ class Configure(config.base.Configure):
       compilerFlagsArg = self.getCompilerFlagsArg(1) # compiler only
       oldCompilerFlags = getattr(self, compilerFlagsArg)
       for testFlag in self.generatePICGuesses():
-        self.logPrint('Trying '+language+' compiler flag '+testFlag)
+        if testFlag:
+          self.logPrint('Trying '+language+' compiler flag '+testFlag+' for PIC code')
+        else:
+          self.logPrint('Trying '+language+' for PIC code without any compiler flag')
         acceptedPIC = 1
         try:
           self.addCompilerFlag(testFlag, compilerOnly = 1)
@@ -1100,7 +1109,10 @@ class Configure(config.base.Configure):
           self.logPrint('Rejected '+language+' compiler flag '+testFlag+' because shared linker cannot handle it')
           setattr(self, compilerFlagsArg, oldCompilerFlags)
           continue
-        self.logPrint('Accepted '+language+' compiler flag '+testFlag)
+        if testFlag:
+          self.logPrint('Accepted '+language+' compiler flag '+testFlag+' for PIC code')
+        else:
+          self.logPrint('Accepted '+language+' PIC code without compiler flag')
         self.isPIC = 1
         break
       self.popLanguage()
@@ -1336,7 +1348,7 @@ class Configure(config.base.Configure):
           except RuntimeError:
             accepted = 0
           if accepted:
-            goodFlags = filter(self.checkLinkerFlag, flags)
+            goodFlags = list(filter(self.checkLinkerFlag, flags))
             self.sharedLinker = self.LD_SHARED
             self.sharedLibraryFlags = goodFlags
             self.sharedLibraryExt = ext
@@ -1406,10 +1418,10 @@ class Configure(config.base.Configure):
       for testFlag in ['-Wl,-multiply_defined,suppress', '-Wl,-multiply_defined -Wl,suppress', '-Wl,-commons,use_dylibs', '-Wl,-search_paths_first', '-Wl,-no_compact_unwind']:
         if self.checkLinkerFlag(testFlag):
           # expand to CC_LINKER_FLAGS or CXX_LINKER_FLAGS or FC_LINKER_FLAGS
-	  linker_flag_var = langMap[language]+'_LINKER_FLAGS'
+          linker_flag_var = langMap[language]+'_LINKER_FLAGS'
           val = getattr(self,linker_flag_var)
-	  val.append(testFlag)
-	  setattr(self,linker_flag_var,val)
+          val.append(testFlag)
+          setattr(self,linker_flag_var,val)
       self.popLanguage()
     return
 
@@ -1517,7 +1529,7 @@ class Configure(config.base.Configure):
       self.logPrint('Checking dynamic linker '+linker+' using flags '+str(flags))
       if self.getExecutable(linker, resultName = 'dynamicLinker'):
         flagsArg = self.getLinkerFlagsArg()
-        goodFlags = filter(self.checkLinkerFlag, flags)
+        goodFlags = list(filter(self.checkLinkerFlag, flags))
         self.dynamicLibraryFlags = goodFlags
         self.dynamicLibraryExt = ext
         testMethod = 'foo'
@@ -1618,12 +1630,12 @@ if (dlclose(handle)) {
           if (opt in self.argDB  and self.argDB[opt] != '0'):
             # check if corresponding mpi wrapper exists
             for cplr in cplrs:
-              mpicplr = os.path.join(self.argDB['with-mpi-dir'], 'bin', cplr)
-              if os.path.exists(mpicplr):
-                msg = '--'+opt+'='+self.argDB[opt]+' is specified along with --with-mpi-dir='+self.argDB['with-mpi-dir']+' which implies using '+mpicplr+'.\n\
+              for mpicplr in [os.path.join(self.argDB['with-mpi-dir'], 'bin', cplr),os.path.join(self.argDB['with-mpi-dir'], 'intel64', 'bin', cplr)]:
+                if os.path.exists(mpicplr):
+                  msg = '--'+opt+'='+self.argDB[opt]+' is specified along with --with-mpi-dir='+self.argDB['with-mpi-dir']+' which implies using '+mpicplr+'.\n\
   configure is confused and does not know which compiler to select and use! Please specify either [mpi] compilers or --with-mpi-dir - but not both!\n\
   In most cases, specifying --with-mpi-dir - and not explicitly listing compilers could be preferable.'
-                raise RuntimeError(msg)
+                  raise RuntimeError(msg)
     return
 
   def requireMpiLdPath(self):
@@ -1637,7 +1649,7 @@ if (dlclose(handle)) {
 
   def printEnvVariables(self):
     buf = '**** printenv ****'
-    for key,val in os.environ.iteritems():
+    for key,val in os.environ.items():
       buf += '\n'+str(key)+'='+str(val)
     self.logPrint(buf)
     return
