@@ -5,21 +5,9 @@ if not hasattr(sys, 'version_info'):
   print('*** Python version 1 is not supported. Please get the latest version from www.python.org ***')
   sys.exit(4)
 
-import cPickle
+import pickle
 
-try:
-  import subprocess
-  USE_SUBPROCESS = 1
-except ImportError:
-  USE_SUBPROCESS = 0
-
-# Some features related to detecting login failures cannot be easily
-# implemented with the 'subprocess' module. Disable it for now ...
-USE_SUBPROCESS = 0
-# In Python 2.6 and above, the 'popen2' module is deprecated
-if sys.version_info[:2] >= (2, 6) and not USE_SUBPROCESS:
-  import warnings
-  warnings.filterwarnings('ignore', category=DeprecationWarning, module=__name__)
+import subprocess
 
 import nargs
 useThreads = nargs.Arg.findArgument('useThreads', sys.argv[1:])
@@ -120,8 +108,8 @@ class Script(logger.Logger):
     return
 
   def checkPython(self):
-    if not hasattr(sys, 'version_info') or float(sys.version_info[0]) != 2 or float(sys.version_info[1]) < 6:
-      raise RuntimeError('BuildSystem requires Python2 version 2.6 or higher. Get Python at http://www.python.org')
+    if not hasattr(sys, 'version_info') or sys.version_info < (2,6):
+      raise RuntimeError('BuildSystem requires Python version 2.6 or higher. Get Python at http://www.python.org')
     return
 
   def getModule(root, name):
@@ -146,109 +134,59 @@ class Script(logger.Logger):
     return module
   importModule = staticmethod(importModule)
 
-  if USE_SUBPROCESS:
+  @staticmethod
+  def runShellCommand(command, log=None, cwd=None):
+    return Script.runShellCommandSeq([command], log=log, cwd=cwd)
 
-    def runShellCommand(command, log=None, cwd=None):
-      Popen = subprocess.Popen
-      PIPE  = subprocess.PIPE
+  @staticmethod
+  def runShellCommandSeq(commandseq, log=None, cwd=None):
+    Popen = subprocess.Popen
+    PIPE  = subprocess.PIPE
+    output = ''
+    error = ''
+    for command in commandseq:
+      useShell = isinstance(command, str) or isinstance(command, bytes)
       if log: log.write('Executing: %s\n' % (command,))
-      pipe = Popen(command, cwd=cwd, stdin=None, stdout=PIPE, stderr=PIPE,
-                   bufsize=-1, shell=True, universal_newlines=True)
-      (out, err) = pipe.communicate()
-      ret = pipe.returncode
-      return (out, err, ret)
-
-  else:
-
-    def openPipe(command):
-      '''We need to use the asynchronous version here since we want to avoid blocking reads'''
-      import popen2
-
-      pipe = None
-      if hasattr(popen2, 'Popen3'):
-        pipe   = popen2.Popen3(command, 1)
-        input  = pipe.tochild
-        output = pipe.fromchild
-        err    = pipe.childerr
-      else:
-        import os
-        (input, output, err) = os.popen3(command)
-      return (input, output, err, pipe)
-    openPipe = staticmethod(openPipe)
-
-    def runShellCommand(command, log = None, cwd = None):
-      import select, os
-
-      ret        = None
-      out        = ''
-      err        = ''
-      loginError = 0
-      if cwd is not None:
-        oldpath = os.getcwd()
-        os.chdir(cwd)
-      if log: log.write('Executing: %s\n' % (command,))
-      (input, output, error, pipe) = Script.openPipe(command)
-      if cwd is not None:
-        os.chdir(oldpath)
-      input.close()
-      if useSelect:
-        outputClosed = 0
-        errorClosed  = 0
-        lst = [output, error]
-        while 1:
-          try:
-            ready = select.select(lst, [], [])
-          except Exception as e:
-            if log: log.write('** Error calling select() : '+str(e)+'\n')
-            continue
-          if len(ready[0]):
-            if error in ready[0]:
-              msg = error.readline()
-              if msg:
-                err += msg
-              else:
-                errorClosed = 1
-                lst.remove(error)
-            if output in ready[0]:
-              msg = output.readline()
-              if msg:
-                out += msg
-              else:
-                outputClosed = 1
-                lst.remove(output)
-            if msg and msg.find('password:') >= 0:
-              loginError = 1
-              break
-          if outputClosed and errorClosed:
-            break
-      else:
-        out = output.read()
-        err = error.read()
-      output.close()
-      error.close()
-      if pipe:
-        # We would like the NOHANG argument here
-        ret = pipe.wait()
-      if loginError:
-        raise RuntimeError('Could not login to site')
-      return (out, err, ret)
-
-  runShellCommand = staticmethod(runShellCommand)
+      try:
+        pipe = Popen(command, cwd=cwd, stdin=None, stdout=PIPE, stderr=PIPE,
+                     shell=useShell)
+        (out, err) = pipe.communicate()
+        if sys.version_info >= (3,0):
+          out = out.decode(encoding='UTF-8',errors='replace')
+          err = err.decode(encoding='UTF-8',errors='replace')
+        ret = pipe.returncode
+      except OSError as e:
+        return ('', e.message, e.errno)
+      except FileNotFoundError as e:
+        return ('', e.message, e.errno)
+      output += out
+      error += err
+      if ret:
+        break
+    return (output, error, ret)
 
   def defaultCheckCommand(command, status, output, error):
     '''Raise an error if the exit status is nonzero'''
     if status: raise RuntimeError('Could not execute "%s":\n%s' % (command,output+error))
   defaultCheckCommand = staticmethod(defaultCheckCommand)
 
-  def executeShellCommand(command, checkCommand = None, timeout = 600.0, log = None, lineLimit = 0, cwd=None):
+  @staticmethod
+  def executeShellCommand(command, checkCommand = None, timeout = 600.0, log = None, lineLimit = 0, cwd=None, logOutputflg = True):
     '''Execute a shell command returning the output, and optionally provide a custom error checker
        - This returns a tuple of the (output, error, statuscode)'''
+    return Script.executeShellCommandSeq([command], checkCommand=checkCommand, timeout=timeout, log=log, lineLimit=lineLimit, cwd=cwd,logOutputflg = logOutputflg)
+
+  @staticmethod
+  def executeShellCommandSeq(commandseq, checkCommand = None, timeout = 600.0, log = None, lineLimit = 0, cwd=None, logOutputflg = True):
+    '''Execute a sequence of shell commands (an && chain) returning the output, and optionally provide a custom error checker
+       - This returns a tuple of the (output, error, statuscode)'''
     if not checkCommand:
       checkCommand = Script.defaultCheckCommand
     if log is None:
       log = logger.Logger.defaultLog
-    def logOutput(log, output):
+    def logOutput(log, output, logOutputflg):
       import re
+      if not logOutputflg: return output
       # get rid of multiple blank lines
       output = re.sub('\n+','\n', output).strip()
       if output:
@@ -259,7 +197,7 @@ class Script(logger.Logger):
         else:
           log.write('stdout: '+output+'\n')
       return output
-    def runInShell(command, log, cwd):
+    def runInShell(commandseq, log, cwd):
       if useThreads:
         import threading
         class InShell(threading.Thread):
@@ -269,7 +207,7 @@ class Script(logger.Logger):
             self.setDaemon(1)
           def run(self):
             (self.output, self.error, self.status) = ('', '', -1) # So these fields exist even if command fails with no output
-            (self.output, self.error, self.status) = Script.runShellCommand(command, log, cwd)
+            (self.output, self.error, self.status) = Script.runShellCommandSeq(commandseq, log, cwd)
         thread = InShell()
         thread.start()
         thread.join(timeout)
@@ -280,13 +218,12 @@ class Script(logger.Logger):
         else:
           return (thread.output, thread.error, thread.status)
       else:
-        return Script.runShellCommand(command, log, cwd)
+        return Script.runShellCommandSeq(commandseq, log, cwd)
 
-    (output, error, status) = runInShell(command, log, cwd)
-    output = logOutput(log, output)
-    checkCommand(command, status, output, error)
+    (output, error, status) = runInShell(commandseq, log, cwd)
+    output = logOutput(log, output,logOutputflg)
+    checkCommand(commandseq, status, output, error)
     return (output, error, status)
-  executeShellCommand = staticmethod(executeShellCommand)
 
   def loadConfigure(self, argDB = None):
     if argDB is None:
@@ -296,11 +233,11 @@ class Script(logger.Logger):
       return None
     try:
       cache = argDB['configureCache']
-      framework = cPickle.loads(cache)
+      framework = pickle.loads(cache)
       framework.framework = framework
       framework.argDB = argDB
       self.logPrint('Loaded configure to cache: size '+str(len(cache)))
-    except cPickle.UnpicklingError as e:
+    except pickle.UnpicklingError as e:
       framework = None
       self.logPrint('Invalid cached configure: '+str(e))
     return framework
