@@ -3,15 +3,7 @@ import config.base
 import os
 import sys
 import re
-import cPickle
-import string
-
-# The sorted() builtin is not available with python-2.3
-try: sorted
-except NameError:
-  def sorted(lst):
-    lst.sort()
-    return lst
+import pickle
 
 class Configure(config.base.Configure):
   def __init__(self, framework):
@@ -52,6 +44,7 @@ class Configure(config.base.Configure):
     help.addArgument('PETSc', '-with-xsdk-defaults', nargs.ArgBool(None, 0, 'Set the following as defaults for the xSDK standard: --enable-debug=1, --enable-shared=1, --with-precision=double, --with-index-size=32, locate blas/lapack automatically'))
     help.addArgument('PETSc', '-known-has-attribute-aligned=<bool>',nargs.ArgBool(None, None, 'Indicates __attribute((aligned(16)) directive works (the usual test will be skipped)'))
     help.addArgument('PETSc', '-with-display=<x11display>',       nargs.Arg(None, '', 'Specifiy DISPLAY env variable for use with matlab test)'))
+    help.addArgument('PETSc', '-with-package-scripts=<pyscripts>',nargs.ArgFileList(None,None,'Specify configure package scripts for user provided packages'))
     return
 
   def registerPythonFile(self,filename,directory):
@@ -125,16 +118,15 @@ class Configure(config.base.Configure):
     self.functions.headerPrefix  = self.headerPrefix
     self.libraries.headerPrefix  = self.headerPrefix
 
-    # Look for any user provided --download-xxx=directory packages
-    for arg in sys.argv:
-      if arg.startswith('--download-') and arg.find('=') > -1:
-        pname = arg[11:arg.find('=')]
-        if not hasattr(self,pname):
-          dname = os.path.dirname(arg[arg.find('=')+1:])
-          if os.path.isdir(dname) and not os.path.isfile(os.path.join(dname,pname+'.py')):
-            self.framework.logPrint('User is registering a new package: '+arg)
-            sys.path.append(dname)
-            self.registerPythonFile(pname+'.py','')
+    # Register user provided package scripts
+    if 'with-package-scripts' in self.framework.argDB:
+      for script in self.framework.argDB['with-package-scripts']:
+        if os.path.splitext(script)[1] != '.py':
+          raise RuntimeError('Only python scripts compatible with configure package script format should be specified! Invalid option -with-package-scripts='+script)
+        self.framework.logPrint('User is registering a new package script: '+script)
+        dname,fname = os.path.split(script)
+        if dname: sys.path.append(dname)
+        self.registerPythonFile(fname,'')
 
     # test for a variety of basic headers and functions
     headersC = map(lambda name: name+'.h', ['setjmp','dos', 'endian', 'fcntl', 'float', 'io', 'limits', 'malloc', 'pwd', 'search', 'strings',
@@ -176,7 +168,7 @@ class Configure(config.base.Configure):
     fd.write('cflags_dep='+self.compilers.dependenciesGenerationFlag.get('C','')+'\n')
     fd.write('ldflag_rpath='+self.setCompilers.CSharedLinkerFlag+'\n')
     self.setCompilers.popLanguage()
-    if hasattr(self.compilers, 'C++'):
+    if hasattr(self.compilers, 'CXX'):
       self.setCompilers.pushLanguage('C++')
       fd.write('cxxcompiler='+self.setCompilers.getCompiler()+'\n')
       fd.write('cxxflags_extra='+self.setCompilers.getCompilerFlags().strip()+'\n')
@@ -402,8 +394,6 @@ prepend-path PATH "%s"
     self.complibs = self.compilers.flibs+self.compilers.cxxlibs+self.compilers.LIBS.split()
     self.PETSC_WITH_EXTERNAL_LIB = self.libraries.toStringNoDupes(['-L${PETSC_DIR}/${PETSC_ARCH}/lib', self.petsclib]+self.packagelibs+self.complibs)
     self.PETSC_EXTERNAL_LIB_BASIC = self.libraries.toStringNoDupes(self.packagelibs+self.complibs)
-    if self.framework.argDB['prefix'] and self.setCompilers.CSharedLinkerFlag not in ['-L']:
-      string.replace(self.PETSC_EXTERNAL_LIB_BASIC,self.setCompilers.CSharedLinkerFlag+os.path.join(self.petscdir.dir,self.arch.arch,'lib'),self.setCompilers.CSharedLinkerFlag+os.path.join(self.installdir.dir,'lib'))
 
     self.addMakeMacro('PETSC_EXTERNAL_LIB_BASIC',self.PETSC_EXTERNAL_LIB_BASIC)
     allincludes = petscincludes + includes
@@ -468,7 +458,7 @@ prepend-path PATH "%s"
 
   def dumpConfigInfo(self):
     import time
-    fd = file(os.path.join(self.arch.arch,'include','petscconfiginfo.h'),'w')
+    fd = open(os.path.join(self.arch.arch,'include','petscconfiginfo.h'),'w')
     fd.write('static const char *petscconfigureoptions = "'+self.framework.getOptionsString(['configModules', 'optionsModule']).replace('\"','\\"')+'";\n')
     fd.close()
     return
@@ -480,7 +470,7 @@ prepend-path PATH "%s"
     import script
     def escape(s):
       return s.replace('"',r'\"').replace(r'\ ',r'\\ ')
-    fd = file(os.path.join(self.arch.arch,'include','petscmachineinfo.h'),'w')
+    fd = open(os.path.join(self.arch.arch,'include','petscmachineinfo.h'),'w')
     fd.write('static const char *petscmachineinfo = \"\\n\"\n')
     fd.write('\"-----------------------------------------\\n\"\n')
     buildhost = platform.node()
@@ -559,9 +549,6 @@ prepend-path PATH "%s"
         if elem not in unique and elem != '':
           unique.append(elem)
       return unique
-    try: reversed # reversed was added in Python-2.4
-    except NameError:
-      def reversed(lst): return lst[::-1]
     def nublast(lst):
       'Return a list containing the last occurrence of each unique entry in a list'
       return reversed(nub(reversed(lst)))
@@ -628,7 +615,7 @@ prepend-path PATH "%s"
         libvars.append(libvar)
       fd.write('mark_as_advanced (' + ' '.join(libvars) + ')\n')
       fd.write('set (PETSC_PACKAGE_LIBS ' + ' '.join(map(cmakeexpand,libvars)) + ')\n')
-      includes = filter(notstandardinclude,includes)
+      includes = list(filter(notstandardinclude,includes))
       fd.write('set (PETSC_PACKAGE_INCLUDES ' + ' '.join(map(lambda i: '"'+i+'"',includes)) + ')\n')
     fd = open(os.path.join(self.arch.arch,'lib','petsc','conf','PETScBuildInternal.cmake'), 'w')
     writeMacroDefinitions(fd)
@@ -770,6 +757,10 @@ prepend-path PATH "%s"
       self.addDefine('DEPRECATED(why)', '__attribute((deprecated))')
     else:
       self.addDefine('DEPRECATED(why)', ' ')
+    if self.checkCompile("""enum E {oldval __attribute((deprecated)), newval };""", ''):
+      self.addDefine('DEPRECATED_ENUM(why)', '__attribute((deprecated))')
+    else:
+      self.addDefine('DEPRECATED_ENUM(why)', ' ')
     self.popLanguage()
 
   def configureAlign(self):
@@ -797,7 +788,7 @@ fprintf(f, "%lu\\n", (unsigned long)sizeof(struct mystruct));
       self.pushLanguage(self.languages.clanguage)
       try:
         if self.checkRun(includes, body) and os.path.exists(filename):
-          f    = file(filename)
+          f    = open(filename)
           size = int(f.read())
           f.close()
           os.remove(filename)
@@ -983,7 +974,7 @@ fprintf(f, "%lu\\n", (unsigned long)sizeof(struct mystruct));
   def configureDefaultArch(self):
     conffile = os.path.join('lib','petsc','conf', 'petscvariables')
     if self.framework.argDB['with-default-arch']:
-      fd = file(conffile, 'w')
+      fd = open(conffile, 'w')
       fd.write('PETSC_ARCH='+self.arch.arch+'\n')
       fd.write('PETSC_DIR='+self.petscdir.dir+'\n')
       fd.write('include '+os.path.join('$(PETSC_DIR)','$(PETSC_ARCH)','lib','petsc','conf','petscvariables')+'\n')
@@ -1013,7 +1004,7 @@ fprintf(f, "%lu\\n", (unsigned long)sizeof(struct mystruct));
         del args['optionsModule']
     if not 'PETSC_ARCH' in args:
       args['PETSC_ARCH'] = 'PETSC_ARCH='+str(self.arch.arch)
-    f = file(scriptName, 'w')
+    f = open(scriptName, 'w')
     f.write('#!'+sys.executable+'\n')
     f.write('if __name__ == \'__main__\':\n')
     f.write('  import sys\n')
@@ -1113,7 +1104,7 @@ fprintf(f, "%lu\\n", (unsigned long)sizeof(struct mystruct));
     self.dumpCMakeLists()
     # need to save the current state of BuildSystem so that postProcess() packages can read it in and perhaps run make install
     self.framework.storeSubstitutions(self.framework.argDB)
-    self.framework.argDB['configureCache'] = cPickle.dumps(self.framework)
+    self.framework.argDB['configureCache'] = pickle.dumps(self.framework)
     self.framework.argDB.save(force = True)
     self.cmakeBoot()
     self.DumpPkgconfig()
