@@ -4,6 +4,10 @@ configure_options = [
 
   '--useThreads=0', # for some reason cmake hangs when invoked from configure on bsd?
 
+  'COPTFLAGS=-g -O',
+  'FOPTFLAGS=-g -O',
+  'CXXOPTFLAGS=-g -O',
+
   '--with-clanguage=cxx',
   '--with-scalar-type=complex',
 
@@ -26,6 +30,7 @@ configure_options = [
   #'--download-suitesparse=1',
   #'--download-chaco=1',
   #'--download-spai=1',
+  '--download-p4est=1',
 
   ]
 
