@@ -28,6 +28,8 @@ configure_options = [
   '--download-netcdf=1',
   '--download-moab=1',
   '--download-saws',
+  '--download-codipack=1',
+  '--download-adblaslapack=1',
   ]
 
 if __name__ == '__main__':
