@@ -4,11 +4,14 @@ configure_options = [
   '--with-cc=/home/petsc/soft/mpich-3.3b1/bin/mpicc',
   '--with-fc=/home/petsc/soft/mpich-3.3b1/bin/mpif90',
   '--with-cxx=/home/petsc/soft/mpich-3.3b1/bin/mpicxx',
+  '--with-debugging=1',
+  'COPTFLAGS=-g -O',
+  'FOPTFLAGS=-g -O',
+  'CXXOPTFLAGS=-g -O',
   '--with-clanguage=cxx',
   '--with-scalar-type=complex',
   '--with-cxx-dialect=C++11',
   '--with-64-bit-indices=1',
-  '--with-debugging=1',
   'DATAFILESPATH=/home/petsc/datafiles',
   ]
 
