@@ -1,8 +1,11 @@
 #!/usr/bin/env python
 
 configure_options = [
+  'COPTFLAGS=-g -O',
+  'FOPTFLAGS=-g -O',
+  'CXXOPTFLAGS=-g -O',
   '--download-mpich=1',
-  #'--with-gcov=1'
+  '--with-gcov=1'
   ]
 
 if __name__ == '__main__':
