@@ -1,6 +1,9 @@
 #!/usr/bin/env python
 
 configure_options = [
+  'COPTFLAGS=-g -O',
+  'FOPTFLAGS=-g -O',
+  'CXXOPTFLAGS=-g -O',
   '--with-64-bit-indices=1',
   '--download-openmpi=1', #download-mpich works - but system mpich gives wierd errors with superlu_dist+parmeits [with shared/64-bit-indices]?
   '--download-metis=1',
@@ -12,6 +15,8 @@ configure_options = [
   '--donwload-suitesparse=1',
   '--with-cxx-dialect=C++11',
   '--download-cmake',  # superlu_dist requires a newer cmake
+  '--download-p4est=1',
+  '--with-zlib=1',
   ]
 
 if __name__ == '__main__':
