@@ -1,6 +1,9 @@
 #!/usr/bin/env python
 
 configure_options = [
+  'COPTFLAGS=-g -O',
+  'FOPTFLAGS=-g -O',
+  'CXXOPTFLAGS=-g -O',
   '--with-gcov=1',
   #'--download-mpich=1', use system MPI as elemental fails with this
   '--download-fblaslapack=1',
@@ -31,6 +34,7 @@ configure_options = [
   '--download-yaml',
   '--download-ml',
   '--download-sundials',
+  '--download-p4est=1',
   ]
 
 if __name__ == '__main__':
