@@ -1,6 +1,9 @@
 #!/usr/bin/env python
 
 configure_options = [
+  'COPTFLAGS=-g -O',
+  'FOPTFLAGS=-g -O',
+  'CXXOPTFLAGS=-g -O',
   '--download-mpich=1',
   '--download-fblaslapack=1',
   '--download-hypre=1',
@@ -22,6 +25,10 @@ configure_options = [
   '--download-moab=1',
   '--download-chaco=1',
   '--download-revolve=1',
+  '--download-codipack=1',
+  '--download-adblaslapack=1',
+  '--download-p4est=1',
+  '--download-zlib=1',
   ]
 
 if __name__ == '__main__':
