@@ -37,6 +37,9 @@ configure_options = [
   '--download-saws',
   '--download-revolve=1',
   '--download-ctetgen=1',
+  '--download-mfem=1',
+  '--download-glvis=1',
+  '--with-opengl=1',
   ]
 
 if __name__ == '__main__':
