@@ -125,7 +125,7 @@ def parseLoopArgs(varset):
   Given:   String containing loop variables
   Return: tuple containing separate/shared and string of loop vars
   """
-  keynm=varset.split("{{")[0].strip()
+  keynm=varset.split("{{")[0].strip().lstrip('-')
   if not keynm.strip(): keynm='nsize'
   lvars=varset.split('{{')[1].split('}')[0]
   suffx=varset.split('{{')[1].split('}')[1]
@@ -148,7 +148,7 @@ def _getSeparateTestvars(testDict):
 
   # Now check args
   if 'args' not in testDict: return sepvars
-  for varset in re.split('-(?=[a-zA-Z])',testDict['args']):
+  for varset in re.split('(^|\W)-(?=[a-zA-Z])',testDict['args']):
     if not varset.strip(): continue
     if '{{' in varset:
       # Assuming only one for loop per var specification
@@ -165,7 +165,7 @@ def _getNewArgs(args):
   """
   newargs=''
   if not args.strip(): return args
-  for varset in re.split('-(?=[a-zA-Z])',args):
+  for varset in re.split('(^|\W)-(?=[a-zA-Z])',args):
     if not varset.strip(): continue
     if '{{' not in varset:
       if 'separate' not in varset:
@@ -196,7 +196,7 @@ def _getVarVals(findvar,testDict):
   if not save_vals: raise Exception("Could not find separate_testvar: "+findvar)
   return save_vals
 
-def genTestsSeparateTestvars(intests,indicts):
+def genTestsSeparateTestvars(intests,indicts,final=False):
   """
   Given: testname, sdict with 'separate_testvars
   Return: testnames,sdicts: List of generated tests
@@ -239,6 +239,11 @@ def genTestsSeparateTestvars(intests,indicts):
       testnames+=sep_testnames
       sdicts+=sep_dicts
     else:
+      # These are plain vanilla tests (no subtests, no loops) that 
+      # do not have a suffix.  This makes the targets match up with
+      # the output file (testname_1.out)
+      if final:
+          if '_' not in testname: testname+='_1'
       testnames.append(testname)
       sdicts.append(sdict)
   return testnames,sdicts
@@ -304,7 +309,7 @@ def splitTests(testname,sdict):
   # Order: Parent sep_tv, subtests suffix, subtests sep_tv
   testnames,sdicts=genTestsSeparateTestvars([testname],[sdict])
   testnames,sdicts=genTestsSubtestSuffix(testnames,sdicts)
-  testnames,sdicts=genTestsSeparateTestvars(testnames,sdicts)
+  testnames,sdicts=genTestsSeparateTestvars(testnames,sdicts,final=True)
 
   # Because I am altering the list, I do this in passes.  Inelegant 
 
@@ -346,7 +351,10 @@ def parseTest(testStr,srcfile,verbosity):
     if not var in acceptedkeys: raise Exception("Not a defined key: "+var+" from:  "+line)
     # Start by seeing if we are in a subtest
     if line.startswith(" "):
-      subdict[subtestname][var]=val
+      if var in subdict[subtestname]:
+        subdict[subtestname][var]+=" "+val 
+      else: 
+        subdict[subtestname][var]=val
       if not indentlevel: indentlevel=indentcount
       #if indentlevel!=indentcount: print("Error in indentation:", ln)
     # Determine subtest name and make dict
@@ -368,11 +376,11 @@ def parseTest(testStr,srcfile,verbosity):
         subdict[var]=val
       if var=="suffix":
         if len(val)>0:
-          testname=testname+"_"+val
+          testname+="_"+val
 
   if len(comments): subdict['comments']="\n".join(comments).lstrip("\n")
-  # A test block can create multiple tests.  This does
-  # that logic
+
+  # A test block can create multiple tests.  This does that logic
   testnames,subdicts=splitTests(testname,subdict)
   return testnames,subdicts
 
@@ -490,7 +498,7 @@ def printExParseDict(rDict):
   indent="   "
   for sfile in rDict:
     print(sfile)
-    sortkeys=rDict[sfile].keys()
+    sortkeys=list(rDict[sfile].keys())
     sortkeys.sort()
     for runex in sortkeys:
       print(indent+runex)
