@@ -9,6 +9,7 @@
 #include <petscmatlab.h>
 #include <petscdraw.h>
 
+#include <petscsf.h>
 #include <petscdmda.h>
 #include <petscdmcomposite.h>
 #include <petscdmpatch.h>
@@ -16,6 +17,8 @@
 #include <petscdmredundant.h>
 #include <petscdmshell.h>
 #include <petscdmsliced.h>
+#include <petscdmswarm.h>
+#include <petscdmstag.h>
 
 #include <petscdt.h>
 #include <petscds.h>
