@@ -162,6 +162,17 @@ struct _n_Boundary {
   DMBoundary  next;
 };
 
+typedef struct _n_Field {
+  PetscObject disc;         /* Field discretization, or a PetscContainer with the field name */
+  DMLabel     label;        /* Label defining the domain of definition of the field */
+  PetscBool   adjacency[2]; /* Flags for defining variable influence (adjacency) for each field [use cone() or support() first, use the transitive closure] */
+} RegionField;
+
+typedef struct _n_Space {
+  PetscDS ds;
+  DMLabel label;
+} DMSpace;
+
 PETSC_EXTERN PetscErrorCode DMDestroyLabelLinkList(DM);
 
 struct _p_DM {
@@ -186,6 +197,7 @@ struct _p_DM {
   PetscBool               structure_only; /* Flag indicating the DMCreateMatrix() create matrix structure without values */
   PetscInt                levelup,leveldown;  /* if the DM has been obtained by refining (or coarsening) this indicates how many times that process has been used to generate this DM */
   PetscBool               setupcalled;        /* Indicates that the DM has been set up, methods that modify a DM such that a fresh setup is required should reset this flag */
+  PetscBool               setfromoptionscalled;
   void                    *data;
   /* Hierarchy / Submeshes */
   DM                      coarseMesh;
@@ -204,6 +216,7 @@ struct _p_DM {
   PetscSF                 sfNatural;            /* SF mapping to the "natural" ordering */
   PetscBool               useNatural;           /* Create the natural SF */
   /* Allows a non-standard data layout */
+  PetscBool               adjacency[2];         /* [use cone() or support() first, use the transitive closure] */
   PetscSection            defaultSection;       /* Layout for local vectors */
   PetscSection            defaultGlobalSection; /* Layout for global vectors */
   PetscLayout             map;
@@ -222,9 +235,13 @@ struct _p_DM {
   /* Null spaces -- of course I should make this have a variable number of fields */
   /*   I now believe this might not be the right way: see below */
   NullSpaceFunc           nullspaceConstructors[10];
+  NullSpaceFunc           nearnullspaceConstructors[10];
   /* Fields are represented by objects */
-  PetscDS                 prob;
-  DMBoundary              boundary;          /* List of boundary conditions */
+  PetscInt                Nf;                   /* Number of fields defined on the total domain */
+  RegionField            *fields;               /* Array of discretization fields with regions of validity */
+  DMBoundary              boundary;             /* List of boundary conditions */
+  PetscInt                Nds;                  /* Number of discrete systems defined on the total domain */
+  DMSpace                *probs;                /* Array of discrete systems */
   /* Output structures */
   DM                      dmBC;                 /* The DM with boundary conditions in the global DM */
   PetscInt                outputSequenceNum;    /* The current sequence number for output */
@@ -244,8 +261,8 @@ PETSC_EXTERN PetscLogEvent DM_CreateRestriction;
 
 PETSC_EXTERN PetscErrorCode DMCreateGlobalVector_Section_Private(DM,Vec*);
 PETSC_EXTERN PetscErrorCode DMCreateLocalVector_Section_Private(DM,Vec*);
-PETSC_EXTERN PetscErrorCode DMCreateSubDM_Section_Private(DM,PetscInt,const PetscInt[],IS*,DM*);
-PETSC_EXTERN PetscErrorCode DMCreateSuperDM_Section_Private(DM[],PetscInt,IS**,DM*);
+
+PETSC_EXTERN PetscErrorCode DMView_GLVis(DM,PetscViewer,PetscErrorCode(*)(DM,PetscViewer));
 
 /*
 
