@@ -120,9 +120,9 @@ struct _p_KSP {
   void       *data;                      /* holder for misc stuff associated
                                    with a particular iterative solver */
 
-  PetscBool         view,   viewPre,   viewReason,   viewMat,   viewPMat,   viewRhs,   viewSol,   viewMatExp,   viewEV,   viewSV,   viewEVExp,   viewFinalRes,   viewPOpExp;
-  PetscViewer       viewer, viewerPre, viewerReason, viewerMat, viewerPMat, viewerRhs, viewerSol, viewerMatExp, viewerEV, viewerSV, viewerEVExp, viewerFinalRes, viewerPOpExp;
-  PetscViewerFormat format, formatPre, formatReason, formatMat, formatPMat, formatRhs, formatSol, formatMatExp, formatEV, formatSV, formatEVExp, formatFinalRes, formatPOpExp;
+  PetscBool         view,   viewPre,   viewReason,   viewMat,   viewPMat,   viewRhs,   viewSol,   viewMatExp,   viewEV,   viewSV,   viewEVExp,   viewFinalRes,   viewPOpExp,   viewDScale;
+  PetscViewer       viewer, viewerPre, viewerReason, viewerMat, viewerPMat, viewerRhs, viewerSol, viewerMatExp, viewerEV, viewerSV, viewerEVExp, viewerFinalRes, viewerPOpExp, viewerDScale;
+  PetscViewerFormat format, formatPre, formatReason, formatMat, formatPMat, formatRhs, formatSol, formatMatExp, formatEV, formatSV, formatEVExp, formatFinalRes, formatPOpExp, formatDScale;
 
   /* ----------------Default work-area management -------------------- */
   PetscInt       nwork;
@@ -342,9 +342,27 @@ PETSC_EXTERN PetscLogEvent KSP_Solve_FS_U;
 PETSC_INTERN PetscErrorCode MatGetSchurComplement_Basic(Mat,IS,IS,IS,IS,MatReuse,Mat*,MatSchurComplementAinvType,MatReuse,Mat*);
 PETSC_INTERN PetscErrorCode PCPreSolveChangeRHS(PC,PetscBool*);
 
-/*
-    Either generate an error or mark as diverged when a scalar from an inner product is Nan or Inf
-*/
+/*MC
+   KSPCheckDot - Checks if the result of a dot product used by the corresponding KSP contains Inf or NaN. These indicate that the previous 
+      application of the preconditioner generated an error
+
+   Collective on KSP
+
+   Input Parameter:
+.  ksp - the linear solver (KSP) context.
+
+   Output Parameter:
+.  beta - the result of the inner product
+
+   Level: developer
+
+   Developer Note:
+   this is used to manage returning from KSP solvers whose preconditioners have failed in some way
+
+.keywords: KSP, PC, divergence, convergence
+
+.seealso: KSPCreate(), KSPSetType(), KSP, KSPCheckNorm(), KSPCheckSolve()
+M*/
 #define KSPCheckDot(ksp,beta)           \
   if (PetscIsInfOrNanScalar(beta)) { \
     if (ksp->errorifnotconverged) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_NOT_CONVERGED,"KSPSolve has not converged due to Nan or Inf inner product");\
@@ -352,11 +370,11 @@ PETSC_INTERN PetscErrorCode PCPreSolveChangeRHS(PC,PetscBool*);
       PetscErrorCode ierr;\
       PCFailedReason pcreason;\
       PetscInt       sendbuf,pcreason_max; \
-      ierr = PCGetSetUpFailedReason(ksp->pc,&pcreason);CHKERRQ(ierr);\
+      ierr = PCGetFailedReason(ksp->pc,&pcreason);CHKERRQ(ierr);\
       sendbuf = (PetscInt)pcreason; \
       ierr = MPI_Allreduce(&sendbuf,&pcreason_max,1,MPIU_INT,MPI_MAX,PetscObjectComm((PetscObject)ksp));CHKERRQ(ierr); \
       if (pcreason_max) {\
-        ksp->reason = KSP_DIVERGED_PCSETUP_FAILED;\
+        ksp->reason = KSP_DIVERGED_PC_FAILED;\
         ierr        = VecSetInf(ksp->vec_sol);CHKERRQ(ierr);\
       } else {\
         ksp->reason = KSP_DIVERGED_NANORINF;\
@@ -365,9 +383,27 @@ PETSC_INTERN PetscErrorCode PCPreSolveChangeRHS(PC,PetscBool*);
     }\
   }
 
-/*
-    Either generate an error or mark as diverged when a real from a norm is Nan or Inf
-*/
+/*MC
+   KSPCheckNorm - Checks if the result of a norm used by the corresponding KSP contains Inf or NaN. These indicate that the previous
+      application of the preconditioner generated an error
+
+   Collective on KSP
+
+   Input Parameter:
+.  ksp - the linear solver (KSP) context.
+
+   Output Parameter:
+.  beta - the result of the norm
+
+   Level: developer
+
+   Developer Note:
+   this is used to manage returning from KSP solvers whose preconditioners have failed in some way
+
+.keywords: KSP, PC, divergence, convergence
+
+.seealso: KSPCreate(), KSPSetType(), KSP, KSPCheckDot(), KSPCheckSolve()
+M*/
 #define KSPCheckNorm(ksp,beta)           \
   if (PetscIsInfOrNanReal(beta)) { \
     if (ksp->errorifnotconverged) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_NOT_CONVERGED,"KSPSolve has not converged due to Nan or Inf norm");\
@@ -375,11 +411,11 @@ PETSC_INTERN PetscErrorCode PCPreSolveChangeRHS(PC,PetscBool*);
       PetscErrorCode ierr;\
       PCFailedReason pcreason;\
       PetscInt       sendbuf,pcreason_max; \
-      ierr = PCGetSetUpFailedReason(ksp->pc,&pcreason);CHKERRQ(ierr);\
+      ierr = PCGetFailedReason(ksp->pc,&pcreason);CHKERRQ(ierr);\
       sendbuf = (PetscInt)pcreason; \
       ierr = MPI_Allreduce(&sendbuf,&pcreason_max,1,MPIU_INT,MPI_MAX,PetscObjectComm((PetscObject)ksp));CHKERRQ(ierr); \
       if (pcreason_max) {\
-        ksp->reason = KSP_DIVERGED_PCSETUP_FAILED;\
+        ksp->reason = KSP_DIVERGED_PC_FAILED;\
         ierr        = VecSetInf(ksp->vec_sol);CHKERRQ(ierr);\
       } else {\
         ksp->reason = KSP_DIVERGED_NANORINF;\
