@@ -70,7 +70,7 @@ struct _MatOps {
   PetscErrorCode (*ilufactorsymbolic)(Mat,Mat,IS,IS,const MatFactorInfo*);
   PetscErrorCode (*iccfactorsymbolic)(Mat,Mat,IS,const MatFactorInfo*);
   PetscErrorCode (*getdiagonalblock)(Mat,Mat*);
-  PetscErrorCode (*placeholder_33)(Mat);
+  PetscErrorCode (*freeintermediatedatastructures)(Mat);
   /*34*/
   PetscErrorCode (*duplicate)(Mat,MatDuplicateOption,Mat*);
   PetscErrorCode (*forwardsolve)(Mat,Vec,Vec);
@@ -217,13 +217,13 @@ struct _MatOps {
 PETSC_EXTERN PetscErrorCode MatRegisterOp(MPI_Comm, const char[], PetscVoidFunction, const char[], PetscInt, ...);
 PETSC_EXTERN PetscErrorCode MatQueryOp(MPI_Comm, PetscVoidFunction*, const char[], PetscInt, ...);
 
-typedef struct _p_MatBaseName* MatBaseName;
-struct _p_MatBaseName {
-  char        *bname,*sname,*mname;
-  MatBaseName next;
+typedef struct _p_MatRootName* MatRootName;
+struct _p_MatRootName {
+  char        *rname,*sname,*mname;
+  MatRootName next;
 };
 
-PETSC_EXTERN MatBaseName MatBaseNameList;
+PETSC_EXTERN MatRootName MatRootNameList;
 
 /*
    Utility private matrix routines
@@ -306,8 +306,8 @@ struct _MatStash {
   PetscInt      reproduce_count;
 
   /* The following variables are used for BTS communication */
-  PetscBool      subset_off_proc; /* Subsequent assemblies will set a subset (perhaps equal) of off-process entries set on first assembly */
-  PetscBool      use_status;      /* Use MPI_Status to determine number of items in each message */
+  PetscBool      first_assembly_done;   /* Is the first time matrix assembly done? */
+  PetscBool      use_status;            /* Use MPI_Status to determine number of items in each message */
   PetscMPIInt    nsendranks;
   PetscMPIInt    nrecvranks;
   PetscMPIInt    *sendranks;
@@ -333,6 +333,9 @@ struct _MatStash {
   InsertMode     *insertmode;   /* Pointer to check mat->insertmode and set upon message arrival in case no local values have been set. */
 };
 
+#if !defined(PETSC_HAVE_MPIUNI)
+PETSC_INTERN PetscErrorCode MatStashScatterDestroy_BTS(MatStash*);
+#endif
 PETSC_INTERN PetscErrorCode MatStashCreate_Private(MPI_Comm,PetscInt,MatStash*);
 PETSC_INTERN PetscErrorCode MatStashDestroy_Private(MatStash*);
 PETSC_INTERN PetscErrorCode MatStashScatterEnd_Private(MatStash*);
@@ -394,10 +397,10 @@ struct _p_Mat {
   PetscBool              symmetric_set,hermitian_set,structurally_symmetric_set,spd_set; /* if true, then corresponding flag is correct*/
   PetscBool              symmetric_eternal;
   PetscBool              nooffprocentries,nooffproczerorows;
-  PetscBool              subsetoffprocentries;
+  PetscBool              assembly_subset; /* set by MAT_SUBSET_OFF_PROC_ENTRIES */
   PetscBool              submat_singleis; /* for efficient PCSetUP_ASM() */
   PetscBool              structure_only;
-#if defined(PETSC_HAVE_VIENNACL) || defined(PETSC_HAVE_VECCUDA)
+#if defined(PETSC_HAVE_VIENNACL) || defined(PETSC_HAVE_CUDA)
   PetscOffloadFlag       valid_GPU_matrix; /* flag pointing to the matrix on the gpu*/
 #endif
   void                   *spptr;          /* pointer for special library like SuperLU */
@@ -417,6 +420,7 @@ struct _p_Mat {
 
 PETSC_INTERN PetscErrorCode MatAXPY_Basic(Mat,PetscScalar,Mat,MatStructure);
 PETSC_INTERN PetscErrorCode MatAXPY_BasicWithPreallocation(Mat,Mat,PetscScalar,Mat,MatStructure);
+PETSC_INTERN PetscErrorCode MatAXPY_Basic_Preallocate(Mat,Mat,Mat*);
 
 /*
     Utility for MatFactor (Schur complement)
@@ -442,6 +446,7 @@ struct _MatPartitioningOps {
   PetscErrorCode (*setfromoptions)(PetscOptionItems*,MatPartitioning);
   PetscErrorCode (*destroy)(MatPartitioning);
   PetscErrorCode (*view)(MatPartitioning,PetscViewer);
+  PetscErrorCode (*improve)(MatPartitioning,IS*);
 };
 
 struct _p_MatPartitioning {
