@@ -36,26 +36,27 @@ struct _p_PetscDS {
   void        *data;              /* Implementation object */
   PetscDS     *subprobs;          /* The subspaces for each dimension */
   PetscBool    setup;             /* Flag for setup */
-  PetscInt     Nf;                /* The number of solution fields */
-  PetscBool   *implicit;          /* Flag for implicit or explicit solve for each field */
-  PetscBool    defaultAdj[2];     /* [use cone() or support() first, use the transitive closure] for the case of no fields */
-  PetscBool   *adjacency;         /* Flags for defining variable influence (adjacency) for each field [use cone() or support() first, use the transitive closure] */
-  PetscBool    useJacPre;         /* Flag for using the Jacobian preconditioner */
-  PetscObject *disc;              /* The discretization for each solution field (PetscFE, PetscFV, etc.) */
-  PetscPointFunc   *obj;          /* Scalar integral (like an objective function) */
-  PetscPointFunc   *f;            /* Weak form integrands for F, f_0, f_1 */
-  PetscPointJac    *g;            /* Weak form integrands for J = dF/du, g_0, g_1, g_2, g_3 */
-  PetscPointJac    *gp;           /* Weak form integrands for preconditioner for J, g_0, g_1, g_2, g_3 */
-  PetscPointJac    *gt;           /* Weak form integrands for dF/du_t, g_0, g_1, g_2, g_3 */
-  PetscBdPointFunc *fBd;          /* Weak form boundary integrands F_bd, f_0, f_1 */
-  PetscBdPointJac  *gBd;          /* Weak form boundary integrands J_bd = dF_bd/du, g_0, g_1, g_2, g_3 */
-  PetscRiemannFunc *r;            /* Riemann solvers */
-  PetscPointFunc   *update;       /* Direct update of field coefficients */
-  PetscSimplePointFunc *exactSol; /* Exact solutions for each field */
-  PetscInt          numConstants; /* Number of constants passed to point functions */
-  PetscScalar      *constants;    /* Array of constants passed to point functions */
-  void       **ctx;               /* User contexts for each field */
+  PetscBool    isHybrid;          /* Flag for hybrid cell (this is crappy, but the only thing I can see to do now) */
   PetscInt     dimEmbed;          /* The real space coordinate dimension */
+  PetscInt     Nf;                /* The number of solution fields */
+  PetscObject *disc;              /* The discretization for each solution field (PetscFE, PetscFV, etc.) */
+  /* Equations */
+  DSBoundary            boundary;      /* Linked list of boundary conditions */
+  PetscBool             useJacPre;     /* Flag for using the Jacobian preconditioner */
+  PetscBool            *implicit;      /* Flag for implicit or explicit solve for each field */
+  PetscPointFunc       *obj;           /* Scalar integral (like an objective function) */
+  PetscPointFunc       *f;             /* Weak form integrands for F, f_0, f_1 */
+  PetscPointJac        *g;             /* Weak form integrands for J = dF/du, g_0, g_1, g_2, g_3 */
+  PetscPointJac        *gp;            /* Weak form integrands for preconditioner for J, g_0, g_1, g_2, g_3 */
+  PetscPointJac        *gt;            /* Weak form integrands for dF/du_t, g_0, g_1, g_2, g_3 */
+  PetscBdPointFunc     *fBd;           /* Weak form boundary integrands F_bd, f_0, f_1 */
+  PetscBdPointJac      *gBd;           /* Weak form boundary integrands J_bd = dF_bd/du, g_0, g_1, g_2, g_3 */
+  PetscRiemannFunc     *r;             /* Riemann solvers */
+  PetscPointFunc       *update;        /* Direct update of field coefficients */
+  PetscSimplePointFunc *exactSol;      /* Exact solutions for each field */
+  PetscInt              numConstants;  /* Number of constants passed to point functions */
+  PetscScalar          *constants;     /* Array of constants passed to point functions */
+  void                 **ctx;          /* User contexts for each field */
   /* Computed sizes */
   PetscInt     totDim;            /* Total system dimension */
   PetscInt     totComp;           /* Total field components */
@@ -75,7 +76,6 @@ struct _p_PetscDS {
   PetscReal   *x;                 /* Workspace for computing real coordinates */
   PetscScalar *f0, *f1;           /* Point evaluations of weak form residual integrands */
   PetscScalar *g0, *g1, *g2, *g3; /* Point evaluations of weak form Jacobian integrands */
-  DSBoundary   boundary;          /* Linked list of boundary conditions */
 };
 
 typedef struct {
