@@ -76,8 +76,41 @@ struct _TSTrajectoryOps {
   PetscErrorCode (*setup)(TSTrajectory,TS);
 };
 
+/* TSHistory is an helper object that allows inquiring
+   the TSTrajectory by time and not by the step number only */
+typedef struct _n_TSHistory* TSHistory;
+
 struct _p_TSTrajectory {
   PETSCHEADER(struct _TSTrajectoryOps);
+  TSHistory tsh;        /* associates times to unique step ids */
+  /* stores necessary data to reconstruct states and derivatives via Lagrangian interpolation */
+  struct {
+    PetscInt    order;  /* interpolation order */
+    Vec         *W;     /* work vectors */
+    PetscScalar *L;     /* workspace for Lagrange basis */
+    PetscReal   *T;     /* Lagrange times (stored) */
+    Vec         *WW;    /* just an array of pointers */
+    PetscBool   *TT;    /* workspace for Lagrange */
+    PetscReal   *TW;    /* Lagrange times (workspace) */
+
+    /* caching */
+    PetscBool caching;
+    struct {
+      PetscObjectId    id;
+      PetscObjectState state;
+      PetscReal        time;
+      PetscInt         step;
+    } Ucached;
+    struct {
+      PetscObjectId    id;
+      PetscObjectState state;
+      PetscReal        time;
+      PetscInt         step;
+    } Udotcached;
+  } lag;
+  Vec            U,Udot;                  /* used by TSTrajectory{Get|Restore}UpdatedHistoryVecs */
+  PetscBool      solution_only;           /* whether we dump just the solution or also the stages */
+  PetscBool      adjoint_solve_mode;      /* whether we will use the Trajectory inside a TSAdjointSolve() or not */
   PetscViewer    monitor;
   PetscInt       setupcalled;             /* true if setup has been called */
   PetscInt       recomps;                 /* counter for recomputations in the adjoint run */
@@ -264,9 +297,11 @@ struct _p_TSAdapt {
   PetscReal   clip[2];            /* admissible time step decrease/increase factors */
   PetscReal   dt_min,dt_max;      /* admissible minimum and maximum time step */
   PetscReal   scale_solve_failed; /* scale step by this factor if solver (linear or nonlinear) fails. */
+  PetscReal   matchstepfac[2];    /* factors to control the behaviour of matchstep */
   NormType    wnormtype;
   PetscViewer monitor;
-  PetscInt    timestepjustincreased;
+  PetscInt    timestepjustdecreased_delay; /* number of timesteps after a decrease in the timestep before the timestep can be increased */
+  PetscInt    timestepjustdecreased;
 };
 
 typedef struct _p_DMTS *DMTS;
@@ -337,7 +372,7 @@ struct _n_TSEvent {
   PetscScalar    *fvalue_right;    /* value of event function at the right end-point of the event interval */
   PetscInt       *side;            /* Used for detecting repetition of end-point, -1 => left, +1 => right */
   PetscReal       timestep_prev;   /* previous time step */
-  PetscReal       timestep_orig;   /* initial time step */
+  PetscReal       timestep_posteventinterval;  /* time step immediately after the event interval */
   PetscBool      *zerocrossing;    /* Flag to signal zero crossing detection */
   PetscErrorCode  (*eventhandler)(TS,PetscReal,Vec,PetscScalar*,void*); /* User event handler function */
   PetscErrorCode  (*postevent)(TS,PetscInt,PetscInt[],PetscReal,Vec,PetscBool,void*); /* User post event function */
@@ -395,6 +430,12 @@ struct _n_TSMonitorLGCtx {
   void           *transformctx;
 };
 
+struct _n_TSMonitorSPCtx{
+  PetscDrawSP    sp;
+  PetscInt       howoften; /* when > 0 uses step % howoften, when negative only final solution plotted */
+  PetscInt       ksp_its, snes_its;
+};
+
 struct _n_TSMonitorEnvelopeCtx {
   Vec max,min;
 };
@@ -415,8 +456,15 @@ PETSC_STATIC_INLINE PetscErrorCode TSCheckImplicitTerm(TS ts)
   PetscFunctionReturn(0);
 }
 
+PETSC_EXTERN PetscErrorCode TSGetRHSMats_Private(TS,Mat*,Mat*);
+/* this is declared here as TSHistory is not public */
+PETSC_EXTERN PetscErrorCode TSAdaptHistorySetTSHistory(TSAdapt,TSHistory,PetscBool);
+
+PETSC_INTERN PetscErrorCode TSTrajectoryReconstruct_Private(TSTrajectory,TS,PetscReal,Vec,Vec);
+
 PETSC_EXTERN PetscLogEvent TSTrajectory_Set;
 PETSC_EXTERN PetscLogEvent TSTrajectory_Get;
+PETSC_EXTERN PetscLogEvent TSTrajectory_GetVecs;
 PETSC_EXTERN PetscLogEvent TSTrajectory_DiskWrite;
 PETSC_EXTERN PetscLogEvent TSTrajectory_DiskRead;
 
