@@ -283,8 +283,11 @@ PETSC_STATIC_INLINE PetscErrorCode VecScatterMemcpyPlanExecute_Scatter(PetscInt
   PetscFunctionReturn(0);
 }
 
-PETSC_INTERN PetscErrorCode VecScatterGetTypes_Private(VecScatter,VecScatterFormat*,VecScatterFormat*);
-PETSC_INTERN PetscErrorCode VecScatterIsSequential_Private(VecScatter_Common*,PetscBool*);
+PETSC_EXTERN PetscErrorCode VecScatterGetRemoteCount_Private(VecScatter,PetscBool,PetscInt*,PetscInt*);
+PETSC_EXTERN PetscErrorCode VecScatterGetRemote_Private(VecScatter,PetscBool,PetscInt*,const PetscInt**,const PetscInt**,const PetscMPIInt**,PetscInt*);
+PETSC_EXTERN PetscErrorCode VecScatterGetRemoteOrdered_Private(VecScatter,PetscBool,PetscInt*,const PetscInt**,const PetscInt**,const PetscMPIInt**,PetscInt*);
+PETSC_EXTERN PetscErrorCode VecScatterRestoreRemote_Private(VecScatter,PetscBool,PetscInt*,const PetscInt**,const PetscInt**,const PetscMPIInt**,PetscInt*);
+PETSC_EXTERN PetscErrorCode VecScatterRestoreRemoteOrdered_Private(VecScatter,PetscBool,PetscInt*,const PetscInt**,const PetscInt**,const PetscMPIInt**,PetscInt*);
 
 typedef struct _VecScatterOps *VecScatterOps;
 struct _VecScatterOps {
@@ -292,28 +295,67 @@ struct _VecScatterOps {
   PetscErrorCode (*end)(VecScatter,Vec,Vec,InsertMode,ScatterMode);
   PetscErrorCode (*copy)(VecScatter,VecScatter);
   PetscErrorCode (*destroy)(VecScatter);
+  PetscErrorCode (*setup)(VecScatter);
   PetscErrorCode (*view)(VecScatter,PetscViewer);
   PetscErrorCode (*viewfromoptions)(VecScatter,const char prefix[],const char name[]);
-  PetscErrorCode (*remap)(VecScatter,PetscInt *,PetscInt*);
+  PetscErrorCode (*remap)(VecScatter,const PetscInt *,const PetscInt*);
   PetscErrorCode (*getmerged)(VecScatter,PetscBool *);
+  PetscErrorCode (*getremotecount)(VecScatter,PetscBool,PetscInt*,PetscInt*);
+  PetscErrorCode (*getremote)(VecScatter,PetscBool,PetscInt*,const PetscInt**,const PetscInt**,const PetscMPIInt**,PetscInt*);
+  PetscErrorCode (*getremoteordered)(VecScatter,PetscBool,PetscInt*,const PetscInt**,const PetscInt**,const PetscMPIInt**,PetscInt*);
+  PetscErrorCode (*restoreremote)(VecScatter,PetscBool,PetscInt*,const PetscInt**,const PetscInt**,const PetscMPIInt**,PetscInt*);
+  PetscErrorCode (*restoreremoteordered)(VecScatter,PetscBool,PetscInt*,const PetscInt**,const PetscInt**,const PetscMPIInt**,PetscInt*);
 };
 
 struct _p_VecScatter {
   PETSCHEADER(struct _VecScatterOps);
-  PetscInt       to_n,from_n;
-  PetscBool      inuse;                /* prevents corruption from mixing two scatters */
-  PetscBool      beginandendtogether;  /* indicates that the scatter begin and end  function are called together, VecScatterEnd() is then treated as a nop */
-  void           *fromdata,*todata;
-  void           *spptr;
-  PetscBool      is_duplicate;         /* IS has duplicate indices, would cause writing error in the case StoP of VecScatterEndMPI3Node */
-  Vec            to_v,from_v;          /* used in VecScatterCreate() */
-  IS             to_is,from_is;        /* used in VecScatterCreate() */
+  PetscInt          to_n,from_n;
+  PetscBool         inuse;                /* prevents corruption from mixing two scatters */
+  PetscBool         beginandendtogether;  /* indicates that the scatter begin and end  function are called together, VecScatterEnd() is then treated as a nop */
+  void              *fromdata,*todata;
+  void              *spptr;
+  PetscBool         is_duplicate;         /* IS has duplicate indices, would cause writing error in the case StoP of VecScatterEndMPI3Node */
+  Vec               to_v,from_v;          /* used in VecScatterCreate() */
+  IS                to_is,from_is;        /* used in VecScatterCreate() */
+  const PetscScalar *xdata;               /* vector data to read from */
+  PetscScalar       *ydata;               /* vector data to write to */
+
+  void              *data;                /* implementation specific data */
 };
 
 PETSC_INTERN PetscErrorCode VecScatterCreate_Seq(VecScatter);
 PETSC_INTERN PetscErrorCode VecScatterCreate_MPI1(VecScatter);
 PETSC_INTERN PetscErrorCode VecScatterCreate_MPI3(VecScatter);
 PETSC_INTERN PetscErrorCode VecScatterCreate_MPI3Node(VecScatter);
+PETSC_INTERN PetscErrorCode VecScatterCreate_SF(VecScatter);
+
+PETSC_INTERN PetscErrorCode VecScatterSetUp_vectype_private(VecScatter,PetscErrorCode (*)(PetscInt,const PetscInt*,PetscInt,const PetscInt*,Vec,Vec,PetscInt,VecScatter),PetscErrorCode (*)(PetscInt,const PetscInt*,PetscInt,const PetscInt*,Vec,Vec,PetscInt,VecScatter),PetscErrorCode (*)(PetscInt,const PetscInt*,PetscInt,const PetscInt*,Vec,Vec,PetscInt,VecScatter));
+
+PETSC_INTERN PetscErrorCode VecScatterDestroy_SGToSG(VecScatter);
+PETSC_INTERN PetscErrorCode VecScatterDestroy_SGToSS(VecScatter);
+PETSC_INTERN PetscErrorCode VecScatterDestroy_SSToSG(VecScatter);
+PETSC_INTERN PetscErrorCode VecScatterDestroy_SSToSS(VecScatter);
+PETSC_INTERN PetscErrorCode VecScatterBegin_SGToSG(VecScatter,Vec,Vec,InsertMode,ScatterMode);
+PETSC_INTERN PetscErrorCode VecScatterBegin_SGToSS_Stride1(VecScatter,Vec,Vec,InsertMode,ScatterMode);
+PETSC_INTERN PetscErrorCode VecScatterBegin_SGToSS(VecScatter,Vec,Vec,InsertMode,ScatterMode);
+PETSC_INTERN PetscErrorCode VecScatterBegin_SSToSG_Stride1(VecScatter,Vec,Vec,InsertMode,ScatterMode);
+PETSC_INTERN PetscErrorCode VecScatterBegin_SSToSG(VecScatter,Vec,Vec,InsertMode,ScatterMode);
+PETSC_INTERN PetscErrorCode VecScatterView_SSToSG(VecScatter,PetscViewer);
+PETSC_INTERN PetscErrorCode VecScatterBegin_SSToSS(VecScatter,Vec,Vec,InsertMode,ScatterMode);
+PETSC_INTERN PetscErrorCode VecScatterCopy_SGToSG(VecScatter,VecScatter);
+PETSC_INTERN PetscErrorCode VecScatterView_SGToSG(VecScatter,PetscViewer);
+PETSC_INTERN PetscErrorCode VecScatterCopy_SGToSS(VecScatter,VecScatter);
+PETSC_INTERN PetscErrorCode VecScatterView_SGToSS(VecScatter,PetscViewer);
+PETSC_INTERN PetscErrorCode VecScatterCopy_SSToSS(VecScatter,VecScatter);
+PETSC_INTERN PetscErrorCode VecScatterView_SSToSS(VecScatter,PetscViewer);
+PETSC_INTERN PetscErrorCode VecScatterMemcpyPlanCreate_SGToSG(PetscInt,VecScatter_Seq_General*,VecScatter_Seq_General*);
+PETSC_INTERN PetscErrorCode GetInputISType_private(VecScatter,PetscInt,PetscInt,PetscInt*,IS*,PetscInt*,IS*);
+
+#define VEC_SEQ_ID 0
+#define VEC_MPI_ID 1
+#define IS_GENERAL_ID 0
+#define IS_STRIDE_ID  1
+#define IS_BLOCK_ID   2
 
 PETSC_EXTERN PetscMPIInt Petsc_Reduction_keyval;
 
