@@ -10,14 +10,17 @@ PETSC_EXTERN PetscClassId PETSC_DRAW_CLASSID;
 
 PETSC_EXTERN PetscFunctionList PetscDrawList;
 PETSC_EXTERN PetscErrorCode PetscDrawInitializePackage(void);
+PETSC_EXTERN PetscErrorCode PetscDrawFinalizePackage(void);
 PETSC_EXTERN PetscErrorCode PetscDrawRegister(const char[],PetscErrorCode(*)(PetscDraw));
 
 PETSC_EXTERN PetscErrorCode PetscDrawGetType(PetscDraw,PetscDrawType*);
 PETSC_EXTERN PetscErrorCode PetscDrawSetType(PetscDraw,PetscDrawType);
 PETSC_EXTERN PetscErrorCode PetscDrawCreate(MPI_Comm,const char[],const char[],int,int,int,int,PetscDraw*);
+PETSC_EXTERN PetscErrorCode PetscDrawSetOptionsPrefix(PetscDraw,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawSetFromOptions(PetscDraw);
-PETSC_EXTERN PetscErrorCode PetscDrawSetSave(PetscDraw,const char*,PetscBool);
-PETSC_EXTERN PetscErrorCode PetscDrawSetSaveFinalImage(PetscDraw,const char*);
+PETSC_EXTERN PetscErrorCode PetscDrawSetSave(PetscDraw,const char[]);
+PETSC_EXTERN PetscErrorCode PetscDrawSetSaveMovie(PetscDraw,const char[]);
+PETSC_EXTERN PetscErrorCode PetscDrawSetSaveFinalImage(PetscDraw,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawView(PetscDraw,PetscViewer);
 PETSC_STATIC_INLINE PetscErrorCode PetscDrawViewFromOptions(PetscDraw A,PetscObject obj,const char name[]) {return PetscObjectViewFromOptions((PetscObject)A,obj,name);}
 
@@ -62,27 +65,59 @@ PETSC_STATIC_INLINE PetscErrorCode PetscDrawViewFromOptions(PetscDraw A,PetscObj
 #define PETSC_DRAW_LIMEGREEN       30
 #define PETSC_DRAW_LAVENDERBLUSH   31
 #define PETSC_DRAW_PLUM            32
+#define PETSC_DRAW_MAXCOLOR       256
+
+/*MC
+
+   PetscDrawRealToColor - Maps a real value within an interval to a color.
+   The color is an integer value in the range [PETSC_DRAW_BASIC_COLORS to 255]
+   that can be passed to various drawing routines.
+
+   Synopsis:
+   #include <petscdraw.h>
+   int PetscDrawRealToColor(PetscReal value,PetscReal min,PetscReal max)
+
+   Not Collective
+
+   Input Parameter:
++  value - value to map within the interval [min,max]
+.  min - lower end of interval
+-  max - upper end of interval
+
+   Notes:
+    Values outside the interval [min,max] are clipped.
+
+   Level: intermediate
+
+.seealso: PetscDrawPointPixel(), PetscDrawPoint(), PetscDrawLine(), PetscDrawTriangle(), PetscDrawRectangle()
+
+M*/
+PETSC_STATIC_INLINE int PetscDrawRealToColor(PetscReal value,PetscReal min,PetscReal max)
+{
+  value = PetscClipInterval(value,min,max);
+  return PETSC_DRAW_BASIC_COLORS + (int)((255-PETSC_DRAW_BASIC_COLORS)*(value-min)/(max-min));
+}
 
 PETSC_EXTERN PetscErrorCode PetscDrawOpenX(MPI_Comm,const char[],const char[],int,int,int,int,PetscDraw*);
-PETSC_EXTERN PetscErrorCode PetscDrawOpenGLUT(MPI_Comm,const char[],const char[],int,int,int,int,PetscDraw*);
 
 #define PETSC_DRAW_FULL_SIZE    -3
 #define PETSC_DRAW_HALF_SIZE    -4
 #define PETSC_DRAW_THIRD_SIZE   -5
 #define PETSC_DRAW_QUARTER_SIZE -6
 
+PETSC_EXTERN PetscErrorCode PetscDrawOpenImage(MPI_Comm,const char[],int,int,PetscDraw*);
 PETSC_EXTERN PetscErrorCode PetscDrawOpenNull(MPI_Comm,PetscDraw *);
 PETSC_EXTERN PetscErrorCode PetscDrawDestroy(PetscDraw*);
 PETSC_EXTERN PetscErrorCode PetscDrawIsNull(PetscDraw,PetscBool *);
 
 PETSC_EXTERN PetscErrorCode PetscDrawGetPopup(PetscDraw,PetscDraw*);
-PETSC_EXTERN PetscErrorCode PetscDrawCheckResizedWindow(PetscDraw);
-PETSC_EXTERN PetscErrorCode PetscDrawResizeWindow(PetscDraw,int,int);
-
 PETSC_EXTERN PetscErrorCode PetscDrawScalePopup(PetscDraw,PetscReal,PetscReal);
 
-PETSC_EXTERN PetscErrorCode PetscDrawPixelToCoordinate(PetscDraw,PetscInt,PetscInt,PetscReal*,PetscReal*);
-PETSC_EXTERN PetscErrorCode PetscDrawCoordinateToPixel(PetscDraw,PetscReal,PetscReal,PetscInt*,PetscInt*);
+PETSC_EXTERN PetscErrorCode PetscDrawCheckResizedWindow(PetscDraw);
+PETSC_EXTERN PetscErrorCode PetscDrawResizeWindow(PetscDraw,int,int);
+PETSC_EXTERN PetscErrorCode PetscDrawGetWindowSize(PetscDraw,int*,int*);
+PETSC_EXTERN PetscErrorCode PetscDrawPixelToCoordinate(PetscDraw,int,int,PetscReal*,PetscReal*);
+PETSC_EXTERN PetscErrorCode PetscDrawCoordinateToPixel(PetscDraw,PetscReal,PetscReal,int*,int*);
 
 PETSC_EXTERN PetscErrorCode PetscDrawIndicatorFunction(PetscDraw,PetscReal,PetscReal,PetscReal,PetscReal,int,PetscErrorCode (*)(void*,PetscReal,PetscReal,PetscBool*),void*);
 
@@ -111,7 +146,7 @@ PETSC_EXTERN PetscErrorCode PetscDrawSetMarkerType(PetscDraw,PetscDrawMarkerType
 PETSC_EXTERN PetscErrorCode PetscDrawGetMarkerType(PetscDraw,PetscDrawMarkerType*);
 
 PETSC_EXTERN PetscErrorCode PetscDrawPoint(PetscDraw,PetscReal,PetscReal,int);
-PETSC_EXTERN PetscErrorCode PetscDrawPointPixel(PetscDraw,PetscInt,PetscInt,int);
+PETSC_EXTERN PetscErrorCode PetscDrawPointPixel(PetscDraw,int,int,int);
 PETSC_EXTERN PetscErrorCode PetscDrawPointSetSize(PetscDraw,PetscReal);
 
 PETSC_EXTERN PetscErrorCode PetscDrawRectangle(PetscDraw,PetscReal,PetscReal,PetscReal,PetscReal,int,int,int,int);
@@ -123,7 +158,6 @@ PETSC_EXTERN PetscErrorCode PetscDrawTensorContour(PetscDraw,int,int,const Petsc
 PETSC_EXTERN PetscErrorCode PetscDrawString(PetscDraw,PetscReal,PetscReal,int,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawStringCentered(PetscDraw,PetscReal,PetscReal,int,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawStringBoxed(PetscDraw,PetscReal,PetscReal,int,int,const char[],PetscReal*,PetscReal*);
-PETSC_EXTERN PetscErrorCode PetscDrawStringBoxedSize(PetscDraw,const char[],PetscReal*,PetscReal*);
 PETSC_EXTERN PetscErrorCode PetscDrawStringVertical(PetscDraw,PetscReal,PetscReal,int,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawStringSetSize(PetscDraw,PetscReal,PetscReal);
 PETSC_EXTERN PetscErrorCode PetscDrawStringGetSize(PetscDraw,PetscReal*,PetscReal*);
@@ -137,17 +171,16 @@ PETSC_EXTERN PetscErrorCode PetscDrawGetCoordinates(PetscDraw,PetscReal*,PetscRe
 
 PETSC_EXTERN PetscErrorCode PetscDrawSetTitle(PetscDraw,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawAppendTitle(PetscDraw,const char[]);
-PETSC_EXTERN PetscErrorCode PetscDrawGetTitle(PetscDraw,char **);
+PETSC_EXTERN PetscErrorCode PetscDrawGetTitle(PetscDraw,const char *[]);
 
 PETSC_EXTERN PetscErrorCode PetscDrawSetPause(PetscDraw,PetscReal);
 PETSC_EXTERN PetscErrorCode PetscDrawGetPause(PetscDraw,PetscReal*);
 PETSC_EXTERN PetscErrorCode PetscDrawPause(PetscDraw);
 PETSC_EXTERN PetscErrorCode PetscDrawSetDoubleBuffer(PetscDraw);
-PETSC_EXTERN PetscErrorCode PetscDrawFlush(PetscDraw);
-PETSC_EXTERN PetscErrorCode PetscDrawSynchronizedFlush(PetscDraw);
 PETSC_EXTERN PetscErrorCode PetscDrawClear(PetscDraw);
+PETSC_EXTERN PetscErrorCode PetscDrawFlush(PetscDraw);
 PETSC_EXTERN PetscErrorCode PetscDrawSave(PetscDraw);
-PETSC_EXTERN PetscErrorCode PetscDrawSynchronizedClear(PetscDraw);
+PETSC_EXTERN PetscErrorCode PetscDrawSaveMovie(PetscDraw);
 PETSC_EXTERN PetscErrorCode PetscDrawBOP(PetscDraw);
 PETSC_EXTERN PetscErrorCode PetscDrawEOP(PetscDraw);
 
@@ -166,12 +199,14 @@ PETSC_EXTERN PetscErrorCode PetscDrawGetBoundingBox(PetscDraw,PetscReal*,PetscRe
 
    Level: intermediate
 
-.seealso: PetscDrawGetMouseButton(), PetscDrawSynchronizedGetMouseButton()
+.seealso: PetscDrawGetMouseButton()
 E*/
-typedef enum {PETSC_BUTTON_NONE,PETSC_BUTTON_LEFT,PETSC_BUTTON_CENTER,PETSC_BUTTON_RIGHT,PETSC_BUTTON_LEFT_SHIFT,PETSC_BUTTON_CENTER_SHIFT,PETSC_BUTTON_RIGHT_SHIFT} PetscDrawButton;
+typedef enum { PETSC_BUTTON_NONE = 0,
+               PETSC_BUTTON_LEFT,PETSC_BUTTON_CENTER,PETSC_BUTTON_RIGHT,
+               PETSC_BUTTON_WHEEL_UP,PETSC_BUTTON_WHEEL_DOWN,
+               PETSC_BUTTON_LEFT_SHIFT,PETSC_BUTTON_CENTER_SHIFT,PETSC_BUTTON_RIGHT_SHIFT } PetscDrawButton;
 
 PETSC_EXTERN PetscErrorCode PetscDrawGetMouseButton(PetscDraw,PetscDrawButton *,PetscReal*,PetscReal *,PetscReal *,PetscReal *);
-PETSC_EXTERN PetscErrorCode PetscDrawSynchronizedGetMouseButton(PetscDraw,PetscDrawButton *,PetscReal*,PetscReal *,PetscReal *,PetscReal *);
 
 PETSC_EXTERN PetscErrorCode PetscDrawZoom(PetscDraw,PetscErrorCode (*)(PetscDraw,void *),void *);
 
@@ -192,7 +227,6 @@ typedef struct {
   PetscReal *yr;
   PetscDraw draw;
   PetscReal port_xl,port_yl,port_xr,port_yr;   /* original port of parent PetscDraw */
-
 } PetscDrawViewPorts;
 PETSC_EXTERN PetscErrorCode PetscDrawViewPortsCreate(PetscDraw,PetscInt,PetscDrawViewPorts**);
 PETSC_EXTERN PetscErrorCode PetscDrawViewPortsCreateRect(PetscDraw,PetscInt,PetscInt,PetscDrawViewPorts**);
@@ -201,59 +235,63 @@ PETSC_EXTERN PetscErrorCode PetscDrawViewPortsSet(PetscDrawViewPorts*,PetscInt);
 
 PETSC_EXTERN PetscClassId PETSC_DRAWAXIS_CLASSID;
 
-PETSC_EXTERN PetscErrorCode PetscDrawAxisCreate(PetscDraw,PetscDrawAxis *);
+PETSC_EXTERN PetscErrorCode PetscDrawAxisCreate(PetscDraw,PetscDrawAxis*);
 PETSC_EXTERN PetscErrorCode PetscDrawAxisDestroy(PetscDrawAxis*);
 PETSC_EXTERN PetscErrorCode PetscDrawAxisDraw(PetscDrawAxis);
 PETSC_EXTERN PetscErrorCode PetscDrawAxisSetLimits(PetscDrawAxis,PetscReal,PetscReal,PetscReal,PetscReal);
 PETSC_EXTERN PetscErrorCode PetscDrawAxisGetLimits(PetscDrawAxis,PetscReal*,PetscReal*,PetscReal*,PetscReal*);
-PETSC_EXTERN PetscErrorCode PetscDrawAxisSetHoldLimits(PetscDrawAxis,PetscBool );
+PETSC_EXTERN PetscErrorCode PetscDrawAxisSetHoldLimits(PetscDrawAxis,PetscBool);
 PETSC_EXTERN PetscErrorCode PetscDrawAxisSetColors(PetscDrawAxis,int,int,int);
 PETSC_EXTERN PetscErrorCode PetscDrawAxisSetLabels(PetscDrawAxis,const char[],const char[],const char[]);
 
 PETSC_EXTERN PetscClassId PETSC_DRAWLG_CLASSID;
 
-PETSC_EXTERN PetscErrorCode PetscDrawLGCreate(PetscDraw,PetscInt,PetscDrawLG *);
+PETSC_EXTERN PetscErrorCode PetscDrawLGCreate(PetscDraw,PetscInt,PetscDrawLG*);
 PETSC_EXTERN PetscErrorCode PetscDrawLGDestroy(PetscDrawLG*);
 PETSC_EXTERN PetscErrorCode PetscDrawLGAddPoint(PetscDrawLG,const PetscReal*,const PetscReal*);
 PETSC_EXTERN PetscErrorCode PetscDrawLGAddCommonPoint(PetscDrawLG,const PetscReal,const PetscReal*);
 PETSC_EXTERN PetscErrorCode PetscDrawLGAddPoints(PetscDrawLG,PetscInt,PetscReal**,PetscReal**);
 PETSC_EXTERN PetscErrorCode PetscDrawLGDraw(PetscDrawLG);
+PETSC_EXTERN PetscErrorCode PetscDrawLGSave(PetscDrawLG);
 PETSC_EXTERN PetscErrorCode PetscDrawLGView(PetscDrawLG,PetscViewer);
 PETSC_EXTERN PetscErrorCode PetscDrawLGReset(PetscDrawLG);
 PETSC_EXTERN PetscErrorCode PetscDrawLGSetDimension(PetscDrawLG,PetscInt);
 PETSC_EXTERN PetscErrorCode PetscDrawLGGetDimension(PetscDrawLG,PetscInt*);
 PETSC_EXTERN PetscErrorCode PetscDrawLGSetLegend(PetscDrawLG,const char *const*);
-PETSC_EXTERN PetscErrorCode PetscDrawLGGetAxis(PetscDrawLG,PetscDrawAxis *);
-PETSC_EXTERN PetscErrorCode PetscDrawLGGetDraw(PetscDrawLG,PetscDraw *);
+PETSC_EXTERN PetscErrorCode PetscDrawLGGetAxis(PetscDrawLG,PetscDrawAxis*);
+PETSC_EXTERN PetscErrorCode PetscDrawLGGetDraw(PetscDrawLG,PetscDraw*);
 PETSC_EXTERN PetscErrorCode PetscDrawLGSetUseMarkers(PetscDrawLG,PetscBool);
 PETSC_EXTERN PetscErrorCode PetscDrawLGSetLimits(PetscDrawLG,PetscReal,PetscReal,PetscReal,PetscReal);
-PETSC_EXTERN PetscErrorCode PetscDrawLGSetColors(PetscDrawLG,const int*);
+PETSC_EXTERN PetscErrorCode PetscDrawLGSetColors(PetscDrawLG,const int[]);
+PETSC_EXTERN PetscErrorCode PetscDrawLGSetOptionsPrefix(PetscDrawLG,const char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawLGSetFromOptions(PetscDrawLG);
 
 PETSC_EXTERN PetscClassId PETSC_DRAWSP_CLASSID;
 
-PETSC_EXTERN PetscErrorCode PetscDrawSPCreate(PetscDraw,int,PetscDrawSP *);
+PETSC_EXTERN PetscErrorCode PetscDrawSPCreate(PetscDraw,int,PetscDrawSP*);
 PETSC_EXTERN PetscErrorCode PetscDrawSPDestroy(PetscDrawSP*);
 PETSC_EXTERN PetscErrorCode PetscDrawSPAddPoint(PetscDrawSP,PetscReal*,PetscReal*);
 PETSC_EXTERN PetscErrorCode PetscDrawSPAddPoints(PetscDrawSP,int,PetscReal**,PetscReal**);
 PETSC_EXTERN PetscErrorCode PetscDrawSPDraw(PetscDrawSP,PetscBool);
+PETSC_EXTERN PetscErrorCode PetscDrawSPSave(PetscDrawSP);
 PETSC_EXTERN PetscErrorCode PetscDrawSPReset(PetscDrawSP);
 PETSC_EXTERN PetscErrorCode PetscDrawSPSetDimension(PetscDrawSP,int);
-PETSC_EXTERN PetscErrorCode PetscDrawSPGetAxis(PetscDrawSP,PetscDrawAxis *);
-PETSC_EXTERN PetscErrorCode PetscDrawSPGetDraw(PetscDrawSP,PetscDraw *);
+PETSC_EXTERN PetscErrorCode PetscDrawSPGetAxis(PetscDrawSP,PetscDrawAxis*);
+PETSC_EXTERN PetscErrorCode PetscDrawSPGetDraw(PetscDrawSP,PetscDraw*);
 PETSC_EXTERN PetscErrorCode PetscDrawSPSetLimits(PetscDrawSP,PetscReal,PetscReal,PetscReal,PetscReal);
 PETSC_EXTERN PetscErrorCode PetscDrawLGSPDraw(PetscDrawLG,PetscDrawSP);
 
 PETSC_EXTERN PetscClassId PETSC_DRAWHG_CLASSID;
 
-PETSC_EXTERN PetscErrorCode PetscDrawHGCreate(PetscDraw,int,PetscDrawHG *);
+PETSC_EXTERN PetscErrorCode PetscDrawHGCreate(PetscDraw,int,PetscDrawHG*);
 PETSC_EXTERN PetscErrorCode PetscDrawHGDestroy(PetscDrawHG*);
 PETSC_EXTERN PetscErrorCode PetscDrawHGAddValue(PetscDrawHG,PetscReal);
 PETSC_EXTERN PetscErrorCode PetscDrawHGDraw(PetscDrawHG);
+PETSC_EXTERN PetscErrorCode PetscDrawHGSave(PetscDrawHG);
 PETSC_EXTERN PetscErrorCode PetscDrawHGView(PetscDrawHG,PetscViewer);
 PETSC_EXTERN PetscErrorCode PetscDrawHGReset(PetscDrawHG);
-PETSC_EXTERN PetscErrorCode PetscDrawHGGetAxis(PetscDrawHG,PetscDrawAxis *);
-PETSC_EXTERN PetscErrorCode PetscDrawHGGetDraw(PetscDrawHG,PetscDraw *);
+PETSC_EXTERN PetscErrorCode PetscDrawHGGetAxis(PetscDrawHG,PetscDrawAxis*);
+PETSC_EXTERN PetscErrorCode PetscDrawHGGetDraw(PetscDrawHG,PetscDraw*);
 PETSC_EXTERN PetscErrorCode PetscDrawHGSetLimits(PetscDrawHG,PetscReal,PetscReal,int,int);
 PETSC_EXTERN PetscErrorCode PetscDrawHGSetNumberBins(PetscDrawHG,int);
 PETSC_EXTERN PetscErrorCode PetscDrawHGSetColor(PetscDrawHG,int);
@@ -263,9 +301,10 @@ PETSC_EXTERN PetscErrorCode PetscDrawHGIntegerBins(PetscDrawHG, PetscBool );
 PETSC_EXTERN PetscClassId PETSC_DRAWBAR_CLASSID;
 
 PETSC_EXTERN PetscErrorCode PetscDrawBarCreate(PetscDraw,PetscDrawBar*);
-PETSC_EXTERN PetscErrorCode PetscDrawBarSetData(PetscDrawBar,PetscInt,const PetscReal[],const char *const *);
+PETSC_EXTERN PetscErrorCode PetscDrawBarSetData(PetscDrawBar,PetscInt,const PetscReal[],const char *const*);
 PETSC_EXTERN PetscErrorCode PetscDrawBarDestroy(PetscDrawBar*);
 PETSC_EXTERN PetscErrorCode PetscDrawBarDraw(PetscDrawBar);
+PETSC_EXTERN PetscErrorCode PetscDrawBarSave(PetscDrawBar);
 PETSC_EXTERN PetscErrorCode PetscDrawBarSetColor(PetscDrawBar,int);
 PETSC_EXTERN PetscErrorCode PetscDrawBarSetLimits(PetscDrawBar,PetscReal,PetscReal);
 PETSC_EXTERN PetscErrorCode PetscDrawBarSort(PetscDrawBar,PetscBool,PetscReal);
@@ -273,16 +312,49 @@ PETSC_EXTERN PetscErrorCode PetscDrawBarSetFromOptions(PetscDrawBar);
 PETSC_EXTERN PetscErrorCode PetscDrawBarGetAxis(PetscDrawBar,PetscDrawAxis*);
 PETSC_EXTERN PetscErrorCode PetscDrawBarGetDraw(PetscDrawBar,PetscDraw*);
 
-/*
-    PetscViewer routines that allow you to access underlying PetscDraw objects
-*/
-PETSC_EXTERN PetscErrorCode PetscViewerDrawGetDraw(PetscViewer,PetscInt,PetscDraw*);
-PETSC_EXTERN PetscErrorCode PetscViewerDrawBaseAdd(PetscViewer,PetscInt);
-PETSC_EXTERN PetscErrorCode PetscViewerDrawBaseSet(PetscViewer,PetscInt);
-PETSC_EXTERN PetscErrorCode PetscViewerDrawGetDrawLG(PetscViewer,PetscInt,PetscDrawLG*);
-PETSC_EXTERN PetscErrorCode PetscViewerDrawGetDrawAxis(PetscViewer,PetscInt,PetscDrawAxis*);
-
-PETSC_EXTERN PetscErrorCode PetscDrawUtilitySetCmapHue(unsigned char *,unsigned char *,unsigned char *,int);
+PETSC_EXTERN PetscErrorCode PetscDrawUtilitySetCmap(const char[],int,unsigned char[],unsigned char[],unsigned char[]);
 PETSC_EXTERN PetscErrorCode PetscDrawUtilitySetGamma(PetscReal);
 
+/*
+    Handling of X11 I/O errors in parallel
+*/
+#if defined(PETSC_HAVE_X) && defined(PETSC_HAVE_SETJMP_H)
+
+#include <setjmp.h>
+
+PETSC_EXTERN jmp_buf PetscXIOErrorHandlerJumpBuf;
+PETSC_EXTERN void    PetscXIOErrorHandlerJump(void *);
+PETSC_EXTERN_TYPEDEF typedef void (*PetscXIOErrorHandler)(void *);
+PETSC_EXTERN PetscXIOErrorHandler PetscSetXIOErrorHandler(PetscXIOErrorHandler);
+
+#define PetscDrawCollectiveBegin(draw) 0; do { \
+  PetscErrorCode                _Petsc_ierr; \
+  jmp_buf                       _Petsc_jmpbuf; \
+  volatile PetscXIOErrorHandler _Petsc_xioerrhdl = NULL; \
+  PetscBool                     _Petsc_isdrawx, _Petsc_xioerr, _Petsc_xioerr_local = PETSC_FALSE; \
+  _Petsc_ierr = PetscObjectTypeCompare((PetscObject)(draw),PETSC_DRAW_X,&_Petsc_isdrawx);CHKERRQ(_Petsc_ierr); \
+  if (_Petsc_isdrawx) { \
+    _Petsc_ierr = PetscMemcpy(&_Petsc_jmpbuf,&PetscXIOErrorHandlerJumpBuf,sizeof(_Petsc_jmpbuf));CHKERRQ(_Petsc_ierr); \
+    _Petsc_xioerrhdl = PetscSetXIOErrorHandler(PetscXIOErrorHandlerJump); \
+    if (setjmp(PetscXIOErrorHandlerJumpBuf)) {_Petsc_xioerr_local = PETSC_TRUE; do {_Petsc_ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(_Petsc_ierr);} \
+  } do {} while(0)
+
+#define PetscDrawCollectiveEnd(draw) 0; \
+  if (_Petsc_isdrawx) { \
+    (void)PetscSetXIOErrorHandler(_Petsc_xioerrhdl); \
+    _Petsc_ierr = PetscMemcpy(&PetscXIOErrorHandlerJumpBuf,&_Petsc_jmpbuf,sizeof(PetscXIOErrorHandlerJumpBuf));CHKERRQ(_Petsc_ierr); \
+    _Petsc_ierr = MPI_Allreduce(&_Petsc_xioerr_local,&_Petsc_xioerr,1,MPIU_BOOL,MPI_LOR,PetscObjectComm((PetscObject)(draw)));CHKERRQ(_Petsc_ierr); \
+    if (_Petsc_xioerr) { \
+      _Petsc_ierr = PetscDrawSetType((draw),PETSC_DRAW_NULL);CHKERRQ(_Petsc_ierr); \
+      PetscFunctionReturn(0); \
+    } \
+  } } while(0)
+
+#else
+
+#define PetscDrawCollectiveBegin(draw) 0; do {} while(0)
+#define PetscDrawCollectiveEnd(draw)   0; do {} while(0)
+
+#endif
+
 #endif
