@@ -128,6 +128,8 @@ typedef struct {
   PetscLogDouble flops, flops2, flopsTmp; /* The flops and flops^2 used in this event */
   PetscLogDouble time, time2, timeTmp;    /* The time and time^2 taken for this event */
   PetscLogDouble syncTime;                /* The synchronization barrier time */
+  PetscLogDouble dof[8];        /* The number of degrees of freedom associated with this event */
+  PetscLogDouble errors[8];     /* The errors (user-defined) associated with this event */
   PetscLogDouble numMessages;   /* The number of messages in this event */
   PetscLogDouble messageLength; /* The total message lengths in this event */
   PetscLogDouble numReductions; /* The number of reductions in this event */
@@ -175,7 +177,6 @@ struct _n_PetscStageLog {
 PETSC_EXTERN PetscErrorCode PetscLogObjectParent(PetscObject,PetscObject);
 PETSC_EXTERN PetscErrorCode PetscLogObjectMemory(PetscObject,PetscLogDouble);
 
-
 #if defined(PETSC_USE_LOG)  /* --- Logging is turned on --------------------------------*/
 PETSC_EXTERN PetscStageLog petsc_stageLog;
 PETSC_EXTERN PetscErrorCode PetscLogGetStageLog(PetscStageLog*);
@@ -266,6 +267,8 @@ PETSC_EXTERN PetscErrorCode PetscLogEventActivateClass(PetscClassId);
 PETSC_EXTERN PetscErrorCode PetscLogEventDeactivateClass(PetscClassId);
 PETSC_EXTERN PetscErrorCode PetscLogEventGetId(const char[],PetscLogEvent*);
 PETSC_EXTERN PetscErrorCode PetscLogEventGetPerfInfo(int,PetscLogEvent,PetscEventPerfInfo*);
+PETSC_EXTERN PetscErrorCode PetscLogEventSetDof(PetscLogEvent, PetscInt, PetscLogDouble);
+PETSC_EXTERN PetscErrorCode PetscLogEventSetError(PetscLogEvent, PetscInt, PetscLogDouble);
 
 /* Global counters */
 PETSC_EXTERN PetscLogDouble petsc_irecv_ct;
@@ -285,6 +288,11 @@ PETSC_EXTERN PetscLogDouble petsc_wait_all_ct;
 PETSC_EXTERN PetscLogDouble petsc_sum_of_waits_ct;
 
 PETSC_EXTERN PetscBool PetscLogSyncOn;  /* true if logging synchronization is enabled */
+PETSC_EXTERN PetscErrorCode PetscLogEventSynchronize(PetscLogEvent, MPI_Comm);
+
+#define PetscLogEventSync(e,comm) \
+  (((PetscLogPLB && petsc_stageLog->stageInfo[petsc_stageLog->curStage].perfInfo.active && petsc_stageLog->stageInfo[petsc_stageLog->curStage].eventLog->eventInfo[e].active) ? \
+    PetscLogEventSynchronize((e),(comm)) : 0 ))
 
 #define PetscLogEventBegin(e,o1,o2,o3,o4) \
   (((PetscLogPLB && petsc_stageLog->stageInfo[petsc_stageLog->curStage].perfInfo.active && petsc_stageLog->stageInfo[petsc_stageLog->curStage].eventLog->eventInfo[e].active) ? \
@@ -352,14 +360,14 @@ PETSC_STATIC_INLINE int PetscMPIParallelComm(MPI_Comm comm)
 #define MPI_Isend(buf,count,datatype,dest,tag,comm,request) \
   ((petsc_isend_ct++,0) || PetscMPITypeSize(&(petsc_isend_len),(count),(datatype)) || MPI_Isend((buf),(count),(datatype),(dest),(tag),(comm),(request)))
 
-#define MPI_Startall_irecv(count,number,requests) \
-  ((petsc_irecv_ct += (PetscLogDouble)(number),0) || PetscMPITypeSize(&(petsc_irecv_len),(count),(MPIU_SCALAR)) || MPI_Startall((number),(requests)))
+#define MPI_Startall_irecv(count,datatype,number,requests) \
+  ((petsc_irecv_ct += (PetscLogDouble)(number),0) || PetscMPITypeSize(&(petsc_irecv_len),(count),(datatype)) || ((number) && MPI_Startall((number),(requests))))
 
-#define MPI_Startall_isend(count,number,requests) \
-  ((petsc_isend_ct += (PetscLogDouble)(number),0) || PetscMPITypeSize(&(petsc_isend_len),(count),(MPIU_SCALAR)) || MPI_Startall((number),(requests)))
+#define MPI_Startall_isend(count,datatype,number,requests) \
+  ((petsc_isend_ct += (PetscLogDouble)(number),0) || PetscMPITypeSize(&(petsc_isend_len),(count),(datatype)) || ((number) && MPI_Startall((number),(requests))))
 
-#define MPI_Start_isend(count,requests) \
-  ((petsc_isend_ct++,0) || PetscMPITypeSize((&petsc_isend_len),(count),(MPIU_SCALAR)) || MPI_Start((requests)))
+#define MPI_Start_isend(count,datatype,requests) \
+  ((petsc_isend_ct++,0) || PetscMPITypeSize((&petsc_isend_len),(count),(datatype)) || MPI_Start((requests)))
 
 #define MPI_Recv(buf,count,datatype,source,tag,comm,status) \
   ((petsc_recv_ct++,0) || PetscMPITypeSize((&petsc_recv_len),(count),(datatype)) || MPI_Recv((buf),(count),(datatype),(source),(tag),(comm),(status)))
@@ -411,13 +419,13 @@ PETSC_STATIC_INLINE int PetscMPIParallelComm(MPI_Comm comm)
 
 #else
 
-#define MPI_Startall_irecv(count,number,requests) \
-  (MPI_Startall((number),(requests)))
+#define MPI_Startall_irecv(count,datatype,number,requests) \
+  ((number) && MPI_Startall((number),(requests)))
 
-#define MPI_Startall_isend(count,number,requests) \
-  (MPI_Startall((number),(requests)))
+#define MPI_Startall_isend(count,datatype,number,requests) \
+  ((number) && MPI_Startall((number),(requests)))
 
-#define MPI_Start_isend(count,requests) \
+#define MPI_Start_isend(count,datatype,requests) \
   (MPI_Start((requests)))
 
 #endif /* !__MPIUNI_H && ! PETSC_HAVE_BROKEN_RECURSIVE_MACRO */
@@ -447,6 +455,8 @@ PETSC_STATIC_INLINE int PetscMPIParallelComm(MPI_Comm comm)
 #define PetscLogEventSetActiveAll(a,b)     0
 #define PetscLogEventGetId(a,b)            (*(b)=0,0)
 #define PetscLogEventGetPerfInfo(a,b,c)    0
+#define PetscLogEventSetDof(a,b,c)         0
+#define PetscLogEventSetError(a,b,c)       0
 
 #define PetscLogPLB                        0
 #define PetscLogPLE                        0
@@ -471,13 +481,14 @@ PETSC_EXTERN PetscErrorCode PetscLogObjectState(PetscObject,const char[],...);
 #define PetscLogViewFromOptions()          0
 #define PetscLogDump(c)                    0
 
+#define PetscLogEventSync(e,comm)          0
 #define PetscLogEventBegin(e,o1,o2,o3,o4)  0
 #define PetscLogEventEnd(e,o1,o2,o3,o4)    0
 
 /* If PETSC_USE_LOG is NOT defined, these still need to be! */
-#define MPI_Startall_irecv(count,number,requests) MPI_Startall(number,requests)
-#define MPI_Startall_isend(count,number,requests) MPI_Startall(number,requests)
-#define MPI_Start_isend(count,requests)           MPI_Start(requests)
+#define MPI_Startall_irecv(count,datatype,number,requests) ((number) && MPI_Startall(number,requests))
+#define MPI_Startall_isend(count,datatype,number,requests) ((number) && MPI_Startall(number,requests))
+#define MPI_Start_isend(count,datatype,requests)           MPI_Start(requests)
 
 #endif   /* PETSC_USE_LOG */
 
