@@ -11,6 +11,18 @@
 #if !defined(__PETSCMATH_H)
 #define __PETSCMATH_H
 #include <math.h>
+#include <petscsystypes.h>
+
+/*MC
+   MPIU_REAL - MPI datatype corresponding to PetscReal
+
+   Notes:
+   In MPI calls that require an MPI datatype that matches a PetscReal or array of PetscReal values, pass this value.
+
+   Level: beginner
+
+.seealso: PetscReal, PetscScalar, PetscComplex, PetscInt, MPIU_SCALAR, MPIU_COMPLEX, MPIU_INT
+M*/
 
 /*
 
@@ -23,7 +35,6 @@
 
 #if defined(PETSC_USE_REAL_SINGLE)
 #define MPIU_REAL   MPI_FLOAT
-typedef float PetscReal;
 #define PetscRoundReal(a)   roundf(a)
 #define PetscSqrtReal(a)    sqrtf(a)
 #define PetscExpReal(a)     expf(a)
@@ -49,7 +60,6 @@ typedef float PetscReal;
 #define PetscTGamma(a)      tgammaf(a)
 #elif defined(PETSC_USE_REAL_DOUBLE)
 #define MPIU_REAL   MPI_DOUBLE
-typedef double PetscReal;
 #define PetscRoundReal(a)   round(a)
 #define PetscSqrtReal(a)    sqrt(a)
 #define PetscExpReal(a)     exp(a)
@@ -83,7 +93,6 @@ extern "C" {
 #endif
 PETSC_EXTERN MPI_Datatype MPIU___FLOAT128 PetscAttrMPITypeTag(__float128);
 #define MPIU_REAL MPIU___FLOAT128
-typedef __float128 PetscReal;
 #define PetscRoundReal(a)   roundq(a)
 #define PetscSqrtReal(a)    sqrtq(a)
 #define PetscExpReal(a)     expq(a)
@@ -110,7 +119,6 @@ typedef __float128 PetscReal;
 #elif defined(PETSC_USE_REAL___FP16)
 PETSC_EXTERN MPI_Datatype MPIU___FP16 PetscAttrMPITypeTag(__fp16);
 #define MPIU_REAL MPIU___FP16
-typedef __fp16 PetscReal;
 #define PetscRoundReal(a)   roundf(a)
 #define PetscSqrtReal(a)    sqrtf(a)
 #define PetscExpReal(a)     expf(a)
@@ -136,49 +144,50 @@ typedef __fp16 PetscReal;
 #define PetscTGamma(a)      tgammaf(a)
 #endif /* PETSC_USE_REAL_* */
 
+/*MC
+   MPIU_COMPLEX - MPI datatype corresponding to PetscComplex
+
+   Notes:
+   In MPI calls that require an MPI datatype that matches a PetscComplex or array of PetscComplex values, pass this value.
+
+   Level: beginner
+
+.seealso: PetscReal, PetscScalar, PetscComplex, PetscInt, MPIU_REAL, MPIU_SCALAR, MPIU_COMPLEX, MPIU_INT, PETSC_i
+M*/
+
 /*
     Complex number definitions
  */
 #if defined(__cplusplus) && defined(PETSC_HAVE_CXX_COMPLEX) && !defined(PETSC_USE_REAL___FLOAT128)
-#if !defined(PETSC_SKIP_COMPLEX)
-#define PETSC_HAVE_COMPLEX 1
+#if defined(PETSC_HAVE_COMPLEX)
 /* C++ support of complex number */
-#if defined(PETSC_HAVE_VECCUDA) && __CUDACC_VER_MAJOR__ > 6
-/* complex headers in thrust only available in CUDA 7.0 and above */
-#define complexlib thrust
-#include <thrust/complex.h>
-#else
-#define complexlib std
-#include <complex>
-#endif
 
 #define PetscRealPartComplex(a)      (a).real()
 #define PetscImaginaryPartComplex(a) (a).imag()
-#define PetscAbsComplex(a)           complexlib::abs(a)
-#define PetscConjComplex(a)          complexlib::conj(a)
-#define PetscSqrtComplex(a)          complexlib::sqrt(a)
-#define PetscPowComplex(a,b)         complexlib::pow(a,b)
-#define PetscExpComplex(a)           complexlib::exp(a)
-#define PetscLogComplex(a)           complexlib::log(a)
-#define PetscSinComplex(a)           complexlib::sin(a)
-#define PetscCosComplex(a)           complexlib::cos(a)
-#define PetscAsinComplex(a)          complexlib::asin(a)
-#define PetscAcosComplex(a)          complexlib::acos(a)
+#define PetscAbsComplex(a)           petsccomplexlib::abs(a)
+#define PetscConjComplex(a)          petsccomplexlib::conj(a)
+#define PetscSqrtComplex(a)          petsccomplexlib::sqrt(a)
+#define PetscPowComplex(a,b)         petsccomplexlib::pow(a,b)
+#define PetscExpComplex(a)           petsccomplexlib::exp(a)
+#define PetscLogComplex(a)           petsccomplexlib::log(a)
+#define PetscSinComplex(a)           petsccomplexlib::sin(a)
+#define PetscCosComplex(a)           petsccomplexlib::cos(a)
+#define PetscAsinComplex(a)          petsccomplexlib::asin(a)
+#define PetscAcosComplex(a)          petsccomplexlib::acos(a)
 #if defined(PETSC_HAVE_TANCOMPLEX)
-#define PetscTanComplex(a)           complexlib::tan(a)
+#define PetscTanComplex(a)           petsccomplexlib::tan(a)
 #else
 #define PetscTanComplex(a)           PetscSinComplex(a)/PetscCosComplex(a)
 #endif
-#define PetscSinhComplex(a)          complexlib::sinh(a)
-#define PetscCoshComplex(a)          complexlib::cosh(a)
+#define PetscSinhComplex(a)          petsccomplexlib::sinh(a)
+#define PetscCoshComplex(a)          petsccomplexlib::cosh(a)
 #if defined(PETSC_HAVE_TANHCOMPLEX)
-#define PetscTanhComplex(a)          complexlib::tanh(a)
+#define PetscTanhComplex(a)          petsccomplexlib::tanh(a)
 #else
 #define PetscTanhComplex(a)          PetscSinhComplex(a)/PetscCoshComplex(a)
 #endif
 
 #if defined(PETSC_USE_REAL_SINGLE)
-typedef complexlib::complex<float> PetscComplex;
 #if defined(PETSC_USE_CXX_COMPLEX_FLOAT_WORKAROUND)
 static inline PetscComplex operator+(const PetscComplex& lhs, const double& rhs) { return lhs + float(rhs); }
 static inline PetscComplex operator+(const double& lhs, const PetscComplex& rhs) { return float(lhs) + rhs; }
@@ -194,7 +203,6 @@ static inline bool operator!=(const PetscComplex& lhs, const double& rhs) { retu
 static inline bool operator!=(const double& lhs, const PetscComplex& rhs) { return rhs.imag() != float(0) || rhs.real() != float(lhs); }
 #endif  /* PETSC_USE_CXX_COMPLEX_FLOAT_WORKAROUND */
 #elif defined(PETSC_USE_REAL_DOUBLE)
-typedef complexlib::complex<double> PetscComplex;
 #if defined(PETSC_USE_CXX_COMPLEX_FLOAT_WORKAROUND)
 static inline PetscComplex operator+(const PetscComplex& lhs, const PetscInt& rhs) { return lhs + double(rhs); }
 static inline PetscComplex operator+(const PetscInt& lhs, const PetscComplex& rhs) { return double(lhs) + rhs; }
@@ -210,19 +218,14 @@ static inline bool operator!=(const PetscComplex& lhs, const PetscInt& rhs) { re
 static inline bool operator!=(const PetscInt& lhs, const PetscComplex& rhs) { return rhs.imag() != double(0) || rhs.real() != double(lhs); }
 #endif  /* PETSC_USE_CXX_COMPLEX_FLOAT_WORKAROUND */
 #elif defined(PETSC_USE_REAL___FLOAT128)
-typedef complexlib::complex<__float128> PetscComplex; /* Notstandard and not expected to work, use __complex128 */
 PETSC_EXTERN MPI_Datatype MPIU___COMPLEX128;
-#endif  /* PETSC_USE_REAL_ */
-#endif  /* ! PETSC_SKIP_COMPLEX */
+#endif
+#endif  /* PETSC_HAVE_COMPLEX */
 
 #elif defined(PETSC_HAVE_C99_COMPLEX) && !defined(PETSC_USE_REAL___FP16)
-#if !defined(PETSC_SKIP_COMPLEX)
-#define PETSC_HAVE_COMPLEX 1
-#include <complex.h>
+#if defined(PETSC_HAVE_COMPLEX)
 
 #if defined(PETSC_USE_REAL_SINGLE) || defined(PETSC_USE_REAL___FP16)
-typedef float _Complex PetscComplex;
-
 #define PetscRealPartComplex(a)      crealf(a)
 #define PetscImaginaryPartComplex(a) cimagf(a)
 #define PetscAbsComplex(a)           cabsf(a)
@@ -241,8 +244,6 @@ typedef float _Complex PetscComplex;
 #define PetscTanhComplex(a)          ctanhf(a)
 
 #elif defined(PETSC_USE_REAL_DOUBLE)
-typedef double _Complex PetscComplex;
-
 #define PetscRealPartComplex(a)      creal(a)
 #define PetscImaginaryPartComplex(a) cimag(a)
 #define PetscAbsComplex(a)           cabs(a)
@@ -261,7 +262,6 @@ typedef double _Complex PetscComplex;
 #define PetscTanhComplex(a)          ctanh(a)
 
 #elif defined(PETSC_USE_REAL___FLOAT128)
-typedef __complex128 PetscComplex;
 PETSC_EXTERN MPI_Datatype MPIU___COMPLEX128 PetscAttrMPITypeTag(__complex128);
 
 #define PetscRealPartComplex(a)      crealq(a)
@@ -282,9 +282,7 @@ PETSC_EXTERN MPI_Datatype MPIU___COMPLEX128 PetscAttrMPITypeTag(__complex128);
 #define PetscTanhComplex(a)          ctanhq(a)
 
 #endif /* PETSC_USE_REAL_* */
-#elif (defined(PETSC_USE_COMPLEX) && !defined(PETSC_SKIP_COMPLEX))
-#error "PETSc was configured --with-scalar-type=complex, but a language-appropriate complex library is not available"
-#endif /* !PETSC_SKIP_COMPLEX */
+#endif /* PETSC_HAVE_COMPLEX */
 #endif /* (__cplusplus && PETSC_HAVE_CXX_COMPLEX) else-if (!__cplusplus && PETSC_HAVE_C99_COMPLEX) */
 
 #if defined(PETSC_HAVE_COMPLEX)
@@ -293,8 +291,8 @@ PETSC_EXTERN MPI_Datatype MPIU___COMPLEX128 PetscAttrMPITypeTag(__complex128);
 #define MPIU_C_COMPLEX MPI_C_COMPLEX
 #else
 # if defined(__cplusplus) && defined(PETSC_HAVE_CXX_COMPLEX) && !defined(PETSC_USE_REAL___FLOAT128)
-  typedef complexlib::complex<double> petsc_mpiu_c_double_complex;
-  typedef complexlib::complex<float> petsc_mpiu_c_complex;
+  typedef petsccomplexlib::complex<double> petsc_mpiu_c_double_complex;
+  typedef petsccomplexlib::complex<float> petsc_mpiu_c_complex;
 # elif !defined(__cplusplus) && defined(PETSC_HAVE_C99_COMPLEX)
   typedef double _Complex petsc_mpiu_c_double_complex;
   typedef float _Complex petsc_mpiu_c_complex;
@@ -317,8 +315,18 @@ PETSC_EXTERN MPI_Datatype MPIU_C_COMPLEX PetscAttrMPITypeTagLayoutCompatible(pet
 #  endif /* PETSC_USE_REAL_* */
 #endif
 
+/*MC
+   MPIU_SCALAR - MPI datatype corresponding to PetscScalar
+
+   Notes:
+   In MPI calls that require an MPI datatype that matches a PetscScalar or array of PetscScalar values, pass this value.
+
+   Level: beginner
+
+.seealso: PetscReal, PetscScalar, PetscComplex, PetscInt, MPIU_REAL, MPIU_COMPLEX, MPIU_INT
+M*/
+
 #if (defined(PETSC_USE_COMPLEX) && !defined(PETSC_SKIP_COMPLEX))
-typedef PetscComplex PetscScalar;
 
 /*MC
    PetscRealPart - Returns the real part of a PetscScalar
@@ -381,7 +389,6 @@ M*/
     real number definitions
  */
 #else /* PETSC_USE_COMPLEX */
-typedef PetscReal PetscScalar;
 #define MPIU_SCALAR MPIU_REAL
 
 #define PetscRealPart(a)      (a)
@@ -679,6 +686,7 @@ PETSC_STATIC_INLINE PetscBool PetscIsNanScalar(PetscScalar v) {return PetscIsNan
 PETSC_STATIC_INLINE PetscBool PetscIsInfOrNanScalar(PetscScalar v) {return PetscIsInfOrNanReal(PetscAbsScalar(v));}
 PETSC_STATIC_INLINE PetscBool PetscIsNormalScalar(PetscScalar v) {return PetscIsNormalReal(PetscAbsScalar(v));}
 
+PETSC_EXTERN PetscBool PetscIsCloseAtTol(PetscReal,PetscReal,PetscReal,PetscReal);
 PETSC_EXTERN PetscBool PetscEqualReal(PetscReal,PetscReal);
 PETSC_EXTERN PetscBool PetscEqualScalar(PetscScalar,PetscScalar);
 
@@ -692,7 +700,8 @@ typedef PetscReal MatReal;
 
 struct petsc_mpiu_2scalar {PetscScalar a,b;};
 PETSC_EXTERN MPI_Datatype MPIU_2SCALAR PetscAttrMPITypeTagLayoutCompatible(struct petsc_mpiu_2scalar);
-#if defined(PETSC_USE_64BIT_INDICES) || !defined(MPI_2INT)
+
+#if defined(PETSC_USE_64BIT_INDICES)
 struct petsc_mpiu_2int {PetscInt a,b;};
 PETSC_EXTERN MPI_Datatype MPIU_2INT PetscAttrMPITypeTagLayoutCompatible(struct petsc_mpiu_2int);
 #else
