@@ -3,18 +3,18 @@
 
 #define PETSC_VERSION_RELEASE    1
 #define PETSC_VERSION_MAJOR      3
-#define PETSC_VERSION_MINOR      10
-#define PETSC_VERSION_SUBMINOR   5
+#define PETSC_VERSION_MINOR      11
+#define PETSC_VERSION_SUBMINOR   3
 #define PETSC_VERSION_PATCH      0
-#define PETSC_RELEASE_DATE       "Sep, 12, 2018"
-#define PETSC_VERSION_DATE       "Mar, 28, 2019"
+#define PETSC_RELEASE_DATE       "Mar, 29, 2019"
+#define PETSC_VERSION_DATE       "Jun, 26, 2019"
 
 #if !defined (PETSC_VERSION_GIT)
-#define PETSC_VERSION_GIT        "v3.10.5"
+#define PETSC_VERSION_GIT        "v3.11.3"
 #endif
 
 #if !defined(PETSC_VERSION_DATE_GIT)
-#define PETSC_VERSION_DATE_GIT   "2019-03-28 18:42:07 -0500"
+#define PETSC_VERSION_DATE_GIT   "2019-06-26 15:29:32 -0500"
 #endif
 
 #define PETSC_VERSION_EQ(MAJOR,MINOR,SUBMINOR) \
