@@ -173,20 +173,24 @@ PetscErrorCode PetscFEGetType(PetscFE fem, PetscFEType *name)
 
   Input Parameter:
 + fem - the PetscFE object to view
-- v   - the viewer
+- viewer   - the viewer
 
   Level: developer
 
 .seealso PetscFEDestroy()
 @*/
-PetscErrorCode PetscFEView(PetscFE fem, PetscViewer v)
+PetscErrorCode PetscFEView(PetscFE fem, PetscViewer viewer)
 {
+  PetscBool      iascii;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fem, PETSCFE_CLASSID, 1);
-  if (!v) {ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject) fem), &v);CHKERRQ(ierr);}
-  if (fem->ops->view) {ierr = (*fem->ops->view)(fem, v);CHKERRQ(ierr);}
+  if (viewer) PetscValidHeaderSpecific(viewer, PETSC_VIEWER_CLASSID, 2);
+  if (!viewer) {ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject) fem), &viewer);CHKERRQ(ierr);}
+  ierr = PetscObjectPrintClassNamePrefixType((PetscObject)fem, viewer);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject) viewer, PETSCVIEWERASCII, &iascii);CHKERRQ(ierr);
+  if (fem->ops->view) {ierr = (*fem->ops->view)(fem, viewer);CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
@@ -917,6 +921,7 @@ PETSC_EXTERN PetscErrorCode PetscFECreatePointTrace(PetscFE fe, PetscInt refPoin
   DM             dm;
   DMLabel        label;
   PetscReal      *xi, *v, *J, detJ;
+  const char     *name;
   PetscQuadrature origin, fullQuad, subQuad;
   PetscErrorCode ierr;
 
@@ -953,6 +958,8 @@ PETSC_EXTERN PetscErrorCode PetscFECreatePointTrace(PetscFE fe, PetscInt refPoin
   ierr = PetscFESetNumComponents(*trFE,numComp);CHKERRQ(ierr);
   ierr = PetscFESetBasisSpace(*trFE,bsubsp);CHKERRQ(ierr);
   ierr = PetscFESetDualSpace(*trFE,dsubsp);CHKERRQ(ierr);
+  ierr = PetscObjectGetName((PetscObject) fe, &name);CHKERRQ(ierr);
+  if (name) {ierr = PetscFESetName(*trFE, name);CHKERRQ(ierr);}
   ierr = PetscFEGetQuadrature(fe,&fullQuad);CHKERRQ(ierr);
   ierr = PetscQuadratureGetOrder(fullQuad,&order);CHKERRQ(ierr);
   ierr = DMPlexGetConeSize(dm,refPoint,&coneSize);CHKERRQ(ierr);
@@ -1372,11 +1379,14 @@ PetscErrorCode PetscFEGetHeightSubspace(PetscFE fe, PetscInt height, PetscFE *su
   if (!fe->subspaces) {ierr = PetscCalloc1(dim, &fe->subspaces);CHKERRQ(ierr);}
   if (height <= dim) {
     if (!fe->subspaces[height-1]) {
-      PetscFE sub;
+      PetscFE     sub;
+      const char *name;
 
       ierr = PetscSpaceGetHeightSubspace(P, height, &subP);CHKERRQ(ierr);
       ierr = PetscDualSpaceGetHeightSubspace(Q, height, &subQ);CHKERRQ(ierr);
       ierr = PetscFECreate(PetscObjectComm((PetscObject) fe), &sub);CHKERRQ(ierr);
+      ierr = PetscObjectGetName((PetscObject) fe,  &name);CHKERRQ(ierr);
+      ierr = PetscObjectSetName((PetscObject) sub,  name);CHKERRQ(ierr);
       ierr = PetscFEGetType(fe, &fetype);CHKERRQ(ierr);
       ierr = PetscFESetType(sub, fetype);CHKERRQ(ierr);
       ierr = PetscFESetBasisSpace(sub, subP);CHKERRQ(ierr);
@@ -1525,8 +1535,7 @@ PetscErrorCode PetscFECreateDefault(MPI_Comm comm, PetscInt dim, PetscInt Nc, Pe
   if (isSimplex) {
     ierr = PetscDTGaussJacobiQuadrature(dim,   1, quadPointsPerEdge, -1.0, 1.0, &q);CHKERRQ(ierr);
     ierr = PetscDTGaussJacobiQuadrature(dim-1, 1, quadPointsPerEdge, -1.0, 1.0, &fq);CHKERRQ(ierr);
-  }
-  else {
+  } else {
     ierr = PetscDTGaussTensorQuadrature(dim,   1, quadPointsPerEdge, -1.0, 1.0, &q);CHKERRQ(ierr);
     ierr = PetscDTGaussTensorQuadrature(dim-1, 1, quadPointsPerEdge, -1.0, 1.0, &fq);CHKERRQ(ierr);
   }
@@ -1537,3 +1546,31 @@ PetscErrorCode PetscFECreateDefault(MPI_Comm comm, PetscInt dim, PetscInt Nc, Pe
   PetscFunctionReturn(0);
 }
 
+/*@C
+  PetscFESetName - Names the FE and its subobjects
+
+  Not collective
+
+  Input Parameters:
++ fe   - The PetscFE
+- name - The name
+
+  Level: beginner
+
+.keywords: PetscFE, finite element
+.seealso: PetscFECreate(), PetscSpaceCreate(), PetscDualSpaceCreate()
+@*/
+PetscErrorCode PetscFESetName(PetscFE fe, const char name[])
+{
+  PetscSpace     P;
+  PetscDualSpace Q;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscFEGetBasisSpace(fe, &P);CHKERRQ(ierr);
+  ierr = PetscFEGetDualSpace(fe, &Q);CHKERRQ(ierr);
+  ierr = PetscObjectSetName((PetscObject) fe, name);CHKERRQ(ierr);
+  ierr = PetscObjectSetName((PetscObject) P,  name);CHKERRQ(ierr);
+  ierr = PetscObjectSetName((PetscObject) Q,  name);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
