@@ -7,13 +7,11 @@
 #include <petsc/private/dmdaimpl.h>     /*I   "petscdmda.h"    I*/
 
 #include <petscdraw.h>
-#undef __FUNCT__
-#define __FUNCT__ "DMView_DA_3d"
-PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
+static PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   PetscMPIInt    rank;
-  PetscBool      iascii,isdraw,isbinary;
+  PetscBool      iascii,isdraw,isglvis,isbinary;
   DM_DA          *dd = (DM_DA*)da->data;
 #if defined(PETSC_HAVE_MATLAB_ENGINE)
   PetscBool ismatlab;
@@ -24,6 +22,7 @@ PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
 
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERGLVIS,&isglvis);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERBINARY,&isbinary);CHKERRQ(ierr);
 #if defined(PETSC_HAVE_MATLAB_ENGINE)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERMATLAB,&ismatlab);CHKERRQ(ierr);
@@ -31,9 +30,28 @@ PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
   if (iascii) {
     PetscViewerFormat format;
 
-    ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
     ierr = PetscViewerGetFormat(viewer, &format);CHKERRQ(ierr);
-    if (format != PETSC_VIEWER_ASCII_VTK && format != PETSC_VIEWER_ASCII_VTK_CELL) {
+    if (format == PETSC_VIEWER_LOAD_BALANCE) {
+      PetscInt      i,nmax = 0,nmin = PETSC_MAX_INT,navg = 0,*nz,nzlocal;
+      DMDALocalInfo info;
+      PetscMPIInt   size;
+      ierr = MPI_Comm_size(PetscObjectComm((PetscObject)da),&size);CHKERRQ(ierr);
+      ierr = DMDAGetLocalInfo(da,&info);CHKERRQ(ierr);
+      nzlocal = info.xm*info.ym*info.zm;
+      ierr = PetscMalloc1(size,&nz);CHKERRQ(ierr);
+      ierr = MPI_Allgather(&nzlocal,1,MPIU_INT,nz,1,MPIU_INT,PetscObjectComm((PetscObject)da));CHKERRQ(ierr);
+      for (i=0; i<(PetscInt)size; i++) {
+        nmax = PetscMax(nmax,nz[i]);
+        nmin = PetscMin(nmin,nz[i]);
+        navg += nz[i];
+      }
+      ierr = PetscFree(nz);CHKERRQ(ierr);
+      navg = navg/size;
+      ierr = PetscViewerASCIIPrintf(viewer,"  Load Balance - Grid Points: Min %D  avg %D  max %D\n",nmin,navg,nmax);CHKERRQ(ierr);
+      PetscFunctionReturn(0);
+    }
+    if (format != PETSC_VIEWER_ASCII_VTK && format != PETSC_VIEWER_ASCII_VTK_CELL && format != PETSC_VIEWER_ASCII_GLVIS) {
       DMDALocalInfo info;
       ierr = DMDAGetLocalInfo(da,&info);CHKERRQ(ierr);
       ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Processor [%d] M %D N %D P %D m %D n %D p %D w %D s %D\n",rank,dd->M,dd->N,dd->P,dd->m,dd->n,dd->p,dd->w,dd->s);CHKERRQ(ierr);
@@ -51,7 +69,9 @@ PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
       }
 #endif
       ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
-      ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_FALSE);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
+    } else if (format == PETSC_VIEWER_ASCII_GLVIS) {
+      ierr = DMView_DA_GLVis(da,viewer);CHKERRQ(ierr);
     } else {
       ierr = DMView_DA_VTK(da,viewer);CHKERRQ(ierr);
     }
@@ -65,10 +85,14 @@ PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
     PetscBool      isnull;
 
     ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
-    ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr); if (isnull) PetscFunctionReturn(0);
-    ierr = PetscDrawSetCoordinates(draw,xmin,ymin,xmax,ymax);CHKERRQ(ierr);
-    ierr = PetscDrawSynchronizedClear(draw);CHKERRQ(ierr);
+    ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr);
+    if (isnull) PetscFunctionReturn(0);
 
+    ierr = PetscDrawCheckResizedWindow(draw);CHKERRQ(ierr);
+    ierr = PetscDrawClear(draw);CHKERRQ(ierr);
+    ierr = PetscDrawSetCoordinates(draw,xmin,ymin,xmax,ymax);CHKERRQ(ierr);
+
+    ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
     /* first processor draw all node lines */
     if (!rank) {
       for (k=0; k<dd->P; k++) {
@@ -76,17 +100,19 @@ PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
         for (xmin=(PetscReal)(k*(dd->M+1)); xmin<(PetscReal)(dd->M+(k*(dd->M+1))); xmin++) {
           ierr = PetscDrawLine(draw,xmin,ymin,xmin,ymax,PETSC_DRAW_BLACK);CHKERRQ(ierr);
         }
-
         xmin = (PetscReal)(k*(dd->M+1)); xmax = xmin + (PetscReal)(dd->M - 1);
         for (ymin=0; ymin<(PetscReal)dd->N; ymin++) {
           ierr = PetscDrawLine(draw,xmin,ymin,xmax,ymin,PETSC_DRAW_BLACK);CHKERRQ(ierr);
         }
       }
     }
-    ierr = PetscDrawSynchronizedFlush(draw);CHKERRQ(ierr);
+    ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(ierr);
+    ierr = PetscDrawFlush(draw);CHKERRQ(ierr);
     ierr = PetscDrawPause(draw);CHKERRQ(ierr);
 
-    for (k=0; k<dd->P; k++) {  /*Go through and draw for each plane*/
+    ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
+    /*Go through and draw for each plane*/
+    for (k=0; k<dd->P; k++) {
       if ((k >= dd->zs) && (k < dd->ze)) {
         /* draw my box */
         ymin = dd->ys;
@@ -102,29 +128,28 @@ PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
         xmin = dd->xs/dd->w;
         xmax =(dd->xe-1)/dd->w;
 
+        /* identify which processor owns the box */
+        ierr = PetscSNPrintf(node,sizeof(node),"%d",(int)rank);CHKERRQ(ierr);
+        ierr = PetscDrawString(draw,xmin+(dd->M+1)*k+.2,ymin+.3,PETSC_DRAW_RED,node);CHKERRQ(ierr);
         /* put in numbers*/
         base = (dd->base+(dd->xe-dd->xs)*(dd->ye-dd->ys)*(k-dd->zs))/dd->w;
-
-        /* Identify which processor owns the box */
-        sprintf(node,"%d",rank);
-        ierr = PetscDrawString(draw,xmin+(dd->M+1)*k+.2,ymin+.3,PETSC_DRAW_RED,node);CHKERRQ(ierr);
-
         for (y=ymin; y<=ymax; y++) {
           for (x=xmin+(dd->M+1)*k; x<=xmax+(dd->M+1)*k; x++) {
-            sprintf(node,"%d",(int)base++);
+            ierr = PetscSNPrintf(node,sizeof(node),"%d",(int)base++);CHKERRQ(ierr);
             ierr = PetscDrawString(draw,x,y,PETSC_DRAW_BLACK,node);CHKERRQ(ierr);
           }
         }
 
       }
     }
-    ierr = PetscDrawSynchronizedFlush(draw);CHKERRQ(ierr);
+    ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(ierr);
+    ierr = PetscDrawFlush(draw);CHKERRQ(ierr);
     ierr = PetscDrawPause(draw);CHKERRQ(ierr);
 
+    ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
     for (k=0-dd->s; k<dd->P+dd->s; k++) {
       /* Go through and draw for each plane */
       if ((k >= dd->Zs) && (k < dd->Ze)) {
-
         /* overlay ghost numbers, useful for error checking */
         base = (dd->Xe-dd->Xs)*(dd->Ye-dd->Ys)*(k-dd->Zs)/dd->w;
         ierr = ISLocalToGlobalMappingGetBlockIndices(da->ltogmap,&idx);CHKERRQ(ierr);
@@ -141,10 +166,8 @@ PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
             ycoord = y;
             /*Keep y wrap around points on drawing */
             if (y<0) ycoord = dd->N+y;
-
             if (y>=dd->N) ycoord = y-dd->N;
             xcoord = x;   /* Keep x wrap points on drawing */
-
             if (x<xmin) xcoord = xmax - (xmin-x);
             if (x>=xmax) xcoord = xmin + (x-xmax);
             ierr = PetscDrawString(draw,xcoord/dd->w,ycoord,PETSC_DRAW_BLUE,node);CHKERRQ(ierr);
@@ -154,8 +177,12 @@ PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
         ierr = ISLocalToGlobalMappingRestoreBlockIndices(da->ltogmap,&idx);CHKERRQ(ierr);
       }
     }
-    ierr = PetscDrawSynchronizedFlush(draw);CHKERRQ(ierr);
+    ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(ierr);
+    ierr = PetscDrawFlush(draw);CHKERRQ(ierr);
     ierr = PetscDrawPause(draw);CHKERRQ(ierr);
+    ierr = PetscDrawSave(draw);CHKERRQ(ierr);
+  } else if (isglvis) {
+    ierr = DMView_DA_GLVis(da,viewer);CHKERRQ(ierr);
   } else if (isbinary) {
     ierr = DMView_DA_Binary(da,viewer);CHKERRQ(ierr);
 #if defined(PETSC_HAVE_MATLAB_ENGINE)
@@ -166,8 +193,6 @@ PetscErrorCode DMView_DA_3d(DM da,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSetUp_DA_3D"
 PetscErrorCode  DMSetUp_DA_3D(DM da)
 {
   DM_DA            *dd          = (DM_DA*)da->data;
@@ -208,7 +233,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
   if (stencil_type == DMDA_STENCIL_BOX && (bx == DM_BOUNDARY_MIRROR || by == DM_BOUNDARY_MIRROR || bz == DM_BOUNDARY_MIRROR)) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_SUP,"Mirror boundary and box stencil");
   ierr = PetscObjectGetComm((PetscObject) da, &comm);CHKERRQ(ierr);
 #if !defined(PETSC_USE_64BIT_INDICES)
-  if (((Petsc64bitInt) M)*((Petsc64bitInt) N)*((Petsc64bitInt) P)*((Petsc64bitInt) dof) > (Petsc64bitInt) PETSC_MPI_INT_MAX) SETERRQ3(comm,PETSC_ERR_INT_OVERFLOW,"Mesh of %D by %D by %D (dof) is too large for 32 bit indices",M,N,dof);
+  if (((PetscInt64) M)*((PetscInt64) N)*((PetscInt64) P)*((PetscInt64) dof) > (PetscInt64) PETSC_MPI_INT_MAX) SETERRQ4(comm,PETSC_ERR_INT_OVERFLOW,"Mesh of %D by %D by %D by %D (dof) is too large for 32 bit indices",M,N,P,dof);
 #endif
 
   ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
@@ -426,23 +451,11 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
   dd->nlocal = (Xe-Xs)*(Ye-Ys)*(Ze-Zs)*dof;
   ierr       = VecCreateSeqWithArray(PETSC_COMM_SELF,dof,dd->nlocal,NULL,&local);CHKERRQ(ierr);
 
-  /* generate appropriate vector scatters */
-  /* local to global inserts non-ghost point region into global */
-  ierr   = PetscMalloc1((IXe-IXs)*(IYe-IYs)*(IZe-IZs),&idx);CHKERRQ(ierr);
-  left   = xs - Xs; right = left + x;
-  bottom = ys - Ys; top = bottom + y;
-  down   = zs - Zs; up  = down + z;
-  count  = 0;
-  for (i=down; i<up; i++) {
-    for (j=bottom; j<top; j++) {
-      for (k=left; k<right; k++) {
-        idx[count++] = (i*(Ye-Ys) + j)*(Xe-Xs) + k;
-      }
-    }
-  }
+  /* generate global to local vector scatter and local to global mapping*/
 
   /* global to local must include ghost points within the domain,
      but not ghost points outside the domain that aren't periodic */
+  ierr   = PetscMalloc1((IXe-IXs)*(IYe-IYs)*(IZe-IZs),&idx);CHKERRQ(ierr);
   if (stencil_type == DMDA_STENCIL_BOX) {
     left   = IXs - Xs; right = left + (IXe-IXs);
     bottom = IYs - Ys; top = bottom + (IYe-IYs);
@@ -802,7 +815,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
         if (twod && (s_t < 0)) s_t = bases[n4] + i*x_t + x_t*y_t*z_t - x_t*y_t; /* 2D case */
         for (j=0; j<x_t; j++) idx[nn++] = s_t++;
       } else if (bz == DM_BOUNDARY_MIRROR) {
-        for (j=0; j<x; j++) idx[nn++] = 0;
+        for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + x*i + (s_z - k - 1)*x*y;
       }
 
       if (n5 >= 0) { /* directly right */
@@ -860,7 +873,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
         s_t = bases[n10] - (s_y+1-i)*x_t + (k+1)*x_t*y_t;
         for (j=0; j<x_t; j++) idx[nn++] = s_t++;
       }  else if (by == DM_BOUNDARY_MIRROR) {
-        for (j=0; j<x; j++) idx[nn++] = 0;
+        for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + k*x*y + (s_y - i)*x;
       }
       if (n11 >= 0) { /* right below */
         x_t = lx[n11 % m];
@@ -879,7 +892,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
         s_t = bases[n12] + (i+1)*x_t - s_x + k*x_t*y_t;
         for (j=0; j<s_x; j++) idx[nn++] = s_t++;
       }  else if (bx == DM_BOUNDARY_MIRROR) {
-        for (j=0; j<s_x; j++) idx[nn++] = 0;
+        for (j=0; j<s_x; j++) idx[nn++] = bases[rank] + s_x - j - 1 + k*x*y + i*x;
       }
 
       /* Interior */
@@ -893,7 +906,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
         s_t = bases[n14] + i*x_t + k*x_t*y_t;
         for (j=0; j<s_x; j++) idx[nn++] = s_t++;
       } else if (bx == DM_BOUNDARY_MIRROR) {
-        for (j=0; j<s_x; j++) idx[nn++] = 0;
+        for (j=0; j<s_x; j++) idx[nn++] = bases[rank] + x - j - 1 + k*x*y + i*x;
       }
     }
 
@@ -912,7 +925,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
         s_t = bases[n16] + (i-1)*x_t + k*x_t*y_t;
         for (j=0; j<x_t; j++) idx[nn++] = s_t++;
       } else if (by == DM_BOUNDARY_MIRROR) {
-        for (j=0; j<x; j++) idx[nn++] = 0;
+        for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + k*x*y + (y-i)*x;
       }
       if (n17 >= 0) { /* right above */
         x_t = lx[n17 % m];
@@ -971,7 +984,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
         if (twod && (s_t >= M*N*P)) s_t = bases[n22] + i*x_t; /* 2d case */
         for (j=0; j<x_t; j++) idx[nn++] = s_t++;
       } else if (bz == DM_BOUNDARY_MIRROR) {
-        for (j=0; j<x; j++) idx[nn++] = 0;
+        for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + (z-k-1)*x*y + i*x;
       }
 
       if (n23 >= 0) { /* directly right */
@@ -1025,10 +1038,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
     n26 = sn26;
   }
 
-  if (((stencil_type == DMDA_STENCIL_STAR) ||
-      (bx != DM_BOUNDARY_PERIODIC && bx) ||
-      (by != DM_BOUNDARY_PERIODIC && by) ||
-       (bz != DM_BOUNDARY_PERIODIC && bz))) {
+  if (((stencil_type == DMDA_STENCIL_STAR) || (bx != DM_BOUNDARY_PERIODIC && bx) || (by != DM_BOUNDARY_PERIODIC && by) || (bz != DM_BOUNDARY_PERIODIC && bz))) {
     /*
         Recompute the local to global mappings, this time keeping the
       information about the cross corner processor numbers.
@@ -1085,7 +1095,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
           for (j=0; j<x_t; j++) idx[nn++] = s_t++;
         } else if (Zs-zs < 0) {
           if (bz == DM_BOUNDARY_MIRROR) {
-            for (j=0; j<x; j++) idx[nn++] = 0;
+            for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + x*i + (s_z - k - 1)*x*y;
           } else {
             for (j=0; j<x; j++) idx[nn++] = -1;
           }
@@ -1153,7 +1163,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
           for (j=0; j<x_t; j++) idx[nn++] = s_t++;
         } else if (Ys-ys < 0) {
           if (by == DM_BOUNDARY_MIRROR) {
-            for (j=0; j<x; j++) idx[nn++] = -1;
+            for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + k*x*y + (s_y - i)*x;;
           } else {
             for (j=0; j<x; j++) idx[nn++] = -1;
           }
@@ -1178,7 +1188,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
           for (j=0; j<s_x; j++) idx[nn++] = s_t++;
         } else if (Xs-xs < 0) {
           if (bx == DM_BOUNDARY_MIRROR) {
-            for (j=0; j<s_x; j++) idx[nn++] = 0;
+            for (j=0; j<s_x; j++) idx[nn++] = bases[rank] + s_x - j - 1 + k*x*y + i*x;
           } else {
             for (j=0; j<s_x; j++) idx[nn++] = -1;
           }
@@ -1196,7 +1206,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
           for (j=0; j<s_x; j++) idx[nn++] = s_t++;
         } else if (xe-Xe < 0) {
           if (bx == DM_BOUNDARY_MIRROR) {
-            for (j=0; j<s_x; j++) idx[nn++] = 0;
+            for (j=0; j<s_x; j++) idx[nn++] = bases[rank] + x - j - 1 + k*x*y + i*x;
           } else {
             for (j=0; j<s_x; j++) idx[nn++] = -1;
           }
@@ -1221,7 +1231,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
           for (j=0; j<x_t; j++) idx[nn++] = s_t++;
         } else if (ye-Ye < 0) {
           if (by == DM_BOUNDARY_MIRROR) {
-            for (j=0; j<x; j++) idx[nn++] = 0;
+            for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + k*x*y + (y-i)*x;
           } else {
             for (j=0; j<x; j++) idx[nn++] = -1;
           }
@@ -1289,7 +1299,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
           for (j=0; j<x_t; j++) idx[nn++] = s_t++;
         } else if (ze-Ze < 0) {
           if (bz == DM_BOUNDARY_MIRROR) {
-            for (j=0; j<x; j++) idx[nn++] = 0;
+            for (j=0; j<x; j++) idx[nn++] = bases[rank] + j + (z-k-1)*x*y + i*x;
           } else {
             for (j=0; j<x; j++) idx[nn++] = -1;
           }
@@ -1362,8 +1372,6 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDACreate3d"
 /*@C
    DMDACreate3d - Creates an object that will manage the communication of three-dimensional
    regular array data that is distributed across some processors.
@@ -1375,8 +1383,7 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
 .  bx,by,bz - type of ghost nodes the array have.
          Use one of DM_BOUNDARY_NONE, DM_BOUNDARY_GHOSTED, DM_BOUNDARY_PERIODIC.
 .  stencil_type - Type of stencil (DMDA_STENCIL_STAR or DMDA_STENCIL_BOX)
-.  M,N,P - global dimension in each direction of the array (use -M, -N, and or -P to indicate that it may be set to a different value
-            from the command line with -da_grid_x <M> -da_grid_y <N> -da_grid_z <P>)
+.  M,N,P - global dimension in each direction of the array 
 .  m,n,p - corresponding number of processors in each dimension
            (or PETSC_DECIDE to have calculated)
 .  dof - number of degrees of freedom per node
@@ -1392,16 +1399,16 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
 
    Options Database Key:
 +  -dm_view - Calls DMView() at the conclusion of DMDACreate3d()
-.  -da_grid_x <nx> - number of grid points in x direction, if M < 0
-.  -da_grid_y <ny> - number of grid points in y direction, if N < 0
-.  -da_grid_z <nz> - number of grid points in z direction, if P < 0
+.  -da_grid_x <nx> - number of grid points in x direction
+.  -da_grid_y <ny> - number of grid points in y direction
+.  -da_grid_z <nz> - number of grid points in z direction
 .  -da_processors_x <MX> - number of processors in x direction
 .  -da_processors_y <MY> - number of processors in y direction
 .  -da_processors_z <MZ> - number of processors in z direction
 .  -da_refine_x <rx> - refinement ratio in x direction
 .  -da_refine_y <ry> - refinement ratio in y direction
 .  -da_refine_z <rz>- refinement ratio in z directio
--  -da_refine <n> - refine the DMDA n times before creating it, , if M, N, or P < 0
+-  -da_refine <n> - refine the DMDA n times before creating it
 
    Level: beginner
 
@@ -1414,6 +1421,11 @@ PetscErrorCode  DMSetUp_DA_3D(DM da)
    The appropriate vector objects can be obtained with calls to DMCreateGlobalVector()
    and DMCreateLocalVector() and calls to VecDuplicate() if more are needed.
 
+   You must call DMSetUp() after this call before using this DM.
+
+   If you wish to use the options database to change values in the DMDA call DMSetFromOptions() after this call
+   but before DMSetUp().
+
 .keywords: distributed array, create, three-dimensional
 
 .seealso: DMDestroy(), DMView(), DMDACreate1d(), DMDACreate2d(), DMGlobalToLocalBegin(), DMDAGetRefinementFactor(),
@@ -1436,8 +1448,5 @@ PetscErrorCode  DMDACreate3d(MPI_Comm comm,DMBoundaryType bx,DMBoundaryType by,D
   ierr = DMDASetStencilType(*da, stencil_type);CHKERRQ(ierr);
   ierr = DMDASetStencilWidth(*da, s);CHKERRQ(ierr);
   ierr = DMDASetOwnershipRanges(*da, lx, ly, lz);CHKERRQ(ierr);
-  /* This violates the behavior for other classes, but right now users expect negative dimensions to be handled this way */
-  ierr = DMSetFromOptions(*da);CHKERRQ(ierr);
-  ierr = DMSetUp(*da);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
