@@ -128,9 +128,9 @@ PetscErrorCode DMForestTemplate(DM dm, MPI_Comm comm, DM *tdm)
   DMType                     type;
   DM                         base;
   DMForestTopology           topology;
+  MatType                    mtype;
   PetscInt                   dim, overlap, ref, factor;
   DMForestAdaptivityStrategy strat;
-  PetscDS                    ds;
   void                       *ctx;
   PetscErrorCode             (*map)(DM, PetscInt, PetscInt, const PetscReal[], PetscReal[], void*);
   void                       *mapCtx;
@@ -163,8 +163,7 @@ PetscErrorCode DMForestTemplate(DM dm, MPI_Comm comm, DM *tdm)
     ierr = (forest->ftemplate)(dm, *tdm);CHKERRQ(ierr);
   }
   ierr = DMForestSetAdaptivityForest(*tdm,dm);CHKERRQ(ierr);
-  ierr = DMGetDS(dm,&ds);CHKERRQ(ierr);
-  ierr = DMSetDS(*tdm,ds);CHKERRQ(ierr);
+  ierr = DMCopyDisc(dm,*tdm);CHKERRQ(ierr);
   ierr = DMGetApplicationContext(dm,&ctx);CHKERRQ(ierr);
   ierr = DMSetApplicationContext(*tdm,&ctx);CHKERRQ(ierr);
   {
@@ -176,6 +175,8 @@ PetscErrorCode DMForestTemplate(DM dm, MPI_Comm comm, DM *tdm)
     ierr = DMSetPeriodicity(*tdm,isper,maxCell,L,bd);CHKERRQ(ierr);
   }
   ierr = DMCopyBoundary(dm,*tdm);CHKERRQ(ierr);
+  ierr = DMGetMatType(dm,&mtype);CHKERRQ(ierr);
+  ierr = DMSetMatType(*tdm,mtype);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -301,11 +302,19 @@ PetscErrorCode DMForestSetBaseDM(DM dm, DM base)
   ierr         = DMDestroy(&forest->base);CHKERRQ(ierr);
   forest->base = base;
   if (base) {
+    PetscBool        isper;
+    const PetscReal *maxCell, *L;
+    const DMBoundaryType *bd;
+
     PetscValidHeaderSpecific(base, DM_CLASSID, 2);
     ierr = DMGetDimension(base,&dim);CHKERRQ(ierr);
     ierr = DMSetDimension(dm,dim);CHKERRQ(ierr);
     ierr = DMGetCoordinateDim(base,&dimEmbed);CHKERRQ(ierr);
     ierr = DMSetCoordinateDim(dm,dimEmbed);CHKERRQ(ierr);
+    ierr = DMGetPeriodicity(base,&isper,&maxCell,&L,&bd);CHKERRQ(ierr);
+    ierr = DMSetPeriodicity(dm,isper,maxCell,L,bd);CHKERRQ(ierr);
+  } else {
+    ierr = DMSetPeriodicity(dm,PETSC_FALSE,NULL,NULL,NULL);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -323,6 +332,9 @@ PetscErrorCode DMForestSetBaseDM(DM dm, DM base)
   Output Parameter:
 . base - the base DM of the forest
 
+  Notes:
+    After DMSetUp(), the base DM will be redundantly distributed across MPI processes
+
   Level: intermediate
 
 .seealso(); DMForestSetBaseDM()
@@ -411,6 +423,7 @@ PetscErrorCode DMForestSetAdaptivityForest(DM dm,DM adapt)
     ierr = DMSetCoarseDM(dm,adapt);CHKERRQ(ierr);
     break;
   case DM_ADAPT_COARSEN:
+  case DM_ADAPT_COARSEN_LAST:
     ierr = DMSetFineDM(dm,adapt);CHKERRQ(ierr);
     break;
   default:
@@ -450,6 +463,7 @@ PetscErrorCode DMForestGetAdaptivityForest(DM dm, DM *adapt)
     ierr = DMGetCoarseDM(dm,adapt);CHKERRQ(ierr);
     break;
   case DM_ADAPT_COARSEN:
+  case DM_ADAPT_COARSEN_LAST:
     ierr = DMGetFineDM(dm,adapt);CHKERRQ(ierr);
     break;
   default:
@@ -471,11 +485,11 @@ PetscErrorCode DMForestGetAdaptivityForest(DM dm, DM *adapt)
 
   Input Parameters:
 + dm - the forest
-- purpose - the adaptivity purpose (DM_ADAPT_DETERMINE/DM_ADAPT_REFINE/DM_ADAPT_COARSEN)
+- purpose - the adaptivity purpose
 
   Level: advanced
 
-.seealso: DMForestTemplate(), DMForestSetAdaptivityForest(), DMForestGetAdaptivityForest()
+.seealso: DMForestTemplate(), DMForestSetAdaptivityForest(), DMForestGetAdaptivityForest(), DMAdaptFlag
 @*/
 PetscErrorCode DMForestSetAdaptivityPurpose(DM dm, DMAdaptFlag purpose)
 {
@@ -502,8 +516,9 @@ PetscErrorCode DMForestSetAdaptivityPurpose(DM dm, DMAdaptFlag purpose)
 
 /*@
   DMForestGetAdaptivityPurpose - Get whether the current DM is being adapted from its source (set with
-  DMForestSetAdaptivityForest()) for the purpose of refinement (DM_ADAPT_REFINE), coarsening (DM_ADAPT_COARSEN), or
-  undefined (DM_ADAPT_DETERMINE).  This only matters for the purposes of reference counting: during DMDestroy(), cyclic
+  DMForestSetAdaptivityForest()) for the purpose of refinement (DM_ADAPT_REFINE), coarsening (DM_ADAPT_COARSEN),
+  coarsening only the last level (DM_ADAPT_COARSEN_LAST) or undefined (DM_ADAPT_DETERMINE).
+  This only matters for the purposes of reference counting: during DMDestroy(), cyclic
   references can be found between DMs only if the cyclic reference is due to a fine/coarse relationship (see
   DMSetFineDM()/DMSetCoarseDM()).  If the purpose is not refinement or coarsening, and the user does not maintain a
   reference to the post-adaptation forest (i.e., the one created by DMForestTemplate()), then this can cause a memory
@@ -515,11 +530,11 @@ PetscErrorCode DMForestSetAdaptivityPurpose(DM dm, DMAdaptFlag purpose)
 . dm - the forest
 
   Output Parameter:
-. purpose - the adaptivity purpose (DM_ADAPT_DETERMINE/DM_ADAPT_REFINE/DM_ADAPT_COARSEN)
+. purpose - the adaptivity purpose
 
   Level: advanced
 
-.seealso: DMForestTemplate(), DMForestSetAdaptivityForest(), DMForestGetAdaptivityForest()
+.seealso: DMForestTemplate(), DMForestSetAdaptivityForest(), DMForestGetAdaptivityForest(), DMAdaptFlag
 @*/
 PetscErrorCode DMForestGetAdaptivityPurpose(DM dm, DMAdaptFlag *purpose)
 {
@@ -787,7 +802,7 @@ PetscErrorCode DMForestSetInitialRefinement(DM dm, PetscInt initRefinement)
 . dm - the forest
 
   Output Paramater:
-. initefinement - default PETSC_DEFAULT (interpreted by the subtype of DMForest)
+. initRefinement - default PETSC_DEFAULT (interpreted by the subtype of DMForest)
 
   Level: intermediate
 
@@ -990,6 +1005,21 @@ PetscErrorCode DMForestTransferVec(DM dmIn, Vec vecIn, DM dmOut, Vec vecOut, Pet
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode DMForestTransferVecFromBase(DM dm, Vec vecIn, Vec vecOut)
+{
+  DM_Forest      *forest;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(dm   ,DM_CLASSID  ,1);
+  PetscValidHeaderSpecific(vecIn  ,VEC_CLASSID ,2);
+  PetscValidHeaderSpecific(vecOut ,VEC_CLASSID ,3);
+  forest = (DM_Forest *) dm->data;
+  if (!forest->transfervecfrombase) SETERRQ(PetscObjectComm((PetscObject)dm),PETSC_ERR_SUP,"DMForestTransferVecFromBase() not implemented");
+  ierr = (forest->transfervecfrombase)(dm,vecIn,vecOut);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
   DMForestGetComputeAdaptivitySF - Get whether transfer PetscSFs should be computed relating the cells of the
   pre-adaptation forest to the post-adaptiation forest.  After DMSetUp() is called, these transfer PetscSFs can be
@@ -1250,9 +1280,9 @@ PetscErrorCode DMForestSetAdaptivityLabel(DM dm, DMLabel adaptLabel)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
-  adaptLabel->refct++;
   if (forest->adaptLabel) {ierr = DMLabelDestroy(&forest->adaptLabel);CHKERRQ(ierr);}
   forest->adaptLabel = adaptLabel;
+  ierr = PetscObjectReference((PetscObject) adaptLabel);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1415,7 +1445,6 @@ PetscErrorCode DMForestGetWeightCapacity(DM dm, PetscReal *capacity)
 
 PETSC_EXTERN PetscErrorCode DMSetFromOptions_Forest(PetscOptionItems *PetscOptionsObject,DM dm)
 {
-  DM_Forest                  *forest = (DM_Forest*) dm->data;
   PetscBool                  flg, flg1, flg2, flg3, flg4;
   DMForestTopology           oldTopo;
   char                       stringBuffer[256];
@@ -1428,7 +1457,6 @@ PETSC_EXTERN PetscErrorCode DMSetFromOptions_Forest(PetscOptionItems *PetscOptio
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
-  forest->setfromoptionscalled = PETSC_TRUE;
   ierr                         = DMForestGetTopology(dm, &oldTopo);CHKERRQ(ierr);
   ierr                         = PetscOptionsHead(PetscOptionsObject,"DMForest Options");CHKERRQ(ierr);
   ierr                         = PetscOptionsString("-dm_forest_topology","the topology of the forest's base mesh","DMForestSetTopology",oldTopo,stringBuffer,256,&flg1);CHKERRQ(ierr);
@@ -1545,7 +1573,7 @@ PetscErrorCode DMCreateSubDM_Forest(DM dm, PetscInt numFields, const PetscInt fi
 
   PetscFunctionBegin;
   if (subdm) {ierr = DMClone(dm, subdm);CHKERRQ(ierr);}
-  ierr = DMCreateSubDM_Section_Private(dm, numFields, fields, is, subdm);CHKERRQ(ierr);
+  ierr = DMCreateSectionSubDM(dm, numFields, fields, is, subdm);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1565,11 +1593,10 @@ PetscErrorCode DMRefine_Forest(DM dm, MPI_Comm comm, DM *dmRefined)
   ierr = DMForestTemplate(dm,comm,dmRefined);CHKERRQ(ierr);
   ierr = DMGetLabel(dm,"refine",&refine);CHKERRQ(ierr);
   if (!refine) {
-    ierr = DMLabelCreate("refine",&refine);CHKERRQ(ierr);
+    ierr = DMLabelCreate(PETSC_COMM_SELF, "refine",&refine);CHKERRQ(ierr);
     ierr = DMLabelSetDefaultValue(refine,DM_ADAPT_REFINE);CHKERRQ(ierr);
-  }
-  else {
-    refine->refct++;
+  } else {
+    ierr = PetscObjectReference((PetscObject) refine);CHKERRQ(ierr);
   }
   ierr = DMForestSetAdaptivityLabel(*dmRefined,refine);CHKERRQ(ierr);
   ierr = DMLabelDestroy(&refine);CHKERRQ(ierr);
@@ -1597,13 +1624,13 @@ PetscErrorCode DMCoarsen_Forest(DM dm, MPI_Comm comm, DM *dmCoarsened)
     PetscFunctionReturn(0);
   }
   ierr = DMForestTemplate(dm,comm,dmCoarsened);CHKERRQ(ierr);
-  ierr = DMForestSetAdaptivityPurpose(coarseDM,DM_ADAPT_COARSEN);CHKERRQ(ierr);
+  ierr = DMForestSetAdaptivityPurpose(*dmCoarsened,DM_ADAPT_COARSEN);CHKERRQ(ierr);
   ierr = DMGetLabel(dm,"coarsen",&coarsen);CHKERRQ(ierr);
   if (!coarsen) {
-    ierr = DMLabelCreate("coarsen",&coarsen);CHKERRQ(ierr);
+    ierr = DMLabelCreate(PETSC_COMM_SELF, "coarsen",&coarsen);CHKERRQ(ierr);
     ierr = DMLabelSetDefaultValue(coarsen,DM_ADAPT_COARSEN);CHKERRQ(ierr);
   } else {
-    coarsen->refct++;
+    ierr = PetscObjectReference((PetscObject) coarsen);CHKERRQ(ierr);
   }
   ierr = DMForestSetAdaptivityLabel(*dmCoarsened,coarsen);CHKERRQ(ierr);
   ierr = DMLabelDestroy(&coarsen);CHKERRQ(ierr);
@@ -1674,7 +1701,6 @@ PETSC_EXTERN PetscErrorCode DMCreate_Forest(DM dm)
   dm->data                     = forest;
   forest->refct                = 1;
   forest->data                 = NULL;
-  forest->setfromoptionscalled = PETSC_FALSE;
   forest->topology             = NULL;
   forest->adapt                = NULL;
   forest->base                 = NULL;
@@ -1697,4 +1723,3 @@ PETSC_EXTERN PetscErrorCode DMCreate_Forest(DM dm)
   ierr                         = DMInitialize_Forest(dm);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-
