@@ -10,7 +10,7 @@ static PetscErrorCode TestInsertion()
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = DMLabelCreate("Test Label", &label);CHKERRQ(ierr);
+  ierr = DMLabelCreate(PETSC_COMM_SELF, "Test Label", &label);CHKERRQ(ierr);
   ierr = DMLabelSetDefaultValue(label, -100);CHKERRQ(ierr);
   for (i = 0; i < N; ++i) {
     ierr = DMLabelSetValue(label, i, values[i%5]);CHKERRQ(ierr);
@@ -118,20 +118,27 @@ static PetscErrorCode TestEmptyStrata(MPI_Comm comm)
   }
   ierr = DMPlexSymmetrize(dm);CHKERRQ(ierr);
   /* Create a user managed depth label, so that we can leave out edges */
-  if (!rank) {
-    DMLabel  label;
-    PetscInt i;
+  {
+    DMLabel label;
+    PetscInt numValues, maxValues = 0, v;
 
     ierr = DMCreateLabel(dm, "depth");CHKERRQ(ierr);
     ierr = DMPlexGetDepthLabel(dm, &label);CHKERRQ(ierr);
-    for (i = 0; i < 25; ++i) {
-      if (i < 2)       {ierr = DMLabelSetValue(label, i, 3);CHKERRQ(ierr);}
-      else if (i < 13) {ierr = DMLabelSetValue(label, i, 2);CHKERRQ(ierr);}
-      else             {
-        if (i==13) {ierr = DMLabelAddStratum(label, 1);CHKERRQ(ierr);}
-        ierr = DMLabelSetValue(label, i, 0);CHKERRQ(ierr);
+    if (!rank) {
+      PetscInt i;
+
+      for (i = 0; i < 25; ++i) {
+        if (i < 2)       {ierr = DMLabelSetValue(label, i, 3);CHKERRQ(ierr);}
+        else if (i < 13) {ierr = DMLabelSetValue(label, i, 2);CHKERRQ(ierr);}
+        else             {
+          if (i==13) {ierr = DMLabelAddStratum(label, 1);CHKERRQ(ierr);}
+          ierr = DMLabelSetValue(label, i, 0);CHKERRQ(ierr);
+        }
       }
     }
+    ierr = DMLabelGetNumValues(label, &numValues);CHKERRQ(ierr);
+    ierr = MPI_Allreduce(&numValues, &maxValues, 1, MPIU_INT, MPI_MAX, PetscObjectComm((PetscObject) dm));CHKERRQ(ierr);
+    for (v = numValues; v < maxValues; ++v) {ierr = DMLabelAddStratum(label,v);CHKERRQ(ierr);}
   }
   ierr = DMPlexDistribute(dm, 1, NULL, &dmDist);CHKERRQ(ierr);
   if (dmDist) {
@@ -146,7 +153,8 @@ static PetscErrorCode TestEmptyStrata(MPI_Comm comm)
     PetscInt     dof[]     = {0,0,0,1};
     PetscInt     N;
 
-    ierr = DMPlexCreateSection(dm, dim, 1, numComp, dof, 0, NULL, NULL, NULL, NULL, &s);CHKERRQ(ierr);
+    ierr = DMSetNumFields(dm, 1);CHKERRQ(ierr);
+    ierr = DMPlexCreateSection(dm, NULL, numComp, dof, 0, NULL, NULL, NULL, NULL, &s);CHKERRQ(ierr);
     ierr = DMSetSection(dm, s);CHKERRQ(ierr);
     ierr = PetscSectionDestroy(&s);CHKERRQ(ierr);
     ierr = DMCreateGlobalVector(dm, &v);CHKERRQ(ierr);
@@ -178,8 +186,7 @@ static PetscErrorCode TestDistribution(MPI_Comm comm)
   if (!flg) PetscFunctionReturn(0);
   ierr = PetscOptionsGetInt(NULL, NULL, "-overlap", &overlap, NULL);CHKERRQ(ierr);
   ierr = DMPlexCreateFromFile(comm, filename, PETSC_TRUE, &dm); CHKERRQ(ierr);
-  ierr = DMPlexSetAdjacencyUseCone(dm, PETSC_TRUE);CHKERRQ(ierr);
-  ierr = DMPlexSetAdjacencyUseClosure(dm, PETSC_FALSE);CHKERRQ(ierr);
+  ierr = DMSetBasicAdjacency(dm, PETSC_TRUE, PETSC_FALSE);CHKERRQ(ierr);
   ierr = DMCreateLabel(dm, name);CHKERRQ(ierr);
   ierr = DMGetLabel(dm, name, &label);CHKERRQ(ierr);
   ierr = DMPlexGetHeightStratum(dm, 0, &cStart, &cEnd);CHKERRQ(ierr);
