@@ -472,7 +472,7 @@ int main(int argc, char **argv) {
 /*TEST
 
   build:
-    requires: exodusii pnetcdf
+    requires: exodusii pnetcdf !complex
   # 2D seq
   test:
     suffix: 0
