@@ -419,7 +419,8 @@ static PetscErrorCode SetupSection(DM dm, AppCtx *user)
     ierr = ISDestroy(&aIS);CHKERRQ(ierr);
   }
   ierr = DMSetNumFields(dm, 1);CHKERRQ(ierr);
-  ierr = DMSetField(dm, 0, (PetscObject) user->fe);CHKERRQ(ierr);
+  ierr = DMSetField(dm, 0, NULL, (PetscObject) user->fe);CHKERRQ(ierr);
+  ierr = DMCreateDS(dm);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)dm,DMPLEX,&isPlex);CHKERRQ(ierr);
   if (!isPlex) {
     PetscSection    section;
@@ -550,7 +551,7 @@ static PetscErrorCode TestFEJacobian(DM dm, AppCtx *user)
     MatNullSpace sp;
     PetscBool    isNullSpace, hasConst;
     PetscInt     n, i;
-    Vec          res, localX, localRes;
+    Vec          res = NULL, localX, localRes;
     PetscDS      ds;
 
     if (user->numComponents != user->dim) SETERRQ2(PetscObjectComm((PetscObject) dm), PETSC_ERR_ARG_OUTOFRANGE, "The number of components %d must be equal to the dimension %d for this test", user->numComponents, user->dim);
@@ -561,7 +562,7 @@ static PetscErrorCode TestFEJacobian(DM dm, AppCtx *user)
     ierr = DMPlexSNESComputeJacobianFEM(dm,local,E,E,NULL);CHKERRQ(ierr);
     ierr = DMPlexCreateRigidBody(dm,&sp);CHKERRQ(ierr);
     ierr = MatNullSpaceGetVecs(sp,&hasConst,&n,&vecs);CHKERRQ(ierr);
-    ierr = VecDuplicate(vecs[0],&res);CHKERRQ(ierr);
+    if (n) {ierr = VecDuplicate(vecs[0],&res);CHKERRQ(ierr);}
     ierr = DMCreateLocalVector(dm,&localX);CHKERRQ(ierr);
     ierr = DMCreateLocalVector(dm,&localRes);CHKERRQ(ierr);
     for (i = 0; i < n; i++) { /* also test via matrix-free Jacobian application */
@@ -634,8 +635,7 @@ static PetscErrorCode TestFVGrad(DM dm, AppCtx *user)
   PetscFunctionBeginUser;
   comm = PetscObjectComm((PetscObject)dm);
   /* duplicate DM, give dup. a FV discretization */
-  ierr = DMPlexSetAdjacencyUseCone(dm,PETSC_TRUE);CHKERRQ(ierr);
-  ierr = DMPlexSetAdjacencyUseClosure(dm,PETSC_FALSE);CHKERRQ(ierr);
+  ierr = DMSetBasicAdjacency(dm,PETSC_TRUE,PETSC_FALSE);CHKERRQ(ierr);
   ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
   dmRedist = NULL;
   if (size > 1) {
@@ -654,13 +654,10 @@ static PetscErrorCode TestFVGrad(DM dm, AppCtx *user)
   ierr = DMPlexConstructGhostCells(dmRedist,NULL,NULL,&dmfv);CHKERRQ(ierr);
   ierr = DMDestroy(&dmRedist);CHKERRQ(ierr);
   ierr = DMSetNumFields(dmfv,1);CHKERRQ(ierr);
-  ierr = DMSetField(dmfv, 0, (PetscObject) fv);CHKERRQ(ierr);
+  ierr = DMSetField(dmfv, 0, NULL, (PetscObject) fv);CHKERRQ(ierr);
+  ierr = DMCreateDS(dmfv);CHKERRQ(ierr);
   ierr = DMPlexGetReferenceTree(dm,&refTree);CHKERRQ(ierr);
-  if (refTree) {
-    PetscDS ds;
-    ierr = DMGetDS(dmfv,&ds);CHKERRQ(ierr);
-    ierr = DMSetDS(refTree,ds);CHKERRQ(ierr);
-  }
+  if (refTree) {ierr = DMCopyDisc(dmfv,refTree);CHKERRQ(ierr);}
   ierr = DMPlexSNESGetGradientDM(dmfv, fv, &dmgrad);CHKERRQ(ierr);
   ierr = DMPlexGetHeightStratum(dmfv,0,&cStart,&cEnd);CHKERRQ(ierr);
   nvecs = user->dim * (user->dim+1) / 2;
