@@ -550,12 +550,6 @@ int main(int argc,char **args)
       requires: datafilespath double !define(PETSC_USE_64BIT_INDICES)
       args: -ksp_type fgmres -pc_type ksp -f0 ${DATAFILESPATH}/matrices/medium -ksp_fgmres_modifypcksp -ksp_monitor_short
 
-   testset:
-      TODO: Need to determine goal of this test
-      suffix: 11
-      nsize: 2
-      args: -f0 http://ftp.mcs.anl.gov/pub/petsc/Datafiles/matrices/testmatrix.gz
-
    testset:
       suffix: 12
       requires: matlab
@@ -587,6 +581,10 @@ int main(int argc,char **args)
          suffix: 17
          requires: hypre
          args: -pc_type hypre -pc_hypre_type boomeramg
+      test:
+         suffix: 18
+         requires: hypre
+         args: -pc_type hypre -pc_hypre_type euclid
 
    testset:
       suffix: 19
@@ -611,7 +609,7 @@ int main(int argc,char **args)
 
    testset:
       suffix: aijcusparse
-      requires: datafilespath double !define(PETSC_USE_64BIT_INDICES) veccuda
+      requires: datafilespath double !define(PETSC_USE_64BIT_INDICES) cuda
       args: -f0 ${DATAFILESPATH}/matrices/medium -ksp_monitor_short -ksp_view -mat_view ascii::ascii_info -mat_type aijcusparse -pc_factor_mat_solver_type cusparse -pc_type ilu -vec_type cuda
 
    testset:
