@@ -48,6 +48,7 @@ static PetscErrorCode KSPChebyshevEstEigSet_Chebyshev(KSP ksp,PetscReal a,PetscR
   if (a != 0.0 || b != 0.0 || c != 0.0 || d != 0.0) {
     if (!cheb->kspest) { /* should this block of code be moved to KSPSetUp_Chebyshev()? */
       ierr = KSPCreate(PetscObjectComm((PetscObject)ksp),&cheb->kspest);CHKERRQ(ierr);
+      ierr = KSPSetErrorIfNotConverged(cheb->kspest,ksp->errorifnotconverged);CHKERRQ(ierr);
       ierr = PetscObjectIncrementTabLevel((PetscObject)cheb->kspest,(PetscObject)ksp,1);CHKERRQ(ierr);
       ierr = KSPSetPC(cheb->kspest,ksp->pc);CHKERRQ(ierr);
       /* use PetscObjectSet/AppendOptionsPrefix() instead of KSPSet/AppendOptionsPrefix() so that the PC prefix is not changed */
@@ -311,7 +312,7 @@ static PetscErrorCode KSPSolve_Chebyshev(KSP ksp)
 {
   KSP_Chebyshev  *cheb = (KSP_Chebyshev*)ksp->data;
   PetscErrorCode ierr;
-  PetscInt       k,kp1,km1,maxit,ktmp,i;
+  PetscInt       k,kp1,km1,ktmp,i;
   PetscScalar    alpha,omegaprod,mu,omega,Gamma,c[3],scale;
   PetscReal      rnorm = 0.0;
   Vec            sol_orig,b,p[3],r;
@@ -326,6 +327,8 @@ static PetscErrorCode KSPSolve_Chebyshev(KSP ksp)
   if (cheb->kspest) {
     PetscObjectId    amatid,    pmatid;
     PetscObjectState amatstate, pmatstate;
+    PCFailedReason   pcreason;
+    PC               pc;
 
     ierr = PetscObjectGetId((PetscObject)Amat,&amatid);CHKERRQ(ierr);
     ierr = PetscObjectGetId((PetscObject)Pmat,&pmatid);CHKERRQ(ierr);
@@ -365,21 +368,19 @@ static PetscErrorCode KSPSolve_Chebyshev(KSP ksp)
         }
       }
       ierr = KSPSolve(cheb->kspest,B,ksp->work[0]);CHKERRQ(ierr);
-
       ierr = KSPGetConvergedReason(cheb->kspest,&reason);CHKERRQ(ierr);
-      if (reason < 0) {
+      ierr = KSPGetPC(cheb->kspest,&pc);CHKERRQ(ierr);
+      ierr = PCGetFailedReason(pc,&pcreason);CHKERRQ(ierr);
+      if (reason < 0 || pcreason) {
         if (reason == KSP_DIVERGED_ITS) {
           ierr = PetscInfo(ksp,"Eigen estimator ran for prescribed number of iterations\n");CHKERRQ(ierr);
         } else {
           PetscInt its;
           ierr = KSPGetIterationNumber(cheb->kspest,&its);CHKERRQ(ierr);
-          if (reason == KSP_DIVERGED_PCSETUP_FAILED) {
-            ierr = PetscInfo(ksp,"Eigen estimator KSP_DIVERGED_PCSETUP_FAILED\n");CHKERRQ(ierr);
-            ksp->reason = KSP_DIVERGED_PCSETUP_FAILED;
-            ierr = VecSetInf(ksp->vec_sol);CHKERRQ(ierr);
-          } else {
-            SETERRQ2(PetscObjectComm((PetscObject)ksp),PETSC_ERR_PLIB,"Eigen estimator failed: %s at iteration %D",KSPConvergedReasons[reason],its);
-          }
+          ksp->reason = KSP_DIVERGED_PC_FAILED;
+          ierr = VecSetInf(ksp->vec_sol);CHKERRQ(ierr);
+          ierr = PetscInfo3(ksp,"Eigen estimator failed: %s %s at iteration %D",KSPConvergedReasons[reason],PCFailedReasons[pcreason],its);CHKERRQ(ierr);
+          PetscFunctionReturn(0);
         }
       } else if (reason==KSP_CONVERGED_RTOL ||reason==KSP_CONVERGED_ATOL) {
         ierr = PetscInfo(ksp,"Eigen estimator converged prematurely. Should not happen except for small or low rank problem\n");CHKERRQ(ierr);
@@ -400,10 +401,9 @@ static PetscErrorCode KSPSolve_Chebyshev(KSP ksp)
       cheb->pmatstate = pmatstate;
     }
   }
-
+  ierr = PetscObjectSAWsTakeAccess((PetscObject)ksp);CHKERRQ(ierr);
   ksp->its = 0;
-  maxit    = ksp->max_it;
-
+  ierr   = PetscObjectSAWsGrantAccess((PetscObject)ksp);CHKERRQ(ierr);
   /* These three point to the three active solutions, we
      rotate these three at each solution update */
   km1      = 0; k = 1; kp1 = 2;
@@ -427,35 +427,71 @@ static PetscErrorCode KSPSolve_Chebyshev(KSP ksp)
   c[k]   = mu;
 
   if (!ksp->guess_zero) {
-    ierr = KSP_MatMult(ksp,Amat,p[km1],r);CHKERRQ(ierr);     /*  r = b - A*p[km1] */
+    ierr = KSP_MatMult(ksp,Amat,sol_orig,r);CHKERRQ(ierr);     /*  r = b - A*p[km1] */
     ierr = VecAYPX(r,-1.0,b);CHKERRQ(ierr);
   } else {
     ierr = VecCopy(b,r);CHKERRQ(ierr);
   }
 
-  ierr = KSP_PCApply(ksp,r,p[k]);CHKERRQ(ierr);  /* p[k] = scale B^{-1}r + p[km1] */
-  ierr = VecAYPX(p[k],scale,p[km1]);CHKERRQ(ierr);
+  /* calculate residual norm if requested, we have done one iteration */
+  if (ksp->normtype) {
+    switch (ksp->normtype) {
+    case KSP_NORM_PRECONDITIONED:
+      ierr = KSP_PCApply(ksp,r,p[k]);CHKERRQ(ierr);  /* p[k] = B^{-1}r */
+      ierr = VecNorm(p[k],NORM_2,&rnorm);CHKERRQ(ierr);
+      break;
+    case KSP_NORM_UNPRECONDITIONED:
+    case KSP_NORM_NATURAL:
+      ierr = VecNorm(r,NORM_2,&rnorm);CHKERRQ(ierr);
+      break;
+    case KSP_NORM_NONE:
+      rnorm = 0.0;
+      break;
+    default: SETERRQ1(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP,"%s",KSPNormTypes[ksp->normtype]);
+    }
+    ierr         = PetscObjectSAWsTakeAccess((PetscObject)ksp);CHKERRQ(ierr);
+    ksp->rnorm   = rnorm;
+    ierr = PetscObjectSAWsGrantAccess((PetscObject)ksp);CHKERRQ(ierr);
+    ierr = KSPLogResidualHistory(ksp,rnorm);CHKERRQ(ierr);
+    ierr = KSPMonitor(ksp,0,rnorm);CHKERRQ(ierr);
+    ierr = (*ksp->converged)(ksp,0,rnorm,&ksp->reason,ksp->cnvP);CHKERRQ(ierr);
+  }
+  else ksp->reason = KSP_CONVERGED_ITERATING;
+  if (ksp->reason || ksp->max_it==0) {
+    if (ksp->max_it==0) ksp->reason = KSP_DIVERGED_ITS; /* This for a V(0,x) cycle */
+    PetscFunctionReturn(0);
+  }
+  if (ksp->normtype != KSP_NORM_PRECONDITIONED) {
+    ierr = KSP_PCApply(ksp,r,p[k]);CHKERRQ(ierr);  /* p[k] = B^{-1}r */
+  }
+  ierr = VecAYPX(p[k],scale,p[km1]);CHKERRQ(ierr);  /* p[k] = scale B^{-1}r + p[km1] */
+  ierr = PetscObjectSAWsTakeAccess((PetscObject)ksp);CHKERRQ(ierr);
+  ksp->its = 1;
+  ierr   = PetscObjectSAWsGrantAccess((PetscObject)ksp);CHKERRQ(ierr);
 
-  for (i=0; i<maxit; i++) {
+  for (i=1; i<ksp->max_it; i++) {
     ierr = PetscObjectSAWsTakeAccess((PetscObject)ksp);CHKERRQ(ierr);
-
     ksp->its++;
     ierr   = PetscObjectSAWsGrantAccess((PetscObject)ksp);CHKERRQ(ierr);
-    c[kp1] = 2.0*mu*c[k] - c[km1];
-    omega  = omegaprod*c[k]/c[kp1];
 
     ierr = KSP_MatMult(ksp,Amat,p[k],r);CHKERRQ(ierr);          /*  r = b - Ap[k]    */
     ierr = VecAYPX(r,-1.0,b);CHKERRQ(ierr);
-    ierr = KSP_PCApply(ksp,r,p[kp1]);CHKERRQ(ierr);             /*  p[kp1] = B^{-1}r  */
-    ksp->vec_sol = p[k];
-
     /* calculate residual norm if requested */
-    if (ksp->normtype != KSP_NORM_NONE || ksp->numbermonitors) {
-      if (ksp->normtype == KSP_NORM_UNPRECONDITIONED) {
-        ierr = VecNorm(r,NORM_2,&rnorm);CHKERRQ(ierr);
-      } else {
+    if (ksp->normtype) {
+      switch (ksp->normtype) {
+      case KSP_NORM_PRECONDITIONED:
+        ierr = KSP_PCApply(ksp,r,p[kp1]);CHKERRQ(ierr);             /*  p[kp1] = B^{-1}r  */
         ierr = VecNorm(p[kp1],NORM_2,&rnorm);CHKERRQ(ierr);
+        break;
+      case KSP_NORM_UNPRECONDITIONED:
+      case KSP_NORM_NATURAL:
+        ierr = VecNorm(r,NORM_2,&rnorm);CHKERRQ(ierr);
+        break;
+      default:
+        rnorm = 0.0;
+        break;
       }
+      KSPCheckNorm(ksp,rnorm);
       ierr         = PetscObjectSAWsTakeAccess((PetscObject)ksp);CHKERRQ(ierr);
       ksp->rnorm   = rnorm;
       ierr = PetscObjectSAWsGrantAccess((PetscObject)ksp);CHKERRQ(ierr);
@@ -463,7 +499,16 @@ static PetscErrorCode KSPSolve_Chebyshev(KSP ksp)
       ierr = KSPMonitor(ksp,i,rnorm);CHKERRQ(ierr);
       ierr = (*ksp->converged)(ksp,i,rnorm,&ksp->reason,ksp->cnvP);CHKERRQ(ierr);
       if (ksp->reason) break;
+      if (ksp->normtype != KSP_NORM_PRECONDITIONED) {
+        ierr = KSP_PCApply(ksp,r,p[kp1]);CHKERRQ(ierr);             /*  p[kp1] = B^{-1}r  */
+      }
+    } else {
+      ierr = KSP_PCApply(ksp,r,p[kp1]);CHKERRQ(ierr);             /*  p[kp1] = B^{-1}r  */
     }
+    ksp->vec_sol = p[k];
+
+    c[kp1] = 2.0*mu*c[k] - c[km1];
+    omega  = omegaprod*c[k]/c[kp1];
 
     /* y^{k+1} = omega(y^{k} - y^{k-1} + Gamma*r^{k}) + y^{k-1} */
     ierr = VecAXPBYPCZ(p[kp1],1.0-omega,omega,omega*Gamma*scale,p[km1],p[k]);CHKERRQ(ierr);
@@ -474,19 +519,26 @@ static PetscErrorCode KSPSolve_Chebyshev(KSP ksp)
     kp1  = ktmp;
   }
   if (!ksp->reason) {
-    if (ksp->normtype != KSP_NORM_NONE) {
+    if (ksp->normtype) {
       ierr = KSP_MatMult(ksp,Amat,p[k],r);CHKERRQ(ierr);       /*  r = b - Ap[k]    */
       ierr = VecAYPX(r,-1.0,b);CHKERRQ(ierr);
-      if (ksp->normtype == KSP_NORM_UNPRECONDITIONED) {
-        ierr = VecNorm(r,NORM_2,&rnorm);CHKERRQ(ierr);
-      } else {
+      switch (ksp->normtype) {
+      case KSP_NORM_PRECONDITIONED:
         ierr = KSP_PCApply(ksp,r,p[kp1]);CHKERRQ(ierr); /* p[kp1] = B^{-1}r */
         ierr = VecNorm(p[kp1],NORM_2,&rnorm);CHKERRQ(ierr);
+        break;
+      case KSP_NORM_UNPRECONDITIONED:
+      case KSP_NORM_NATURAL:
+        ierr = VecNorm(r,NORM_2,&rnorm);CHKERRQ(ierr);
+        break;
+      default:
+        rnorm = 0.0;
+        break;
       }
+      KSPCheckNorm(ksp,rnorm);
       ierr         = PetscObjectSAWsTakeAccess((PetscObject)ksp);CHKERRQ(ierr);
       ksp->rnorm   = rnorm;
       ierr         = PetscObjectSAWsGrantAccess((PetscObject)ksp);CHKERRQ(ierr);
-      ksp->vec_sol = p[k];
       ierr = KSPLogResidualHistory(ksp,rnorm);CHKERRQ(ierr);
       ierr = KSPMonitor(ksp,i,rnorm);CHKERRQ(ierr);
     }
