@@ -116,13 +116,16 @@ static PetscErrorCode KSPSolve_FCG(KSP ksp)
     case KSP_NORM_PRECONDITIONED:
       ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*   z <- Br         */
       ierr = VecNorm(Z,NORM_2,&dp);CHKERRQ(ierr);              /*   dp <- dqrt(z'*z) = sqrt(e'*A'*B'*B*A*e)     */
+      KSPCheckNorm(ksp,dp);
       break;
     case KSP_NORM_UNPRECONDITIONED:
       ierr = VecNorm(R,NORM_2,&dp);CHKERRQ(ierr);              /*   dp <- sqrt(r'*r) = sqrt(e'*A'*A*e)     */
+      KSPCheckNorm(ksp,dp);
       break;
     case KSP_NORM_NATURAL:
       ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*   z <- Br         */
       ierr = VecXDot(R,Z,&s);CHKERRQ(ierr);
+      KSPCheckDot(ksp,s);
       dp = PetscSqrtReal(PetscAbsScalar(s));                   /*   dp <- sqrt(r'*z) = sqrt(e'*A'*B*A*e)  */
       break;
     case KSP_NORM_NONE:
@@ -202,6 +205,7 @@ static PetscErrorCode KSPSolve_FCG(KSP ksp)
     /* Update X and R */
     betaold = beta;
     ierr = VecXDot(Pcurr,R,&beta);CHKERRQ(ierr);                 /*  beta <- pi'*r       */
+    KSPCheckDot(ksp,beta);
     ierr = KSP_MatMult(ksp,Amat,Pcurr,Ccurr);CHKERRQ(ierr);      /*  w <- A*pi (stored in ci)   */
     ierr = VecXDot(Pcurr,Ccurr,&dpi);CHKERRQ(ierr);              /*  dpi <- pi'*w        */
     alphaold = alpha;
@@ -214,13 +218,16 @@ static PetscErrorCode KSPSolve_FCG(KSP ksp)
       case KSP_NORM_PRECONDITIONED:
         ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*   z <- Br             */
         ierr = VecNorm(Z,NORM_2,&dp);CHKERRQ(ierr);              /*   dp <- sqrt(z'*z) = sqrt(e'*A'*B'*B*A*e)  */
-        break;
+        KSPCheckNorm(ksp,dp);
+      break;
       case KSP_NORM_UNPRECONDITIONED:
         ierr = VecNorm(R,NORM_2,&dp);CHKERRQ(ierr);              /*   dp <- sqrt(r'*r) = sqrt(e'*A'*A*e)   */
+        KSPCheckNorm(ksp,dp);
         break;
       case KSP_NORM_NATURAL:
         ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*   z <- Br             */
         ierr = VecXDot(R,Z,&s);CHKERRQ(ierr);
+        KSPCheckDot(ksp,s);
         dp = PetscSqrtReal(PetscAbsScalar(s));                   /*   dp <- sqrt(r'*z) = sqrt(e'*A'*B*A*e)  */
         break;
       case KSP_NORM_NONE:
