@@ -1,4 +1,4 @@
-#include <petsc-private/fortranimpl.h>
+#include <petsc/private/fortranimpl.h>
 #include <petscksp.h>
 #include <petscviewer.h>
 
@@ -12,7 +12,7 @@
 #define kspview_                   kspview
 #endif
 
-PETSC_EXTERN void PETSC_STDCALL kspgettype_(KSP *ksp,CHAR name PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
+PETSC_EXTERN void PETSC_STDCALL kspgettype_(KSP *ksp,char* name PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   const char *tname;
 
@@ -22,12 +22,12 @@ PETSC_EXTERN void PETSC_STDCALL kspgettype_(KSP *ksp,CHAR name PETSC_MIXED_LEN(l
 
 }
 
-PETSC_EXTERN void PETSC_STDCALL kspsettype_(KSP *ksp,CHAR type PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
+PETSC_EXTERN void PETSC_STDCALL kspsettype_(KSP *ksp,char* type PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   char *t;
 
   FIXCHAR(type,len,t);
-  *ierr = KSPSetType(*ksp,t);
+  *ierr = KSPSetType(*ksp,t);if (*ierr) return;
   FREECHAR(type,t);
 }
 
