@@ -221,9 +221,9 @@ PetscErrorCode  KSPSetUpOnBlocks(KSP ksp)
   PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
   ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
   ierr = PCSetUpOnBlocks(pc);CHKERRQ(ierr);
-  ierr = PCGetSetUpFailedReason(pc,&pcreason);CHKERRQ(ierr);
+  ierr = PCGetFailedReason(pc,&pcreason);CHKERRQ(ierr);
   if (pcreason) {
-    ksp->reason = KSP_DIVERGED_PCSETUP_FAILED;
+    ksp->reason = KSP_DIVERGED_PC_FAILED;
   }
   PetscFunctionReturn(0);
 }
@@ -389,9 +389,9 @@ PetscErrorCode KSPSetUp(KSP ksp)
   ierr = PetscLogEventEnd(KSP_SetUp,ksp,ksp->vec_rhs,ksp->vec_sol,0);CHKERRQ(ierr);
   ierr = PCSetErrorIfFailure(ksp->pc,ksp->errorifnotconverged);CHKERRQ(ierr);
   ierr = PCSetUp(ksp->pc);CHKERRQ(ierr);
-  ierr = PCGetSetUpFailedReason(ksp->pc,&pcreason);CHKERRQ(ierr); 
+  ierr = PCGetFailedReason(ksp->pc,&pcreason);CHKERRQ(ierr); 
   if (pcreason) {
-    ksp->reason = KSP_DIVERGED_PCSETUP_FAILED;
+    ksp->reason = KSP_DIVERGED_PC_FAILED;
   }
 
   ierr = MatGetNullSpace(mat,&nullsp);CHKERRQ(ierr);
@@ -428,10 +428,10 @@ static PetscErrorCode KSPReasonView_Internal(KSP ksp, PetscViewer viewer, PetscV
       } else {
         ierr = PetscViewerASCIIPrintf(viewer,"Linear solve did not converge due to %s iterations %D\n",KSPConvergedReasons[ksp->reason],ksp->its);CHKERRQ(ierr);
       }
-      if (ksp->reason == KSP_DIVERGED_PCSETUP_FAILED) {
+      if (ksp->reason == KSP_DIVERGED_PC_FAILED) {
         PCFailedReason reason;
-        ierr = PCGetSetUpFailedReason(ksp->pc,&reason);CHKERRQ(ierr);
-        ierr = PetscViewerASCIIPrintf(viewer,"               PCSETUP_FAILED due to %s \n",PCFailedReasons[reason]);CHKERRQ(ierr);
+        ierr = PCGetFailedReason(ksp->pc,&reason);CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(viewer,"               PC_FAILED due to %s \n",PCFailedReasons[reason]);CHKERRQ(ierr);
       }
     }
     ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)ksp)->tablevel);CHKERRQ(ierr);
@@ -492,7 +492,7 @@ PetscErrorCode KSPReasonViewFromOptions(KSP ksp)
   PetscErrorCode    ierr;
 
   PetscFunctionBegin;
-  ierr   = PetscOptionsGetViewer(PetscObjectComm((PetscObject)ksp),((PetscObject)ksp)->prefix,"-ksp_converged_reason",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr   = PetscOptionsGetViewer(PetscObjectComm((PetscObject)ksp),((PetscObject)ksp)->options,((PetscObject)ksp)->prefix,"-ksp_converged_reason",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = KSPReasonView_Internal(ksp, viewer, format);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
@@ -697,12 +697,14 @@ PetscErrorCode KSPSolve(KSP ksp,Vec b,Vec x)
   }
   if (ksp->viewPre) {ierr = ObjectView((PetscObject) ksp, ksp->viewerPre, ksp->formatPre);CHKERRQ(ierr);}
 
+  ksp->transpose_solve = PETSC_FALSE;
+
   if (ksp->presolve) {ierr = (*ksp->presolve)(ksp,ksp->vec_rhs,ksp->vec_sol,ksp->prectx);CHKERRQ(ierr);}
-  ierr = PetscLogEventBegin(KSP_Solve,ksp,ksp->vec_rhs,ksp->vec_sol,0);CHKERRQ(ierr);
 
   /* reset the residual history list if requested */
   if (ksp->res_hist_reset) ksp->res_hist_len = 0;
-  ksp->transpose_solve = PETSC_FALSE;
+
+  ierr = PetscLogEventBegin(KSP_Solve,ksp,ksp->vec_rhs,ksp->vec_sol,0);CHKERRQ(ierr);
 
   if (ksp->guess) {
     PetscObjectState ostate,state;
@@ -723,7 +725,7 @@ PetscErrorCode KSPSolve(KSP ksp,Vec b,Vec x)
   ierr = KSPSetUp(ksp);CHKERRQ(ierr);
   ierr = KSPSetUpOnBlocks(ksp);CHKERRQ(ierr);
 
-  VecLocked(ksp->vec_sol,3);
+  ierr = VecSetErrorIfLocked(ksp->vec_sol,3);CHKERRQ(ierr);
 
   ierr = PCGetOperators(ksp->pc,&mat,&pmat);CHKERRQ(ierr);
   /* diagonal scale RHS if called for */
@@ -773,13 +775,13 @@ PetscErrorCode KSPSolve(KSP ksp,Vec b,Vec x)
     vec_rhs      = ksp->vec_rhs;
     ksp->vec_rhs = btmp;
   }
-  ierr = VecLockPush(ksp->vec_rhs);CHKERRQ(ierr);
-  if (ksp->reason == KSP_DIVERGED_PCSETUP_FAILED) {
+  ierr = VecLockReadPush(ksp->vec_rhs);CHKERRQ(ierr);
+  if (ksp->reason == KSP_DIVERGED_PC_FAILED) {
     ierr = VecSetInf(ksp->vec_sol);CHKERRQ(ierr);
   }
   ierr = (*ksp->ops->solve)(ksp);CHKERRQ(ierr);
 
-  ierr = VecLockPop(ksp->vec_rhs);CHKERRQ(ierr);
+  ierr = VecLockReadPop(ksp->vec_rhs);CHKERRQ(ierr);
   if (nullsp) {
     ksp->vec_rhs = vec_rhs;
     ierr = VecDestroy(&btmp);CHKERRQ(ierr);
@@ -810,23 +812,24 @@ PetscErrorCode KSPSolve(KSP ksp,Vec b,Vec x)
     }
   }
   ierr = PetscLogEventEnd(KSP_Solve,ksp,ksp->vec_rhs,ksp->vec_sol,0);CHKERRQ(ierr);
-  if (ksp->postsolve) {
-    ierr = (*ksp->postsolve)(ksp,ksp->vec_rhs,ksp->vec_sol,ksp->postctx);CHKERRQ(ierr);
-  }
   if (ksp->guess) {
     ierr = KSPGuessUpdate(ksp->guess,ksp->vec_rhs,ksp->vec_sol);CHKERRQ(ierr);
   }
+  if (ksp->postsolve) {
+    ierr = (*ksp->postsolve)(ksp,ksp->vec_rhs,ksp->vec_sol,ksp->postctx);CHKERRQ(ierr);
+  }
 
   ierr = PCGetOperators(ksp->pc,&mat,&pmat);CHKERRQ(ierr);
   if (ksp->viewEV)       {ierr = KSPViewEigenvalues_Internal(ksp, PETSC_FALSE, ksp->viewerEV,    ksp->formatEV);CHKERRQ(ierr);}
   if (ksp->viewEVExp)    {ierr = KSPViewEigenvalues_Internal(ksp, PETSC_TRUE,  ksp->viewerEVExp, ksp->formatEVExp);CHKERRQ(ierr);}
   if (ksp->viewSV)       {ierr = KSPViewSingularvalues_Internal(ksp, ksp->viewerSV, ksp->formatSV);CHKERRQ(ierr);}
   if (ksp->viewFinalRes) {ierr = KSPViewFinalResidual_Internal(ksp, ksp->viewerFinalRes, ksp->formatFinalRes);CHKERRQ(ierr);}
-  if (ksp->viewMat)      {ierr = ObjectView((PetscObject) mat,          ksp->viewerMat,  ksp->formatMat);CHKERRQ(ierr);}
-  if (ksp->viewPMat)     {ierr = ObjectView((PetscObject) pmat,         ksp->viewerPMat, ksp->formatPMat);CHKERRQ(ierr);}
-  if (ksp->viewRhs)      {ierr = ObjectView((PetscObject) ksp->vec_rhs, ksp->viewerRhs,  ksp->formatRhs);CHKERRQ(ierr);}
-  if (ksp->viewSol)      {ierr = ObjectView((PetscObject) ksp->vec_sol, ksp->viewerSol,  ksp->formatSol);CHKERRQ(ierr);}
-  if (ksp->view)         {ierr = ObjectView((PetscObject) ksp,          ksp->viewer,     ksp->format);CHKERRQ(ierr);}
+  if (ksp->viewMat)      {ierr = ObjectView((PetscObject) mat,           ksp->viewerMat,    ksp->formatMat);CHKERRQ(ierr);}
+  if (ksp->viewPMat)     {ierr = ObjectView((PetscObject) pmat,          ksp->viewerPMat,   ksp->formatPMat);CHKERRQ(ierr);}
+  if (ksp->viewRhs)      {ierr = ObjectView((PetscObject) ksp->vec_rhs,  ksp->viewerRhs,    ksp->formatRhs);CHKERRQ(ierr);}
+  if (ksp->viewSol)      {ierr = ObjectView((PetscObject) ksp->vec_sol,  ksp->viewerSol,    ksp->formatSol);CHKERRQ(ierr);}
+  if (ksp->view)         {ierr = ObjectView((PetscObject) ksp,           ksp->viewer,       ksp->format);CHKERRQ(ierr);}
+  if (ksp->viewDScale)   {ierr = ObjectView((PetscObject) ksp->diagonal, ksp->viewerDScale, ksp->formatDScale);CHKERRQ(ierr);}
   if (ksp->viewMatExp)   {
     Mat A, B;
 
@@ -848,7 +851,7 @@ PetscErrorCode KSPSolve(KSP ksp,Vec b,Vec x)
     ierr = VecDestroy(&x);CHKERRQ(ierr);
   }
   ierr = PetscObjectSAWsBlock((PetscObject)ksp);CHKERRQ(ierr);
-  if (ksp->errorifnotconverged && ksp->reason < 0) SETERRQ(comm,PETSC_ERR_NOT_CONVERGED,"KSPSolve has not converged");
+  if (ksp->errorifnotconverged && ksp->reason < 0 && ksp->reason != KSP_DIVERGED_ITS) SETERRQ1(comm,PETSC_ERR_NOT_CONVERGED,"KSPSolve has not converged, reason %s",KSPConvergedReasons[ksp->reason]);
   PetscFunctionReturn(0);
 }
 
@@ -903,6 +906,8 @@ PetscErrorCode  KSPSolveTranspose(KSP ksp,Vec b,Vec x)
   ksp->vec_sol         = x;
   ksp->transpose_solve = PETSC_TRUE;
 
+  if (ksp->presolve) {ierr = (*ksp->presolve)(ksp,ksp->vec_rhs,ksp->vec_sol,ksp->prectx);CHKERRQ(ierr);}
+
   if (ksp->guess) {
     PetscObjectState ostate,state;
 
@@ -933,6 +938,7 @@ PetscErrorCode  KSPSolveTranspose(KSP ksp,Vec b,Vec x)
   }
 
   ierr = (*ksp->ops->solve)(ksp);CHKERRQ(ierr);
+  ksp->totalits += ksp->its;
   if (nullsp) {
     ksp->vec_rhs = vec_rhs;
     ierr = VecDestroy(&btmp);CHKERRQ(ierr);
@@ -942,11 +948,15 @@ PetscErrorCode  KSPSolveTranspose(KSP ksp,Vec b,Vec x)
   if (ksp->guess) {
     ierr = KSPGuessUpdate(ksp->guess,ksp->vec_rhs,ksp->vec_sol);CHKERRQ(ierr);
   }
+  if (ksp->postsolve) {
+    ierr = (*ksp->postsolve)(ksp,ksp->vec_rhs,ksp->vec_sol,ksp->postctx);CHKERRQ(ierr);
+  }
 
   if (ksp->viewMat)      {ierr = ObjectView((PetscObject) mat,          ksp->viewerMat,  ksp->formatMat);CHKERRQ(ierr);}
   if (ksp->viewPMat)     {ierr = ObjectView((PetscObject) pmat,         ksp->viewerPMat, ksp->formatPMat);CHKERRQ(ierr);}
   if (ksp->viewRhs)      {ierr = ObjectView((PetscObject) ksp->vec_rhs, ksp->viewerRhs,  ksp->formatRhs);CHKERRQ(ierr);}
   if (ksp->viewSol)      {ierr = ObjectView((PetscObject) ksp->vec_sol, ksp->viewerSol,  ksp->formatSol);CHKERRQ(ierr);}
+  if (ksp->view)         {ierr = ObjectView((PetscObject) ksp,          ksp->viewer,     ksp->format);CHKERRQ(ierr);}
 
   if (inXisinB) {
     ierr = VecCopy(x,b);CHKERRQ(ierr);
@@ -956,6 +966,58 @@ PetscErrorCode  KSPSolveTranspose(KSP ksp,Vec b,Vec x)
   PetscFunctionReturn(0);
 }
 
+/*@
+   KSPResetViewers - Resets all the viewers set from the options database during KSPSetFromOptions()
+
+   Collective on KSP
+
+   Input Parameter:
+.  ksp - iterative context obtained from KSPCreate()
+
+   Level: beginner
+
+.keywords: destroy
+
+.seealso: KSPCreate(), KSPSetUp(), KSPSolve(), KSPSetFromOptions(), KSP
+@*/
+PetscErrorCode  KSPResetViewers(KSP ksp)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (ksp) PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
+  if (!ksp) PetscFunctionReturn(0);
+  ierr = PetscViewerDestroy(&ksp->viewer);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&ksp->viewerPre);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&ksp->viewerReason);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&ksp->viewerMat);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&ksp->viewerPMat);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&ksp->viewerRhs);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&ksp->viewerSol);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&ksp->viewerMatExp);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&ksp->viewerEV);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&ksp->viewerSV);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&ksp->viewerEVExp);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&ksp->viewerFinalRes);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&ksp->viewerPOpExp);CHKERRQ(ierr);
+  ierr = PetscViewerDestroy(&ksp->viewerDScale);CHKERRQ(ierr);
+  ksp->view         = PETSC_FALSE;
+  ksp->viewPre      = PETSC_FALSE;
+  ksp->viewReason   = PETSC_FALSE;
+  ksp->viewMat      = PETSC_FALSE;
+  ksp->viewPMat     = PETSC_FALSE;
+  ksp->viewRhs      = PETSC_FALSE;
+  ksp->viewSol      = PETSC_FALSE;
+  ksp->viewMatExp   = PETSC_FALSE;
+  ksp->viewEV       = PETSC_FALSE;
+  ksp->viewSV       = PETSC_FALSE;
+  ksp->viewEVExp    = PETSC_FALSE;
+  ksp->viewFinalRes = PETSC_FALSE;
+  ksp->viewPOpExp   = PETSC_FALSE;
+  ksp->viewDScale   = PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
 /*@
    KSPReset - Resets a KSP context to the kspsetupcalled = 0 state and removes any allocated Vecs and Mats
 
@@ -991,19 +1053,7 @@ PetscErrorCode  KSPReset(KSP ksp)
   ierr = VecDestroy(&ksp->diagonal);CHKERRQ(ierr);
   ierr = VecDestroy(&ksp->truediagonal);CHKERRQ(ierr);
 
-  ierr = PetscViewerDestroy(&ksp->viewer);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&ksp->viewerPre);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&ksp->viewerReason);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&ksp->viewerMat);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&ksp->viewerPMat);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&ksp->viewerRhs);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&ksp->viewerSol);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&ksp->viewerMatExp);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&ksp->viewerEV);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&ksp->viewerSV);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&ksp->viewerEVExp);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&ksp->viewerFinalRes);CHKERRQ(ierr);
-  ierr = PetscViewerDestroy(&ksp->viewerPOpExp);CHKERRQ(ierr);
+  ierr = KSPResetViewers(ksp);CHKERRQ(ierr);
 
   ksp->setupstage = KSP_SETUP_NEW;
   PetscFunctionReturn(0);
@@ -1683,6 +1733,7 @@ PetscErrorCode  KSPGetPC(KSP ksp,PC *pc)
     ierr = PCCreate(PetscObjectComm((PetscObject)ksp),&ksp->pc);CHKERRQ(ierr);
     ierr = PetscObjectIncrementTabLevel((PetscObject)ksp->pc,(PetscObject)ksp,0);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)ksp,(PetscObject)ksp->pc);CHKERRQ(ierr);
+    ierr = PetscObjectSetOptions((PetscObject)ksp->pc,((PetscObject)ksp)->options);CHKERRQ(ierr);
   }
   *pc = ksp->pc;
   PetscFunctionReturn(0);
@@ -2006,7 +2057,7 @@ $     converge (KSP ksp, int it, PetscReal rnorm, KSPConvergedReason *reason,voi
 
 .keywords: set, convergence, test, context
 
-.seealso: KSPConvergedDefault(), KSPGetConvergenceContext(), KSPSetTolerances(), KSP
+.seealso: KSPConvergedDefault(), KSPGetConvergenceContext(), KSPSetTolerances(), KSP, KSPGetConvergenceTest(), KSPGetAndClearConvergenceTest()
 @*/
 PetscErrorCode  KSPSetConvergenceTest(KSP ksp,PetscErrorCode (*converge)(KSP,PetscInt,PetscReal,KSPConvergedReason*,void*),void *cctx,PetscErrorCode (*destroy)(void*))
 {
@@ -2023,6 +2074,90 @@ PetscErrorCode  KSPSetConvergenceTest(KSP ksp,PetscErrorCode (*converge)(KSP,Pet
   PetscFunctionReturn(0);
 }
 
+/*@C
+   KSPGetConvergenceTest - Gets the function to be used to determine
+   convergence.
+
+   Logically Collective on KSP
+
+   Input Parameter:
+.   ksp - iterative context obtained from KSPCreate()
+
+   Output Parameter:
++  converge - pointer to convergence test function
+.  cctx    - context for private data for the convergence routine (may be null)
+-  destroy - a routine for destroying the context (may be null)
+
+   Calling sequence of converge:
+$     converge (KSP ksp, int it, PetscReal rnorm, KSPConvergedReason *reason,void *mctx)
+
++  ksp - iterative context obtained from KSPCreate()
+.  it - iteration number
+.  rnorm - (estimated) 2-norm of (preconditioned) residual
+.  reason - the reason why it has converged or diverged
+-  cctx  - optional convergence context, as set by KSPSetConvergenceTest()
+
+   Level: advanced
+
+.keywords: set, convergence, test, context
+
+.seealso: KSPConvergedDefault(), KSPGetConvergenceContext(), KSPSetTolerances(), KSP, KSPSetConvergenceTest(), KSPGetAndClearConvergenceTest()
+@*/
+PetscErrorCode  KSPGetConvergenceTest(KSP ksp,PetscErrorCode (**converge)(KSP,PetscInt,PetscReal,KSPConvergedReason*,void*),void **cctx,PetscErrorCode (**destroy)(void*))
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
+  if (converge) *converge = ksp->converged;
+  if (destroy)  *destroy  = ksp->convergeddestroy;
+  if (cctx)     *cctx     = ksp->cnvP;
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   KSPGetAndClearConvergenceTest - Gets the function to be used to determine convergence. Removes the current test without calling destroy on the test context
+
+   Logically Collective on KSP
+
+   Input Parameter:
+.   ksp - iterative context obtained from KSPCreate()
+
+   Output Parameter:
++  converge - pointer to convergence test function
+.  cctx    - context for private data for the convergence routine
+-  destroy - a routine for destroying the context
+
+   Calling sequence of converge:
+$     converge (KSP ksp, int it, PetscReal rnorm, KSPConvergedReason *reason,void *mctx)
+
++  ksp - iterative context obtained from KSPCreate()
+.  it - iteration number
+.  rnorm - (estimated) 2-norm of (preconditioned) residual
+.  reason - the reason why it has converged or diverged
+-  cctx  - optional convergence context, as set by KSPSetConvergenceTest()
+
+   Level: advanced
+
+   Notes: This is intended to be used to allow transfering the convergence test (and its context) to another testing object (for example another KSP) and then calling
+          KSPSetConvergenceTest() on this original KSP. If you just called KSPGetConvergenceTest() followed by KSPSetConvergenceTest() the original context information
+          would be destroyed and hence the transfered context would be invalid and trigger a crash on use
+
+.keywords: set, convergence, test, context
+
+.seealso: KSPConvergedDefault(), KSPGetConvergenceContext(), KSPSetTolerances(), KSP, KSPSetConvergenceTest(), KSPGetConvergenceTest()
+@*/
+PetscErrorCode  KSPGetAndClearConvergenceTest(KSP ksp,PetscErrorCode (**converge)(KSP,PetscInt,PetscReal,KSPConvergedReason*,void*),void **cctx,PetscErrorCode (**destroy)(void*))
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
+  *converge             = ksp->converged;
+  *destroy              = ksp->convergeddestroy;
+  *cctx                 = ksp->cnvP;
+  ksp->converged        = NULL;
+  ksp->cnvP             = NULL;
+  ksp->convergeddestroy = NULL;
+  PetscFunctionReturn(0);
+}
+
 /*@C
    KSPGetConvergenceContext - Gets the convergence context set with
    KSPSetConvergenceTest().
@@ -2385,6 +2520,9 @@ $  func(KSP ksp,Vec x,void *ctx)
 .  x - solution vector
 -  ctx - optional user-provided context
 
+   Notes: This should only be used in conjunction with KSPSetComputeRHS(), KSPSetComputeOperators(), otherwise
+   call KSPSetInitialGuessNonzero() and set the initial guess values in the solution vector passed to KSPSolve().
+
    Level: beginner
 
 .seealso: KSPSolve(), KSPSetComputeRHS(), KSPSetComputeOperators(), DMKSPSetComputeInitialGuess()
