@@ -1,34 +1,67 @@
 
 #include <../src/ksp/pc/impls/factor/factor.h>  /*I "petscpc.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetUpMatSolverPackage"
+static PetscErrorCode PCFactorSetReuseOrdering_Factor(PC pc,PetscBool flag)
+{
+  PC_Factor *lu = (PC_Factor*)pc->data;
+
+  PetscFunctionBegin;
+  lu->reuseordering = flag;
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode PCFactorSetReuseFill_Factor(PC pc,PetscBool flag)
+{
+  PC_Factor *lu = (PC_Factor*)pc->data;
+
+  PetscFunctionBegin;
+  lu->reusefill = flag;
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode  PCFactorSetUseInPlace_Factor(PC pc,PetscBool flg)
+{
+  PC_Factor *dir = (PC_Factor*)pc->data;
+
+  PetscFunctionBegin;
+  dir->inplace = flg;
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode  PCFactorGetUseInPlace_Factor(PC pc,PetscBool *flg)
+{
+  PC_Factor *dir = (PC_Factor*)pc->data;
+
+  PetscFunctionBegin;
+  *flg = dir->inplace;
+  PetscFunctionReturn(0);
+}
+
 /*@
-    PCFactorSetUpMatSolverPackage - Can be called after KSPSetOperators() or PCSetOperators(), causes MatGetFactor() to be called so then one may
+    PCFactorSetUpMatSolverType - Can be called after KSPSetOperators() or PCSetOperators(), causes MatGetFactor() to be called so then one may
        set the options for that particular factorization object.
 
   Input Parameter:
 .  pc  - the preconditioner context
 
-  Notes: After you have called this function (which has to be after the KSPSetOperators() or PCSetOperators()) you can call PCFactorGetMatrix() and then set factor options on that matrix.
+  Notes:
+    After you have called this function (which has to be after the KSPSetOperators() or PCSetOperators()) you can call PCFactorGetMatrix() and then set factor options on that matrix.
 
-.seealso: PCFactorSetMatSolverPackage(), PCFactorGetMatrix()
+.seealso: PCFactorSetMatSolverType(), PCFactorGetMatrix()
 
   Level: intermediate
 
 @*/
-PetscErrorCode PCFactorSetUpMatSolverPackage(PC pc)
+PetscErrorCode PCFactorSetUpMatSolverType(PC pc)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
-  ierr = PetscTryMethod(pc,"PCFactorSetUpMatSolverPackage_C",(PC),(pc));CHKERRQ(ierr);
+  ierr = PetscTryMethod(pc,"PCFactorSetUpMatSolverType_C",(PC),(pc));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetZeroPivot"
 /*@
    PCFactorSetZeroPivot - Sets the size at which smaller pivots are declared to be zero
 
@@ -58,8 +91,6 @@ PetscErrorCode  PCFactorSetZeroPivot(PC pc,PetscReal zero)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetShiftType"
 /*@
    PCFactorSetShiftType - adds a particular type of quantity to the diagonal of the matrix during
      numerical factorization, thus the matrix has nonzero pivots
@@ -90,8 +121,6 @@ PetscErrorCode  PCFactorSetShiftType(PC pc,MatFactorShiftType shifttype)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetShiftAmount"
 /*@
    PCFactorSetShiftAmount - adds a quantity to the diagonal of the matrix during
      numerical factorization, thus the matrix has nonzero pivots
@@ -122,8 +151,6 @@ PetscErrorCode  PCFactorSetShiftAmount(PC pc,PetscReal shiftamount)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetDropTolerance"
 /*
    PCFactorSetDropTolerance - The preconditioner will use an ILU
    based on a drop tolerance. (Under development)
@@ -159,8 +186,84 @@ PetscErrorCode  PCFactorSetDropTolerance(PC pc,PetscReal dt,PetscReal dtcol,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorGetLevels"
+/*@
+   PCFactorGetZeroPivot - Gets the tolerance used to define a zero privot
+
+   Not Collective
+
+   Input Parameters:
+.  pc - the preconditioner context
+
+   Output Parameter:
+.  pivot - the tolerance
+
+   Level: intermediate
+
+
+.seealso: PCFactorSetZeroPivot()
+@*/
+PetscErrorCode  PCFactorGetZeroPivot(PC pc,PetscReal *pivot)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  ierr = PetscUseMethod(pc,"PCFactorGetZeroPivot_C",(PC,PetscReal*),(pc,pivot));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   PCFactorGetShiftAmount - Gets the tolerance used to define a zero privot
+
+   Not Collective
+
+   Input Parameters:
+.  pc - the preconditioner context
+
+   Output Parameter:
+.  shift - how much to shift the diagonal entry
+
+   Level: intermediate
+
+
+.seealso: PCFactorSetShiftAmount(), PCFactorSetShiftType(), PCFactorGetShiftType()
+@*/
+PetscErrorCode  PCFactorGetShiftAmount(PC pc,PetscReal *shift)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  ierr = PetscUseMethod(pc,"PCFactorGetShiftAmount_C",(PC,PetscReal*),(pc,shift));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   PCFactorGetShiftType - Gets the type of shift, if any, done when a zero pivot is detected
+
+   Not Collective
+
+   Input Parameters:
+.  pc - the preconditioner context
+
+   Output Parameter:
+.  type - one of MAT_SHIFT_NONE, MAT_SHIFT_NONZERO,  MAT_SHIFT_POSITIVE_DEFINITE, or MAT_SHIFT_INBLOCKS
+
+   Level: intermediate
+
+
+.seealso: PCFactorSetShiftType(), MatFactorShiftType, PCFactorSetShiftAmount(), PCFactorGetShiftAmount()
+@*/
+PetscErrorCode  PCFactorGetShiftType(PC pc,MatFactorShiftType *type)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  ierr = PetscUseMethod(pc,"PCFactorGetShiftType_C",(PC,MatFactorShiftType*),(pc,type));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    PCFactorGetLevels - Gets the number of levels of fill to use.
 
@@ -186,8 +289,6 @@ PetscErrorCode  PCFactorGetLevels(PC pc,PetscInt *levels)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetLevels"
 /*@
    PCFactorSetLevels - Sets the number of levels of fill to use.
 
@@ -216,8 +317,6 @@ PetscErrorCode  PCFactorSetLevels(PC pc,PetscInt levels)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetAllowDiagonalFill"
 /*@
    PCFactorSetAllowDiagonalFill - Causes all diagonal matrix entries to be
    treated as level 0 fill even if there is no non-zero location.
@@ -251,8 +350,6 @@ PetscErrorCode  PCFactorSetAllowDiagonalFill(PC pc,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorGetAllowDiagonalFill"
 /*@
    PCFactorGetAllowDiagonalFill - Determines if all diagonal matrix entries are
        treated as level 0 fill even if there is no non-zero location.
@@ -288,8 +385,6 @@ PetscErrorCode  PCFactorGetAllowDiagonalFill(PC pc,PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorReorderForNonzeroDiagonal"
 /*@
    PCFactorReorderForNonzeroDiagonal - reorders rows/columns of matrix to remove zeros from diagonal
 
@@ -319,10 +414,8 @@ PetscErrorCode  PCFactorReorderForNonzeroDiagonal(PC pc,PetscReal rtol)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetMatSolverPackage"
 /*@C
-   PCFactorSetMatSolverPackage - sets the software that is used to perform the factorization
+   PCFactorSetMatSolverType - sets the software that is used to perform the factorization
 
    Logically Collective on PC
 
@@ -331,7 +424,7 @@ PetscErrorCode  PCFactorReorderForNonzeroDiagonal(PC pc,PetscReal rtol)
 -  stype - for example, superlu, superlu_dist
 
    Options Database Key:
-.  -pc_factor_mat_solver_package <stype> - petsc, superlu, superlu_dist, mumps, cusparse
+.  -pc_factor_mat_solver_type <stype> - petsc, superlu, superlu_dist, mumps, cusparse
 
    Level: intermediate
 
@@ -341,23 +434,21 @@ PetscErrorCode  PCFactorReorderForNonzeroDiagonal(PC pc,PetscReal rtol)
 
 .keywords: PC, set, factorization, direct, fill
 
-.seealso: MatGetFactor(), MatSolverPackage, PCFactorGetMatSolverPackage()
+.seealso: MatGetFactor(), MatSolverType, PCFactorGetMatSolverType()
 
 @*/
-PetscErrorCode  PCFactorSetMatSolverPackage(PC pc,const MatSolverPackage stype)
+PetscErrorCode  PCFactorSetMatSolverType(PC pc,MatSolverType stype)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
-  ierr = PetscTryMethod(pc,"PCFactorSetMatSolverPackage_C",(PC,const MatSolverPackage),(pc,stype));CHKERRQ(ierr);
+  ierr = PetscTryMethod(pc,"PCFactorSetMatSolverType_C",(PC,MatSolverType),(pc,stype));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorGetMatSolverPackage"
 /*@C
-   PCFactorGetMatSolverPackage - gets the software that is used to perform the factorization
+   PCFactorGetMatSolverType - gets the software that is used to perform the factorization
 
    Not Collective
 
@@ -372,16 +463,16 @@ PetscErrorCode  PCFactorSetMatSolverPackage(PC pc,const MatSolverPackage stype)
 
 .keywords: PC, set, factorization, direct, fill
 
-.seealso: MatGetFactor(), MatSolverPackage, PCFactorGetMatSolverPackage()
+.seealso: MatGetFactor(), MatSolverType, PCFactorGetMatSolverType()
 
 @*/
-PetscErrorCode  PCFactorGetMatSolverPackage(PC pc,const MatSolverPackage *stype)
+PetscErrorCode  PCFactorGetMatSolverType(PC pc,MatSolverType *stype)
 {
-  PetscErrorCode ierr,(*f)(PC,const MatSolverPackage*);
+  PetscErrorCode ierr,(*f)(PC,MatSolverType*);
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
-  ierr = PetscObjectQueryFunction((PetscObject)pc,"PCFactorGetMatSolverPackage_C",&f);CHKERRQ(ierr);
+  ierr = PetscObjectQueryFunction((PetscObject)pc,"PCFactorGetMatSolverType_C",&f);CHKERRQ(ierr);
   if (f) {
     ierr = (*f)(pc,stype);CHKERRQ(ierr);
   } else {
@@ -390,8 +481,6 @@ PetscErrorCode  PCFactorGetMatSolverPackage(PC pc,const MatSolverPackage *stype)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetFill"
 /*@
    PCFactorSetFill - Indicate the amount of fill you expect in the factored matrix,
    fill = number nonzeros in factor/number nonzeros in original matrix.
@@ -430,8 +519,6 @@ PetscErrorCode  PCFactorSetFill(PC pc,PetscReal fill)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetUseInPlace"
 /*@
    PCFactorSetUseInPlace - Tells the system to do an in-place factorization.
    For dense matrices, this enables the solution of much larger problems.
@@ -473,8 +560,6 @@ PetscErrorCode  PCFactorSetUseInPlace(PC pc,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorGetUseInPlace"
 /*@
    PCFactorGetUseInPlace - Determines if an in-place factorization is being used.
 
@@ -502,8 +587,6 @@ PetscErrorCode  PCFactorGetUseInPlace(PC pc,PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetMatOrderingType"
 /*@C
     PCFactorSetMatOrderingType - Sets the ordering routine (to reduce fill) to
     be used in the LU factorization.
@@ -519,7 +602,8 @@ PetscErrorCode  PCFactorGetUseInPlace(PC pc,PetscBool *flg)
 
     Level: intermediate
 
-    Notes: nested dissection is used by default
+    Notes:
+    nested dissection is used by default
 
     For Cholesky and ICC and the SBAIJ format reorderings are not available,
     since only the upper triangular part of the matrix is stored. You can use the
@@ -536,8 +620,6 @@ PetscErrorCode  PCFactorSetMatOrderingType(PC pc,MatOrderingType ordering)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetColumnPivot"
 /*@
     PCFactorSetColumnPivot - Determines when column pivoting is done during matrix factorization.
       For PETSc dense matrices column pivoting is always done, for PETSc sparse matrices
@@ -567,8 +649,6 @@ PetscErrorCode  PCFactorSetColumnPivot(PC pc,PetscReal dtcol)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetPivotInBlocks"
 /*@
     PCFactorSetPivotInBlocks - Determines if pivoting is done while factoring each block
       with BAIJ or SBAIJ matrices
@@ -597,10 +677,8 @@ PetscErrorCode  PCFactorSetPivotInBlocks(PC pc,PetscBool pivot)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetReuseFill"
 /*@
-   PCFactorSetReuseFill - When matrices with same different nonzero structure are factored,
+   PCFactorSetReuseFill - When matrices with different nonzero structure are factored,
    this causes later ones to use the fill ratio computed in the initial factorization.
 
    Logically Collective on PC
@@ -628,3 +706,39 @@ PetscErrorCode  PCFactorSetReuseFill(PC pc,PetscBool flag)
   ierr = PetscTryMethod(pc,"PCFactorSetReuseFill_C",(PC,PetscBool),(pc,flag));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
+PetscErrorCode PCFactorInitialize(PC pc)
+{
+  PetscErrorCode ierr;
+  PC_Factor       *fact = (PC_Factor*)pc->data;
+
+  PetscFunctionBegin;
+  ierr                       = MatFactorInfoInitialize(&fact->info);CHKERRQ(ierr);
+  fact->info.shifttype       = (PetscReal)MAT_SHIFT_NONE;
+  fact->info.shiftamount     = 100.0*PETSC_MACHINE_EPSILON;
+  fact->info.zeropivot       = 100.0*PETSC_MACHINE_EPSILON;
+  fact->info.pivotinblocks   = 1.0;
+  pc->ops->getfactoredmatrix = PCFactorGetMatrix_Factor;
+
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetZeroPivot_C",PCFactorSetZeroPivot_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetZeroPivot_C",PCFactorGetZeroPivot_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetShiftType_C",PCFactorSetShiftType_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetShiftType_C",PCFactorGetShiftType_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetShiftAmount_C",PCFactorSetShiftAmount_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetShiftAmount_C",PCFactorGetShiftAmount_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetMatSolverType_C",PCFactorGetMatSolverType_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetMatSolverType_C",PCFactorSetMatSolverType_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetUpMatSolverType_C",PCFactorSetUpMatSolverType_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetFill_C",PCFactorSetFill_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetMatOrderingType_C",PCFactorSetMatOrderingType_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetLevels_C",PCFactorSetLevels_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetLevels_C",PCFactorGetLevels_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetAllowDiagonalFill_C",PCFactorSetAllowDiagonalFill_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetAllowDiagonalFill_C",PCFactorGetAllowDiagonalFill_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetPivotInBlocks_C",PCFactorSetPivotInBlocks_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetUseInPlace_C",PCFactorSetUseInPlace_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetUseInPlace_C",PCFactorGetUseInPlace_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetReuseOrdering_C",PCFactorSetReuseOrdering_Factor);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetReuseFill_C",PCFactorSetReuseFill_Factor);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
