@@ -35,7 +35,7 @@ PetscErrorCode PCReset_GAMG(PC pc)
   PC_GAMG        *pc_gamg = (PC_GAMG*)mg->innerctx;
 
   PetscFunctionBegin;
-  if (pc_gamg->data) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_PLIB,"This should not happen, cleaned up in SetUp\n");
+  ierr = PetscFree(pc_gamg->data);CHKERRQ(ierr);
   pc_gamg->data_sz = 0;
   ierr = PetscFree(pc_gamg->orig_data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -532,15 +532,15 @@ PetscErrorCode PCSetUp_GAMG(PC pc)
           ierr = pc_gamg->ops->optprolongator(pc, Aarr[level], &Prol11);CHKERRQ(ierr);
         }
 
+        if (pc_gamg->use_aggs_in_asm) {
+          PetscInt bs;
+          ierr = MatGetBlockSizes(Prol11, &bs, NULL);CHKERRQ(ierr);
+          ierr = PetscCDGetASMBlocks(agg_lists, bs, Gmat, &nASMBlocksArr[level], &ASMLocalIDsArr[level]);CHKERRQ(ierr);
+        }
+
         Parr[level1] = Prol11;
       } else Parr[level1] = NULL; /* failed to coarsen */
 
-      if (pc_gamg->use_aggs_in_asm) {
-        PetscInt bs;
-        ierr = MatGetBlockSizes(Prol11, &bs, NULL);CHKERRQ(ierr);
-        ierr = PetscCDGetASMBlocks(agg_lists, bs, Gmat, &nASMBlocksArr[level], &ASMLocalIDsArr[level]);CHKERRQ(ierr);
-      }
-
       ierr = MatDestroy(&Gmat);CHKERRQ(ierr);
       ierr = PetscCDDestroy(agg_lists);CHKERRQ(ierr);
     } /* construct prolongator scope */
@@ -766,6 +766,9 @@ static PetscErrorCode PCGAMGSetProcEqLim_GAMG(PC pc, PetscInt n)
    Options Database Key:
 .  -pc_gamg_coarse_eq_limit <limit>
 
+   Notes: For example -pc_gamg_coarse_eq_limit 1000 will stop coarsening once the coarse grid 
+     has less than 1000 unknowns.
+
    Level: intermediate
 
    Concepts: Unstructured multigrid preconditioner
@@ -1162,12 +1165,45 @@ static PetscErrorCode PCGAMGSetType_GAMG(PC pc, PCGAMGType type)
   PetscFunctionReturn(0);
 }
 
+/* -------------------------------------------------------------------------- */
+/*
+   PCMGGetGridComplexity - compute coarse grid complexity of MG hierarchy
+
+   Input Parameter:
+.  pc - the preconditioner context
+
+   Output Parameter:
+.  gc - grid complexity = sum_i(nnz_i) / nnz_0
+
+   Level: advanced
+*/
+static PetscErrorCode PCMGGetGridComplexity(PC pc, PetscReal *gc)
+{
+  PetscErrorCode ierr;
+  PC_MG          *mg      = (PC_MG*)pc->data;
+  PC_MG_Levels   **mglevels = mg->levels;
+  PetscInt       lev, nnz0 = -1;
+  MatInfo        info;
+  PetscFunctionBegin;
+  if (!mg->nlevels) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"MG has no levels");
+  for (lev=0, *gc=0; lev<mg->nlevels; lev++) {
+    Mat dB;
+    ierr = KSPGetOperators(mglevels[lev]->smoothd,NULL,&dB);CHKERRQ(ierr);
+    ierr = MatGetInfo(dB,MAT_GLOBAL_SUM,&info);CHKERRQ(ierr); /* global reduction */
+    *gc += (PetscReal)info.nz_used;
+    if (lev==mg->nlevels-1) nnz0 = info.nz_used;
+  }
+  if (nnz0) *gc /= (PetscReal)nnz0;
+  else *gc = 0;
+  PetscFunctionReturn(0);
+}
+
 static PetscErrorCode PCView_GAMG(PC pc,PetscViewer viewer)
 {
   PetscErrorCode ierr,i;
   PC_MG          *mg      = (PC_MG*)pc->data;
   PC_GAMG        *pc_gamg = (PC_GAMG*)mg->innerctx;
-
+  PetscReal       gc=0;
   PetscFunctionBegin;
   ierr = PetscViewerASCIIPrintf(viewer,"    GAMG specific options\n");CHKERRQ(ierr);
   ierr = PetscViewerASCIIPrintf(viewer,"      Threshold for dropping small values in graph on each level =");CHKERRQ(ierr);
@@ -1185,6 +1221,8 @@ static PetscErrorCode PCView_GAMG(PC pc,PetscViewer viewer)
   if (pc_gamg->ops->view) {
     ierr = (*pc_gamg->ops->view)(pc,viewer);CHKERRQ(ierr);
   }
+  ierr = PCMGGetGridComplexity(pc,&gc);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"      Complexity:    grid = %g\n",gc);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1308,6 +1346,8 @@ PETSC_EXTERN PetscErrorCode PCCreate_GAMG(PC pc)
   pc->ops->destroy        = PCDestroy_GAMG;
   mg->view                = PCView_GAMG;
 
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCMGGetLevels_C",PCMGGetLevels_MG);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCMGSetLevels_C",PCMGSetLevels_MG);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCGAMGSetProcEqLim_C",PCGAMGSetProcEqLim_GAMG);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCGAMGSetCoarseEqLim_C",PCGAMGSetCoarseEqLim_GAMG);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCGAMGSetRepartition_C",PCGAMGSetRepartition_GAMG);CHKERRQ(ierr);
@@ -1338,8 +1378,7 @@ PETSC_EXTERN PetscErrorCode PCCreate_GAMG(PC pc)
 
 /*@C
  PCGAMGInitializePackage - This function initializes everything in the PCGAMG package. It is called
-    from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to PCCreate_GAMG()
-    when using static libraries.
+    from PCInitializePackage().
 
  Level: developer
 
