@@ -3,7 +3,6 @@
  */
 
 #include <../src/ksp/pc/impls/gamg/gamg.h>        /*I "petscpc.h" I*/
-#include <petsc/private/kspimpl.h>
 
 #if defined(PETSC_HAVE_TRIANGLE)
 #define REAL PetscReal
@@ -18,7 +17,7 @@ typedef struct {
   PetscInt degree;         /* vertex degree */
 } GAMGNode;
 
-int petsc_geo_mg_compare(const void *a, const void *b)
+PETSC_STATIC_INLINE int petsc_geo_mg_compare(const void *a, const void *b)
 {
   return (((GAMGNode*)a)->degree - ((GAMGNode*)b)->degree);
 }
@@ -30,43 +29,48 @@ int petsc_geo_mg_compare(const void *a, const void *b)
    Input Parameter:
    .  pc - the preconditioner context
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCSetCoordinates_GEO"
 PetscErrorCode PCSetCoordinates_GEO(PC pc, PetscInt ndm, PetscInt a_nloc, PetscReal *coords)
 {
   PC_MG          *mg      = (PC_MG*)pc->data;
   PC_GAMG        *pc_gamg = (PC_GAMG*)mg->innerctx;
   PetscErrorCode ierr;
-  PetscInt       arrsz,bs,my0,kk,ii,nloc,Iend;
+  PetscInt       arrsz,bs,my0,kk,ii,nloc,Iend,aloc;
   Mat            Amat = pc->pmat;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(Amat, MAT_CLASSID, 1);
   ierr = MatGetBlockSize(Amat, &bs);CHKERRQ(ierr);
-
   ierr = MatGetOwnershipRange(Amat, &my0, &Iend);CHKERRQ(ierr);
+  aloc = (Iend-my0);
   nloc = (Iend-my0)/bs;
 
-  if (nloc!=a_nloc) SETERRQ2(PetscObjectComm((PetscObject)Amat),PETSC_ERR_ARG_WRONG, "Stokes not supported nloc = %d %d.",a_nloc,nloc);
-  if ((Iend-my0)%bs!=0) SETERRQ1(PetscObjectComm((PetscObject)Amat),PETSC_ERR_ARG_WRONG, "Bad local size %d.",nloc);
+  if (nloc!=a_nloc && aloc!=a_nloc) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Number of local blocks %D must be %D or %D.",a_nloc,nloc,aloc);
 
   pc_gamg->data_cell_rows = 1;
-  if (coords==0 && nloc > 0) SETERRQ(PetscObjectComm((PetscObject)Amat),PETSC_ERR_ARG_WRONG, "Need coordinates for pc_gamg_type 'geo'.");
+  if (!coords && nloc > 0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG, "Need coordinates for pc_gamg_type 'geo'.");
   pc_gamg->data_cell_cols = ndm; /* coordinates */
 
   arrsz = nloc*pc_gamg->data_cell_rows*pc_gamg->data_cell_cols;
 
   /* create data - syntactic sugar that should be refactored at some point */
-  if (pc_gamg->data==0 || (pc_gamg->data_sz != arrsz)) {
+  if (!pc_gamg->data || (pc_gamg->data_sz != arrsz)) {
     ierr = PetscFree(pc_gamg->data);CHKERRQ(ierr);
     ierr = PetscMalloc1(arrsz+1, &pc_gamg->data);CHKERRQ(ierr);
   }
   for (kk=0; kk<arrsz; kk++) pc_gamg->data[kk] = -999.;
   pc_gamg->data[arrsz] = -99.;
   /* copy data in - column oriented */
-  for (kk = 0; kk < nloc; kk++) {
-    for (ii = 0; ii < ndm; ii++) {
-      pc_gamg->data[ii*nloc + kk] =  coords[kk*ndm + ii];
+  if (nloc == a_nloc) {
+    for (kk = 0; kk < nloc; kk++) {
+      for (ii = 0; ii < ndm; ii++) {
+        pc_gamg->data[ii*nloc + kk] =  coords[kk*ndm + ii];
+      }
+    }
+  } else { /* assumes the coordinates are blocked */
+    for (kk = 0; kk < nloc; kk++) {
+      for (ii = 0; ii < ndm; ii++) {
+        pc_gamg->data[ii*nloc + kk] =  coords[bs*kk*ndm + ii];
+      }
     }
   }
   if (pc_gamg->data[arrsz] != -99.) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"pc_gamg->data[arrsz %D] %g != -99.",arrsz,pc_gamg->data[arrsz]);
@@ -81,8 +85,6 @@ PetscErrorCode PCSetCoordinates_GEO(PC pc, PetscInt ndm, PetscInt a_nloc, PetscR
   Input Parameter:
    . pc -
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCSetData_GEO"
 PetscErrorCode PCSetData_GEO(PC pc, Mat m)
 {
   PetscFunctionBegin;
@@ -96,9 +98,7 @@ PetscErrorCode PCSetData_GEO(PC pc, Mat m)
   Input Parameter:
    . pc -
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCSetFromOptions_GEO"
-PetscErrorCode PCSetFromOptions_GEO(PetscOptions *PetscOptionsObject,PC pc)
+PetscErrorCode PCSetFromOptions_GEO(PetscOptionItems *PetscOptionsObject,PC pc)
 {
   PetscErrorCode ierr;
 
@@ -136,8 +136,6 @@ PetscErrorCode PCSetFromOptions_GEO(PetscOptions *PetscOptionsObject,PC pc)
    . a_Prol - prolongation operator
    . a_worst_best - measure of worst missed fine vertex, 0 is no misses
 */
-#undef __FUNCT__
-#define __FUNCT__ "triangulateAndFormProl"
 static PetscErrorCode triangulateAndFormProl(IS selected_2,PetscInt data_stride,PetscReal coords[],PetscInt nselected_1,const PetscInt clid_lid_1[],const PetscCoarsenData *agg_lists_1,
                                              const PetscInt crsGID[],PetscInt bs,Mat a_Prol,PetscReal *a_worst_best)
 {
@@ -159,7 +157,7 @@ static PetscErrorCode triangulateAndFormProl(IS selected_2,PetscInt data_stride,
   if (nselected_2 == 1 || nselected_2 == 2) { /* 0 happens on idle processors */
     *a_worst_best = 100.0; /* this will cause a stop, but not globalized (should not happen) */
   } else *a_worst_best = 0.0;
-  ierr = MPI_Allreduce(a_worst_best, &tm, 1, MPIU_REAL, MPIU_MAX, comm);CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(a_worst_best, &tm, 1, MPIU_REAL, MPIU_MAX, comm);CHKERRQ(ierr);
   if (tm > 0.0) {
     *a_worst_best = 100.0;
     PetscFunctionReturn(0);
@@ -304,14 +302,14 @@ static PetscErrorCode triangulateAndFormProl(IS selected_2,PetscInt data_stride,
       ierr = PetscCDEmptyAt(agg_lists_1,mm,&ise);CHKERRQ(ierr);
       if (!ise) {
         const PetscInt lid = mm;
-        /* for (clid_iterator=0;clid_iterator<nselected_1;clid_iterator++) { */
-        PetscScalar  AA[3][3];
-        PetscBLASInt N=3,NRHS=1,LDA=3,IPIV[3],LDB=3,INFO;
-        PetscCDPos   pos;
+        PetscScalar    AA[3][3];
+        PetscBLASInt   N=3,NRHS=1,LDA=3,IPIV[3],LDB=3,INFO;
+        PetscCDIntNd   *pos;
+
         ierr = PetscCDGetHeadPos(agg_lists_1,lid,&pos);CHKERRQ(ierr);
         while (pos) {
           PetscInt flid;
-          ierr = PetscLLNGetID(pos, &flid);CHKERRQ(ierr);
+          ierr = PetscCDIntNdGetID(pos, &flid);CHKERRQ(ierr);
           ierr = PetscCDGetNextPos(agg_lists_1,lid,&pos);CHKERRQ(ierr);
 
           if (flid < nFineLoc) {  /* could be a ghost */
@@ -436,8 +434,6 @@ static PetscErrorCode triangulateAndFormProl(IS selected_2,PetscInt data_stride,
    . a_Gmat_2 - graph that is squared of 'Gmat_1'
    . a_crsGID[a_selected_2.size()] - map of global IDs of coarse grid nodes
 */
-#undef __FUNCT__
-#define __FUNCT__ "getGIDsOnSquareGraph"
 static PetscErrorCode getGIDsOnSquareGraph(PetscInt nselected_1,const PetscInt clid_lid_1[],const Mat Gmat1,IS *a_selected_2,Mat *a_Gmat_2,PetscInt **a_crsGID)
 {
   PetscErrorCode ierr;
@@ -539,14 +535,12 @@ static PetscErrorCode getGIDsOnSquareGraph(PetscInt nselected_1,const PetscInt c
   Output Parameter:
    . a_Gmat
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGGraph_GEO"
 PetscErrorCode PCGAMGGraph_GEO(PC pc,Mat Amat,Mat *a_Gmat)
 {
   PetscErrorCode  ierr;
   PC_MG           *mg      = (PC_MG*)pc->data;
   PC_GAMG         *pc_gamg = (PC_GAMG*)mg->innerctx;
-  const PetscReal vfilter  = pc_gamg->threshold;
+  const PetscReal vfilter  = pc_gamg->threshold[0];
   MPI_Comm        comm;
   Mat             Gmat;
   PetscBool       set,flg,symm;
@@ -576,8 +570,6 @@ PetscErrorCode PCGAMGGraph_GEO(PC pc,Mat Amat,Mat *a_Gmat)
   Output Parameter:
    . a_llist_parent - linked list from selected indices for data locality only
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGCoarsen_GEO"
 PetscErrorCode PCGAMGCoarsen_GEO(PC a_pc,Mat *a_Gmat,PetscCoarsenData **a_llist_parent)
 {
   PetscErrorCode ierr;
@@ -608,14 +600,17 @@ PetscErrorCode PCGAMGCoarsen_GEO(PC a_pc,Mat *a_Gmat,PetscCoarsenData **a_llist_
     }
     ierr = MatRestoreRow(Gmat,Ii,&ncols,0,0);CHKERRQ(ierr);
   }
-  /* randomize */
-  srand(1); /* make deterministic */
   if (PETSC_TRUE) {
-    PetscBool *bIndexSet;
-    ierr = PetscMalloc1(nloc, &bIndexSet);CHKERRQ(ierr);
-    for (Ii = 0; Ii < nloc; Ii++) bIndexSet[Ii] = PETSC_FALSE;
+    PetscRandom  rand;
+    PetscBool    *bIndexSet;
+    PetscReal    rr;
+    PetscInt     iSwapIndex;
+
+    ierr = PetscRandomCreate(comm,&rand);CHKERRQ(ierr);
+    ierr = PetscCalloc1(nloc, &bIndexSet);CHKERRQ(ierr);
     for (Ii = 0; Ii < nloc; Ii++) {
-      PetscInt iSwapIndex = rand()%nloc;
+      ierr = PetscRandomGetValueReal(rand,&rr);CHKERRQ(ierr);
+      iSwapIndex = (PetscInt) (rr*nloc);
       if (!bIndexSet[iSwapIndex] && iSwapIndex != Ii) {
         GAMGNode iTemp = gnodes[iSwapIndex];
         gnodes[iSwapIndex]    = gnodes[Ii];
@@ -624,15 +619,15 @@ PetscErrorCode PCGAMGCoarsen_GEO(PC a_pc,Mat *a_Gmat,PetscCoarsenData **a_llist_
         bIndexSet[iSwapIndex] = PETSC_TRUE;
       }
     }
+    ierr = PetscRandomDestroy(&rand);CHKERRQ(ierr);
     ierr = PetscFree(bIndexSet);CHKERRQ(ierr);
   }
   /* only sort locals */
   qsort(gnodes, nloc, sizeof(GAMGNode), petsc_geo_mg_compare);
   /* create IS of permutation */
   for (kk=0; kk<nloc; kk++) permute[kk] = gnodes[kk].lid; /* locals only */
-  ierr = ISCreateGeneral(PETSC_COMM_SELF, nloc, permute, PETSC_OWN_POINTER, &perm);CHKERRQ(ierr);
-
   ierr = PetscFree(gnodes);CHKERRQ(ierr);
+  ierr = ISCreateGeneral(PETSC_COMM_SELF, nloc, permute, PETSC_OWN_POINTER, &perm);CHKERRQ(ierr);
 
   /* get MIS aggs */
 
@@ -663,8 +658,6 @@ PetscErrorCode PCGAMGCoarsen_GEO(PC a_pc,Mat *a_Gmat,PetscCoarsenData **a_llist_
  Output Parameter:
  . a_P_out - prolongation operator to the next level
  */
-#undef __FUNCT__
-#define __FUNCT__ "PCGAMGProlongator_GEO"
 PetscErrorCode PCGAMGProlongator_GEO(PC pc,Mat Amat,Mat Gmat,PetscCoarsenData *agg_lists,Mat *a_P_out)
 {
   PC_MG          *mg      = (PC_MG*)pc->data;
@@ -686,7 +679,7 @@ PetscErrorCode PCGAMGProlongator_GEO(PC pc,Mat Amat,Mat Gmat,PetscCoarsenData *a
   ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
   ierr = MatGetOwnershipRange(Amat, &Istart, &Iend);CHKERRQ(ierr);
   ierr = MatGetBlockSize(Amat, &bs);CHKERRQ(ierr);
-  nloc = (Iend-Istart)/bs; my0 = Istart/bs; 
+  nloc = (Iend-Istart)/bs; my0 = Istart/bs;
   if ((Iend-Istart) % bs) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_PLIB,"(Iend %D - Istart %D) % bs %D",Iend,Istart,bs);
 
   /* get 'nLocalSelected' */
@@ -716,7 +709,7 @@ PetscErrorCode PCGAMGProlongator_GEO(PC pc,Mat Amat,Mat Gmat,PetscCoarsenData *a
 
   /* can get all points "removed" - but not on geomg */
   ierr =  MatGetSize(Prol, &kk, &jj);CHKERRQ(ierr);
-  if (jj==0) {
+  if (!jj) {
     ierr = PetscInfo(pc,"ERROE: no selected points on coarse grid\n");CHKERRQ(ierr);
     ierr = PetscFree(clid_flid);CHKERRQ(ierr);
     ierr = MatDestroy(&Prol);CHKERRQ(ierr);
@@ -765,7 +758,7 @@ PetscErrorCode PCGAMGProlongator_GEO(PC pc,Mat Amat,Mat Gmat,PetscCoarsenData *a
       /* clean up and create coordinates for coarse grid (output) */
       if (size > 1) ierr = PetscFree(coords);CHKERRQ(ierr);
 
-      ierr = MPI_Allreduce(&metric, &tm, 1, MPIU_REAL, MPIU_MAX, comm);CHKERRQ(ierr);
+      ierr = MPIU_Allreduce(&metric, &tm, 1, MPIU_REAL, MPIU_MAX, comm);CHKERRQ(ierr);
       if (tm > 1.) { /* needs to be globalized - should not happen */
         ierr = PetscInfo1(pc," failed metric for coarse grid %e\n",(double)tm);CHKERRQ(ierr);
         ierr = MatDestroy(&Prol);CHKERRQ(ierr);
@@ -794,8 +787,6 @@ PetscErrorCode PCGAMGProlongator_GEO(PC pc,Mat Amat,Mat Gmat,PetscCoarsenData *a
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy_GAMG_GEO"
 static PetscErrorCode PCDestroy_GAMG_GEO(PC pc)
 {
   PetscErrorCode ierr;
@@ -812,8 +803,6 @@ static PetscErrorCode PCDestroy_GAMG_GEO(PC pc)
   Input Parameter:
    . pc -
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCCreateGAMG_GEO"
 PetscErrorCode  PCCreateGAMG_GEO(PC pc)
 {
   PetscErrorCode ierr;
