@@ -32,6 +32,7 @@ PetscErrorCode PCISSetUseStiffnessScaling(PC pc, PetscBool use)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  PetscValidLogicalCollectiveInt(pc,use,2);
   ierr = PetscTryMethod(pc,"PCISSetUseStiffnessScaling_C",(PC,PetscBool),(pc,use));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -45,6 +46,18 @@ static PetscErrorCode PCISSetSubdomainDiagonalScaling_IS(PC pc, Vec scaling_fact
   ierr    = PetscObjectReference((PetscObject)scaling_factors);CHKERRQ(ierr);
   ierr    = VecDestroy(&pcis->D);CHKERRQ(ierr);
   pcis->D = scaling_factors;
+  if (pc->setupcalled) {
+    PetscInt sn;
+
+    ierr = VecGetSize(pcis->D,&sn);CHKERRQ(ierr);
+    if (sn == pcis->n) {
+      ierr = VecScatterBegin(pcis->N_to_B,pcis->D,pcis->vec1_B,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+      ierr = VecScatterEnd(pcis->N_to_B,pcis->D,pcis->vec1_B,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+      ierr = VecDestroy(&pcis->D);CHKERRQ(ierr);
+      ierr = VecDuplicate(pcis->vec1_B,&pcis->D);CHKERRQ(ierr);
+      ierr = VecCopy(pcis->vec1_B,pcis->D);CHKERRQ(ierr);
+    } else if (sn != pcis->n_B) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Invalid size for scaling vector. Expected %D (or full %D), found %D",pcis->n_B,pcis->n,sn); 
+  }
   PetscFunctionReturn(0);
 }
 
@@ -70,6 +83,7 @@ PetscErrorCode PCISSetSubdomainDiagonalScaling(PC pc, Vec scaling_factors)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  PetscValidHeaderSpecific(scaling_factors,VEC_CLASSID,2);
   ierr = PetscTryMethod(pc,"PCISSetSubdomainDiagonalScaling_C",(PC,Vec),(pc,scaling_factors));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -80,6 +94,11 @@ static PetscErrorCode PCISSetSubdomainScalingFactor_IS(PC pc, PetscScalar scal)
 
   PetscFunctionBegin;
   pcis->scaling_factor = scal;
+  if (pcis->D) {
+    PetscErrorCode ierr;
+
+    ierr = VecSet(pcis->D,pcis->scaling_factor);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
@@ -124,8 +143,12 @@ PetscErrorCode  PCISSetUp(PC pc, PetscBool computematrices, PetscBool computesol
 
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)pc->pmat,MATIS,&flg);CHKERRQ(ierr);
-  if (!flg) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONG,"Preconditioner type of Neumann Neumman requires matrix of type MATIS");
+  if (!flg) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONG,"Requires preconditioning matrix of type MATIS");
   matis = (Mat_IS*)pc->pmat->data;
+  if (pc->useAmat) {
+    ierr = PetscObjectTypeCompare((PetscObject)pc->mat,MATIS,&flg);CHKERRQ(ierr);
+    if (!flg) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONG,"Requires linear system matrix of type MATIS");
+  }
 
   /* first time creation, get info on substructuring */
   if (!pc->setupcalled) {
@@ -210,6 +233,19 @@ PetscErrorCode  PCISSetUp(PC pc, PetscBool computematrices, PetscBool computesol
     ierr = ISLocalToGlobalMappingCreateIS(pcis->is_B_local,&pcis->BtoNmap);CHKERRQ(ierr);
   }
 
+  {
+    PetscInt sn;
+
+    ierr = VecGetSize(pcis->D,&sn);CHKERRQ(ierr);
+    if (sn == pcis->n) {
+      ierr = VecScatterBegin(pcis->N_to_B,pcis->D,pcis->vec1_B,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+      ierr = VecScatterEnd(pcis->N_to_B,pcis->D,pcis->vec1_B,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+      ierr = VecDestroy(&pcis->D);CHKERRQ(ierr);
+      ierr = VecDuplicate(pcis->vec1_B,&pcis->D);CHKERRQ(ierr);
+      ierr = VecCopy(pcis->vec1_B,pcis->D);CHKERRQ(ierr);
+    } else if (sn != pcis->n_B) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Invalid size for scaling vector. Expected %D (or full %D), found %D",pcis->n_B,pcis->n,sn); 
+  }
+
   /*
     Extracting the blocks A_II, A_BI, A_IB and A_BB from A. If the numbering
     is such that interior nodes come first than the interface ones, we have
@@ -219,6 +255,9 @@ PetscErrorCode  PCISSetUp(PC pc, PetscBool computematrices, PetscBool computesol
         [ A_BI | A_BB ]
   */
   if (computematrices) {
+    PetscBool amat = (PetscBool)(pc->mat != pc->pmat && pc->useAmat);
+    PetscInt  bs,ibs;
+
     reuse = MAT_INITIAL_MATRIX;
     if (pcis->reusesubmatrices && pc->setupcalled) {
       if (pc->flag == SAME_NONZERO_PATTERN) {
@@ -229,12 +268,25 @@ PetscErrorCode  PCISSetUp(PC pc, PetscBool computematrices, PetscBool computesol
     }
     if (reuse == MAT_INITIAL_MATRIX) {
       ierr = MatDestroy(&pcis->A_II);CHKERRQ(ierr);
+      ierr = MatDestroy(&pcis->pA_II);CHKERRQ(ierr);
       ierr = MatDestroy(&pcis->A_IB);CHKERRQ(ierr);
       ierr = MatDestroy(&pcis->A_BI);CHKERRQ(ierr);
       ierr = MatDestroy(&pcis->A_BB);CHKERRQ(ierr);
     }
 
-    ierr = MatCreateSubMatrix(matis->A,pcis->is_I_local,pcis->is_I_local,reuse,&pcis->A_II);CHKERRQ(ierr);
+    ierr = ISLocalToGlobalMappingGetBlockSize(pcis->mapping,&ibs);CHKERRQ(ierr);
+    ierr = MatGetBlockSize(matis->A,&bs);CHKERRQ(ierr);
+    ierr = MatCreateSubMatrix(matis->A,pcis->is_I_local,pcis->is_I_local,reuse,&pcis->pA_II);CHKERRQ(ierr);
+    if (amat) {
+      Mat_IS *amatis = (Mat_IS*)pc->mat->data;
+      ierr = MatCreateSubMatrix(amatis->A,pcis->is_I_local,pcis->is_I_local,reuse,&pcis->A_II);CHKERRQ(ierr);
+    } else {
+      ierr = PetscObjectReference((PetscObject)pcis->pA_II);CHKERRQ(ierr);
+      ierr = MatDestroy(&pcis->A_II);CHKERRQ(ierr);
+      pcis->A_II = pcis->pA_II;
+    }
+    ierr = MatSetBlockSize(pcis->A_II,bs == ibs ? bs : 1);CHKERRQ(ierr);
+    ierr = MatSetBlockSize(pcis->pA_II,bs == ibs ? bs : 1);CHKERRQ(ierr);
     ierr = MatCreateSubMatrix(matis->A,pcis->is_B_local,pcis->is_B_local,reuse,&pcis->A_BB);CHKERRQ(ierr);
     ierr = PetscObjectTypeCompare((PetscObject)matis->A,MATSEQSBAIJ,&issbaij);CHKERRQ(ierr);
     if (!issbaij) {
@@ -248,6 +300,7 @@ PetscErrorCode  PCISSetUp(PC pc, PetscBool computematrices, PetscBool computesol
       ierr = MatCreateSubMatrix(newmat,pcis->is_B_local,pcis->is_I_local,reuse,&pcis->A_BI);CHKERRQ(ierr);
       ierr = MatDestroy(&newmat);CHKERRQ(ierr);
     }
+    ierr = MatSetBlockSize(pcis->A_BB,bs == ibs ? bs : 1);CHKERRQ(ierr);
   }
 
   /* Creating scaling vector D */
@@ -275,7 +328,6 @@ PetscErrorCode  PCISSetUp(PC pc, PetscBool computematrices, PetscBool computesol
   ierr = VecScatterBegin(pcis->global_to_B,pcis->vec1_global,pcis->vec1_B,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
   ierr = VecScatterEnd(pcis->global_to_B,pcis->vec1_global,pcis->vec1_B,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
   ierr = VecPointwiseDivide(pcis->D,pcis->D,pcis->vec1_B);CHKERRQ(ierr);
-
   /* See historical note 01, at the bottom of this file. */
 
   /* Creating the KSP contexts for the local Dirichlet and Neumann problems */
@@ -375,6 +427,7 @@ PetscErrorCode  PCISDestroy(PC pc)
   ierr = ISDestroy(&pcis->is_B_global);CHKERRQ(ierr);
   ierr = ISDestroy(&pcis->is_I_global);CHKERRQ(ierr);
   ierr = MatDestroy(&pcis->A_II);CHKERRQ(ierr);
+  ierr = MatDestroy(&pcis->pA_II);CHKERRQ(ierr);
   ierr = MatDestroy(&pcis->A_IB);CHKERRQ(ierr);
   ierr = MatDestroy(&pcis->A_BI);CHKERRQ(ierr);
   ierr = MatDestroy(&pcis->A_BB);CHKERRQ(ierr);
@@ -453,6 +506,7 @@ PetscErrorCode  PCISApplySchur(PC pc, Vec v, Vec vec1_B, Vec vec2_B, Vec vec1_D,
   ierr = MatMult(pcis->A_BB,v,vec1_B);CHKERRQ(ierr);
   ierr = MatMult(pcis->A_IB,v,vec1_D);CHKERRQ(ierr);
   ierr = KSPSolve(pcis->ksp_D,vec1_D,vec2_D);CHKERRQ(ierr);
+  ierr = KSPCheckSolve(pcis->ksp_D,pc,vec2_D);CHKERRQ(ierr);
   ierr = MatMult(pcis->A_BI,vec2_D,vec2_B);CHKERRQ(ierr);
   ierr = VecAXPY(vec1_B,-1.0,vec2_B);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -565,6 +619,7 @@ PetscErrorCode  PCISApplyInvSchur(PC pc, Vec b, Vec x, Vec vec1_N, Vec vec2_N)
   }
   /* Solving the system for vec2_N */
   ierr = KSPSolve(pcis->ksp_N,vec1_N,vec2_N);CHKERRQ(ierr);
+  ierr = KSPCheckSolve(pcis->ksp_N,pc,vec2_N);CHKERRQ(ierr);
   /* Extracting the local interface vector out of the solution */
   ierr = VecScatterBegin(pcis->N_to_B,vec2_N,x,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
   ierr = VecScatterEnd  (pcis->N_to_B,vec2_N,x,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
