@@ -10,8 +10,6 @@ typedef struct {
   Mat       L;             /* keep a copy to reuse when obtained with L = A10*A01 */
 } PC_LSC;
 
-#undef __FUNCT__
-#define __FUNCT__ "PCLSCAllocate_Private"
 static PetscErrorCode PCLSCAllocate_Private(PC pc)
 {
   PC_LSC         *lsc = (PC_LSC*)pc->data;
@@ -36,8 +34,6 @@ static PetscErrorCode PCLSCAllocate_Private(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp_LSC"
 static PetscErrorCode PCSetUp_LSC(PC pc)
 {
   PC_LSC         *lsc = (PC_LSC*)pc->data;
@@ -70,8 +66,6 @@ static PetscErrorCode PCSetUp_LSC(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_LSC"
 static PetscErrorCode PCApply_LSC(PC pc,Vec x,Vec y)
 {
   PC_LSC         *lsc = (PC_LSC*)pc->data;
@@ -81,6 +75,7 @@ static PetscErrorCode PCApply_LSC(PC pc,Vec x,Vec y)
   PetscFunctionBegin;
   ierr = MatSchurComplementGetSubMatrices(pc->mat,&A,NULL,&B,&C,NULL);CHKERRQ(ierr);
   ierr = KSPSolve(lsc->kspL,x,lsc->x1);CHKERRQ(ierr);
+  ierr = KSPCheckSolve(lsc->kspL,pc,lsc->x1);CHKERRQ(ierr);
   ierr = MatMult(B,lsc->x1,lsc->x0);CHKERRQ(ierr);
   if (lsc->scale) {
     ierr = VecPointwiseMult(lsc->x0,lsc->x0,lsc->scale);CHKERRQ(ierr);
@@ -91,11 +86,10 @@ static PetscErrorCode PCApply_LSC(PC pc,Vec x,Vec y)
   }
   ierr = MatMult(C,lsc->y0,lsc->x1);CHKERRQ(ierr);
   ierr = KSPSolve(lsc->kspL,lsc->x1,y);CHKERRQ(ierr);
+  ierr = KSPCheckSolve(lsc->kspL,pc,y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCReset_LSC"
 static PetscErrorCode PCReset_LSC(PC pc)
 {
   PC_LSC         *lsc = (PC_LSC*)pc->data;
@@ -111,8 +105,6 @@ static PetscErrorCode PCReset_LSC(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy_LSC"
 static PetscErrorCode PCDestroy_LSC(PC pc)
 {
   PetscErrorCode ierr;
@@ -123,9 +115,7 @@ static PetscErrorCode PCDestroy_LSC(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetFromOptions_LSC"
-static PetscErrorCode PCSetFromOptions_LSC(PetscOptions *PetscOptionsObject,PC pc)
+static PetscErrorCode PCSetFromOptions_LSC(PetscOptionItems *PetscOptionsObject,PC pc)
 {
   PC_LSC         *lsc = (PC_LSC*)pc->data;
   PetscErrorCode ierr;
@@ -139,8 +129,6 @@ static PetscErrorCode PCSetFromOptions_LSC(PetscOptions *PetscOptionsObject,PC p
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCView_LSC"
 static PetscErrorCode PCView_LSC(PC pc,PetscViewer viewer)
 {
   PC_LSC         *jac = (PC_LSC*)pc->data;
@@ -211,8 +199,8 @@ static PetscErrorCode PCView_LSC(PC pc,PetscViewer viewer)
    Since we do not use the values in Sp, you can still put an assembled matrix there to use normal preconditioners.
 
    References:
-+  Elman, Howle, Shadid, Shuttleworth, and Tuminaro, Block preconditioners based on approximate commutators, 2006.
--  Silvester, Elman, Kay, Wathen, Efficient preconditioning of the linearized Navier-Stokes equations for incompressible flow, 2001.
++  1. - Elman, Howle, Shadid, Shuttleworth, and Tuminaro, Block preconditioners based on approximate commutators, 2006.
+-  2. - Silvester, Elman, Kay, Wathen, Efficient preconditioning of the linearized Navier Stokes equations for incompressible flow, 2001.
 
    Concepts: physics based preconditioners, block preconditioners
 
@@ -221,8 +209,6 @@ static PetscErrorCode PCView_LSC(PC pc,PetscViewer viewer)
            MatCreateSchurComplement()
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate_LSC"
 PETSC_EXTERN PetscErrorCode PCCreate_LSC(PC pc)
 {
   PC_LSC         *lsc;
