@@ -8,21 +8,19 @@
 /* Logging support */
 PetscClassId  PC_CLASSID;
 PetscLogEvent PC_SetUp, PC_SetUpOnBlocks, PC_Apply, PC_ApplyCoarse, PC_ApplyMultiple, PC_ApplySymmetricLeft;
-PetscLogEvent PC_ApplySymmetricRight, PC_ModifySubMatrices, PC_ApplyOnBlocks, PC_ApplyTransposeOnBlocks, PC_ApplyOnMproc;
+PetscLogEvent PC_ApplySymmetricRight, PC_ModifySubMatrices, PC_ApplyOnBlocks, PC_ApplyTransposeOnBlocks;
+PetscInt      PetscMGLevelId;
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGetDefaultType_Private"
 PetscErrorCode PCGetDefaultType_Private(PC pc,const char *type[])
 {
   PetscErrorCode ierr;
   PetscMPIInt    size;
-  PetscBool      flg1,flg2,set,flg3;
+  PetscBool      hasop,flg1,flg2,set,flg3;
 
   PetscFunctionBegin;
   ierr = MPI_Comm_size(PetscObjectComm((PetscObject)pc),&size);CHKERRQ(ierr);
   if (pc->pmat) {
-    PetscErrorCode (*f)(Mat,MatReuse,Mat*);
-    ierr = PetscObjectQueryFunction((PetscObject)pc->pmat,"MatGetDiagonalBlock_C",&f);CHKERRQ(ierr);
+    ierr = MatHasOperation(pc->pmat,MATOP_GET_DIAGONAL_BLOCK,&hasop);CHKERRQ(ierr);
     if (size == 1) {
       ierr = MatGetFactorAvailable(pc->pmat,"petsc",MAT_FACTOR_ICC,&flg1);CHKERRQ(ierr);
       ierr = MatGetFactorAvailable(pc->pmat,"petsc",MAT_FACTOR_ILU,&flg2);CHKERRQ(ierr);
@@ -31,13 +29,13 @@ PetscErrorCode PCGetDefaultType_Private(PC pc,const char *type[])
         *type = PCICC;
       } else if (flg2) {
         *type = PCILU;
-      } else if (f) { /* likely is a parallel matrix run on one processor */
+      } else if (hasop) { /* likely is a parallel matrix run on one processor */
         *type = PCBJACOBI;
       } else {
         *type = PCNONE;
       }
     } else {
-       if (f) {
+       if (hasop) {
         *type = PCBJACOBI;
       } else {
         *type = PCNONE;
@@ -53,8 +51,6 @@ PetscErrorCode PCGetDefaultType_Private(PC pc,const char *type[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCReset"
 /*@
    PCReset - Resets a PC context to the pcsetupcalled = 0 state and removes any allocated Vecs and Mats
 
@@ -65,7 +61,8 @@ PetscErrorCode PCGetDefaultType_Private(PC pc,const char *type[])
 
    Level: developer
 
-   Notes: This allows a PC to be reused for a different sized linear system but using the same options that have been previously set in the PC
+   Notes:
+    This allows a PC to be reused for a different sized linear system but using the same options that have been previously set in the PC
 
 .keywords: PC, destroy
 
@@ -89,8 +86,6 @@ PetscErrorCode  PCReset(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy"
 /*@
    PCDestroy - Destroys PC context that was created with PCCreate().
 
@@ -124,8 +119,6 @@ PetscErrorCode  PCDestroy(PC *pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGetDiagonalScale"
 /*@C
    PCGetDiagonalScale - Indicates if the preconditioner applies an additional left and right
       scaling as needed by certain time-stepping codes.
@@ -140,7 +133,8 @@ PetscErrorCode  PCDestroy(PC *pc)
 
    Level: developer
 
-   Notes: If this returns PETSC_TRUE then the system solved via the Krylov method is
+   Notes:
+    If this returns PETSC_TRUE then the system solved via the Krylov method is
 $           D M A D^{-1} y = D M b  for left preconditioning or
 $           D A M D^{-1} z = D b for right preconditioning
 
@@ -157,8 +151,6 @@ PetscErrorCode  PCGetDiagonalScale(PC pc,PetscBool  *flag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetDiagonalScale"
 /*@
    PCSetDiagonalScale - Indicates the left scaling to use to apply an additional left and right
       scaling as needed by certain time-stepping codes.
@@ -171,7 +163,8 @@ PetscErrorCode  PCGetDiagonalScale(PC pc,PetscBool  *flag)
 
    Level: intermediate
 
-   Notes: The system solved via the Krylov method is
+   Notes:
+    The system solved via the Krylov method is
 $           D M A D^{-1} y = D M b  for left preconditioning or
 $           D A M D^{-1} z = D b for right preconditioning
 
@@ -201,8 +194,6 @@ PetscErrorCode  PCSetDiagonalScale(PC pc,Vec s)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCDiagonalScaleLeft"
 /*@
    PCDiagonalScaleLeft - Scales a vector by the left scaling as needed by certain time-stepping codes.
 
@@ -215,7 +206,8 @@ PetscErrorCode  PCSetDiagonalScale(PC pc,Vec s)
 
    Level: intermediate
 
-   Notes: The system solved via the Krylov method is
+   Notes:
+    The system solved via the Krylov method is
 $           D M A D^{-1} y = D M b  for left preconditioning or
 $           D A M D^{-1} z = D b for right preconditioning
 
@@ -243,8 +235,6 @@ PetscErrorCode  PCDiagonalScaleLeft(PC pc,Vec in,Vec out)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCDiagonalScaleRight"
 /*@
    PCDiagonalScaleRight - Scales a vector by the right scaling as needed by certain time-stepping codes.
 
@@ -257,7 +247,8 @@ PetscErrorCode  PCDiagonalScaleLeft(PC pc,Vec in,Vec out)
 
    Level: intermediate
 
-   Notes: The system solved via the Krylov method is
+   Notes:
+    The system solved via the Krylov method is
 $           D M A D^{-1} y = D M b  for left preconditioning or
 $           D A M D^{-1} z = D b for right preconditioning
 
@@ -285,11 +276,9 @@ PetscErrorCode  PCDiagonalScaleRight(PC pc,Vec in,Vec out)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUseAmat"
 /*@
    PCSetUseAmat - Sets a flag to indicate that when the preconditioner needs to apply (part of) the
-   operator during the preconditioning process it applies the Amat provided to TSSetRHSJacobian(), 
+   operator during the preconditioning process it applies the Amat provided to TSSetRHSJacobian(),
    TSSetIJacobian(), SNESSetJacobian(), KSPSetOperator() or PCSetOperator() not the Pmat.
 
    Logically Collective on PC
@@ -317,8 +306,6 @@ PetscErrorCode  PCSetUseAmat(PC pc,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetErrorIfFailure"
 /*@
    PCSetErrorIfFailure - Causes PC to generate an error if a FPE, for example a zero pivot, is detected.
 
@@ -332,7 +319,7 @@ PetscErrorCode  PCSetUseAmat(PC pc,PetscBool flg)
 
    Notes:
     Normally PETSc continues if a linear solver fails due to a failed setup of a preconditioner, you can call KSPGetConvergedReason() after a KSPSolve()
-    to determine if it has converged or failed. Or use -ksp_error_if_not_converged to cause the program to terminate as soon as lack of convergence is 
+    to determine if it has converged or failed. Or use -ksp_error_if_not_converged to cause the program to terminate as soon as lack of convergence is
     detected.
 
     This is propagated into KSPs used by this PC, which then propagate it into PCs used by those KSPs
@@ -350,8 +337,6 @@ PetscErrorCode  PCSetErrorIfFailure(PC pc,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGetUseAmat"
 /*@
    PCGetUseAmat - Gets a flag to indicate that when the preconditioner needs to apply (part of) the
    operator during the preconditioning process it applies the Amat provided to TSSetRHSJacobian(),
@@ -381,8 +366,6 @@ PetscErrorCode  PCGetUseAmat(PC pc,PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate"
 /*@
    PCCreate - Creates a preconditioner context.
 
@@ -435,8 +418,6 @@ PetscErrorCode  PCCreate(MPI_Comm comm,PC *newpc)
 
 /* -------------------------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply"
 /*@
    PCApply - Applies the preconditioner to a vector.
 
@@ -470,25 +451,21 @@ PetscErrorCode  PCApply(PC pc,Vec x,Vec y)
   ierr = MatGetLocalSize(pc->pmat,&m,&n);CHKERRQ(ierr);
   ierr = VecGetLocalSize(x,&nv);CHKERRQ(ierr);
   ierr = VecGetLocalSize(y,&mv);CHKERRQ(ierr);
-  if (mv != m) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Preconditioner number of local rows %D does not equal resulting vector number of rows %D",m,mv);CHKERRQ(ierr);
-  if (nv != n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Preconditioner number of local columns %D does not equal resulting vector number of rows %D",n,nv);CHKERRQ(ierr);
-  VecLocked(y,3);
+  if (mv != m) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Preconditioner number of local rows %D does not equal resulting vector number of rows %D",m,mv);
+  if (nv != n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Preconditioner number of local columns %D does not equal resulting vector number of rows %D",n,nv);
+  ierr = VecSetErrorIfLocked(y,3);CHKERRQ(ierr);
 
-  if (pc->setupcalled < 2) {
-    ierr = PCSetUp(pc);CHKERRQ(ierr);
-  }
+  ierr = PCSetUp(pc);CHKERRQ(ierr);
   if (!pc->ops->apply) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_SUP,"PC does not have apply");
-  ierr = VecLockPush(x);CHKERRQ(ierr);
+  ierr = VecLockReadPush(x);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(PC_Apply,pc,x,y,0);CHKERRQ(ierr);
   ierr = (*pc->ops->apply)(pc,x,y);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(PC_Apply,pc,x,y,0);CHKERRQ(ierr);
   if (pc->erroriffailure) {ierr = VecValidValues(y,3,PETSC_FALSE);CHKERRQ(ierr);}
-  ierr = VecLockPop(x);CHKERRQ(ierr);
+  ierr = VecLockReadPop(x);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplySymmetricLeft"
 /*@
    PCApplySymmetricLeft - Applies the left part of a symmetric preconditioner to a vector.
 
@@ -520,21 +497,17 @@ PetscErrorCode  PCApplySymmetricLeft(PC pc,Vec x,Vec y)
   PetscValidHeaderSpecific(y,VEC_CLASSID,3);
   if (x == y) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_IDN,"x and y must be different vectors");
   if (pc->erroriffailure) {ierr = VecValidValues(x,2,PETSC_TRUE);CHKERRQ(ierr);}
-  if (pc->setupcalled < 2) {
-    ierr = PCSetUp(pc);CHKERRQ(ierr);
-  }
+  ierr = PCSetUp(pc);CHKERRQ(ierr);
   if (!pc->ops->applysymmetricleft) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_SUP,"PC does not have left symmetric apply");
-  ierr = VecLockPush(x);CHKERRQ(ierr);
+  ierr = VecLockReadPush(x);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(PC_ApplySymmetricLeft,pc,x,y,0);CHKERRQ(ierr);
   ierr = (*pc->ops->applysymmetricleft)(pc,x,y);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(PC_ApplySymmetricLeft,pc,x,y,0);CHKERRQ(ierr);
-  ierr = VecLockPop(x);CHKERRQ(ierr);
+  ierr = VecLockReadPop(x);CHKERRQ(ierr);
   if (pc->erroriffailure) {ierr = VecValidValues(y,3,PETSC_FALSE);CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplySymmetricRight"
 /*@
    PCApplySymmetricRight - Applies the right part of a symmetric preconditioner to a vector.
 
@@ -566,21 +539,17 @@ PetscErrorCode  PCApplySymmetricRight(PC pc,Vec x,Vec y)
   PetscValidHeaderSpecific(y,VEC_CLASSID,3);
   if (x == y) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_IDN,"x and y must be different vectors");
   if (pc->erroriffailure) {ierr = VecValidValues(x,2,PETSC_TRUE);CHKERRQ(ierr);}
-  if (pc->setupcalled < 2) {
-    ierr = PCSetUp(pc);CHKERRQ(ierr);
-  }
+  ierr = PCSetUp(pc);CHKERRQ(ierr);
   if (!pc->ops->applysymmetricright) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_SUP,"PC does not have left symmetric apply");
-  ierr = VecLockPush(x);CHKERRQ(ierr);
+  ierr = VecLockReadPush(x);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(PC_ApplySymmetricRight,pc,x,y,0);CHKERRQ(ierr);
   ierr = (*pc->ops->applysymmetricright)(pc,x,y);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(PC_ApplySymmetricRight,pc,x,y,0);CHKERRQ(ierr);
-  ierr = VecLockPop(x);CHKERRQ(ierr);
+  ierr = VecLockReadPop(x);CHKERRQ(ierr);
   if (pc->erroriffailure) {ierr = VecValidValues(y,3,PETSC_FALSE);CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyTranspose"
 /*@
    PCApplyTranspose - Applies the transpose of preconditioner to a vector.
 
@@ -593,9 +562,11 @@ PetscErrorCode  PCApplySymmetricRight(PC pc,Vec x,Vec y)
    Output Parameter:
 .  y - output vector
 
-   Notes: For complex numbers this applies the non-Hermitian transpose.
+   Notes:
+    For complex numbers this applies the non-Hermitian transpose.
 
-   Developer Notes: We need to implement a PCApplyHermitianTranspose()
+   Developer Notes:
+    We need to implement a PCApplyHermitianTranspose()
 
    Level: developer
 
@@ -613,21 +584,17 @@ PetscErrorCode  PCApplyTranspose(PC pc,Vec x,Vec y)
   PetscValidHeaderSpecific(y,VEC_CLASSID,3);
   if (x == y) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_IDN,"x and y must be different vectors");
   if (pc->erroriffailure) {ierr = VecValidValues(x,2,PETSC_TRUE);CHKERRQ(ierr);}
-  if (pc->setupcalled < 2) {
-    ierr = PCSetUp(pc);CHKERRQ(ierr);
-  }
+  ierr = PCSetUp(pc);CHKERRQ(ierr);
   if (!pc->ops->applytranspose) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_SUP,"PC does not have apply transpose");
-  ierr = VecLockPush(x);CHKERRQ(ierr);
+  ierr = VecLockReadPush(x);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(PC_Apply,pc,x,y,0);CHKERRQ(ierr);
   ierr = (*pc->ops->applytranspose)(pc,x,y);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(PC_Apply,pc,x,y,0);CHKERRQ(ierr);
-  ierr = VecLockPop(x);CHKERRQ(ierr);
+  ierr = VecLockReadPop(x);CHKERRQ(ierr);
   if (pc->erroriffailure) {ierr = VecValidValues(y,3,PETSC_FALSE);CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyTransposeExists"
 /*@
    PCApplyTransposeExists - Test whether the preconditioner has a transpose apply operation
 
@@ -655,8 +622,6 @@ PetscErrorCode  PCApplyTransposeExists(PC pc,PetscBool  *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyBAorAB"
 /*@
    PCApplyBAorAB - Applies the preconditioner and operator to a vector. y = B*A*x or y = A*B*x.
 
@@ -673,7 +638,8 @@ PetscErrorCode  PCApplyTransposeExists(PC pc,PetscBool  *flg)
 
    Level: developer
 
-   Notes: If the PC has had PCSetDiagonalScale() set then D M A D^{-1} for left preconditioning or  D A M D^{-1} is actually applied. Note that the
+   Notes:
+    If the PC has had PCSetDiagonalScale() set then D M A D^{-1} for left preconditioning or  D A M D^{-1} is actually applied. Note that the
    specific KSPSolve() method must also be written to handle the post-solve "correction" for the diagonal scaling.
 
 .keywords: PC, apply, operator
@@ -694,10 +660,7 @@ PetscErrorCode  PCApplyBAorAB(PC pc,PCSide side,Vec x,Vec y,Vec work)
   if (pc->diagonalscale && side == PC_SYMMETRIC) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_SUP,"Cannot include diagonal scaling with symmetric preconditioner application");
   if (pc->erroriffailure) {ierr = VecValidValues(x,3,PETSC_TRUE);CHKERRQ(ierr);}
 
-  if (pc->setupcalled < 2) {
-    ierr = PCSetUp(pc);CHKERRQ(ierr);
-  }
-
+  ierr = PCSetUp(pc);CHKERRQ(ierr);
   if (pc->diagonalscale) {
     if (pc->ops->applyBA) {
       Vec work2; /* this is expensive, but to fix requires a second work vector argument to PCApplyBAorAB() */
@@ -738,8 +701,6 @@ PetscErrorCode  PCApplyBAorAB(PC pc,PCSide side,Vec x,Vec y,Vec work)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyBAorABTranspose"
 /*@
    PCApplyBAorABTranspose - Applies the transpose of the preconditioner
    and operator to a vector. That is, applies tr(B) * tr(A) with left preconditioning,
@@ -757,7 +718,8 @@ PetscErrorCode  PCApplyBAorAB(PC pc,PCSide side,Vec x,Vec y,Vec work)
 .  y - output vector
 
 
-   Notes: this routine is used internally so that the same Krylov code can be used to solve A x = b and A' x = b, with a preconditioner
+   Notes:
+    this routine is used internally so that the same Krylov code can be used to solve A x = b and A' x = b, with a preconditioner
       defined by B'. This is why this has the funny form that it computes tr(B) * tr(A)
 
     Level: developer
@@ -784,10 +746,7 @@ PetscErrorCode  PCApplyBAorABTranspose(PC pc,PCSide side,Vec x,Vec y,Vec work)
   }
   if (side != PC_LEFT && side != PC_RIGHT) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_OUTOFRANGE,"Side must be right or left");
 
-  if (pc->setupcalled < 2) {
-    ierr = PCSetUp(pc);CHKERRQ(ierr);
-  }
-
+  ierr = PCSetUp(pc);CHKERRQ(ierr);
   if (side == PC_RIGHT) {
     ierr = PCApplyTranspose(pc,x,work);CHKERRQ(ierr);
     ierr = MatMultTranspose(pc->mat,work,y);CHKERRQ(ierr);
@@ -802,8 +761,6 @@ PetscErrorCode  PCApplyBAorABTranspose(PC pc,PCSide side,Vec x,Vec y,Vec work)
 
 /* -------------------------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyRichardsonExists"
 /*@
    PCApplyRichardsonExists - Determines whether a particular preconditioner has a
    built-in fast application of Richardson's method.
@@ -826,14 +783,12 @@ PetscErrorCode  PCApplyRichardsonExists(PC pc,PetscBool  *exists)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
-  PetscValidIntPointer(exists,2);
+  PetscValidPointer(exists,2);
   if (pc->ops->applyrichardson) *exists = PETSC_TRUE;
   else *exists = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyRichardson"
 /*@
    PCApplyRichardson - Applies several steps of Richardson iteration with
    the particular preconditioner. This routine is usually used by the
@@ -879,18 +834,14 @@ PetscErrorCode  PCApplyRichardson(PC pc,Vec b,Vec y,Vec w,PetscReal rtol,PetscRe
   PetscValidHeaderSpecific(y,VEC_CLASSID,3);
   PetscValidHeaderSpecific(w,VEC_CLASSID,4);
   if (b == y) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_IDN,"b and y must be different vectors");
-  if (pc->setupcalled < 2) {
-    ierr = PCSetUp(pc);CHKERRQ(ierr);
-  }
+  ierr = PCSetUp(pc);CHKERRQ(ierr);
   if (!pc->ops->applyrichardson) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_SUP,"PC does not have apply richardson");
   ierr = (*pc->ops->applyrichardson)(pc,b,y,w,rtol,abstol,dtol,its,guesszero,outits,reason);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGetSetUpFailedReason"
 /*@
-   PCGetSetUpFailedReason - Gets the reason a PCSetUp() failed or 0 if it did not fail
+   PCGetFailedReason - Gets the reason a PCSetUp() failed or 0 if it did not fail
 
    Logically Collective on PC
 
@@ -898,7 +849,7 @@ PetscErrorCode  PCApplyRichardson(PC pc,Vec b,Vec y,Vec w,PetscReal rtol,PetscRe
 .  pc - the preconditioner context
 
    Output Parameter:
-.  reason - the reason it failed, currently only -1 
+.  reason - the reason it failed, currently only -1
 
    Level: advanced
 
@@ -906,11 +857,11 @@ PetscErrorCode  PCApplyRichardson(PC pc,Vec b,Vec y,Vec w,PetscReal rtol,PetscRe
 
 .seealso: PCCreate(), PCApply(), PCDestroy()
 @*/
-PetscErrorCode  PCGetSetUpFailedReason(PC pc,PetscInt *reason)
+PetscErrorCode PCGetFailedReason(PC pc,PCFailedReason *reason)
 {
   PetscFunctionBegin;
-  if (pc->setupcalled < 0) *reason = pc->setupcalled;
-  else *reason = 0;
+  if (pc->setupcalled < 0) *reason = (PCFailedReason)pc->setupcalled;
+  else *reason = pc->failedreason;
   PetscFunctionReturn(0);
 }
 
@@ -920,8 +871,6 @@ PetscErrorCode  PCGetSetUpFailedReason(PC pc,PetscInt *reason)
                      1 indicates has been previously setup
                     -1 indicates a PCSetUp() was attempted and failed
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp"
 /*@
    PCSetUp - Prepares for the use of a preconditioner.
 
@@ -976,8 +925,8 @@ PetscErrorCode  PCSetUp(PC pc)
     ierr = PCSetType(pc,def);CHKERRQ(ierr);
   }
 
-  ierr = MatSetErrorIfFPE(pc->pmat,pc->erroriffailure);CHKERRQ(ierr);
-  ierr = MatSetErrorIfFPE(pc->mat,pc->erroriffailure);CHKERRQ(ierr);
+  ierr = MatSetErrorIfFailure(pc->pmat,pc->erroriffailure);CHKERRQ(ierr);
+  ierr = MatSetErrorIfFailure(pc->mat,pc->erroriffailure);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(PC_SetUp,pc,0,0,0);CHKERRQ(ierr);
   if (pc->ops->setup) {
     ierr = (*pc->ops->setup)(pc);CHKERRQ(ierr);
@@ -987,8 +936,6 @@ PetscErrorCode  PCSetUp(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUpOnBlocks"
 /*@
    PCSetUpOnBlocks - Sets up the preconditioner for each block in
    the block Jacobi, block Gauss-Seidel, and overlapping Schwarz
@@ -1018,8 +965,6 @@ PetscErrorCode  PCSetUpOnBlocks(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetModifySubMatrices"
 /*@C
    PCSetModifySubMatrices - Sets a user-defined routine for modifying the
    submatrices that arise within certain subdomain-based preconditioners.
@@ -1068,8 +1013,6 @@ PetscErrorCode  PCSetModifySubMatrices(PC pc,PetscErrorCode (*func)(PC,PetscInt,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCModifySubMatrices"
 /*@C
    PCModifySubMatrices - Calls an optional user-defined routine within
    certain preconditioners if one has been set with PCSetModifySubMarices().
@@ -1120,8 +1063,6 @@ PetscErrorCode  PCModifySubMatrices(PC pc,PetscInt nsub,const IS row[],const IS
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetOperators"
 /*@
    PCSetOperators - Sets the matrix associated with the linear system and
    a (possibly) different one associated with the preconditioner.
@@ -1188,8 +1129,6 @@ PetscErrorCode  PCSetOperators(PC pc,Mat Amat,Mat Pmat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetReusePreconditioner"
 /*@
    PCSetReusePreconditioner - reuse the current preconditioner even if the operator in the preconditioner has changed.
 
@@ -1211,8 +1150,6 @@ PetscErrorCode  PCSetReusePreconditioner(PC pc,PetscBool flag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGetReusePreconditioner"
 /*@
    PCGetReusePreconditioner - Determines if the PC reuses the current preconditioner even if the operator in the preconditioner has changed.
 
@@ -1236,9 +1173,7 @@ PetscErrorCode  PCGetReusePreconditioner(PC pc,PetscBool *flag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGetOperators"
-/*@C
+/*@
    PCGetOperators - Gets the matrix associated with the linear system and
    possibly a different one associated with the preconditioner.
 
@@ -1253,7 +1188,8 @@ PetscErrorCode  PCGetReusePreconditioner(PC pc,PetscBool *flag)
 
    Level: intermediate
 
-   Notes: Does not increase the reference count of the matrices, so you should not destroy them
+   Notes:
+    Does not increase the reference count of the matrices, so you should not destroy them
 
    Alternative usage: If the operators have NOT been set with KSP/PCSetOperators() then the operators
       are created in PC and returned to the user. In this case, if both operators
@@ -1336,8 +1272,6 @@ PetscErrorCode  PCGetOperators(PC pc,Mat *Amat,Mat *Pmat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGetOperatorsSet"
 /*@C
    PCGetOperatorsSet - Determines if the matrix associated with the linear system and
    possibly a different one associated with the preconditioner have been set in the PC.
@@ -1366,8 +1300,6 @@ PetscErrorCode  PCGetOperatorsSet(PC pc,PetscBool  *mat,PetscBool  *pmat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorGetMatrix"
 /*@
    PCFactorGetMatrix - Gets the factored matrix from the
    preconditioner context.  This routine is valid only for the LU,
@@ -1383,7 +1315,8 @@ PetscErrorCode  PCGetOperatorsSet(PC pc,PetscBool  *mat,PetscBool  *pmat)
 
    Level: advanced
 
-   Notes: Does not increase the reference count for the matrix so DO NOT destroy it
+   Notes:
+    Does not increase the reference count for the matrix so DO NOT destroy it
 
 .keywords: PC, get, factored, matrix
 @*/
@@ -1400,8 +1333,6 @@ PetscErrorCode  PCFactorGetMatrix(PC pc,Mat *mat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetOptionsPrefix"
 /*@C
    PCSetOptionsPrefix - Sets the prefix used for searching for all
    PC options in the database.
@@ -1433,8 +1364,6 @@ PetscErrorCode  PCSetOptionsPrefix(PC pc,const char prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCAppendOptionsPrefix"
 /*@C
    PCAppendOptionsPrefix - Appends to the prefix used for searching for all
    PC options in the database.
@@ -1466,8 +1395,6 @@ PetscErrorCode  PCAppendOptionsPrefix(PC pc,const char prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGetOptionsPrefix"
 /*@C
    PCGetOptionsPrefix - Gets the prefix used for searching for all
    PC options in the database.
@@ -1480,7 +1407,8 @@ PetscErrorCode  PCAppendOptionsPrefix(PC pc,const char prefix[])
    Output Parameters:
 .  prefix - pointer to the prefix string used, is returned
 
-   Notes: On the fortran side, the user should pass in a string 'prifix' of
+   Notes:
+    On the fortran side, the user should pass in a string 'prifix' of
    sufficient length to hold the prefix.
 
    Level: advanced
@@ -1500,8 +1428,23 @@ PetscErrorCode  PCGetOptionsPrefix(PC pc,const char *prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPreSolve"
+/*
+   Indicates the right hand side will be changed by KSPSolve(), this occurs for a few
+  preconditioners including BDDC and Eisentat that transform the equations before applying
+  the Krylov methods
+*/
+PETSC_INTERN PetscErrorCode  PCPreSolveChangeRHS(PC pc,PetscBool *change)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  PetscValidPointer(change,2);
+  *change = PETSC_FALSE;
+  ierr = PetscTryMethod(pc,"PCPreSolveChangeRHS_C",(PC,PetscBool*),(pc,change));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    PCPreSolve - Optional pre-solve phase, intended for any
    preconditioner-specific actions that must be performed before
@@ -1550,8 +1493,6 @@ PetscErrorCode  PCPreSolve(PC pc,KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPostSolve"
 /*@
    PCPostSolve - Optional post-solve phase, intended for any
    preconditioner-specific actions that must be performed after
@@ -1596,8 +1537,6 @@ PetscErrorCode  PCPostSolve(PC pc,KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCLoad"
 /*@C
   PCLoad - Loads a PC that has been stored in binary  with PCView().
 
@@ -1651,8 +1590,6 @@ PetscErrorCode  PCLoad(PC newdm, PetscViewer viewer)
 #if defined(PETSC_HAVE_SAWS)
 #include <petscviewersaws.h>
 #endif
-#undef __FUNCT__
-#define __FUNCT__ "PCView"
 /*@C
    PCView - Prints the PC data structure.
 
@@ -1681,12 +1618,11 @@ PetscErrorCode  PCLoad(PC newdm, PetscViewer viewer)
 @*/
 PetscErrorCode  PCView(PC pc,PetscViewer viewer)
 {
-  PCType            cstr;
-  PetscErrorCode    ierr;
-  PetscBool         iascii,isstring,isbinary,isdraw;
-  PetscViewerFormat format;
+  PCType         cstr;
+  PetscErrorCode ierr;
+  PetscBool      iascii,isstring,isbinary,isdraw;
 #if defined(PETSC_HAVE_SAWS)
-  PetscBool         issaws;
+  PetscBool      issaws;
 #endif
 
   PetscFunctionBegin;
@@ -1706,7 +1642,6 @@ PetscErrorCode  PCView(PC pc,PetscViewer viewer)
 #endif
 
   if (iascii) {
-    ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)pc,viewer);CHKERRQ(ierr);
     if (!pc->setupcalled) {
       ierr = PetscViewerASCIIPrintf(viewer,"  PC has not been set up so information may be incomplete\n");CHKERRQ(ierr);
@@ -1793,71 +1728,6 @@ PetscErrorCode  PCView(PC pc,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "PCSetInitialGuessNonzero"
-/*@
-   PCSetInitialGuessNonzero - Tells the iterative solver that the
-   initial guess is nonzero; otherwise PC assumes the initial guess
-   is to be zero (and thus zeros it out before solving).
-
-   Logically Collective on PC
-
-   Input Parameters:
-+  pc - iterative context obtained from PCCreate()
--  flg - PETSC_TRUE indicates the guess is non-zero, PETSC_FALSE indicates the guess is zero
-
-   Level: Developer
-
-   Notes:
-    This is a weird function. Since PC's are linear operators on the right hand side they
-    CANNOT use an initial guess. This function is for the "pass-through" preconditioners
-    PCKSP and PCREDUNDANT  and causes the inner KSP object to use the nonzero
-    initial guess. Not currently working for PCREDUNDANT, that has to be rewritten to use KSP.
-
-
-.keywords: PC, set, initial guess, nonzero
-
-.seealso: PCGetInitialGuessNonzero(), PCSetInitialGuessKnoll(), PCGetInitialGuessKnoll()
-@*/
-PetscErrorCode  PCSetInitialGuessNonzero(PC pc,PetscBool flg)
-{
-  PetscFunctionBegin;
-  PetscValidLogicalCollectiveBool(pc,flg,2);
-  pc->nonzero_guess = flg;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PCGetInitialGuessNonzero"
-/*@
-   PCGetInitialGuessNonzero - Determines if the iterative solver assumes that the
-   initial guess is nonzero; otherwise PC assumes the initial guess
-   is to be zero (and thus zeros it out before solving).
-
-   Logically Collective on PC
-
-   Input Parameter:
-.   pc - iterative context obtained from PCCreate()
-
-   Output Parameter:
-.  flg - PETSC_TRUE indicates the guess is non-zero, PETSC_FALSE indicates the guess is zero
-
-   Level: Developer
-
-.keywords: PC, set, initial guess, nonzero
-
-.seealso: PCGetInitialGuessNonzero(), PCSetInitialGuessKnoll(), PCGetInitialGuessKnoll(), PCSetInitialGuessNonzero()
-@*/
-PetscErrorCode  PCGetInitialGuessNonzero(PC pc,PetscBool *flg)
-{
-  PetscFunctionBegin;
-  *flg = pc->nonzero_guess;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PCRegister"
 /*@C
   PCRegister -  Adds a method to the preconditioner package.
 
@@ -1891,12 +1761,11 @@ PetscErrorCode  PCRegister(const char sname[],PetscErrorCode (*function)(PC))
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = PCInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&PCList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCComputeExplicitOperator"
 /*@
     PCComputeExplicitOperator - Computes the explicit preconditioned operator.
 
@@ -1976,13 +1845,12 @@ PetscErrorCode  PCComputeExplicitOperator(PC pc,Mat *mat)
   }
   ierr = PetscFree(rows);CHKERRQ(ierr);
   ierr = VecDestroy(&out);CHKERRQ(ierr);
+  ierr = VecDestroy(&in);CHKERRQ(ierr);
   ierr = MatAssemblyBegin(*mat,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(*mat,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetCoordinates"
 /*@
    PCSetCoordinates - sets the coordinates of all the nodes on the local process
 
@@ -1991,25 +1859,30 @@ PetscErrorCode  PCComputeExplicitOperator(PC pc,Mat *mat)
    Input Parameters:
 +  pc - the solver context
 .  dim - the dimension of the coordinates 1, 2, or 3
--  coords - the coordinates
+.  nloc - the blocked size of the coordinates array
+-  coords - the coordinates array
 
    Level: intermediate
 
-   Notes: coords is an array of the 3D coordinates for the nodes on
-   the local processor.  So if there are 108 equation on a processor
+   Notes:
+   coords is an array of the dim coordinates for the nodes on
+   the local processor, of size dim*nloc.
+   If there are 108 equation on a processor
    for a displacement finite element discretization of elasticity (so
-   that there are 36 = 108/3 nodes) then the array must have 108
+   that there are nloc = 36 = 108/3 nodes) then the array must have 108
    double precision values (ie, 3 * 36).  These x y z coordinates
    should be ordered for nodes 0 to N-1 like so: [ 0.x, 0.y, 0.z, 1.x,
    ... , N-1.z ].
 
-.seealso: MatSetNearNullSpace
+.seealso: MatSetNearNullSpace()
 @*/
 PetscErrorCode PCSetCoordinates(PC pc, PetscInt dim, PetscInt nloc, PetscReal *coords)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  PetscValidLogicalCollectiveInt(pc,dim,2);
   ierr = PetscTryMethod(pc,"PCSetCoordinates_C",(PC,PetscInt,PetscInt,PetscReal*),(pc,dim,nloc,coords));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
