@@ -5,8 +5,6 @@
 PETSC_EXTERN PetscErrorCode MatCoarsenCreate_MIS(MatCoarsen);
 PETSC_EXTERN PetscErrorCode MatCoarsenCreate_HEM(MatCoarsen);
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCoarsenRegisterAll"
 /*@C
   MatCoarsenRegisterAll - Registers all of the matrix Coarsen routines in PETSc.
 
