@@ -6,7 +6,7 @@ Input arguments are:\n\
    ./ex94 -f0 <A_binary> -f1 <B_binary> -matmatmult_mat_view ascii::ascii_info -matmatmulttr_mat_view
    mpiexec -n 3 ./ex94 -f0 medium -f1 medium -f2 arco1 -f3 arco1 -matmatmult_mat_view
 */
- 
+
 #include <petscmat.h>
 
 /*
@@ -183,6 +183,7 @@ int main(int argc,char **args)
 
     /* Test MAT_REUSE_MATRIX - reuse symbolic C */
     ierr = MatTransposeMatMult(P,B,MAT_REUSE_MATRIX,fill,&C);CHKERRQ(ierr);
+    ierr = MatFreeIntermediateDataStructures(C);CHKERRQ(ierr);
 
     /* Compare P^T*B and R*B */
     ierr = MatMatMult(R,B,MAT_INITIAL_MATRIX,fill,&C1);CHKERRQ(ierr);
@@ -219,9 +220,11 @@ int main(int argc,char **args)
   /* 3) Test MatPtAP() */
   /*-------------------*/
   if (Test_MatPtAP) {
-    PetscInt PN;
-    Mat      Cdup;
+    PetscInt  PN;
+    Mat       Cdup;
+    PetscBool view=PETSC_FALSE;
 
+    ierr = PetscOptionsGetBool(NULL,NULL,"-matptap_view",&view,NULL);CHKERRQ(ierr);
     ierr = MatDuplicate(A_save,MAT_COPY_VALUES,&A);CHKERRQ(ierr);
     ierr = MatGetSize(A,&M,&N);CHKERRQ(ierr);
     ierr = MatGetLocalSize(A,&m,&n);CHKERRQ(ierr);
@@ -300,8 +303,11 @@ int main(int argc,char **args)
       ierr = MatDestroy(&Adense);CHKERRQ(ierr);
     }
 
-    /* Test MatDuplicate() of C=PtAP */
+    /* Test MatDuplicate() of C=PtAP and MatView(Cdup,...) */
     ierr = MatDuplicate(C,MAT_COPY_VALUES,&Cdup);CHKERRQ(ierr);
+    if (view) {
+      ierr = MatView(Cdup,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
+    }
     ierr = MatDestroy(&Cdup);CHKERRQ(ierr);
 
     if (size>1 || !seqaij) Test_MatRARt = PETSC_FALSE;
@@ -460,4 +466,11 @@ int main(int argc,char **args)
       args: -f0 ${DATAFILESPATH}/matrices/arco1 -f1 ${DATAFILESPATH}/matrices/arco1 -viewer_binary_skip_info -matptap_via scalable
       output_file: output/ex94_1.out
 
+   test:
+      suffix: view
+      nsize: 2
+      requires: datafilespath !complex double !define(PETSC_USE_64BIT_INDICES)
+      args: -f0 ${DATAFILESPATH}/matrices/tiny -f1 ${DATAFILESPATH}/matrices/tiny -viewer_binary_skip_info -matptap_view
+      output_file: output/ex94_2.out
+
 TEST*/
