@@ -16,24 +16,25 @@ typedef struct { /* used by MatCreateMPIAIJSumSeqAIJ for reusing the merged matr
 } Mat_Merge_SeqsToMPI;
 
 typedef struct { /* used by MatPtAP_MPIAIJ_MPIAIJ() and MatMatMult_MPIAIJ_MPIAIJ() */
-  PetscInt    *startsj_s,*startsj_r;    /* used by MatGetBrowsOfAoCols_MPIAIJ */
-  PetscScalar *bufa;                    /* used by MatGetBrowsOfAoCols_MPIAIJ */
-  Mat         P_loc,P_oth;     /* partial B_seq -- intend to replace B_seq */
-  PetscInt    *api,*apj;       /* symbolic i and j arrays of the local product A_loc*B_seq */
-  PetscScalar *apv;
-  MatReuse    reuse;           /* flag to skip MatGetBrowsOfAoCols_MPIAIJ() and MatMPIAIJGetLocalMat() in 1st call of MatPtAPNumeric_MPIAIJ_MPIAIJ() */
-  PetscScalar *apa;            /* tmp array for store a row of A*P used in MatMatMult() */
-  Mat         A_loc;           /* used by MatTransposeMatMult(), contains api and apj */
-  Mat         Pt;              /* used by MatTransposeMatMult(), Pt = P^T */
-  PetscBool   scalable;        /* flag determines scalable or non-scalable implementation */
-  Mat         Rd,Ro,AP_loc,C_loc,C_oth;
-  PetscInt    algType;         /* implementation algorithm */
+  PetscInt               *startsj_s,*startsj_r;    /* used by MatGetBrowsOfAoCols_MPIAIJ */
+  PetscScalar            *bufa;                    /* used by MatGetBrowsOfAoCols_MPIAIJ */
+  Mat                     P_loc,P_oth;             /* partial B_seq -- intend to replace B_seq */
+  PetscInt                *api,*apj;               /* symbolic i and j arrays of the local product A_loc*B_seq */
+  PetscScalar             *apv;
+  MatReuse                reuse;                   /* flag to skip MatGetBrowsOfAoCols_MPIAIJ() and MatMPIAIJGetLocalMat() in 1st call of MatPtAPNumeric_MPIAIJ_MPIAIJ() */
+  PetscScalar             *apa;                    /* tmp array for store a row of A*P used in MatMatMult() */
+  Mat                     A_loc;                   /* used by MatTransposeMatMult(), contains api and apj */
+  ISLocalToGlobalMapping  ltog;                    /* mapping from local column indices to global column indices for A_loc */
+  Mat                     Pt;                      /* used by MatTransposeMatMult(), Pt = P^T */
+  PetscBool               freestruct;              /* flag for MatFreeIntermediateDataStructures() */
+  Mat                     Rd,Ro,AP_loc,C_loc,C_oth;
+  PetscInt                algType;                 /* implementation algorithm */
 
   Mat_Merge_SeqsToMPI *merge;
   PetscErrorCode (*destroy)(Mat);
   PetscErrorCode (*duplicate)(Mat,MatDuplicateOption,Mat*);
   PetscErrorCode (*view)(Mat,PetscViewer);
-} Mat_PtAPMPI;
+} Mat_APMPI;
 
 typedef struct {
   Mat A,B;                             /* local submatrices: A (diag part),
@@ -71,7 +72,7 @@ typedef struct {
   PetscInt *ld;                    /* number of entries per row left of diagona block */
 
   /* Used by MatMatMult() and MatPtAP() */
-  Mat_PtAPMPI *ptap;
+  Mat_APMPI *ap;
 
   /* used by MatMatMatMult() */
   Mat_MatMatMatMult *matmatmatmult;
@@ -95,7 +96,7 @@ PETSC_INTERN PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat,ISColoring,MatFDColor
 PETSC_INTERN PetscErrorCode MatCreateSubMatrices_MPIAIJ (Mat,PetscInt,const IS[],const IS[],MatReuse,Mat *[]);
 PETSC_INTERN PetscErrorCode MatCreateSubMatricesMPI_MPIAIJ (Mat,PetscInt,const IS[],const IS[],MatReuse,Mat *[]);
 PETSC_INTERN PetscErrorCode MatCreateSubMatrix_MPIAIJ_All(Mat,MatCreateSubMatrixOption,MatReuse,Mat *[]);
-
+PETSC_INTERN PetscErrorCode MatView_MPIAIJ(Mat,PetscViewer);
 
 PETSC_INTERN PetscErrorCode MatCreateSubMatrix_MPIAIJ(Mat,IS,IS,MatReuse,Mat*);
 PETSC_INTERN PetscErrorCode MatCreateSubMatrix_MPIAIJ_nonscalable(Mat,IS,IS,PetscInt,MatReuse,Mat*);
@@ -104,6 +105,7 @@ PETSC_INTERN PetscErrorCode MatCreateSubMatrix_MPIAIJ_SameRowColDist(Mat,IS,IS,M
 PETSC_INTERN PetscErrorCode MatGetMultiProcBlock_MPIAIJ(Mat,MPI_Comm,MatReuse,Mat*);
 
 PETSC_INTERN PetscErrorCode MatLoad_MPIAIJ(Mat,PetscViewer);
+PETSC_INTERN PetscErrorCode MatLoad_MPIAIJ_Binary(Mat,PetscViewer);
 PETSC_INTERN PetscErrorCode MatCreateColmap_MPIAIJ_Private(Mat);
 PETSC_INTERN PetscErrorCode MatMatMult_MPIDense_MPIAIJ(Mat,Mat,MatReuse,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatMatMult_MPIAIJ_MPIAIJ(Mat,Mat,MatReuse,PetscReal,Mat*);
@@ -123,7 +125,8 @@ PETSC_INTERN PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIAIJ(Mat,Mat,Mat);
 
 PETSC_INTERN PetscErrorCode MatPtAPSymbolic_MPIAIJ_MPIAIJ_scalable(Mat,Mat,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatPtAPNumeric_MPIAIJ_MPIAIJ_scalable(Mat,Mat,Mat);
-
+PETSC_INTERN PetscErrorCode MatFreeIntermediateDataStructures_MPIAIJ_AP(Mat);
+PETSC_INTERN PetscErrorCode MatFreeIntermediateDataStructures_MPIAIJ_BC(Mat);
 
 #if defined(PETSC_HAVE_HYPRE)
 PETSC_INTERN PetscErrorCode MatPtAPSymbolic_AIJ_AIJ_wHYPRE(Mat,Mat,PetscReal,Mat*);
