@@ -8,8 +8,6 @@
 #include <engine.h>   /* MATLAB include file */
 #include <mex.h>      /* MATLAB include file */
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSeqAIJToMatlab"
 PETSC_EXTERN mxArray *MatSeqAIJToMatlab(Mat B)
 {
   PetscErrorCode ierr;
@@ -29,8 +27,6 @@ PETSC_EXTERN mxArray *MatSeqAIJToMatlab(Mat B)
   PetscFunctionReturn(mat);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatlabEnginePut_SeqAIJ"
 PETSC_EXTERN PetscErrorCode MatlabEnginePut_SeqAIJ(PetscObject obj,void *mengine)
 {
   PetscErrorCode ierr;
@@ -43,20 +39,6 @@ PETSC_EXTERN PetscErrorCode MatlabEnginePut_SeqAIJ(PetscObject obj,void *mengine
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSeqAIJFromMatlab"
-/*@C
-    MatSeqAIJFromMatlab - Given a MATLAB sparse matrix, fills a SeqAIJ matrix with its transpose.
-
-   Not Collective
-
-   Input Parameters:
-+     mmat - a MATLAB sparse matris
--     mat - an already created MATSEQAIJ
-
-  Level: intermediate
-
-@*/
 PETSC_EXTERN PetscErrorCode MatSeqAIJFromMatlab(mxArray *mmat,Mat mat)
 {
   PetscErrorCode ierr;
@@ -100,13 +82,12 @@ PETSC_EXTERN PetscErrorCode MatSeqAIJFromMatlab(mxArray *mmat,Mat mat)
 
   for (k=0; k<mat->rmap->n; k++) aij->ilen[k] = aij->imax[k] = aij->i[k+1] - aij->i[k];
 
+  mat->nonzerostate++; /* since the nonzero structure can change anytime force the Inode information to always be rebuilt */
   ierr = MatAssemblyBegin(mat,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(mat,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatlabEngineGet_SeqAIJ"
 PETSC_EXTERN PetscErrorCode  MatlabEngineGet_SeqAIJ(PetscObject obj,void *mengine)
 {
   PetscErrorCode ierr;
@@ -119,8 +100,6 @@ PETSC_EXTERN PetscErrorCode  MatlabEngineGet_SeqAIJ(PetscObject obj,void *mengin
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_Matlab"
 PetscErrorCode MatSolve_Matlab(Mat A,Vec b,Vec x)
 {
   PetscErrorCode ierr;
@@ -142,8 +121,6 @@ PetscErrorCode MatSolve_Matlab(Mat A,Vec b,Vec x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatLUFactorNumeric_Matlab"
 PetscErrorCode MatLUFactorNumeric_Matlab(Mat F,Mat A,const MatFactorInfo *info)
 {
   PetscErrorCode ierr;
@@ -153,6 +130,7 @@ PetscErrorCode MatLUFactorNumeric_Matlab(Mat F,Mat A,const MatFactorInfo *info)
 
   PetscFunctionBegin;
   if (F->factortype == MAT_FACTOR_ILU || info->dt > 0) {
+    /* the ILU form is not currently registered */
     if (info->dtcol == PETSC_DEFAULT) dtcol = .01;
     F->ops->solve = MatSolve_Matlab;
     F->factortype = MAT_FACTOR_LU;
@@ -184,8 +162,6 @@ PetscErrorCode MatLUFactorNumeric_Matlab(Mat F,Mat A,const MatFactorInfo *info)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatLUFactorSymbolic_Matlab"
 PetscErrorCode MatLUFactorSymbolic_Matlab(Mat F,Mat A,IS r,IS c,const MatFactorInfo *info)
 {
   PetscFunctionBegin;
@@ -195,17 +171,24 @@ PetscErrorCode MatLUFactorSymbolic_Matlab(Mat F,Mat A,IS r,IS c,const MatFactorI
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFactorGetSolverPackage_seqaij_matlab"
-PetscErrorCode MatFactorGetSolverPackage_seqaij_matlab(Mat A,const MatSolverPackage *type)
+PetscErrorCode MatFactorGetSolverType_seqaij_matlab(Mat A,MatSolverType *type)
 {
   PetscFunctionBegin;
   *type = MATSOLVERMATLAB;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatGetFactor_seqaij_matlab"
+PetscErrorCode MatDestroy_matlab(Mat A)
+{
+  PetscErrorCode ierr;
+  const char     *_A;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectGetName((PetscObject)A,&_A);CHKERRQ(ierr);
+  ierr = PetscMatlabEngineEvaluate(PETSC_MATLAB_ENGINE_(PetscObjectComm((PetscObject)A)),"delete %s l_%s u_%s;",_A,_A,_A);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 PETSC_EXTERN PetscErrorCode MatGetFactor_seqaij_matlab(Mat A,MatFactorType ftype,Mat *F)
 {
   PetscErrorCode ierr;
@@ -214,34 +197,35 @@ PETSC_EXTERN PetscErrorCode MatGetFactor_seqaij_matlab(Mat A,MatFactorType ftype
   if (A->cmap->N != A->rmap->N) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"matrix must be square");
   ierr                         = MatCreate(PetscObjectComm((PetscObject)A),F);CHKERRQ(ierr);
   ierr                         = MatSetSizes(*F,A->rmap->n,A->cmap->n,A->rmap->n,A->cmap->n);CHKERRQ(ierr);
-  ierr                         = MatSetType(*F,((PetscObject)A)->type_name);CHKERRQ(ierr);
-  ierr                         = MatSeqAIJSetPreallocation(*F,0,NULL);CHKERRQ(ierr);
+  ierr                         = PetscStrallocpy("matlab",&((PetscObject)*F)->type_name);CHKERRQ(ierr);
+  ierr                         = MatSetUp(*F);CHKERRQ(ierr);
+
+  (*F)->ops->destroy           = MatDestroy_matlab;
+  (*F)->ops->getinfo           = MatGetInfo_External;  
   (*F)->ops->lufactorsymbolic  = MatLUFactorSymbolic_Matlab;
   (*F)->ops->ilufactorsymbolic = MatLUFactorSymbolic_Matlab;
 
-  ierr = PetscObjectComposeFunction((PetscObject)(*F),"MatFactorGetSolverPackage_C",MatFactorGetSolverPackage_seqaij_matlab);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)(*F),"MatFactorGetSolverType_C",MatFactorGetSolverType_seqaij_matlab);CHKERRQ(ierr);
 
   (*F)->factortype = ftype;
+  ierr = PetscFree((*F)->solvertype);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(MATSOLVERMATLAB,&(*F)->solvertype);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolverPackageRegister_Matlab"
-PETSC_EXTERN PetscErrorCode MatSolverPackageRegister_Matlab(void)
+PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_Matlab(void)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = MatSolverPackageRegister(MATSOLVERMATLAB,MATSEQAIJ,        MAT_FACTOR_LU,MatGetFactor_seqaij_matlab);CHKERRQ(ierr);
+  ierr = MatSolverTypeRegister(MATSOLVERMATLAB,MATSEQAIJ,        MAT_FACTOR_LU,MatGetFactor_seqaij_matlab);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 /* --------------------------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFactorInfo_Matlab"
-PetscErrorCode MatFactorInfo_Matlab(Mat A,PetscViewer viewer)
+PetscErrorCode MatView_Info_Matlab(Mat A,PetscViewer viewer)
 {
   PetscErrorCode ierr;
 
@@ -250,8 +234,6 @@ PetscErrorCode MatFactorInfo_Matlab(Mat A,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatView_Matlab"
 PetscErrorCode MatView_Matlab(Mat A,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -264,7 +246,7 @@ PetscErrorCode MatView_Matlab(Mat A,PetscViewer viewer)
   if (iascii) {
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     if (format == PETSC_VIEWER_ASCII_FACTOR_INFO) {
-      ierr = MatFactorInfo_Matlab(A,viewer);
+      ierr = MatView_Info_Matlab(A,viewer);
     }
   }
   PetscFunctionReturn(0);
@@ -272,20 +254,21 @@ PetscErrorCode MatView_Matlab(Mat A,PetscViewer viewer)
 
 
 /*MC
-  MATSOLVERMATLAB - "matlab" - Providing direct solvers (LU and QR) and drop tolerance
-  based ILU factorization (ILUDT) for sequential matrices via the external package MATLAB.
+  MATSOLVERMATLAB - "matlab" - Providing direct solver LU for sequential aij matrix via the external package MATLAB.
 
 
   Works with MATSEQAIJ matrices.
 
   Options Database Keys:
-. -pc_factor_mat_solver_package matlab - selects MATLAB to do the sparse factorization
+. -pc_factor_mat_solver_type matlab - selects MATLAB to do the sparse factorization
 
+  Notes:
+    You must ./configure with the options --with-matlab --with-matlab-engine
 
   Level: beginner
 
 .seealso: PCLU
 
-.seealso: PCFactorSetMatSolverPackage(), MatSolverPackage
+.seealso: PCFactorSetMatSolverType(), MatSolverType
 M*/
 
