@@ -1,154 +1,170 @@
 
-/*
-  Defines matrix-matrix product routines for pairs of SeqAIJ matrices
-          C = A * B
-*/
+ /*
+   Defines matrix-matrix product routines for pairs of SeqAIJ matrices
+           C = A * B
+ */
 
-#include <../src/mat/impls/aij/seq/aij.h> /*I "petscmat.h" I*/
-#include <../src/mat/utils/freespace.h>
-#include <petscbt.h>
-#include <petsc/private/isimpl.h>
-#include <../src/mat/impls/dense/seq/dense.h>
+ #include <../src/mat/impls/aij/seq/aij.h> /*I "petscmat.h" I*/
+ #include <../src/mat/utils/freespace.h>
+ #include <petscbt.h>
+ #include <petsc/private/isimpl.h>
+ #include <../src/mat/impls/dense/seq/dense.h>
 
-static PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_LLCondensed(Mat,Mat,PetscReal,Mat*);
 
-#if defined(PETSC_HAVE_HYPRE)
-PETSC_INTERN PetscErrorCode MatMatMultSymbolic_AIJ_AIJ_wHYPRE(Mat,Mat,PetscReal,Mat*);
-#endif
+ PETSC_INTERN PetscErrorCode MatMatMult_SeqAIJ_SeqAIJ(Mat A,Mat B,MatReuse scall,PetscReal fill,Mat *C)
+ {
+   PetscErrorCode ierr;
 
-PETSC_INTERN PetscErrorCode MatMatMult_SeqAIJ_SeqAIJ(Mat A,Mat B,MatReuse scall,PetscReal fill,Mat *C)
-{
-  PetscErrorCode ierr;
-#if !defined(PETSC_HAVE_HYPRE)
-  const char     *algTypes[7] = {"sorted","scalable","scalable_fast","heap","btheap","llcondensed","combined"};
-  PetscInt       nalg = 7;
-#else
-  const char     *algTypes[8] = {"sorted","scalable","scalable_fast","heap","btheap","llcondensed","combined","hypre"};
-  PetscInt       nalg = 8;
-#endif
-  PetscInt       alg = 0; /* set default algorithm */
-  PetscBool      combined = PETSC_FALSE;  /* Indicates whether the symbolic stage already computed the numerical values. */
+   PetscFunctionBegin;
+   if (scall == MAT_INITIAL_MATRIX) {
+     ierr = PetscLogEventBegin(MAT_MatMultSymbolic,A,B,0,0);CHKERRQ(ierr);
+     ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ(A,B,fill,C);CHKERRQ(ierr);
+     ierr = PetscLogEventEnd(MAT_MatMultSymbolic,A,B,0,0);CHKERRQ(ierr);
+   }
 
-  PetscFunctionBegin;
-  if (scall == MAT_INITIAL_MATRIX) {
-    ierr = PetscObjectOptionsBegin((PetscObject)A);CHKERRQ(ierr);
-    PetscOptionsObject->alreadyprinted = PETSC_FALSE; /* a hack to ensure the option shows in '-help' */
-    ierr = PetscOptionsEList("-matmatmult_via","Algorithmic approach","MatMatMult",algTypes,nalg,algTypes[0],&alg,NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsEnd();CHKERRQ(ierr);
-    ierr = PetscLogEventBegin(MAT_MatMultSymbolic,A,B,0,0);CHKERRQ(ierr);
-    switch (alg) {
-    case 1:
-      ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable(A,B,fill,C);CHKERRQ(ierr);
-      break;
-    case 2:
-      ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable_fast(A,B,fill,C);CHKERRQ(ierr);
-      break;
-    case 3:
-      ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ_Heap(A,B,fill,C);CHKERRQ(ierr);
-      break;
-    case 4:
-      ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ_BTHeap(A,B,fill,C);CHKERRQ(ierr);
-      break;
-    case 5:
-      ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ_LLCondensed(A,B,fill,C);CHKERRQ(ierr);
-      break;
-    case 6:
-      ierr = MatMatMult_SeqAIJ_SeqAIJ_Combined(A,B,fill,C);CHKERRQ(ierr);
-      combined = PETSC_TRUE;
-      break;
-#if defined(PETSC_HAVE_HYPRE)
-    case 7:
-      ierr = MatMatMultSymbolic_AIJ_AIJ_wHYPRE(A,B,fill,C);CHKERRQ(ierr);
-      break;
-#endif
-    default:
-      ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ(A,B,fill,C);CHKERRQ(ierr);
+   ierr = PetscLogEventBegin(MAT_MatMultNumeric,A,B,0,0);CHKERRQ(ierr);
+   ierr = MatMatMultNumeric_SeqAIJ_SeqAIJ(A,B,*C);CHKERRQ(ierr);
+   ierr = PetscLogEventEnd(MAT_MatMultNumeric,A,B,0,0);CHKERRQ(ierr);
+   PetscFunctionReturn(0);
+ }
+
+ PetscErrorCode MatMatMultNumeric_SeqAIJ_SeqAIJ(Mat A,Mat B,Mat C)
+ {
+   PetscErrorCode ierr;
+
+   PetscFunctionBegin;
+   if (C->ops->matmultnumeric) {
+     ierr = (*C->ops->matmultnumeric)(A,B,C);CHKERRQ(ierr);
+   } else {
+     ierr = MatMatMultNumeric_SeqAIJ_SeqAIJ_Sorted(A,B,C);CHKERRQ(ierr);
+   }
+   PetscFunctionReturn(0);
+ }
+
+ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ(Mat A,Mat B,PetscReal fill,Mat *C)
+ {
+   PetscErrorCode ierr;
+ #if !defined(PETSC_HAVE_HYPRE)
+   const char     *algTypes[8] = {"sorted","scalable","scalable_fast","heap","btheap","llcondensed","combined","rowmerge"};
+   PetscInt       nalg = 8;
+ #else
+   const char     *algTypes[9] = {"sorted","scalable","scalable_fast","heap","btheap","llcondensed","combined","rowmerge","hypre"};
+   PetscInt       nalg = 9;
+ #endif
+   PetscInt       alg = 0; /* set default algorithm */
+
+   PetscFunctionBegin;
+   ierr = PetscOptionsBegin(PetscObjectComm((PetscObject)A),((PetscObject)A)->prefix,"MatMatMult","Mat");CHKERRQ(ierr);
+   ierr = PetscOptionsEList("-matmatmult_via","Algorithmic approach","MatMatMult",algTypes,nalg,algTypes[0],&alg,NULL);CHKERRQ(ierr);
+   ierr = PetscOptionsEnd();CHKERRQ(ierr);
+   switch (alg) {
+   case 1:
+     ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable(A,B,fill,C);CHKERRQ(ierr);
      break;
-    }
-    ierr = PetscLogEventEnd(MAT_MatMultSymbolic,A,B,0,0);CHKERRQ(ierr);
-  }
+   case 2:
+     ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable_fast(A,B,fill,C);CHKERRQ(ierr);
+     break;
+   case 3:
+     ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ_Heap(A,B,fill,C);CHKERRQ(ierr);
+     break;
+   case 4:
+     ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ_BTHeap(A,B,fill,C);CHKERRQ(ierr);
+     break;
+   case 5:
+     ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ_LLCondensed(A,B,fill,C);CHKERRQ(ierr);
+     break;
+   case 6:
+     ierr = MatMatMult_SeqAIJ_SeqAIJ_Combined(A,B,fill,C);CHKERRQ(ierr);
+     break;
+   case 7:
+     ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ_RowMerge(A,B,fill,C);CHKERRQ(ierr);
+     break;
+ #if defined(PETSC_HAVE_HYPRE)
+   case 8:
+     ierr = MatMatMultSymbolic_AIJ_AIJ_wHYPRE(A,B,fill,C);CHKERRQ(ierr);
+     break;
+ #endif
+   default:
+     ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ_Sorted(A,B,fill,C);CHKERRQ(ierr);
+     break;
+   }
+   PetscFunctionReturn(0);
+ }
 
-  ierr = PetscLogEventBegin(MAT_MatMultNumeric,A,B,0,0);CHKERRQ(ierr);
-  if (!combined) {
-    ierr = (*(*C)->ops->matmultnumeric)(A,B,*C);CHKERRQ(ierr);
-  }
-  ierr = PetscLogEventEnd(MAT_MatMultNumeric,A,B,0,0);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
+ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_LLCondensed(Mat A,Mat B,PetscReal fill,Mat *C)
+ {
+   PetscErrorCode     ierr;
+   Mat_SeqAIJ         *a =(Mat_SeqAIJ*)A->data,*b=(Mat_SeqAIJ*)B->data,*c;
+   PetscInt           *ai=a->i,*bi=b->i,*ci,*cj;
+   PetscInt           am =A->rmap->N,bn=B->cmap->N,bm=B->rmap->N;
+   PetscReal          afill;
+   PetscInt           i,j,anzi,brow,bnzj,cnzi,*bj,*aj,*lnk,ndouble=0,Crmax;
+   PetscTable         ta;
+   PetscBT            lnkbt;
+   PetscFreeSpaceList free_space=NULL,current_space=NULL;
 
-static PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_LLCondensed(Mat A,Mat B,PetscReal fill,Mat *C)
-{
-  PetscErrorCode     ierr;
-  Mat_SeqAIJ         *a =(Mat_SeqAIJ*)A->data,*b=(Mat_SeqAIJ*)B->data,*c;
-  PetscInt           *ai=a->i,*bi=b->i,*ci,*cj;
-  PetscInt           am =A->rmap->N,bn=B->cmap->N,bm=B->rmap->N;
-  PetscReal          afill;
-  PetscInt           i,j,anzi,brow,bnzj,cnzi,*bj,*aj,*lnk,ndouble=0,Crmax;
-  PetscTable         ta;
-  PetscBT            lnkbt;
-  PetscFreeSpaceList free_space=NULL,current_space=NULL;
+   PetscFunctionBegin;
+   /* Get ci and cj */
+   /*---------------*/
+   /* Allocate ci array, arrays for fill computation and */
+   /* free space for accumulating nonzero column info */
+   ierr  = PetscMalloc1(am+2,&ci);CHKERRQ(ierr);
+   ci[0] = 0;
 
-  PetscFunctionBegin;
-  /* Get ci and cj */
-  /*---------------*/
-  /* Allocate ci array, arrays for fill computation and */
-  /* free space for accumulating nonzero column info */
-  ierr  = PetscMalloc1(am+2,&ci);CHKERRQ(ierr);
-  ci[0] = 0;
+   /* create and initialize a linked list */
+   ierr = PetscTableCreate(bn,bn,&ta);CHKERRQ(ierr); 
+   MatRowMergeMax_SeqAIJ(b,bm,ta);
+   ierr = PetscTableGetCount(ta,&Crmax);CHKERRQ(ierr);
+   ierr = PetscTableDestroy(&ta);CHKERRQ(ierr);
 
-  /* create and initialize a linked list */
-  ierr = PetscTableCreate(bn,bn,&ta);CHKERRQ(ierr); 
-  MatRowMergeMax_SeqAIJ(b,bm,ta);
-  ierr = PetscTableGetCount(ta,&Crmax);CHKERRQ(ierr);
-  ierr = PetscTableDestroy(&ta);CHKERRQ(ierr);
+   ierr = PetscLLCondensedCreate(Crmax,bn,&lnk,&lnkbt);CHKERRQ(ierr);
 
-  ierr = PetscLLCondensedCreate(Crmax,bn,&lnk,&lnkbt);CHKERRQ(ierr);
+   /* Initial FreeSpace size is fill*(nnz(A)+nnz(B)) */
+   ierr = PetscFreeSpaceGet(PetscRealIntMultTruncate(fill,PetscIntSumTruncate(ai[am],bi[bm])),&free_space);CHKERRQ(ierr);
 
-  /* Initial FreeSpace size is fill*(nnz(A)+nnz(B)) */
-  ierr = PetscFreeSpaceGet(PetscRealIntMultTruncate(fill,PetscIntSumTruncate(ai[am],bi[bm])),&free_space);CHKERRQ(ierr);
+   current_space = free_space;
 
-  current_space = free_space;
+   /* Determine ci and cj */
+   for (i=0; i<am; i++) {
+     anzi = ai[i+1] - ai[i];
+     aj   = a->j + ai[i];
+     for (j=0; j<anzi; j++) {
+       brow = aj[j];
+       bnzj = bi[brow+1] - bi[brow];
+       bj   = b->j + bi[brow];
+       /* add non-zero cols of B into the sorted linked list lnk */
+       ierr = PetscLLCondensedAddSorted(bnzj,bj,lnk,lnkbt);CHKERRQ(ierr);
+     }
+     cnzi = lnk[0];
 
-  /* Determine ci and cj */
-  for (i=0; i<am; i++) {
-    anzi = ai[i+1] - ai[i];
-    aj   = a->j + ai[i];
-    for (j=0; j<anzi; j++) {
-      brow = aj[j];
-      bnzj = bi[brow+1] - bi[brow];
-      bj   = b->j + bi[brow];
-      /* add non-zero cols of B into the sorted linked list lnk */
-      ierr = PetscLLCondensedAddSorted(bnzj,bj,lnk,lnkbt);CHKERRQ(ierr);
-    }
-    cnzi = lnk[0];
+     /* If free space is not available, make more free space */
+     /* Double the amount of total space in the list */
+     if (current_space->local_remaining<cnzi) {
+       ierr = PetscFreeSpaceGet(PetscIntSumTruncate(cnzi,current_space->total_array_size),&current_space);CHKERRQ(ierr);
+       ndouble++;
+     }
 
-    /* If free space is not available, make more free space */
-    /* Double the amount of total space in the list */
-    if (current_space->local_remaining<cnzi) {
-      ierr = PetscFreeSpaceGet(PetscIntSumTruncate(cnzi,current_space->total_array_size),&current_space);CHKERRQ(ierr);
-      ndouble++;
-    }
+     /* Copy data into free space, then initialize lnk */
+     ierr = PetscLLCondensedClean(bn,cnzi,current_space->array,lnk,lnkbt);CHKERRQ(ierr);
 
-    /* Copy data into free space, then initialize lnk */
-    ierr = PetscLLCondensedClean(bn,cnzi,current_space->array,lnk,lnkbt);CHKERRQ(ierr);
+     current_space->array           += cnzi;
+     current_space->local_used      += cnzi;
+     current_space->local_remaining -= cnzi;
 
-    current_space->array           += cnzi;
-    current_space->local_used      += cnzi;
-    current_space->local_remaining -= cnzi;
+     ci[i+1] = ci[i] + cnzi;
+   }
 
-    ci[i+1] = ci[i] + cnzi;
-  }
+   /* Column indices are in the list of free space */
+   /* Allocate space for cj, initialize cj, and */
+   /* destroy list of free space and other temporary array(s) */
+   ierr = PetscMalloc1(ci[am]+1,&cj);CHKERRQ(ierr);
+   ierr = PetscFreeSpaceContiguous(&free_space,cj);CHKERRQ(ierr);
+   ierr = PetscLLCondensedDestroy(lnk,lnkbt);CHKERRQ(ierr);
 
-  /* Column indices are in the list of free space */
-  /* Allocate space for cj, initialize cj, and */
-  /* destroy list of free space and other temporary array(s) */
-  ierr = PetscMalloc1(ci[am]+1,&cj);CHKERRQ(ierr);
-  ierr = PetscFreeSpaceContiguous(&free_space,cj);CHKERRQ(ierr);
-  ierr = PetscLLCondensedDestroy(lnk,lnkbt);CHKERRQ(ierr);
-
-  /* put together the new symbolic matrix */
-  ierr = MatCreateSeqAIJWithArrays(PetscObjectComm((PetscObject)A),am,bn,ci,cj,NULL,C);CHKERRQ(ierr);
-  ierr = MatSetBlockSizesFromMats(*C,A,B);CHKERRQ(ierr);
+   /* put together the new symbolic matrix */
+   ierr = MatCreateSeqAIJWithArrays(PetscObjectComm((PetscObject)A),am,bn,ci,cj,NULL,C);CHKERRQ(ierr);
+   ierr = MatSetBlockSizesFromMats(*C,A,B);CHKERRQ(ierr);
+   ierr = MatSetType(*C,((PetscObject)A)->type_name);CHKERRQ(ierr);
 
   /* MatCreateSeqAIJWithArrays flags matrix so PETSc doesn't free the user's arrays. */
   /* These are PETSc arrays, so change flags so arrays can be deleted by PETSc */
@@ -156,7 +172,7 @@ static PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_LLCondensed(Mat A,Mat B,P
   c->free_a                 = PETSC_FALSE;
   c->free_ij                = PETSC_TRUE;
   c->nonew                  = 0;
-  (*C)->ops->matmultnumeric = MatMatMultNumeric_SeqAIJ_SeqAIJ; /* fast, needs non-scalable O(bn) array 'abdense' */
+  (*C)->ops->matmultnumeric = MatMatMultNumeric_SeqAIJ_SeqAIJ_Sorted; /* fast, needs non-scalable O(bn) array 'abdense' */
 
   /* set MatInfo */
   afill = (PetscReal)ci[am]/(ai[am]+bi[bm]) + 1.e-5;
@@ -178,7 +194,7 @@ static PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_LLCondensed(Mat A,Mat B,P
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode MatMatMultNumeric_SeqAIJ_SeqAIJ(Mat A,Mat B,Mat C)
+PetscErrorCode MatMatMultNumeric_SeqAIJ_SeqAIJ_Sorted(Mat A,Mat B,Mat C)
 {
   PetscErrorCode ierr;
   PetscLogDouble flops=0.0;
@@ -369,6 +385,7 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable_fast(Mat A,Mat B,PetscR
   /* put together the new symbolic matrix */
   ierr = MatCreateSeqAIJWithArrays(PetscObjectComm((PetscObject)A),am,bn,ci,cj,ca,C);CHKERRQ(ierr);
   ierr = MatSetBlockSizesFromMats(*C,A,B);CHKERRQ(ierr);
+  ierr = MatSetType(*C,((PetscObject)A)->type_name);CHKERRQ(ierr);
 
   /* MatCreateSeqAIJWithArrays flags matrix so PETSc doesn't free the user's arrays. */
   /* These are PETSc arrays, so change flags so arrays can be deleted by PETSc */
@@ -475,6 +492,7 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable(Mat A,Mat B,PetscReal f
   /* put together the new symbolic matrix */
   ierr = MatCreateSeqAIJWithArrays(PetscObjectComm((PetscObject)A),am,bn,ci,cj,ca,C);CHKERRQ(ierr);
   ierr = MatSetBlockSizesFromMats(*C,A,B);CHKERRQ(ierr);
+  ierr = MatSetType(*C,((PetscObject)A)->type_name);CHKERRQ(ierr);
 
   /* MatCreateSeqAIJWithArrays flags matrix so PETSc doesn't free the user's arrays. */
   /* These are PETSc arrays, so change flags so arrays can be deleted by PETSc */
@@ -581,6 +599,7 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Heap(Mat A,Mat B,PetscReal fill,
   /* put together the new symbolic matrix */
   ierr = MatCreateSeqAIJWithArrays(PetscObjectComm((PetscObject)A),am,bn,ci,cj,NULL,C);CHKERRQ(ierr);
   ierr = MatSetBlockSizesFromMats(*C,A,B);CHKERRQ(ierr);
+  ierr = MatSetType(*C,((PetscObject)A)->type_name);CHKERRQ(ierr);
 
   /* MatCreateSeqAIJWithArrays flags matrix so PETSc doesn't free the user's arrays. */
   /* These are PETSc arrays, so change flags so arrays can be deleted by PETSc */
@@ -589,7 +608,7 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Heap(Mat A,Mat B,PetscReal fill,
   c->free_ij = PETSC_TRUE;
   c->nonew   = 0;
 
-  (*C)->ops->matmultnumeric = MatMatMultNumeric_SeqAIJ_SeqAIJ;
+  (*C)->ops->matmultnumeric = MatMatMultNumeric_SeqAIJ_SeqAIJ_Sorted;
 
   /* set MatInfo */
   afill = (PetscReal)ci[am]/(ai[am]+bi[bm]) + 1.e-5;
@@ -701,6 +720,7 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_BTHeap(Mat A,Mat B,PetscReal fil
   /* put together the new symbolic matrix */
   ierr = MatCreateSeqAIJWithArrays(PetscObjectComm((PetscObject)A),am,bn,ci,cj,NULL,C);CHKERRQ(ierr);
   ierr = MatSetBlockSizesFromMats(*C,A,B);CHKERRQ(ierr);
+  ierr = MatSetType(*C,((PetscObject)A)->type_name);CHKERRQ(ierr);
 
   /* MatCreateSeqAIJWithArrays flags matrix so PETSc doesn't free the user's arrays. */
   /* These are PETSc arrays, so change flags so arrays can be deleted by PETSc */
@@ -709,7 +729,7 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_BTHeap(Mat A,Mat B,PetscReal fil
   c->free_ij = PETSC_TRUE;
   c->nonew   = 0;
 
-  (*C)->ops->matmultnumeric = MatMatMultNumeric_SeqAIJ_SeqAIJ;
+  (*C)->ops->matmultnumeric = MatMatMultNumeric_SeqAIJ_SeqAIJ_Sorted;
 
   /* set MatInfo */
   afill = (PetscReal)ci[am]/(ai[am]+bi[bm]) + 1.e-5;
@@ -731,8 +751,273 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_BTHeap(Mat A,Mat B,PetscReal fil
   PetscFunctionReturn(0);
 }
 
+
+PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_RowMerge(Mat A,Mat B,PetscReal fill,Mat *C)
+{
+  PetscErrorCode     ierr;
+  Mat_SeqAIJ         *a=(Mat_SeqAIJ*)A->data,*b=(Mat_SeqAIJ*)B->data,*c;
+  const PetscInt     *ai=a->i,*bi=b->i,*aj=a->j,*bj=b->j,*inputi,*inputj,*inputcol,*inputcol_L1;
+  PetscInt           *ci,*cj,*outputj,worki_L1[9],worki_L2[9];
+  PetscInt           c_maxmem,a_maxrownnz=0,a_rownnz;
+  const PetscInt     workcol[8]={0,1,2,3,4,5,6,7};
+  const PetscInt     am=A->rmap->N,bn=B->cmap->N,bm=B->rmap->N;
+  const PetscInt     *brow_ptr[8],*brow_end[8];
+  PetscInt           window[8];
+  PetscInt           window_min,old_window_min,ci_nnz,outputi_nnz=0,L1_nrows,L2_nrows;
+  PetscInt           i,k,ndouble=0,L1_rowsleft,rowsleft;
+  PetscReal          afill;
+  PetscInt           *workj_L1,*workj_L2,*workj_L3;
+  PetscInt           L1_nnz,L2_nnz;
+
+  /* Step 1: Get upper bound on memory required for allocation.
+             Because of the way virtual memory works,
+             only the memory pages that are actually needed will be physically allocated. */
+  PetscFunctionBegin;
+  ierr = PetscMalloc1(am+1,&ci);CHKERRQ(ierr);
+  for (i=0; i<am; i++) {
+    const PetscInt anzi  = ai[i+1] - ai[i]; /* number of nonzeros in this row of A, this is the number of rows of B that we merge */
+    const PetscInt *acol = aj + ai[i]; /* column indices of nonzero entries in this row */
+    a_rownnz = 0;
+    for (k=0; k<anzi; ++k) {
+      a_rownnz += bi[acol[k]+1] - bi[acol[k]];
+      if (a_rownnz > bn) {
+        a_rownnz = bn;
+        break;
+      }
+    }
+    a_maxrownnz = PetscMax(a_maxrownnz, a_rownnz);
+  }
+  /* temporary work areas for merging rows */
+  ierr = PetscMalloc1(a_maxrownnz*8,&workj_L1);CHKERRQ(ierr);
+  ierr = PetscMalloc1(a_maxrownnz*8,&workj_L2);CHKERRQ(ierr);
+  ierr = PetscMalloc1(a_maxrownnz,&workj_L3);CHKERRQ(ierr);
+
+  /* This should be enough for almost all matrices. If not, memory is reallocated later. */
+  c_maxmem = 8*(ai[am]+bi[bm]);
+  /* Step 2: Populate pattern for C */
+  ierr  = PetscMalloc1(c_maxmem,&cj);CHKERRQ(ierr);
+
+  ci_nnz       = 0;
+  ci[0]        = 0;
+  worki_L1[0]  = 0;
+  worki_L2[0]  = 0;
+  for (i=0; i<am; i++) {
+    const PetscInt anzi  = ai[i+1] - ai[i]; /* number of nonzeros in this row of A, this is the number of rows of B that we merge */
+    const PetscInt *acol = aj + ai[i];      /* column indices of nonzero entries in this row */
+    rowsleft             = anzi;
+    inputcol_L1          = acol;
+    L2_nnz               = 0;
+    L2_nrows             = 1;  /* Number of rows to be merged on Level 3. output of L3 already exists -> initial value 1   */
+    worki_L2[1]          = 0;
+    outputi_nnz          = 0;
+
+    /* If the number of indices in C so far + the max number of columns in the next row > c_maxmem  -> allocate more memory */
+    while (ci_nnz+a_maxrownnz > c_maxmem) {
+      c_maxmem *= 2;
+      ndouble++;
+      ierr = PetscRealloc(sizeof(PetscInt)*c_maxmem,&cj);CHKERRQ(ierr);
+    }
+
+    while (rowsleft) {
+      L1_rowsleft = PetscMin(64, rowsleft); /* In the inner loop max 64 rows of B can be merged */
+      L1_nrows    = 0;
+      L1_nnz      = 0;
+      inputcol    = inputcol_L1;
+      inputi      = bi;
+      inputj      = bj;
+
+      /* The following macro is used to specialize for small rows in A.
+         This helps with compiler unrolling, improving performance substantially.
+          Input:  inputj   inputi  inputcol  bn
+          Output: outputj  outputi_nnz                       */
+       #define MatMatMultSymbolic_RowMergeMacro(ANNZ)                        \
+         window_min  = bn;                                                   \
+         outputi_nnz = 0;                                                    \
+         for (k=0; k<ANNZ; ++k) {                                            \
+           brow_ptr[k] = inputj + inputi[inputcol[k]];                       \
+           brow_end[k] = inputj + inputi[inputcol[k]+1];                     \
+           window[k]   = (brow_ptr[k] != brow_end[k]) ? *brow_ptr[k] : bn;   \
+           window_min  = PetscMin(window[k], window_min);                    \
+         }                                                                   \
+         while (window_min < bn) {                                           \
+           outputj[outputi_nnz++] = window_min;                              \
+           /* advance front and compute new minimum */                       \
+           old_window_min = window_min;                                      \
+           window_min = bn;                                                  \
+           for (k=0; k<ANNZ; ++k) {                                          \
+             if (window[k] == old_window_min) {                              \
+               brow_ptr[k]++;                                                \
+               window[k] = (brow_ptr[k] != brow_end[k]) ? *brow_ptr[k] : bn; \
+             }                                                               \
+             window_min = PetscMin(window[k], window_min);                   \
+           }                                                                 \
+         }
+
+      /************** L E V E L  1 ***************/
+      /* Merge up to 8 rows of B to L1 work array*/
+      while (L1_rowsleft) {
+        outputi_nnz = 0;
+        if (anzi > 8)  outputj = workj_L1 + L1_nnz;     /* Level 1 rowmerge*/
+        else           outputj = cj + ci_nnz;           /* Merge directly to C */
+
+        switch (L1_rowsleft) {
+        case 1:  brow_ptr[0] = inputj + inputi[inputcol[0]];
+                 brow_end[0] = inputj + inputi[inputcol[0]+1];
+                 for (; brow_ptr[0] != brow_end[0]; ++brow_ptr[0]) outputj[outputi_nnz++] = *brow_ptr[0]; /* copy row in b over */
+                 inputcol    += L1_rowsleft;
+                 rowsleft    -= L1_rowsleft;
+                 L1_rowsleft  = 0;
+                 break;
+        case 2:  MatMatMultSymbolic_RowMergeMacro(2);
+                 inputcol    += L1_rowsleft;
+                 rowsleft    -= L1_rowsleft;
+                 L1_rowsleft  = 0;
+                 break;
+        case 3: MatMatMultSymbolic_RowMergeMacro(3);
+                 inputcol    += L1_rowsleft;
+                 rowsleft    -= L1_rowsleft;
+                 L1_rowsleft  = 0;
+                 break;
+        case 4:  MatMatMultSymbolic_RowMergeMacro(4);
+                 inputcol    += L1_rowsleft;
+                 rowsleft    -= L1_rowsleft;
+                 L1_rowsleft  = 0;
+                 break;
+        case 5:  MatMatMultSymbolic_RowMergeMacro(5);
+                 inputcol    += L1_rowsleft;
+                 rowsleft    -= L1_rowsleft;
+                 L1_rowsleft  = 0;
+                 break;
+        case 6:  MatMatMultSymbolic_RowMergeMacro(6);
+                 inputcol    += L1_rowsleft;
+                 rowsleft    -= L1_rowsleft;
+                 L1_rowsleft  = 0;
+                 break;
+        case 7:  MatMatMultSymbolic_RowMergeMacro(7);
+                 inputcol    += L1_rowsleft;
+                 rowsleft    -= L1_rowsleft;
+                 L1_rowsleft  = 0;
+                 break;
+        default: MatMatMultSymbolic_RowMergeMacro(8);
+                 inputcol    += 8;
+                 rowsleft    -= 8;
+                 L1_rowsleft -= 8;
+                 break;
+        }
+        inputcol_L1           = inputcol;
+        L1_nnz               += outputi_nnz;
+        worki_L1[++L1_nrows]  = L1_nnz;
+      }
+
+      /********************** L E V E L  2 ************************/
+      /* Merge from L1 work array to either C or to L2 work array */
+      if (anzi > 8) {
+        inputi      = worki_L1;
+        inputj      = workj_L1;
+        inputcol    = workcol;
+        outputi_nnz = 0;
+
+        if (anzi <= 64) outputj = cj + ci_nnz;        /* Merge from L1 work array to C */
+        else            outputj = workj_L2 + L2_nnz;  /* Merge from L1 work array to L2 work array */
+
+        switch (L1_nrows) {
+        case 1:  brow_ptr[0] = inputj + inputi[inputcol[0]];
+                 brow_end[0] = inputj + inputi[inputcol[0]+1];
+                 for (; brow_ptr[0] != brow_end[0]; ++brow_ptr[0]) outputj[outputi_nnz++] = *brow_ptr[0]; /* copy row in b over */
+                 break;
+        case 2:  MatMatMultSymbolic_RowMergeMacro(2); break;
+        case 3:  MatMatMultSymbolic_RowMergeMacro(3); break;
+        case 4:  MatMatMultSymbolic_RowMergeMacro(4); break;
+        case 5:  MatMatMultSymbolic_RowMergeMacro(5); break;
+        case 6:  MatMatMultSymbolic_RowMergeMacro(6); break;
+        case 7:  MatMatMultSymbolic_RowMergeMacro(7); break;
+        case 8:  MatMatMultSymbolic_RowMergeMacro(8); break;
+        default: SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"MatMatMult logic error: Not merging 1-8 rows from L1 work array!");
+        }
+        L2_nnz               += outputi_nnz;
+        worki_L2[++L2_nrows]  = L2_nnz;
+
+        /************************ L E V E L  3 **********************/
+        /* Merge from L2 work array to either C or to L2 work array */
+        if (anzi > 64 && (L2_nrows == 8 || rowsleft == 0)) {
+          inputi      = worki_L2;
+          inputj      = workj_L2;
+          inputcol    = workcol;
+          outputi_nnz = 0;
+          if (rowsleft) outputj = workj_L3;
+          else          outputj = cj + ci_nnz;
+          switch (L2_nrows) {
+          case 1:  brow_ptr[0] = inputj + inputi[inputcol[0]];
+                   brow_end[0] = inputj + inputi[inputcol[0]+1];
+                   for (; brow_ptr[0] != brow_end[0]; ++brow_ptr[0]) outputj[outputi_nnz++] = *brow_ptr[0]; /* copy row in b over */
+                   break;
+          case 2:  MatMatMultSymbolic_RowMergeMacro(2); break;
+          case 3:  MatMatMultSymbolic_RowMergeMacro(3); break;
+          case 4:  MatMatMultSymbolic_RowMergeMacro(4); break;
+          case 5:  MatMatMultSymbolic_RowMergeMacro(5); break;
+          case 6:  MatMatMultSymbolic_RowMergeMacro(6); break;
+          case 7:  MatMatMultSymbolic_RowMergeMacro(7); break;
+          case 8:  MatMatMultSymbolic_RowMergeMacro(8); break;
+          default: SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"MatMatMult logic error: Not merging 1-8 rows from L2 work array!");
+          }
+          L2_nrows    = 1;
+          L2_nnz      = outputi_nnz;
+          worki_L2[1] = outputi_nnz;
+          /* Copy to workj_L2 */
+          if (rowsleft) {
+            for (k=0; k<outputi_nnz; ++k)  workj_L2[k] = outputj[k];
+          }
+        }
+      }
+    }  /* while (rowsleft) */
+#undef MatMatMultSymbolic_RowMergeMacro
+
+    /* terminate current row */
+    ci_nnz += outputi_nnz;
+    ci[i+1] = ci_nnz;
+  }
+
+  /* Step 3: Create the new symbolic matrix */
+  ierr = MatCreateSeqAIJWithArrays(PetscObjectComm((PetscObject)A),am,bn,ci,cj,NULL,C);CHKERRQ(ierr);
+  ierr = MatSetBlockSizesFromMats(*C,A,B);CHKERRQ(ierr);
+  ierr = MatSetType(*C,((PetscObject)A)->type_name);CHKERRQ(ierr);
+
+  /* MatCreateSeqAIJWithArrays flags matrix so PETSc doesn't free the user's arrays. */
+  /* These are PETSc arrays, so change flags so arrays can be deleted by PETSc */
+  c          = (Mat_SeqAIJ*)((*C)->data);
+  c->free_a  = PETSC_TRUE;
+  c->free_ij = PETSC_TRUE;
+  c->nonew   = 0;
+
+  (*C)->ops->matmultnumeric = MatMatMultNumeric_SeqAIJ_SeqAIJ_Sorted;
+
+  /* set MatInfo */
+  afill = (PetscReal)ci[am]/(ai[am]+bi[bm]) + 1.e-5;
+  if (afill < 1.0) afill = 1.0;
+  c->maxnz                     = ci[am];
+  c->nz                        = ci[am];
+  (*C)->info.mallocs           = ndouble;
+  (*C)->info.fill_ratio_given  = fill;
+  (*C)->info.fill_ratio_needed = afill;
+
+#if defined(PETSC_USE_INFO)
+  if (ci[am]) {
+    ierr = PetscInfo3((*C),"Reallocs %D; Fill ratio: given %g needed %g.\n",ndouble,(double)fill,(double)afill);CHKERRQ(ierr);
+    ierr = PetscInfo1((*C),"Use MatMatMult(A,B,MatReuse,%g,&C) for best performance.;\n",(double)afill);CHKERRQ(ierr);
+  } else {
+    ierr = PetscInfo((*C),"Empty matrix product\n");CHKERRQ(ierr);
+  }
+#endif
+
+  /* Step 4: Free temporary work areas */
+  ierr = PetscFree(workj_L1);CHKERRQ(ierr);
+  ierr = PetscFree(workj_L2);CHKERRQ(ierr);
+  ierr = PetscFree(workj_L3);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /* concatenate unique entries and then sort */
-PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ(Mat A,Mat B,PetscReal fill,Mat *C)
+PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Sorted(Mat A,Mat B,PetscReal fill,Mat *C)
 {
   PetscErrorCode     ierr;
   Mat_SeqAIJ         *a  = (Mat_SeqAIJ*)A->data,*b=(Mat_SeqAIJ*)B->data,*c;
@@ -789,6 +1074,7 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ(Mat A,Mat B,PetscReal fill,Mat *
   /* put together the new symbolic matrix */
   ierr = MatCreateSeqAIJWithArrays(PetscObjectComm((PetscObject)A),am,bn,ci,cj,NULL,C);CHKERRQ(ierr);
   ierr = MatSetBlockSizesFromMats(*C,A,B);CHKERRQ(ierr);
+  ierr = MatSetType(*C,((PetscObject)A)->type_name);CHKERRQ(ierr);
 
   /* MatCreateSeqAIJWithArrays flags matrix so PETSc doesn't free the user's arrays. */
   /* These are PETSc arrays, so change flags so arrays can be deleted by PETSc */
@@ -797,7 +1083,7 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ(Mat A,Mat B,PetscReal fill,Mat *
   c->free_ij = PETSC_TRUE;
   c->nonew   = 0;
 
-  (*C)->ops->matmultnumeric = MatMatMultNumeric_SeqAIJ_SeqAIJ;
+  (*C)->ops->matmultnumeric = MatMatMultNumeric_SeqAIJ_SeqAIJ_Sorted;
 
   /* set MatInfo */
   afill = (PetscReal)ci[am]/(ai[am]+bi[bm]) + 1.e-5;
@@ -864,6 +1150,7 @@ PetscErrorCode MatMatTransposeMultSymbolic_SeqAIJ_SeqAIJ(Mat A,Mat B,PetscReal f
   ierr = MatGetSymbolicTranspose_SeqAIJ(B,&bti,&btj);CHKERRQ(ierr);
   ierr = MatCreateSeqAIJWithArrays(PETSC_COMM_SELF,B->cmap->n,B->rmap->n,bti,btj,NULL,&Bt);CHKERRQ(ierr);
   ierr = MatSetBlockSizes(Bt,PetscAbs(A->cmap->bs),PetscAbs(B->cmap->bs));CHKERRQ(ierr);
+  ierr = MatSetType(Bt,((PetscObject)A)->type_name);CHKERRQ(ierr);
 
   /* get symbolic C=A*Bt */
   ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ(A,Bt,fill,C);CHKERRQ(ierr);
@@ -1008,8 +1295,10 @@ PetscErrorCode MatDestroy_SeqAIJ_MatTransMatMult(Mat A)
   Mat_MatTransMatMult *atb = a->atb;
 
   PetscFunctionBegin;
-  ierr = MatDestroy(&atb->At);CHKERRQ(ierr);
-  ierr = (atb->destroy)(A);CHKERRQ(ierr);
+  if (atb) {
+    ierr = MatDestroy(&atb->At);CHKERRQ(ierr);
+    ierr = (*atb->destroy)(A);CHKERRQ(ierr);
+  }
   ierr = PetscFree(atb);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -1025,8 +1314,7 @@ PetscErrorCode MatTransposeMatMult_SeqAIJ_SeqAIJ(Mat A,Mat B,MatReuse scall,Pets
 
   PetscFunctionBegin;
   if (scall == MAT_INITIAL_MATRIX) {
-    ierr = PetscObjectOptionsBegin((PetscObject)A);CHKERRQ(ierr);
-    PetscOptionsObject->alreadyprinted = PETSC_FALSE; /* a hack to ensure the option shows in '-help' */
+    ierr = PetscOptionsBegin(PetscObjectComm((PetscObject)A),((PetscObject)A)->prefix,"MatTransposeMatMult","Mat");CHKERRQ(ierr);
     ierr = PetscOptionsEList("-mattransposematmult_via","Algorithmic approach","MatTransposeMatMult",algTypes,2,algTypes[0],&alg,NULL);CHKERRQ(ierr);
     ierr = PetscOptionsEnd();CHKERRQ(ierr);
 
@@ -1071,6 +1359,7 @@ PetscErrorCode MatTransposeMatMultSymbolic_SeqAIJ_SeqAIJ(Mat A,Mat B,PetscReal f
   ierr = MatGetSymbolicTranspose_SeqAIJ(A,&ati,&atj);CHKERRQ(ierr);
   ierr = MatCreateSeqAIJWithArrays(PETSC_COMM_SELF,A->cmap->n,A->rmap->n,ati,atj,NULL,&At);CHKERRQ(ierr);
   ierr = MatSetBlockSizes(At,PetscAbs(A->cmap->bs),PetscAbs(B->cmap->bs));CHKERRQ(ierr);
+  ierr = MatSetType(At,((PetscObject)A)->type_name);CHKERRQ(ierr);
 
   /* get symbolic C=At*B */
   ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ(At,B,fill,C);CHKERRQ(ierr);
@@ -1535,12 +1824,12 @@ PetscErrorCode MatTransposeColoringCreate_SeqAIJ(Mat mat,ISColoring iscoloring,M
   PetscFunctionReturn(0);
 }
 
-/* Needed for MatMatMult_SeqAIJ_SeqAIJ_Combined() */
-/* Append value to an array if the value is not present yet. A bitarray */
-/* was used to determine if there is already an entry at this position. */
-void appendToArray(PetscInt val, PetscInt *array, PetscInt *cnzi)
+/* The combine method has done the symbolic and numeric in the first phase, and so we just return here */
+PetscErrorCode MatMatMultNumeric_SeqAIJ_SeqAIJ_Combined(Mat A,Mat B,Mat C)
 {
-  array[(*cnzi)++] = val;
+  PetscFunctionBegin;
+  /* Empty function */
+  PetscFunctionReturn(0);
 }
 
 /* This algorithm combines the symbolic and numeric phase of matrix-matrix multiplication. */
@@ -1548,83 +1837,88 @@ PetscErrorCode MatMatMult_SeqAIJ_SeqAIJ_Combined(Mat A,Mat B,PetscReal fill,Mat
 {
   PetscErrorCode     ierr;
   PetscLogDouble     flops=0.0;
-  Mat_SeqAIJ         *a  = (Mat_SeqAIJ*)A->data, *b = (Mat_SeqAIJ*)B->data, *c;
-  const PetscInt     *ai = a->i,*bi = b->i, *aj = a->j;
+  Mat_SeqAIJ         *a=(Mat_SeqAIJ*)A->data,*b=(Mat_SeqAIJ*)B->data,*c;
+  const PetscInt     *ai=a->i,*bi=b->i;
   PetscInt           *ci,*cj,*cj_i;
-  PetscScalar        *ca, *ca_i;
-  PetscInt           c_maxmem = 0, a_maxrownnz = 0, a_rownnz, a_col;
-  PetscInt           am = A->rmap->N, bn = B->cmap->N, bm = B->rmap->N;
-  PetscInt           i, k, ndouble = 0;
+  PetscScalar        *ca,*ca_i;
+  PetscInt           b_maxmemrow,c_maxmem,a_col;
+  PetscInt           am=A->rmap->N,bn=B->cmap->N,bm=B->rmap->N;
+  PetscInt           i,k,ndouble=0;
   PetscReal          afill;
   PetscScalar        *c_row_val_dense;
   PetscBool          *c_row_idx_flags;
-  PetscInt           *aj_i = a->j;
-  PetscScalar        *aa_i = a->a;
+  PetscInt           *aj_i=a->j;
+  PetscScalar        *aa_i=a->a;
 
   PetscFunctionBegin;
-  /* Step 1: Determine upper bounds on memory for C */
-  for (i=0; i<am; i++) { /* iterate over all rows of A */
-    const PetscInt anzi  = ai[i+1] - ai[i]; /* number of nonzeros in this row of A, this is the number of rows of B that we merge */
-    const PetscInt *acol = aj + ai[i]; /* column indices of nonzero entries in this row */
-    a_rownnz = 0;
-    for (k=0;k<anzi;++k) a_rownnz += bi[acol[k]+1] - bi[acol[k]];
-    a_maxrownnz = PetscMax(a_maxrownnz, a_rownnz);
-    c_maxmem += a_rownnz;
-  }
-  ierr = PetscMalloc1(am+1, &ci);               CHKERRQ(ierr);
-  ierr = PetscMalloc1(bn, &c_row_val_dense);    CHKERRQ(ierr);
-  ierr = PetscMalloc1(bn, &c_row_idx_flags);    CHKERRQ(ierr);
-  ierr = PetscMalloc1(c_maxmem,&cj);            CHKERRQ(ierr);
-  ierr = PetscMalloc1(c_maxmem,&ca);            CHKERRQ(ierr);
-  ca_i = ca;
-  cj_i = cj;
+
+  /* Step 1: Determine upper bounds on memory for C and allocate memory */
+  /* This should be enough for almost all matrices. If still more memory is needed, it is reallocated later. */
+  c_maxmem    = 8*(ai[am]+bi[bm]);
+  b_maxmemrow = PetscMin(bi[bm],bn);
+  ierr  = PetscMalloc1(am+1,&ci);CHKERRQ(ierr);
+  ierr  = PetscMalloc1(bn,&c_row_val_dense);CHKERRQ(ierr);
+  ierr  = PetscMalloc1(bn,&c_row_idx_flags);CHKERRQ(ierr);
+  ierr  = PetscMalloc1(c_maxmem,&cj);CHKERRQ(ierr);
+  ierr  = PetscMalloc1(c_maxmem,&ca);CHKERRQ(ierr);
+  ca_i  = ca;
+  cj_i  = cj;
   ci[0] = 0;
-  ierr = PetscMemzero(c_row_val_dense, bn * sizeof(PetscScalar));CHKERRQ(ierr);
-  ierr = PetscMemzero(c_row_idx_flags, bn * sizeof(PetscBool));CHKERRQ(ierr);
+  ierr  = PetscMemzero(c_row_val_dense,bn*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr  = PetscMemzero(c_row_idx_flags,bn*sizeof(PetscBool));CHKERRQ(ierr);
   for (i=0; i<am; i++) {
     /* Step 2: Initialize the dense row vector for C  */
-    const PetscInt anzi     = ai[i+1] - ai[i]; /* number of nonzeros in this row of A, this is the number of rows of B that we merge */
-    PetscInt cnzi           = 0;
-    PetscInt *bj_i;
-    PetscScalar *ba_i;
+    const PetscInt anzi = ai[i+1] - ai[i]; /* number of nonzeros in this row of A, this is the number of rows of B that we merge */
+    PetscInt       cnzi = 0;
+    PetscInt       *bj_i;
+    PetscScalar    *ba_i;
+    /* If the number of indices in C so far + the max number of columns in the next row > c_maxmem  -> allocate more memory
+       Usually, there is enough memory in the first place, so this is not executed. */
+    while (ci[i] + b_maxmemrow > c_maxmem) {
+      c_maxmem *= 2;
+      ndouble++;
+      ierr = PetscRealloc(sizeof(PetscInt)*c_maxmem,&cj);CHKERRQ(ierr);
+      ierr = PetscRealloc(sizeof(PetscScalar)*c_maxmem,&ca);CHKERRQ(ierr);
+    }
 
     /* Step 3: Do the numerical calculations */
     for (a_col=0; a_col<anzi; a_col++) {          /* iterate over all non zero values in a row of A */
-      PetscInt a_col_index = aj_i[a_col];
-      const PetscInt bnzi = bi[a_col_index+1] - bi[a_col_index];
+      PetscInt       a_col_index = aj_i[a_col];
+      const PetscInt bnzi        = bi[a_col_index+1] - bi[a_col_index];
       flops += 2*bnzi;
-      bj_i = b->j + bi[a_col_index];   /* points to the current row in bj */
-      ba_i = b->a + bi[a_col_index];   /* points to the current row in ba */
+      bj_i   = b->j + bi[a_col_index];   /* points to the current row in bj */
+      ba_i   = b->a + bi[a_col_index];   /* points to the current row in ba */
       for (k=0; k<bnzi; ++k) { /* iterate over all non zeros of this row in B */
-        if (c_row_idx_flags[ bj_i[k] ] == PETSC_FALSE) {
-          appendToArray(bj_i[k], cj_i, &cnzi);
-          c_row_idx_flags[ bj_i[k] ] = PETSC_TRUE;
+        if (c_row_idx_flags[bj_i[k]] == PETSC_FALSE) {
+          cj_i[cnzi++]             = bj_i[k];
+          c_row_idx_flags[bj_i[k]] = PETSC_TRUE;
         }
-        c_row_val_dense[ bj_i[k] ] += aa_i[a_col] * ba_i[k];
+        c_row_val_dense[bj_i[k]] += aa_i[a_col] * ba_i[k];
       }
     }
 
     /* Sort array */
-    ierr = PetscSortInt(cnzi, cj_i);CHKERRQ(ierr);
+    ierr = PetscSortInt(cnzi,cj_i);CHKERRQ(ierr);
     /* Step 4 */
-    for (k=0; k < cnzi; k++) {
-      ca_i[k] = c_row_val_dense[cj_i[k]];
+    for (k=0; k<cnzi; k++) {
+      ca_i[k]                  = c_row_val_dense[cj_i[k]];
       c_row_val_dense[cj_i[k]] = 0.;
       c_row_idx_flags[cj_i[k]] = PETSC_FALSE;
     }
     /* terminate current row */
-    aa_i += anzi;
-    aj_i += anzi;
-    ca_i += cnzi;
-    cj_i += cnzi;
+    aa_i   += anzi;
+    aj_i   += anzi;
+    ca_i   += cnzi;
+    cj_i   += cnzi;
     ci[i+1] = ci[i] + cnzi;
-    flops += cnzi;
+    flops  += cnzi;
   }
 
   /* Step 5 */
   /* Create the new matrix */
   ierr = MatCreateSeqAIJWithArrays(PetscObjectComm((PetscObject)A),am,bn,ci,cj,NULL,C);CHKERRQ(ierr);
   ierr = MatSetBlockSizesFromMats(*C,A,B);CHKERRQ(ierr);
+  ierr = MatSetType(*C,((PetscObject)A)->type_name);CHKERRQ(ierr);
 
   /* MatCreateSeqAIJWithArrays flags matrix so PETSc doesn't free the user's arrays. */
   /* These are PETSc arrays, so change flags so arrays can be deleted by PETSc */
@@ -1642,6 +1936,7 @@ PetscErrorCode MatMatMult_SeqAIJ_SeqAIJ_Combined(Mat A,Mat B,PetscReal fill,Mat
   (*C)->info.mallocs           = ndouble;
   (*C)->info.fill_ratio_given  = fill;
   (*C)->info.fill_ratio_needed = afill;
+  (*C)->ops->matmultnumeric    = MatMatMultNumeric_SeqAIJ_SeqAIJ_Combined;
 
   ierr = PetscFree(c_row_val_dense);CHKERRQ(ierr);
   ierr = PetscFree(c_row_idx_flags);CHKERRQ(ierr);
