@@ -2090,7 +2090,7 @@ PetscErrorCode MatDuplicate_SeqSELL(Mat A,MatDuplicateOption cpvalues,Mat *B)
  .  A - the matrix
 
  It is recommended that one use the MatCreate(), MatSetType() and/or MatSetFromOptions(),
- MatXXXXSetPreallocation() paradgm instead of this routine directly.
+ MatXXXXSetPreallocation() paradigm instead of this routine directly.
  [MatXXXXSetPreallocation() is, for example, MatSeqSELLSetPreallocation]
 
  Notes:
