@@ -240,7 +240,9 @@ PetscErrorCode MatDuplicate_Shell(Mat mat,MatDuplicateOption op,Mat *M)
   PetscFunctionBegin;
   ierr = MatShellGetContext(mat,&ctx);CHKERRQ(ierr);
   ierr = MatCreateShell(PetscObjectComm((PetscObject)mat),mat->rmap->n,mat->cmap->n,mat->rmap->N,mat->cmap->N,ctx,M);CHKERRQ(ierr);
-  ierr = MatCopy(mat,*M,SAME_NONZERO_PATTERN);CHKERRQ(ierr);
+  if (op != MAT_DO_NOT_COPY_VALUES) {
+    ierr = MatCopy(mat,*M,SAME_NONZERO_PATTERN);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
@@ -308,6 +310,12 @@ PetscErrorCode MatMultTranspose_Shell(Mat A,Vec x,Vec y)
   }
   ierr = MatShellShiftAndScale(A,xx,y);CHKERRQ(ierr);
   ierr = MatShellPostScaleRight(A,y);CHKERRQ(ierr);
+
+  if (shell->axpy) {
+    if (!shell->right_work) {ierr = MatCreateVecs(A,NULL,&shell->right_work);CHKERRQ(ierr);}
+    ierr = MatMultTranspose(shell->axpy,x,shell->right_work);CHKERRQ(ierr);
+    ierr = VecAXPY(y,shell->axpy_vscale,shell->right_work);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
@@ -376,13 +384,12 @@ PetscErrorCode MatShift_Shell(Mat Y,PetscScalar a)
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode MatDiagonalSet_Shell(Mat A,Vec D,InsertMode ins)
+PetscErrorCode MatDiagonalSet_Shell_Private(Mat A,Vec D,PetscScalar s)
 {
   Mat_Shell      *shell = (Mat_Shell*)A->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (ins == INSERT_VALUES) SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_ARG_WRONGSTATE, "Operation not supported with INSERT_VALUES");
   if (!shell->dshift) {ierr = VecDuplicate(D,&shell->dshift);CHKERRQ(ierr);}
   if (shell->left || shell->right) {
     if (!shell->right_work) {ierr = VecDuplicate(shell->left ? shell->left : shell->right, &shell->right_work);CHKERRQ(ierr);}
@@ -398,10 +405,29 @@ PetscErrorCode MatDiagonalSet_Shell(Mat A,Vec D,InsertMode ins)
       ierr = VecDuplicate(shell->left ? shell->left : shell->right, &shell->dshift);CHKERRQ(ierr);
       ierr = VecCopy(shell->dshift,shell->right_work);CHKERRQ(ierr);
     } else {
-      ierr = VecAXPY(shell->dshift,1.0,shell->right_work);CHKERRQ(ierr);
+      ierr = VecAXPY(shell->dshift,s,shell->right_work);CHKERRQ(ierr);
     }
   } else {
-    ierr = VecAXPY(shell->dshift,1.0,D);CHKERRQ(ierr);
+    ierr = VecAXPY(shell->dshift,s,D);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode MatDiagonalSet_Shell(Mat A,Vec D,InsertMode ins)
+{
+  Vec            d;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (ins == INSERT_VALUES) {
+    if (!A->ops->getdiagonal) SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_SUP,"Operation MATOP_GETDIAGONAL must be set first");
+    ierr = VecDuplicate(D,&d);CHKERRQ(ierr);
+    ierr = MatGetDiagonal(A,d);CHKERRQ(ierr);
+    ierr = MatDiagonalSet_Shell_Private(A,d,-1.);CHKERRQ(ierr);
+    ierr = MatDiagonalSet_Shell_Private(A,D,1.);CHKERRQ(ierr);
+    ierr = VecDestroy(&d);CHKERRQ(ierr);
+  } else {
+    ierr = MatDiagonalSet_Shell_Private(A,D,1.);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -741,6 +767,14 @@ $
 
           A is the user provided function.
 
+   KSP/PC uses changes in the Mat's "state" to decide if preconditioners need to be rebuilt: PCSetUp() only calls the setup() for
+   for the PC implementation if the Mat state has increased from the previous call. Thus to get changes in a MATSHELL to trigger
+   an update in the preconditioner you must call MatAssemblyBegin()/MatAssemblyEnd() or PetscObjectStateIncrease((PetscObject)mat);
+   each time the MATSHELL matrix has changed.
+
+   Calling MatAssemblyBegin()/MatAssemblyEnd() on a MATSHELL removes any previously supplied shift and scales that were provided
+   with MatDiagonalSet(), MatShift(), MatScale(), or MatDiagonalScale().
+
 .keywords: matrix, shell, create
 
 .seealso: MatShellSetOperation(), MatHasOperation(), MatShellGetContext(), MatShellSetContext(), MATSHELL, MatShellSetManageScalingShifts()
