@@ -1,12 +1,10 @@
 
-#include <petsc-private/matimpl.h>          /*I "petscmat.h" I*/
+#include <petsc/private/matimpl.h>          /*I "petscmat.h" I*/
 
 typedef struct {
   Mat A;
 } Mat_Transpose;
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_Transpose"
 PetscErrorCode MatMult_Transpose(Mat N,Vec x,Vec y)
 {
   Mat_Transpose  *Na = (Mat_Transpose*)N->data;
@@ -17,8 +15,6 @@ PetscErrorCode MatMult_Transpose(Mat N,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMultAdd_Transpose"
 PetscErrorCode MatMultAdd_Transpose(Mat N,Vec v1,Vec v2,Vec v3)
 {
   Mat_Transpose  *Na = (Mat_Transpose*)N->data;
@@ -29,8 +25,6 @@ PetscErrorCode MatMultAdd_Transpose(Mat N,Vec v1,Vec v2,Vec v3)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMultTranspose_Transpose"
 PetscErrorCode MatMultTranspose_Transpose(Mat N,Vec x,Vec y)
 {
   Mat_Transpose  *Na = (Mat_Transpose*)N->data;
@@ -41,8 +35,6 @@ PetscErrorCode MatMultTranspose_Transpose(Mat N,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMultTransposeAdd_Transpose"
 PetscErrorCode MatMultTransposeAdd_Transpose(Mat N,Vec v1,Vec v2,Vec v3)
 {
   Mat_Transpose  *Na = (Mat_Transpose*)N->data;
@@ -53,8 +45,6 @@ PetscErrorCode MatMultTransposeAdd_Transpose(Mat N,Vec v1,Vec v2,Vec v3)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_Transpose"
 PetscErrorCode MatDestroy_Transpose(Mat N)
 {
   Mat_Transpose  *Na = (Mat_Transpose*)N->data;
@@ -62,12 +52,73 @@ PetscErrorCode MatDestroy_Transpose(Mat N)
 
   PetscFunctionBegin;
   ierr = MatDestroy(&Na->A);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)N,"MatTransposeGetMat_C",NULL);CHKERRQ(ierr);
   ierr = PetscFree(N->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCreateTranspose"
+PetscErrorCode MatDuplicate_Transpose(Mat N, MatDuplicateOption op, Mat* m)
+{
+  Mat_Transpose  *Na = (Mat_Transpose*)N->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (op == MAT_COPY_VALUES) {
+    ierr = MatTranspose(Na->A,MAT_INITIAL_MATRIX,m);CHKERRQ(ierr);
+  } else if (op == MAT_DO_NOT_COPY_VALUES) {
+    ierr = MatDuplicate(Na->A,MAT_DO_NOT_COPY_VALUES,m);CHKERRQ(ierr);
+    ierr = MatTranspose(*m,MAT_INPLACE_MATRIX,m);CHKERRQ(ierr);
+  } else SETERRQ(PetscObjectComm((PetscObject)N),PETSC_ERR_SUP,"MAT_SHARE_NONZERO_PATTERN not supported for this matrix type");
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode MatCreateVecs_Transpose(Mat A,Vec *r, Vec *l)
+{
+  Mat_Transpose  *Aa = (Mat_Transpose*)A->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = MatCreateVecs(Aa->A,l,r);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode MatTransposeGetMat_Transpose(Mat A,Mat *M)
+{
+  Mat_Transpose  *Aa = (Mat_Transpose*)A->data;
+
+  PetscFunctionBegin;
+  *M = Aa->A;
+  PetscFunctionReturn(0);
+}
+
+/*@
+      MatTransposeGetMat - Gets the Mat object stored inside a MATTRANSPOSEMAT
+
+   Logically collective on Mat
+
+   Input Parameter:
+.   A  - the MATTRANSPOSE matrix
+
+   Output Parameter:
+.   M - the matrix object stored inside A
+
+   Level: intermediate
+
+.seealso: MatCreateTranspose()
+
+@*/
+PetscErrorCode MatTransposeGetMat(Mat A,Mat *M)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(A,MAT_CLASSID,1);
+  PetscValidType(A,1);
+  PetscValidPointer(M,2);
+  ierr = PetscUseMethod(A,"MatTransposeGetMat_C",(Mat,Mat*),(A,M));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
       MatCreateTranspose - Creates a new matrix object that behaves like A'
 
@@ -81,7 +132,8 @@ PetscErrorCode MatDestroy_Transpose(Mat N)
 
    Level: intermediate
 
-   Notes: The transpose A' is NOT actually formed! Rather the new matrix
+   Notes:
+    The transpose A' is NOT actually formed! Rather the new matrix
           object performs the matrix-vector product by using the MatMultTranspose() on
           the original matrix
 
@@ -98,9 +150,11 @@ PetscErrorCode  MatCreateTranspose(Mat A,Mat *N)
   ierr = MatGetLocalSize(A,&m,&n);CHKERRQ(ierr);
   ierr = MatCreate(PetscObjectComm((PetscObject)A),N);CHKERRQ(ierr);
   ierr = MatSetSizes(*N,n,m,PETSC_DECIDE,PETSC_DECIDE);CHKERRQ(ierr);
+  ierr = PetscLayoutSetUp((*N)->rmap);CHKERRQ(ierr);
+  ierr = PetscLayoutSetUp((*N)->cmap);CHKERRQ(ierr);
   ierr = PetscObjectChangeTypeName((PetscObject)*N,MATTRANSPOSEMAT);CHKERRQ(ierr);
 
-  ierr       = PetscNewLog(*N,Mat_Transpose,&Na);CHKERRQ(ierr);
+  ierr       = PetscNewLog(*N,&Na);CHKERRQ(ierr);
   (*N)->data = (void*) Na;
   ierr       = PetscObjectReference((PetscObject)A);CHKERRQ(ierr);
   Na->A      = A;
@@ -110,12 +164,12 @@ PetscErrorCode  MatCreateTranspose(Mat A,Mat *N)
   (*N)->ops->multadd          = MatMultAdd_Transpose;
   (*N)->ops->multtranspose    = MatMultTranspose_Transpose;
   (*N)->ops->multtransposeadd = MatMultTransposeAdd_Transpose;
+  (*N)->ops->duplicate        = MatDuplicate_Transpose;
+  (*N)->ops->getvecs          = MatCreateVecs_Transpose;
   (*N)->assembled             = PETSC_TRUE;
 
-  ierr = PetscLayoutSetBlockSize((*N)->rmap,A->cmap->bs);CHKERRQ(ierr);
-  ierr = PetscLayoutSetBlockSize((*N)->cmap,A->rmap->bs);CHKERRQ(ierr);
-  ierr = PetscLayoutSetUp((*N)->rmap);CHKERRQ(ierr);
-  ierr = PetscLayoutSetUp((*N)->cmap);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)(*N),"MatTransposeGetMat_C",MatTransposeGetMat_Transpose);CHKERRQ(ierr);
+  ierr = MatSetBlockSizes(*N,PetscAbs(A->cmap->bs),PetscAbs(A->rmap->bs));CHKERRQ(ierr);
   ierr = MatSetUp(*N);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
