@@ -52,6 +52,7 @@
 #define matdenserestorearrayread_        MATDENSERESTOREARRAYREAD
 #define matconvert_                      MATCONVERT
 #define matcreatesubmatrices_            MATCREATESUBMATRICES
+#define matcreatesubmatricesmpi_         MATCREATESUBMATRICESMPI
 #define matzerorowscolumns_              MATZEROROWSCOLUMNS
 #define matzerorowscolumnsis_            MATZEROROWSCOLUMNSIS
 #define matzerorowsstencil_              MATZEROROWSSTENCIL
@@ -81,14 +82,20 @@
 #define matfactorinfoinitialize_         MATFACTORINFOINITIALIZE
 #define matnullspacesetfunction_         MATNULLSPACESETFUNCTION
 #define matfindnonzerorows_              MATFINDNONZEROROWS
+#define matgetsize_                      MATGETSIZE
 #define matgetsize00_                    MATGETSIZE00
 #define matgetsize10_                    MATGETSIZE10
 #define matgetsize01_                    MATGETSIZE01
+#define matgetlocalsize_                 MATGETLOCALSIZE
 #define matgetlocalsize00_               MATGETLOCALSIZE00
 #define matgetlocalsize10_               MATGETLOCALSIZE10
 #define matgetlocalsize01_               MATGETLOCALSIZE01
 #define matsetnullspace_                 MATSETNULLSPACE
 #define matgetownershiprange_            MATGETOWNERSHIPRANGE
+#define matgetownershiprange00_          MATGETOWNERSHIPRANGE00
+#define matgetownershiprange10_          MATGETOWNERSHIPRANGE10
+#define matgetownershiprange01_          MATGETOWNERSHIPRANGE01
+#define matgetownershiprange11_          MATGETOWNERSHIPRANGE11
 #define matgetownershipis_               MATGETOWNERSHIPIS
 #define matgetownershiprangecolumn_      MATGETOWNERSHIPRANGECOLUMN
 #elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
@@ -142,6 +149,7 @@
 #define matdenserestorearrayread_        matdenserestorearrayread
 #define matconvert_                      matconvert
 #define matcreatesubmatrices_            matcreatesubmatrices
+#define matcreatesubmatricesmpi_         matcreatesubmatricesmpi
 #define matzerorowscolumns_              matzerorowscolumns
 #define matzerorowscolumnsis_            matzerorowscolumnsis
 #define matzerorowsstencil_              matzerorowsstencil
@@ -168,14 +176,20 @@
 #define matfactorinfoinitialize_         matfactorinfoinitialize
 #define matnullspacesetfunction_         matnullspacesetfunction
 #define matfindnonzerorows_              matfindnonzerorows
+#define matgetsize_                      matgetsize
 #define matgetsize00_                    matgetsize00
 #define matgetsize10_                    matgetsize10
 #define matgetsize01_                    matgetsize01
+#define matgetlocalsize_                 matgetlocalsize
 #define matgetlocalsize00_               matgetlocalsize00
 #define matgetlocalsize10_               matgetlocalsize10
 #define matgetlocalsize01_               matgetlocalsize01
 #define matsetnullspace_                 matsetnullspace
 #define matgetownershiprange_            matgetownershiprange
+#define matgetownershiprange00_          matgetownershiprange00
+#define matgetownershiprange10_          matgetownershiprange10
+#define matgetownershiprange01_          matgetownershiprange01
+#define matgetownershiprange11_          matgetownershiprange11
 #define matgetownershipis_               matgetownershipis
 #define matgetownershiprangecolumn_      matgetownershiprangecolumn
 #endif
@@ -187,6 +201,34 @@ PETSC_EXTERN void PETSC_STDCALL  matgetownershiprange_(Mat *mat,PetscInt *m,Pets
   *ierr = MatGetOwnershipRange(*mat,m,n);
 }
 
+PETSC_EXTERN void PETSC_STDCALL  matgetownershiprange00_(Mat *mat,PetscInt *m,PetscInt *n, int *ierr )
+{
+  CHKFORTRANNULLINTEGER(m);
+  CHKFORTRANNULLINTEGER(n);
+  *ierr = MatGetOwnershipRange(*mat,m,n);
+}
+
+PETSC_EXTERN void PETSC_STDCALL  matgetownershiprange10_(Mat *mat,PetscInt *m,PetscInt *n, int *ierr )
+{
+  CHKFORTRANNULLINTEGER(m);
+  CHKFORTRANNULLINTEGER(n);
+  *ierr = MatGetOwnershipRange(*mat,m,n);
+}
+
+PETSC_EXTERN void PETSC_STDCALL  matgetownershiprange01_(Mat *mat,PetscInt *m,PetscInt *n, int *ierr )
+{
+  CHKFORTRANNULLINTEGER(m);
+  CHKFORTRANNULLINTEGER(n);
+  *ierr = MatGetOwnershipRange(*mat,m,n);
+}
+
+PETSC_EXTERN void PETSC_STDCALL  matgetownershiprange11_(Mat *mat,PetscInt *m,PetscInt *n, int *ierr )
+{
+  CHKFORTRANNULLINTEGER(m);
+  CHKFORTRANNULLINTEGER(n);
+  *ierr = MatGetOwnershipRange(*mat,m,n);
+}
+
 PETSC_EXTERN void PETSC_STDCALL  matgetownershipis_(Mat *mat,IS *m,IS *n, int *ierr )
 {
   CHKFORTRANNULLOBJECT(m);
@@ -607,6 +649,27 @@ PETSC_EXTERN void PETSC_STDCALL matcreatesubmatrices_(Mat *mat,PetscInt *n,IS *i
   }
 }
 
+/*
+    MatCreateSubmatrices() is slightly different from C since the
+    Fortran provides the array to hold the submatrix objects,while in C that
+    array is allocated by the MatCreateSubmatrices()
+*/
+PETSC_EXTERN void PETSC_STDCALL matcreatesubmatricesmpi_(Mat *mat,PetscInt *n,IS *isrow,IS *iscol,MatReuse *scall,Mat *smat,PetscErrorCode *ierr)
+{
+  Mat      *lsmat;
+  PetscInt i;
+
+  if (*scall == MAT_INITIAL_MATRIX) {
+    *ierr = MatCreateSubMatricesMPI(*mat,*n,isrow,iscol,*scall,&lsmat);
+    for (i=0; i<=*n; i++) { /* lsmat[*n] might be a dummy matrix for saving data struc */
+      smat[i] = lsmat[i];
+    }
+    *ierr = PetscFree(lsmat);
+  } else {
+    *ierr = MatCreateSubMatricesMPI(*mat,*n,isrow,iscol,*scall,&smat);
+  }
+}
+
 /*
     MatDestroyMatrices() is slightly different from C since the
     Fortran does not free the array of matrix objects, while in C that
