@@ -11,8 +11,6 @@ PetscBool MatRegisterAllCalled = PETSC_FALSE;
 */
 PetscFunctionList MatList = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSetType"
 /*@C
    MatSetType - Builds matrix object for a particular matrix type
 
@@ -38,14 +36,14 @@ PetscFunctionList MatList = 0;
 PetscErrorCode  MatSetType(Mat mat, MatType matype)
 {
   PetscErrorCode ierr,(*r)(Mat);
-  PetscBool      sametype,found;
-  MatBaseName    names = MatBaseNameList;
+  PetscBool      sametype,found,subclass = PETSC_FALSE;
+  MatRootName    names = MatRootNameList;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
 
   while (names) {
-    ierr = PetscStrcmp(matype,names->bname,&found);CHKERRQ(ierr);
+    ierr = PetscStrcmp(matype,names->rname,&found);CHKERRQ(ierr);
     if (found) {
       PetscMPIInt size;
       ierr = MPI_Comm_size(PetscObjectComm((PetscObject)mat),&size);CHKERRQ(ierr);
@@ -62,8 +60,14 @@ PetscErrorCode  MatSetType(Mat mat, MatType matype)
   ierr =  PetscFunctionListFind(MatList,matype,&r);CHKERRQ(ierr);
   if (!r) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown Mat type given: %s",matype);
 
-  /* free the old data structure if it existed */
-  if (mat->ops->destroy) {
+  if (mat->assembled && ((PetscObject)mat)->type_name) {
+    ierr = PetscStrbeginswith(matype,((PetscObject)mat)->type_name,&subclass);CHKERRQ(ierr);
+  }
+  if (subclass) {
+    ierr = MatConvert(mat,matype,MAT_INPLACE_MATRIX,&mat);CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  } if (mat->ops->destroy) {
+    /* free the old data structure if it existed */
     ierr = (*mat->ops->destroy)(mat);CHKERRQ(ierr);
     mat->ops->destroy = NULL;
 
@@ -95,8 +99,6 @@ PetscErrorCode  MatSetType(Mat mat, MatType matype)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatGetType"
 /*@C
    MatGetType - Gets the matrix type as a string from the matrix object.
 
@@ -124,8 +126,6 @@ PetscErrorCode  MatGetType(Mat mat,MatType *type)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "MatRegister"
 /*@C
   MatRegister -  - Adds a new matrix type
 
@@ -152,7 +152,7 @@ $     -mat_type my_mat
 
 .keywords: Mat, register
 
-.seealso: MatRegisterAll(), MatRegisterDestroy()
+.seealso: MatRegisterAll()
 
 
   Level: advanced
@@ -162,39 +162,49 @@ PetscErrorCode  MatRegister(const char sname[],PetscErrorCode (*function)(Mat))
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = MatInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&MatList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-MatBaseName MatBaseNameList = 0;
+MatRootName MatRootNameList = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "MatRegisterBaseName"
 /*@C
-      MatRegisterBaseName - Registers a name that can be used for either a sequential or its corresponding parallel matrix type.
+      MatRegisterRootName - Registers a name that can be used for either a sequential or its corresponding parallel matrix type. MatSetType()
+        and -mat_type will automatically use the sequential or parallel version based on the size of the MPI communicator associated with the
+        matrix.
 
   Input Parameters:
-+     bname - the basename, for example, MATAIJ
++     rname - the rootname, for example, MATAIJ
 .     sname - the name of the sequential matrix type, for example, MATSEQAIJ
 -     mname - the name of the parallel matrix type, for example, MATMPIAIJ
 
+  Notes: The matrix rootname should not be confused with the base type of the function PetscObjectBaseTypeCompare()
+
+  Developer Notes: PETSc vectors have a similar rootname that indicates PETSc should automatically select the appropriate VecType based on the
+      size of the communicator but it is implemented by simply having additional VecCreate_RootName() registerer routines that dispatch to the
+      appropriate creation routine. Why have two different ways of implementing the same functionality for different types of objects? It is
+      confusing.
+
+  Level: developer
+
+.seealso: PetscObjectBaseTypeCompare()
 
-  Level: advanced
 @*/
-PetscErrorCode  MatRegisterBaseName(const char bname[],const char sname[],const char mname[])
+PetscErrorCode  MatRegisterRootName(const char rname[],const char sname[],const char mname[])
 {
   PetscErrorCode ierr;
-  MatBaseName    names;
+  MatRootName    names;
 
   PetscFunctionBegin;
   ierr = PetscNew(&names);CHKERRQ(ierr);
-  ierr = PetscStrallocpy(bname,&names->bname);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(rname,&names->rname);CHKERRQ(ierr);
   ierr = PetscStrallocpy(sname,&names->sname);CHKERRQ(ierr);
   ierr = PetscStrallocpy(mname,&names->mname);CHKERRQ(ierr);
-  if (!MatBaseNameList) {
-    MatBaseNameList = names;
+  if (!MatRootNameList) {
+    MatRootNameList = names;
   } else {
-    MatBaseName next = MatBaseNameList;
+    MatRootName next = MatRootNameList;
     while (next->next) next = next->next;
     next->next = names;
   }
