@@ -329,6 +329,7 @@ $    -mat_partitioning_view
 PetscErrorCode  MatPartitioningApply(MatPartitioning matp,IS *partitioning)
 {
   PetscErrorCode ierr;
+  PetscBool      viewbalance,improve;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(matp,MAT_PARTITIONING_CLASSID,1);
@@ -342,6 +343,119 @@ PetscErrorCode  MatPartitioningApply(MatPartitioning matp,IS *partitioning)
 
   ierr = MatPartitioningViewFromOptions(matp,NULL,"-mat_partitioning_view");CHKERRQ(ierr);
   ierr = ISViewFromOptions(*partitioning,NULL,"-mat_partitioning_view");CHKERRQ(ierr);
+
+  ierr = PetscObjectOptionsBegin((PetscObject)matp);CHKERRQ(ierr);
+  viewbalance = PETSC_FALSE;
+  ierr = PetscOptionsBool("-mat_partitioning_view_imbalance","Display imbalance information of a partition",NULL,PETSC_FALSE,&viewbalance,NULL);CHKERRQ(ierr);
+  improve = PETSC_FALSE;
+  ierr = PetscOptionsBool("-mat_partitioning_improve","Improve the quality of a partition",NULL,PETSC_FALSE,&improve,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsEnd();CHKERRQ(ierr);
+
+  if (improve) {
+    ierr = MatPartitioningImprove(matp,partitioning);CHKERRQ(ierr);
+  }
+
+  if (viewbalance) {
+    ierr = MatPartitioningViewImbalance(matp,*partitioning);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+
+/*@
+   MatPartitioningImprove - Improves the quality of a given partition.
+
+   Collective on Mat
+
+   Input Parameters:
+.  matp - the matrix partitioning object
+.  partitioning - the partitioning. For each local node this tells the processor
+                   number that that node is assigned to.
+
+   Output Parameters:
+.   partitioning - the partitioning. For each local node this tells the processor
+                   number that that node is assigned to.
+
+   Options Database Keys:
+   To improve the quality of the partition
+$    -mat_partitioning_improve
+
+   Level: beginner
+
+
+.keywords: matrix, improve, partitioning
+
+.seealso:  MatPartitioningApply(), MatPartitioningCreate(),
+           MatPartitioningDestroy(), MatPartitioningSetAdjacency(), ISPartitioningToNumbering(),
+           ISPartitioningCount()
+@*/
+PetscErrorCode  MatPartitioningImprove(MatPartitioning matp,IS *partitioning)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(matp,MAT_PARTITIONING_CLASSID,1);
+  PetscValidPointer(partitioning,2);
+  if (!matp->adj->assembled) SETERRQ(PetscObjectComm((PetscObject)matp),PETSC_ERR_ARG_WRONGSTATE,"Not for unassembled matrix");
+  if (matp->adj->factortype) SETERRQ(PetscObjectComm((PetscObject)matp),PETSC_ERR_ARG_WRONGSTATE,"Not for factored matrix");
+  ierr = PetscLogEventBegin(MAT_Partitioning,matp,0,0,0);CHKERRQ(ierr);
+  if (matp->ops->improve) {
+    ierr = (*matp->ops->improve)(matp,partitioning);CHKERRQ(ierr);
+  }
+  ierr = PetscLogEventEnd(MAT_Partitioning,matp,0,0,0);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   MatPartitioningViewImbalance - Display partitioning imbalance information.
+
+   Collective on MatPartitioning
+
+   Input Parameters:
+.  matp - the matrix partitioning object
+.  partitioning - the partitioning. For each local node this tells the processor
+                   number that that node is assigned to.
+
+   Options Database Keys:
+   To see the partitioning imbalance information
+$    -mat_partitioning_view_balance
+
+   Level: beginner
+
+.keywords: matrix, imbalance, partitioning
+
+.seealso:  MatPartitioningApply(), MatPartitioningView()
+@*/
+PetscErrorCode  MatPartitioningViewImbalance(MatPartitioning matp, IS partitioning)
+{
+  PetscErrorCode  ierr;
+  PetscInt        nparts,*subdomainsizes,*subdomainsizes_tmp,nlocal,i,maxsub,minsub,avgsub;
+  const PetscInt  *indices;
+  PetscViewer     viewer;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(matp,MAT_PARTITIONING_CLASSID,1);
+  PetscValidHeaderSpecific(partitioning,IS_CLASSID,2);
+  nparts = matp->n;
+  ierr = PetscCalloc2(nparts,&subdomainsizes,nparts,&subdomainsizes_tmp);CHKERRQ(ierr);
+  ierr = ISGetLocalSize(partitioning,&nlocal);CHKERRQ(ierr);
+  ierr = ISGetIndices(partitioning,&indices);CHKERRQ(ierr);
+  for (i=0;i<nlocal;i++) {
+    subdomainsizes_tmp[indices[i]] += matp->vertex_weights? matp->vertex_weights[i]:1;
+  }
+  ierr = MPI_Allreduce(subdomainsizes_tmp,subdomainsizes,nparts,MPIU_INT,MPI_SUM, PetscObjectComm((PetscObject)matp));CHKERRQ(ierr);
+  ierr = ISRestoreIndices(partitioning,&indices);CHKERRQ(ierr);
+  minsub = PETSC_MAX_INT, maxsub = PETSC_MIN_INT, avgsub=0;
+  for (i=0; i<nparts; i++) {
+    minsub = PetscMin(minsub,subdomainsizes[i]);
+    maxsub = PetscMax(maxsub,subdomainsizes[i]);
+    avgsub += subdomainsizes[i];
+  }
+  avgsub /=nparts;
+  ierr = PetscFree2(subdomainsizes,subdomainsizes_tmp);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)matp),&viewer);CHKERRQ(ierr);
+  ierr = MatPartitioningView(matp,viewer);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"Partitioning Imbalance Info: Max %D, Min %D, Avg %D, R %g\n",maxsub, minsub, avgsub, (double)(maxsub/(PetscReal)minsub));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -599,19 +713,16 @@ PetscErrorCode  MatPartitioningSetType(MatPartitioning part,MatPartitioningType
   if (match) PetscFunctionReturn(0);
 
   if (part->ops->destroy) {
-    ierr =  (*part->ops->destroy)(part);CHKERRQ(ierr);
-
+    ierr = (*part->ops->destroy)(part);CHKERRQ(ierr);
     part->ops->destroy = NULL;
-    part->data         = 0;
-    part->setupcalled  = 0;
   }
+  part->setupcalled = 0;
+  part->data        = 0;
+  ierr = PetscMemzero(part->ops,sizeof(struct _MatPartitioningOps));CHKERRQ(ierr);
 
   ierr = PetscFunctionListFind(MatPartitioningList,type,&r);CHKERRQ(ierr);
   if (!r) SETERRQ1(PetscObjectComm((PetscObject)part),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown partitioning type %s",type);
 
-  part->ops->destroy = (PetscErrorCode (*)(MatPartitioning)) 0;
-  part->ops->view    = (PetscErrorCode (*)(MatPartitioning,PetscViewer)) 0;
-
   ierr = (*r)(part);CHKERRQ(ierr);
 
   ierr = PetscFree(((PetscObject)part)->type_name);CHKERRQ(ierr);
@@ -632,6 +743,7 @@ PetscErrorCode  MatPartitioningSetType(MatPartitioning part,MatPartitioningType
 $  -mat_partitioning_type  <type>
 $      Use -help for a list of available methods
 $      (for instance, parmetis)
+$  -mat_partitioning_nparts - number of subgraphs
 
 
    Notes:
@@ -670,6 +782,9 @@ PetscErrorCode  MatPartitioningSetFromOptions(MatPartitioning part)
   if (flag) {
     ierr = MatPartitioningSetType(part,type);CHKERRQ(ierr);
   }
+
+  ierr = PetscOptionsInt("-mat_partitioning_nparts","number of fine parts",NULL,part->n,& part->n,&flag);CHKERRQ(ierr);
+
   /*
     Set the type if it was never set.
   */
