@@ -22,26 +22,39 @@
 PetscErrorCode MatAXPY(Mat Y,PetscScalar a,Mat X,MatStructure str)
 {
   PetscErrorCode ierr;
+  PetscInt       M1,M2,N1,N2;
   PetscInt       m1,m2,n1,n2;
   PetscBool      sametype;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(X,MAT_CLASSID,3);
   PetscValidHeaderSpecific(Y,MAT_CLASSID,1);
   PetscValidLogicalCollectiveScalar(Y,a,2);
-  ierr = MatGetSize(X,&m1,&n1);CHKERRQ(ierr);
-  ierr = MatGetSize(Y,&m2,&n2);CHKERRQ(ierr);
-  if (m1 != m2 || n1 != n2) SETERRQ4(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Non conforming matrix add: %D %D %D %D",m1,m2,n1,n2);
+  PetscValidHeaderSpecific(X,MAT_CLASSID,3);
+  PetscCheckSameComm(Y,1,X,3);
+  ierr = MatGetSize(X,&M1,&N1);CHKERRQ(ierr);
+  ierr = MatGetSize(Y,&M2,&N2);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(X,&m1,&n1);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(Y,&m2,&n2);CHKERRQ(ierr);
+  if (M1 != M2 || N1 != N2) SETERRQ4(PetscObjectComm((PetscObject)Y),PETSC_ERR_ARG_SIZ,"Non conforming matrix add: global sizes %D x %D, %D x %D",M1,M2,N1,N2);
+  if (m1 != m2 || n1 != n2) SETERRQ4(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Non conforming matrix add: local sizes %D x %D, %D x %D",m1,m2,n1,n2);
 
   ierr = PetscStrcmp(((PetscObject)X)->type_name,((PetscObject)Y)->type_name,&sametype);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(MAT_AXPY,Y,0,0,0);CHKERRQ(ierr);
   if (Y->ops->axpy && sametype) {
     ierr = (*Y->ops->axpy)(Y,a,X,str);CHKERRQ(ierr);
   } else {
-    ierr = MatAXPY_Basic(Y,a,X,str);CHKERRQ(ierr);
+    if (str != DIFFERENT_NONZERO_PATTERN) {
+      ierr = MatAXPY_Basic(Y,a,X,str);CHKERRQ(ierr);
+    } else {
+      Mat B;
+
+      ierr = MatAXPY_Basic_Preallocate(Y,X,&B);CHKERRQ(ierr);
+      ierr = MatAXPY_BasicWithPreallocation(B,Y,a,X,str);CHKERRQ(ierr);
+      ierr = MatHeaderReplace(Y,&B);CHKERRQ(ierr);
+    }
   }
   ierr = PetscLogEventEnd(MAT_AXPY,Y,0,0,0);CHKERRQ(ierr);
-#if defined(PETSC_HAVE_VIENNACL) || defined(PETSC_HAVE_VECCUDA)
+#if defined(PETSC_HAVE_VIENNACL) || defined(PETSC_HAVE_CUDA)
   if (Y->valid_GPU_matrix != PETSC_OFFLOAD_UNALLOCATED) {
     Y->valid_GPU_matrix = PETSC_OFFLOAD_CPU;
   }
@@ -49,6 +62,56 @@ PetscErrorCode MatAXPY(Mat Y,PetscScalar a,Mat X,MatStructure str)
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode MatAXPY_Basic_Preallocate(Mat Y, Mat X, Mat *B)
+{
+  PetscErrorCode ierr;
+  PetscErrorCode (*preall)(Mat,Mat,Mat*) = NULL;
+
+  PetscFunctionBegin;
+  /* look for any available faster alternative to the general preallocator */
+  ierr = PetscObjectQueryFunction((PetscObject)Y,"MatAXPYGetPreallocation_C",&preall);CHKERRQ(ierr);
+  if (preall) {
+    ierr = (*preall)(Y,X,B);CHKERRQ(ierr);
+  } else { /* Use MatPrellocator, assumes same row-col distribution */
+    Mat      preallocator;
+    PetscInt r,rstart,rend;
+    PetscInt m,n,M,N;
+
+    ierr = MatGetRowUpperTriangular(Y);CHKERRQ(ierr);
+    ierr = MatGetRowUpperTriangular(X);CHKERRQ(ierr);
+    ierr = MatGetSize(Y,&M,&N);CHKERRQ(ierr);
+    ierr = MatGetLocalSize(Y,&m,&n);CHKERRQ(ierr);
+    ierr = MatCreate(PetscObjectComm((PetscObject)Y),&preallocator);CHKERRQ(ierr);
+    ierr = MatSetType(preallocator,MATPREALLOCATOR);CHKERRQ(ierr);
+    ierr = MatSetSizes(preallocator,m,n,M,N);CHKERRQ(ierr);
+    ierr = MatSetUp(preallocator);CHKERRQ(ierr);
+    ierr = MatGetOwnershipRange(preallocator,&rstart,&rend);CHKERRQ(ierr);
+    for (r = rstart; r < rend; ++r) {
+      PetscInt          ncols;
+      const PetscInt    *row;
+      const PetscScalar *vals;
+
+      ierr = MatGetRow(Y,r,&ncols,&row,&vals);CHKERRQ(ierr);
+      ierr = MatSetValues(preallocator,1,&r,ncols,row,vals,INSERT_VALUES);CHKERRQ(ierr);
+      ierr = MatRestoreRow(Y,r,&ncols,&row,&vals);CHKERRQ(ierr);
+      ierr = MatGetRow(X,r,&ncols,&row,&vals);CHKERRQ(ierr);
+      ierr = MatSetValues(preallocator,1,&r,ncols,row,vals,INSERT_VALUES);CHKERRQ(ierr);
+      ierr = MatRestoreRow(X,r,&ncols,&row,&vals);CHKERRQ(ierr);
+    }
+    ierr = MatAssemblyBegin(preallocator,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+    ierr = MatAssemblyEnd(preallocator,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+    ierr = MatRestoreRowUpperTriangular(Y);CHKERRQ(ierr);
+    ierr = MatRestoreRowUpperTriangular(X);CHKERRQ(ierr);
+
+    ierr = MatCreate(PetscObjectComm((PetscObject)Y),B);CHKERRQ(ierr);
+    ierr = MatSetType(*B,((PetscObject)Y)->type_name);CHKERRQ(ierr);
+    ierr = MatSetSizes(*B,m,n,M,N);CHKERRQ(ierr);
+    ierr = MatPreallocatorPreallocate(preallocator,PETSC_FALSE,*B);CHKERRQ(ierr);
+    ierr = MatDestroy(&preallocator);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode MatAXPY_Basic(Mat Y,PetscScalar a,Mat X,MatStructure str)
 {
   PetscInt          i,start,end,j,ncols,m,n;
@@ -60,6 +123,7 @@ PetscErrorCode MatAXPY_Basic(Mat Y,PetscScalar a,Mat X,MatStructure str)
   PetscFunctionBegin;
   ierr = MatGetSize(X,&m,&n);CHKERRQ(ierr);
   ierr = MatGetOwnershipRange(X,&start,&end);CHKERRQ(ierr);
+  ierr = MatGetRowUpperTriangular(X);CHKERRQ(ierr);
   if (a == 1.0) {
     for (i = start; i < end; i++) {
       ierr = MatGetRow(X,i,&ncols,&row,&vals);CHKERRQ(ierr);
@@ -67,17 +131,22 @@ PetscErrorCode MatAXPY_Basic(Mat Y,PetscScalar a,Mat X,MatStructure str)
       ierr = MatRestoreRow(X,i,&ncols,&row,&vals);CHKERRQ(ierr);
     }
   } else {
-    ierr = PetscMalloc1(n+1,&val);CHKERRQ(ierr);
+    PetscInt vs = 100;
+    /* realloc if needed, as this function may be used in parallel */
+    ierr = PetscMalloc1(vs,&val);CHKERRQ(ierr);
     for (i=start; i<end; i++) {
       ierr = MatGetRow(X,i,&ncols,&row,&vals);CHKERRQ(ierr);
-      for (j=0; j<ncols; j++) {
-        val[j] = a*vals[j];
+      if (vs < ncols) {
+        vs   = PetscMin(2*ncols,n);
+        ierr = PetscRealloc(vs*sizeof(*val),&val);CHKERRQ(ierr);
       }
+      for (j=0; j<ncols; j++) val[j] = a*vals[j];
       ierr = MatSetValues(Y,1,&i,ncols,row,val,ADD_VALUES);CHKERRQ(ierr);
       ierr = MatRestoreRow(X,i,&ncols,&row,&vals);CHKERRQ(ierr);
     }
     ierr = PetscFree(val);CHKERRQ(ierr);
   }
+  ierr = MatRestoreRowUpperTriangular(X);CHKERRQ(ierr);
   ierr = MatAssemblyBegin(Y,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(Y,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -94,6 +163,8 @@ PetscErrorCode MatAXPY_BasicWithPreallocation(Mat B,Mat Y,PetscScalar a,Mat X,Ma
   PetscFunctionBegin;
   ierr = MatGetSize(X,&m,&n);CHKERRQ(ierr);
   ierr = MatGetOwnershipRange(X,&start,&end);CHKERRQ(ierr);
+  ierr = MatGetRowUpperTriangular(Y);CHKERRQ(ierr);
+  ierr = MatGetRowUpperTriangular(X);CHKERRQ(ierr);
   if (a == 1.0) {
     for (i = start; i < end; i++) {
       ierr = MatGetRow(Y,i,&ncols,&row,&vals);CHKERRQ(ierr);
@@ -105,21 +176,27 @@ PetscErrorCode MatAXPY_BasicWithPreallocation(Mat B,Mat Y,PetscScalar a,Mat X,Ma
       ierr = MatRestoreRow(X,i,&ncols,&row,&vals);CHKERRQ(ierr);
     }
   } else {
-    ierr = PetscMalloc1(n+1,&val);CHKERRQ(ierr);
+    PetscInt vs = 100;
+    /* realloc if needed, as this function may be used in parallel */
+    ierr = PetscMalloc1(vs,&val);CHKERRQ(ierr);
     for (i=start; i<end; i++) {
       ierr = MatGetRow(Y,i,&ncols,&row,&vals);CHKERRQ(ierr);
       ierr = MatSetValues(B,1,&i,ncols,row,vals,ADD_VALUES);CHKERRQ(ierr);
       ierr = MatRestoreRow(Y,i,&ncols,&row,&vals);CHKERRQ(ierr);
 
       ierr = MatGetRow(X,i,&ncols,&row,&vals);CHKERRQ(ierr);
-      for (j=0; j<ncols; j++) {
-        val[j] = a*vals[j];
+      if (vs < ncols) {
+        vs   = PetscMin(2*ncols,n);
+        ierr = PetscRealloc(vs*sizeof(*val),&val);CHKERRQ(ierr);
       }
+      for (j=0; j<ncols; j++) val[j] = a*vals[j];
       ierr = MatSetValues(B,1,&i,ncols,row,val,ADD_VALUES);CHKERRQ(ierr);
       ierr = MatRestoreRow(X,i,&ncols,&row,&vals);CHKERRQ(ierr);
     }
     ierr = PetscFree(val);CHKERRQ(ierr);
   }
+  ierr = MatRestoreRowUpperTriangular(Y);CHKERRQ(ierr);
+  ierr = MatRestoreRowUpperTriangular(X);CHKERRQ(ierr);
   ierr = MatAssemblyBegin(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -167,7 +244,7 @@ PetscErrorCode  MatShift(Mat Y,PetscScalar a)
   }
 
   ierr = PetscObjectStateIncrease((PetscObject)Y);CHKERRQ(ierr);
-#if defined(PETSC_HAVE_VIENNACL) || defined(PETSC_HAVE_VECCUDA)
+#if defined(PETSC_HAVE_VIENNACL) || defined(PETSC_HAVE_CUDA)
   if (Y->valid_GPU_matrix != PETSC_OFFLOAD_UNALLOCATED) {
     Y->valid_GPU_matrix = PETSC_OFFLOAD_CPU;
   }
