@@ -61,6 +61,8 @@ typedef struct {
 extern PetscErrorCode FormJacobian(SNES,Vec,Mat,Mat,void*);
 extern PetscErrorCode FormFunction(SNES,Vec,Vec,void*);
 extern PetscErrorCode FormInitialGuess(AppCtx*,Vec);
+extern PetscErrorCode ConvergenceTest(KSP,PetscInt,PetscReal,KSPConvergedReason*,void*);
+extern PetscErrorCode ConvergenceDestroy(void*);
 
 int main(int argc,char **argv)
 {
@@ -73,7 +75,9 @@ int main(int argc,char **argv)
   PetscMPIInt    size;
   PetscReal      bratu_lambda_max = 6.81,bratu_lambda_min = 0.,history[50];
   MatFDColoring  fdcoloring;
-  PetscBool      matrix_free = PETSC_FALSE,flg,fd_coloring = PETSC_FALSE;
+  PetscBool      matrix_free = PETSC_FALSE,flg,fd_coloring = PETSC_FALSE, use_convergence_test = PETSC_FALSE;
+  KSP            ksp;
+  PetscInt       *testarray;
 
   ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
@@ -88,6 +92,7 @@ int main(int argc,char **argv)
   ierr    = PetscOptionsGetReal(NULL,NULL,"-par",&user.param,NULL);CHKERRQ(ierr);
   if (user.param >= bratu_lambda_max || user.param <= bratu_lambda_min) SETERRQ(PETSC_COMM_SELF,1,"Lambda is out of range");
   N = user.mx*user.my;
+  ierr    = PetscOptionsGetBool(NULL,NULL,"-use_convergence_test",&use_convergence_test,NULL);CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Create nonlinear solver context
@@ -95,6 +100,8 @@ int main(int argc,char **argv)
 
   ierr = SNESCreate(PETSC_COMM_WORLD,&snes);CHKERRQ(ierr);
 
+
+
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Create vector data structures; set function evaluation routine
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -210,6 +217,17 @@ int main(int argc,char **argv)
   */
   ierr = SNESSetConvergenceHistory(snes,history,hist_its,50,PETSC_TRUE);CHKERRQ(ierr);
 
+  /*
+      Add a user provided convergence test; this is to test that SNESNEWTONTR properly calls the
+      user provided test before the specialized test. The convergence context is just an array to
+      test that it gets properly freed at the end
+  */
+  if (use_convergence_test) {
+    ierr = SNESGetKSP(snes,&ksp);CHKERRQ(ierr);
+    ierr = PetscMalloc1(5,&testarray);CHKERRQ(ierr);
+    ierr = KSPSetConvergenceTest(ksp,ConvergenceTest,testarray,ConvergenceDestroy);CHKERRQ(ierr);
+  }
+
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Evaluate initial guess; then solve nonlinear system
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -444,6 +462,29 @@ PetscErrorCode FormJacobian(SNES snes,Vec X,Mat J,Mat jac,void *ptr)
   return 0;
 }
 
+PetscErrorCode ConvergenceTest(KSP ksp,PetscInt it,PetscReal nrm,KSPConvergedReason *reason,void *ctx)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  *reason = KSP_CONVERGED_ITERATING;
+  if (it > 1) {
+    *reason = KSP_CONVERGED_ITS;
+    ierr = PetscInfo(NULL,"User provided convergence test returning after 2 iterations\n");CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode ConvergenceDestroy(void* ctx)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscInfo(NULL,"User provided convergence destroy called\n");CHKERRQ(ierr);
+  ierr = PetscFree(ctx);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 
 
 /*TEST
@@ -458,8 +499,22 @@ PetscErrorCode FormJacobian(SNES snes,Vec X,Mat J,Mat jac,void *ptr)
       suffix: 2
       args: -snes_monitor_short -snes_type newtontr -ksp_gmres_cgs_refinement_type refine_always
 
+   test:
+      suffix: 2a
+      filter: grep -i KSPConvergedDefault > /dev/null && echo "Found KSPConvergedDefault"
+      args: -snes_monitor_short -snes_type newtontr -ksp_gmres_cgs_refinement_type refine_always -info
+      requires: define(PETSC_USE_LOG)
+
+   test:
+      suffix: 2b
+      filter: grep -i  "User provided convergence test" > /dev/null  && echo "Found User provided convergence test"
+      args: -snes_monitor_short -snes_type newtontr -ksp_gmres_cgs_refinement_type refine_always -use_convergence_test -info
+      requires: define(PETSC_USE_LOG)
+
    test:
       suffix: 3
       args: -snes_monitor_short -mat_coloring_type sl -snes_fd_coloring -mx 8 -my 11 -ksp_gmres_cgs_refinement_type refine_always
 
 TEST*/
+
+
