@@ -59,28 +59,33 @@ There are two compile-time options:
 #import <PETSc/petscsnes.h>
 #import <PETSc/petsc/private/dmdaimpl.h>     /* There is not yet a public interface to manipulate dm->ops */
 #else
+
 #include <petscsnes.h>
 #include <petsc/private/dmdaimpl.h>     /* There is not yet a public interface to manipulate dm->ops */
 #endif
 #include <ctype.h>              /* toupper() */
 
-#if !defined __STDC_VERSION__ || __STDC_VERSION__ < 199901L
-#  if defined __cplusplus       /* C++ restrict is nonstandard and compilers have inconsistent rules about where it can be used */
-#    define restrict
-#  else
-#    define restrict PETSC_RESTRICT
-#  endif
+#if defined(__cplusplus) || defined (PETSC_HAVE_WINDOWS_COMPILERS) || defined (__PGI)
+/*  c++ cannot handle  [_restrict_] notation like C does */
+#undef PETSC_RESTRICT
+#define PETSC_RESTRICT
 #endif
+
 #if defined __SSE2__
 #  include <emmintrin.h>
 #endif
 
 /* The SSE2 kernels are only for PetscScalar=double on architectures that support it */
-#define USE_SSE2_KERNELS (!defined NO_SSE2                              \
-                          && !defined PETSC_USE_COMPLEX                 \
-                          && !defined PETSC_USE_REAL_SINGLE             \
-                          && !defined PETSC_USE_REAL___FLOAT128         \
-                          && defined __SSE2__)
+#if !defined NO_SSE2                           \
+     && !defined PETSC_USE_COMPLEX             \
+     && !defined PETSC_USE_REAL_SINGLE         \
+     && !defined PETSC_USE_REAL___FLOAT128     \
+     && !defined PETSC_USE_REAL___FP16         \
+     && defined __SSE2__
+#define USE_SSE2_KERNELS 1
+#else
+#define USE_SSE2_KERNELS 0
+#endif
 
 static PetscClassId THI_CLASSID;
 
@@ -175,8 +180,6 @@ static const PetscReal QuadQDeriv[4][4][2] = {
     (n)[3] = (x)[i][j+1];                       \
   } while (0)
 
-static PetscScalar Sqr(PetscScalar a) {return a*a;}
-
 static void HexGrad(const PetscReal dphi[][3],const PetscReal zn[],PetscReal dz[])
 {
   PetscInt i;
@@ -188,7 +191,7 @@ static void HexGrad(const PetscReal dphi[][3],const PetscReal zn[],PetscReal dz[
   }
 }
 
-static void HexComputeGeometry(PetscInt q,PetscReal hx,PetscReal hy,const PetscReal dz[restrict],PetscReal phi[restrict],PetscReal dphi[restrict][3],PetscReal *restrict jw)
+static void HexComputeGeometry(PetscInt q,PetscReal hx,PetscReal hy,const PetscReal dz[PETSC_RESTRICT],PetscReal phi[PETSC_RESTRICT],PetscReal dphi[PETSC_RESTRICT][3],PetscReal *PETSC_RESTRICT jw)
 {
   const PetscReal jac[3][3]  = {{hx/2,0,0}, {0,hy/2,0}, {dz[0],dz[1],dz[2]}};
   const PetscReal ijac[3][3] = {{1/jac[0][0],0,0}, {0,1/jac[1][1],0}, {-jac[2][0]/(jac[0][0]*jac[2][2]),-jac[2][1]/(jac[1][1]*jac[2][2]),1/jac[2][2]}};
@@ -387,8 +390,6 @@ static void PRangeClear(PRange *p)
   p->cmax = p->max = -1e100;
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PRangeMinMax"
 static PetscErrorCode PRangeMinMax(PRange *p,PetscReal min,PetscReal max)
 {
 
@@ -400,8 +401,6 @@ static PetscErrorCode PRangeMinMax(PRange *p,PetscReal min,PetscReal max)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THIDestroy"
 static PetscErrorCode THIDestroy(THI *thi)
 {
   PetscErrorCode ierr;
@@ -415,8 +414,6 @@ static PetscErrorCode THIDestroy(THI *thi)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THICreate"
 static PetscErrorCode THICreate(MPI_Comm comm,THI *inthi)
 {
   static PetscBool registered = PETSC_FALSE;
@@ -446,7 +443,7 @@ static PetscErrorCode THICreate(MPI_Comm comm,THI *inthi)
   }
   ierr          = PetscOptionsEnd();CHKERRQ(ierr);
   units->Pascal = units->kilogram / (units->meter * PetscSqr(units->second));
-  units->year   = 31556926. * units->second, /* seconds per year */
+  units->year   = 31556926. * units->second; /* seconds per year */
 
   thi->Lx              = 10.e3;
   thi->Ly              = 10.e3;
@@ -559,8 +556,6 @@ static PetscErrorCode THICreate(MPI_Comm comm,THI *inthi)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THIInitializePrm"
 static PetscErrorCode THIInitializePrm(THI thi,DM da2prm,Vec prm)
 {
   PrmNode        **p;
@@ -581,8 +576,6 @@ static PetscErrorCode THIInitializePrm(THI thi,DM da2prm,Vec prm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THISetUpDM"
 static PetscErrorCode THISetUpDM(THI thi,DM dm)
 {
   PetscErrorCode  ierr;
@@ -600,6 +593,7 @@ static PetscErrorCode THISetUpDM(THI thi,DM dm)
   ierr  = DMGetCoarsenLevel(dm,&coarsenlevel);CHKERRQ(ierr);
   level = refinelevel - coarsenlevel;
   ierr  = DMDACreate2d(PetscObjectComm((PetscObject)thi),DM_BOUNDARY_PERIODIC,DM_BOUNDARY_PERIODIC,st,My,Mx,my,mx,sizeof(PrmNode)/sizeof(PetscScalar),s,0,0,&da2prm);CHKERRQ(ierr);
+  ierr  = DMSetUp(da2prm);CHKERRQ(ierr);
   ierr  = DMCreateLocalVector(da2prm,&X);CHKERRQ(ierr);
   {
     PetscReal Lx = thi->Lx / thi->units->meter,Ly = thi->Ly / thi->units->meter,Lz = thi->Lz / thi->units->meter;
@@ -623,8 +617,6 @@ static PetscErrorCode THISetUpDM(THI thi,DM dm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCoarsenHook_THI"
 static PetscErrorCode DMCoarsenHook_THI(DM dmf,DM dmc,void *ctx)
 {
   THI            thi = (THI)ctx;
@@ -640,8 +632,6 @@ static PetscErrorCode DMCoarsenHook_THI(DM dmf,DM dmc,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMRefineHook_THI"
 static PetscErrorCode DMRefineHook_THI(DM dmc,DM dmf,void *ctx)
 {
   THI            thi = (THI)ctx;
@@ -656,8 +646,6 @@ static PetscErrorCode DMRefineHook_THI(DM dmc,DM dmf,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THIDAGetPrm"
 static PetscErrorCode THIDAGetPrm(DM da,PrmNode ***prm)
 {
   PetscErrorCode ierr;
@@ -673,8 +661,6 @@ static PetscErrorCode THIDAGetPrm(DM da,PrmNode ***prm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THIDARestorePrm"
 static PetscErrorCode THIDARestorePrm(DM da,PrmNode ***prm)
 {
   PetscErrorCode ierr;
@@ -690,8 +676,6 @@ static PetscErrorCode THIDARestorePrm(DM da,PrmNode ***prm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THIInitial"
 static PetscErrorCode THIInitial(SNES snes,Vec X,void *ctx)
 {
   THI            thi;
@@ -727,7 +711,7 @@ static PetscErrorCode THIInitial(SNES snes,Vec X,void *ctx)
   PetscFunctionReturn(0);
 }
 
-static void PointwiseNonlinearity(THI thi,const Node n[restrict],const PetscReal phi[restrict],PetscReal dphi[restrict][3],PetscScalar *restrict u,PetscScalar *restrict v,PetscScalar du[restrict],PetscScalar dv[restrict],PetscReal *eta,PetscReal *deta)
+static void PointwiseNonlinearity(THI thi,const Node n[PETSC_RESTRICT],const PetscReal phi[PETSC_RESTRICT],PetscReal dphi[PETSC_RESTRICT][3],PetscScalar *PETSC_RESTRICT u,PetscScalar *PETSC_RESTRICT v,PetscScalar du[PETSC_RESTRICT],PetscScalar dv[PETSC_RESTRICT],PetscReal *eta,PetscReal *deta)
 {
   PetscInt    l,ll;
   PetscScalar gam;
@@ -744,7 +728,7 @@ static void PointwiseNonlinearity(THI thi,const Node n[restrict],const PetscReal
       dv[ll] += dphi[l][ll] * n[l].v;
     }
   }
-  gam = Sqr(du[0]) + Sqr(dv[1]) + du[0]*dv[1] + 0.25*Sqr(du[1]+dv[0]) + 0.25*Sqr(du[2]) + 0.25*Sqr(dv[2]);
+  gam = PetscSqr(du[0]) + PetscSqr(dv[1]) + du[0]*dv[1] + 0.25*PetscSqr(du[1]+dv[0]) + 0.25*PetscSqr(du[2]) + 0.25*PetscSqr(dv[2]);
   THIViscosity(thi,PetscRealPart(gam),eta,deta);
 }
 
@@ -765,12 +749,10 @@ static void PointwiseNonlinearity2D(THI thi,Node n[],PetscReal phi[],PetscReal d
       dv[ll] += dphi[l][ll] * n[l].v;
     }
   }
-  gam = Sqr(du[0]) + Sqr(dv[1]) + du[0]*dv[1] + 0.25*Sqr(du[1]+dv[0]);
+  gam = PetscSqr(du[0]) + PetscSqr(dv[1]) + du[0]*dv[1] + 0.25*PetscSqr(du[1]+dv[0]);
   THIViscosity(thi,PetscRealPart(gam),eta,deta);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THIFunctionLocal"
 static PetscErrorCode THIFunctionLocal(DMDALocalInfo *info,Node ***x,Node ***f,THI thi)
 {
   PetscInt       xs,ys,xm,ym,zm,i,j,k,q,l;
@@ -876,8 +858,6 @@ static PetscErrorCode THIFunctionLocal(DMDALocalInfo *info,Node ***x,Node ***f,T
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THIMatrixStatistics"
 static PetscErrorCode THIMatrixStatistics(THI thi,Mat B,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -898,8 +878,6 @@ static PetscErrorCode THIMatrixStatistics(THI thi,Mat B,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THISurfaceStatistics"
 static PetscErrorCode THISurfaceStatistics(DM da,Vec X,PetscReal *min,PetscReal *max,PetscReal *mean)
 {
   PetscErrorCode ierr;
@@ -929,8 +907,6 @@ static PetscErrorCode THISurfaceStatistics(DM da,Vec X,PetscReal *min,PetscReal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THISolveStatistics"
 static PetscErrorCode THISolveStatistics(THI thi,SNES snes,PetscInt coarsened,const char name[])
 {
   MPI_Comm       comm;
@@ -976,6 +952,7 @@ static PetscErrorCode THISolveStatistics(THI thi,SNES snes,PetscInt coarsened,co
       min[j] *= thi->units->year / thi->units->meter;
       max[j] *= thi->units->year / thi->units->meter;
     }
+    if (min[0] == 0.0) min[0] = 0.0;
     ierr = PetscPrintf(comm,"|X|_2 %g   %g <= u <=  %g   %g <= v <=  %g   %g <= c <=  %g \n",(double)nrm2,(double)min[0],(double)max[0],(double)min[1],(double)max[1],(double)min[2],(double)max[2]);CHKERRQ(ierr);
     {
       PetscReal umin,umax,umean;
@@ -992,8 +969,6 @@ static PetscErrorCode THISolveStatistics(THI thi,SNES snes,PetscInt coarsened,co
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THIJacobianLocal_2D"
 static PetscErrorCode THIJacobianLocal_2D(DMDALocalInfo *info,Node **x,Mat J,Mat B,THI thi)
 {
   PetscInt       xs,ys,xm,ym,i,j,q,l,ll;
@@ -1068,8 +1043,6 @@ static PetscErrorCode THIJacobianLocal_2D(DMDALocalInfo *info,Node **x,Mat J,Mat
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THIJacobianLocal_3D"
 static PetscErrorCode THIJacobianLocal_3D(DMDALocalInfo *info,Node ***x,Mat B,THI thi,THIAssemblyMode amode)
 {
   PetscInt       xs,ys,xm,ym,zm,i,j,k,q,l,ll;
@@ -1087,6 +1060,7 @@ static PetscErrorCode THIJacobianLocal_3D(DMDALocalInfo *info,Node ***x,Mat B,TH
   hy = thi->Ly / info->my;
 
   ierr = MatZeroEntries(B);CHKERRQ(ierr);
+  ierr = MatSetOption(B,MAT_SUBSET_OFF_PROC_ENTRIES,PETSC_TRUE);CHKERRQ(ierr);
   ierr = THIDAGetPrm(info->da,&prm);CHKERRQ(ierr);
 
   for (i=xs; i<xs+xm; i++) {
@@ -1115,7 +1089,7 @@ static PetscErrorCode THIJacobianLocal_3D(DMDALocalInfo *info,Node ***x,Mat B,TH
           jw /= thi->rhog;      /* residuals are scaled by this factor */
           if (q == 0) etabase = eta;
           for (l=ls; l<8; l++) { /* test functions */
-            const PetscReal *restrict dp = dphi[l];
+            const PetscReal *PETSC_RESTRICT dp = dphi[l];
 #if USE_SSE2_KERNELS
             /* gcc (up to my 4.5 snapshot) is really bad at hoisting intrinsics so we do it manually */
             __m128d
@@ -1139,7 +1113,7 @@ static PetscErrorCode THIJacobianLocal_3D(DMDALocalInfo *info,Node ***x,Mat B,TH
 #else
             for (ll=l; ll<8; ll++) {
 #endif
-              const PetscReal *restrict dpl = dphi[ll];
+              const PetscReal *PETSC_RESTRICT dpl = dphi[ll];
               if (amode == THIASSEMBLY_TRIDIAGONAL && (l-ll)%4) continue; /* these entries would not be inserted */
 #if !USE_SSE2_KERNELS
               /* The analytic Jacobian in nice, easy-to-read form */
@@ -1267,8 +1241,6 @@ static PetscErrorCode THIJacobianLocal_3D(DMDALocalInfo *info,Node ***x,Mat B,TH
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THIJacobianLocal_3D_Full"
 static PetscErrorCode THIJacobianLocal_3D_Full(DMDALocalInfo *info,Node ***x,Mat A,Mat B,THI thi)
 {
   PetscErrorCode ierr;
@@ -1278,8 +1250,6 @@ static PetscErrorCode THIJacobianLocal_3D_Full(DMDALocalInfo *info,Node ***x,Mat
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THIJacobianLocal_3D_Tridiagonal"
 static PetscErrorCode THIJacobianLocal_3D_Tridiagonal(DMDALocalInfo *info,Node ***x,Mat A,Mat B,THI thi)
 {
   PetscErrorCode ierr;
@@ -1289,8 +1259,6 @@ static PetscErrorCode THIJacobianLocal_3D_Tridiagonal(DMDALocalInfo *info,Node *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMRefineHierarchy_THI"
 static PetscErrorCode DMRefineHierarchy_THI(DM dac0,PetscInt nlevels,DM hierarchy[])
 {
   PetscErrorCode  ierr;
@@ -1314,6 +1282,7 @@ static PetscErrorCode DMRefineHierarchy_THI(DM dac0,PetscInt nlevels,DM hierarch
 
   /* Creates a 3D DMDA with the same map-plane layout as the 2D one, with contiguous columns */
   ierr = DMDACreate3d(PetscObjectComm((PetscObject)dac),DM_BOUNDARY_NONE,DM_BOUNDARY_PERIODIC,DM_BOUNDARY_PERIODIC,st,thi->zlevels,N,M,1,n,m,dof,s,NULL,NULL,NULL,&daf);CHKERRQ(ierr);
+  ierr = DMSetUp(daf);CHKERRQ(ierr);
 
   daf->ops->creatematrix        = dac->ops->creatematrix;
   daf->ops->createinterpolation = dac->ops->createinterpolation;
@@ -1329,8 +1298,6 @@ static PetscErrorCode DMRefineHierarchy_THI(DM dac0,PetscInt nlevels,DM hierarch
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCreateInterpolation_DA_THI"
 static PetscErrorCode DMCreateInterpolation_DA_THI(DM dac,DM daf,Mat *A,Vec *scale)
 {
   PetscErrorCode ierr;
@@ -1377,8 +1344,6 @@ static PetscErrorCode DMCreateInterpolation_DA_THI(DM dac,DM daf,Mat *A,Vec *sca
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCreateMatrix_THI_Tridiagonal"
 static PetscErrorCode DMCreateMatrix_THI_Tridiagonal(DM da,Mat *J)
 {
   PetscErrorCode         ierr;
@@ -1408,8 +1373,6 @@ static PetscErrorCode DMCreateMatrix_THI_Tridiagonal(DM da,Mat *J)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "THIDAVecView_VTK_XML"
 static PetscErrorCode THIDAVecView_VTK_XML(THI thi,DM da,Vec X,const char filename[])
 {
   const PetscInt    dof   = 2;
@@ -1499,8 +1462,6 @@ static PetscErrorCode THIDAVecView_VTK_XML(THI thi,DM da,Vec X,const char filena
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char *argv[])
 {
   MPI_Comm       comm;
@@ -1509,7 +1470,7 @@ int main(int argc,char *argv[])
   DM             da;
   SNES           snes;
 
-  ierr = PetscInitialize(&argc,&argv,0,help);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&argv,0,help);if (ierr) return ierr;
   comm = PETSC_COMM_WORLD;
 
   ierr = THICreate(comm,&thi);CHKERRQ(ierr);
@@ -1528,14 +1489,17 @@ int main(int argc,char *argv[])
     }
     ierr = PetscOptionsEnd();CHKERRQ(ierr);
     if (thi->coarse2d) {
-      ierr = DMDACreate2d(comm,DM_BOUNDARY_PERIODIC,DM_BOUNDARY_PERIODIC,DMDA_STENCIL_BOX,-N,-M,PETSC_DETERMINE,PETSC_DETERMINE,sizeof(Node)/sizeof(PetscScalar),1,0,0,&da);CHKERRQ(ierr);
-
+      ierr = DMDACreate2d(comm,DM_BOUNDARY_PERIODIC,DM_BOUNDARY_PERIODIC,DMDA_STENCIL_BOX,N,M,PETSC_DETERMINE,PETSC_DETERMINE,sizeof(Node)/sizeof(PetscScalar),1,0,0,&da);CHKERRQ(ierr);
+      ierr = DMSetFromOptions(da);CHKERRQ(ierr);
+      ierr = DMSetUp(da);CHKERRQ(ierr);
       da->ops->refinehierarchy     = DMRefineHierarchy_THI;
       da->ops->createinterpolation = DMCreateInterpolation_DA_THI;
 
       ierr = PetscObjectCompose((PetscObject)da,"THI",(PetscObject)thi);CHKERRQ(ierr);
     } else {
-      ierr = DMDACreate3d(comm,DM_BOUNDARY_NONE,DM_BOUNDARY_PERIODIC,DM_BOUNDARY_PERIODIC, DMDA_STENCIL_BOX,-P,-N,-M,1,PETSC_DETERMINE,PETSC_DETERMINE,sizeof(Node)/sizeof(PetscScalar),1,0,0,0,&da);CHKERRQ(ierr);
+      ierr = DMDACreate3d(comm,DM_BOUNDARY_NONE,DM_BOUNDARY_PERIODIC,DM_BOUNDARY_PERIODIC, DMDA_STENCIL_BOX,P,N,M,1,PETSC_DETERMINE,PETSC_DETERMINE,sizeof(Node)/sizeof(PetscScalar),1,0,0,0,&da);CHKERRQ(ierr);
+      ierr = DMSetFromOptions(da);CHKERRQ(ierr);
+      ierr = DMSetUp(da);CHKERRQ(ierr);
     }
     ierr = DMDASetFieldName(da,0,"x-velocity");CHKERRQ(ierr);
     ierr = DMDASetFieldName(da,1,"y-velocity");CHKERRQ(ierr);
@@ -1574,7 +1538,7 @@ int main(int argc,char *argv[])
   {
     PetscBool flg;
     char      filename[PETSC_MAX_PATH_LEN] = "";
-    ierr = PetscOptionsGetString(NULL,"-o",filename,sizeof(filename),&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsGetString(NULL,NULL,"-o",filename,sizeof(filename),&flg);CHKERRQ(ierr);
     if (flg) {
       Vec X;
       DM  dm;
@@ -1588,5 +1552,41 @@ int main(int argc,char *argv[])
   ierr = SNESDestroy(&snes);CHKERRQ(ierr);
   ierr = THIDestroy(&thi);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
+
+
+/*TEST
+
+   build:
+      requires: c99
+
+   test:
+      args: -M 6 -P 4 -da_refine 1 -snes_monitor_short -snes_converged_reason -ksp_monitor_short -ksp_converged_reason -thi_mat_type sbaij -ksp_type fgmres -pc_type mg -pc_mg_type full -mg_levels_ksp_type gmres -mg_levels_ksp_max_it 1 -mg_levels_pc_type icc
+      requires: !single
+
+   test:
+      suffix: 2
+      nsize: 2
+      args: -M 6 -P 4 -thi_hom z -snes_monitor_short -snes_converged_reason -ksp_monitor_short -ksp_converged_reason -thi_mat_type sbaij -ksp_type fgmres -pc_type mg -pc_mg_type full -mg_levels_ksp_type gmres -mg_levels_ksp_max_it 1 -mg_levels_pc_type asm -mg_levels_pc_asm_blocks 6 -mg_levels_0_pc_type redundant -snes_grid_sequence 1 -mat_partitioning_type current -ksp_atol -1
+      requires: !single
+
+   test:
+      suffix: 3
+      nsize: 3
+      args: -M 7 -P 4 -thi_hom z -da_refine 1 -snes_monitor_short -snes_converged_reason -ksp_monitor_short -ksp_converged_reason -thi_mat_type baij -ksp_type fgmres -pc_type mg -pc_mg_type full -mg_levels_pc_asm_type restrict -mg_levels_pc_type asm -mg_levels_pc_asm_blocks 9 -mg_levels_ksp_type gmres -mg_levels_ksp_max_it 1 -mat_partitioning_type current
+      requires: !single
+
+   test:
+      suffix: 4
+      nsize: 6
+      args: -M 4 -P 2 -da_refine_hierarchy_x 1,1,3 -da_refine_hierarchy_y 2,2,1 -da_refine_hierarchy_z 2,2,1 -snes_grid_sequence 3 -ksp_converged_reason -ksp_type fgmres -ksp_rtol 1e-2 -pc_type mg -mg_levels_ksp_type gmres -mg_levels_ksp_max_it 1 -mg_levels_pc_type bjacobi -mg_levels_1_sub_pc_type cholesky -pc_mg_type multiplicative -snes_converged_reason -snes_stol 1e-12 -thi_L 80e3 -thi_alpha 0.05 -thi_friction_m 1 -thi_hom x -snes_view -mg_levels_0_pc_type redundant -mg_levels_0_ksp_type preonly -ksp_atol -1
+      requires: !single
+
+   test:
+      suffix: 5
+      nsize: 6
+      args: -M 12 -P 5 -snes_monitor_short -ksp_converged_reason -pc_type asm -pc_asm_type restrict -dm_mat_type {{aij baij sbaij}}
+      requires: !single
+
+TEST*/
