@@ -862,7 +862,7 @@ PetscErrorCode NonlinearGS(SNES snes,Vec X, Vec B, void *ctx)
    test:
      suffix: 5_aspin
      nsize: 4
-     args: -snes_monitor_short -ksp_monitor_short -snes_converged_reason -da_refine 4 -da_overlap 3 -snes_type aspin
+     args: -snes_monitor_short -ksp_monitor_short -snes_converged_reason -da_refine 4 -da_overlap 3 -snes_type aspin -snes_view
 
    test:
      suffix: 5_broyden
@@ -942,4 +942,9 @@ PetscErrorCode NonlinearGS(SNES snes,Vec X, Vec B, void *ctx)
      nsize: 4
      args: -snes_converged_reason -ksp_converged_reason -da_grid_x 129 -da_grid_y 129 -pc_type mg -pc_mg_levels 8 -mg_levels_ksp_type chebyshev -mg_levels_ksp_chebyshev_esteig 0,0.5,0,1.1 -mg_levels_ksp_max_it 2
 
+   test:
+     requires: complex !single
+     suffix: complex
+     args: -snes_mf_operator -mat_mffd_complex -snes_monitor
+
 TEST*/
