@@ -1,7 +1,5 @@
 #include <../src/snes/impls/gs/gsimpl.h>      /*I "petscsnes.h"  I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNGSSetTolerances"
 /*@
    SNESNGSSetTolerances - Sets various parameters used in convergence tests.
 
@@ -53,8 +51,6 @@ PetscErrorCode  SNESNGSSetTolerances(SNES snes,PetscReal abstol,PetscReal rtol,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNGSGetTolerances"
 /*@
    SNESNGSGetTolerances - Gets various parameters used in convergence tests.
 
@@ -90,8 +86,6 @@ PetscErrorCode  SNESNGSGetTolerances(SNES snes,PetscReal *atol,PetscReal *rtol,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNGSSetSweeps"
 /*@
    SNESNGSSetSweeps - Sets the number of sweeps of GS to use.
 
@@ -103,7 +97,7 @@ PetscErrorCode  SNESNGSGetTolerances(SNES snes,PetscReal *atol,PetscReal *rtol,P
 
 .keywords: SNES, nonlinear, set, Gauss-Siedel
 
-.seealso: SNESSetNGS(), SNESGetNGS(), SNESSetPC(), SNESNGSGetSweeps()
+.seealso: SNESSetNGS(), SNESGetNGS(), SNESSetNPC(), SNESNGSGetSweeps()
 @*/
 
 PetscErrorCode SNESNGSSetSweeps(SNES snes, PetscInt sweeps)
@@ -116,8 +110,6 @@ PetscErrorCode SNESNGSSetSweeps(SNES snes, PetscInt sweeps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNGSGetSweeps"
 /*@
    SNESNGSGetSweeps - Gets the number of sweeps GS will use.
 
@@ -131,7 +123,7 @@ PetscErrorCode SNESNGSSetSweeps(SNES snes, PetscInt sweeps)
 
 .keywords: SNES, nonlinear, set, Gauss-Siedel
 
-.seealso: SNESSetNGS(), SNESGetNGS(), SNESSetPC(), SNESNGSSetSweeps()
+.seealso: SNESSetNGS(), SNESGetNGS(), SNESSetNPC(), SNESNGSSetSweeps()
 @*/
 PetscErrorCode SNESNGSGetSweeps(SNES snes, PetscInt * sweeps)
 {
@@ -143,26 +135,16 @@ PetscErrorCode SNESNGSGetSweeps(SNES snes, PetscInt * sweeps)
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "SNESDefaultApplyNGS"
-PetscErrorCode SNESDefaultApplyNGS(SNES snes,Vec X,Vec F,void *ctx)
-{
-  PetscFunctionBegin;
-  /* see if there's a coloring on the DM */
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "SNESReset_NGS"
 PetscErrorCode SNESReset_NGS(SNES snes)
 {
+  SNES_NGS       *gs = (SNES_NGS*)snes->data;
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
+  ierr = ISColoringDestroy(&gs->coloring);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESDestroy_NGS"
 PetscErrorCode SNESDestroy_NGS(SNES snes)
 {
   PetscErrorCode ierr;
@@ -173,8 +155,6 @@ PetscErrorCode SNESDestroy_NGS(SNES snes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetUp_NGS"
 PetscErrorCode SNESSetUp_NGS(SNES snes)
 {
   PetscErrorCode ierr;
@@ -188,9 +168,7 @@ PetscErrorCode SNESSetUp_NGS(SNES snes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetFromOptions_NGS"
-PetscErrorCode SNESSetFromOptions_NGS(PetscOptions *PetscOptionsObject,SNES snes)
+PetscErrorCode SNESSetFromOptions_NGS(PetscOptionItems *PetscOptionsObject,SNES snes)
 {
   SNES_NGS       *gs = (SNES_NGS*)snes->data;
   PetscErrorCode ierr;
@@ -213,28 +191,36 @@ PetscErrorCode SNESSetFromOptions_NGS(PetscOptions *PetscOptionsObject,SNES snes
     ierr = SNESNGSSetTolerances(snes,atol,rtol,stol,max_its);CHKERRQ(ierr);
   }
   flg  = PETSC_FALSE;
-  ierr = PetscOptionsBool("-snes_ngs_secant","Use pointwise secant local Jacobian approximation","",flg,&flg,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-snes_ngs_secant","Use finite difference secant approximation with coloring","",flg,&flg,NULL);CHKERRQ(ierr);
   if (flg) {
     ierr = SNESSetNGS(snes,SNESComputeNGSDefaultSecant,NULL);CHKERRQ(ierr);
-    ierr = PetscInfo(snes,"Setting default finite difference coloring Jacobian matrix\n");CHKERRQ(ierr);
+    ierr = PetscInfo(snes,"Setting default finite difference secant approximation with coloring\n");CHKERRQ(ierr);
   }
   ierr = PetscOptionsReal("-snes_ngs_secant_h","Differencing parameter for secant search","",gs->h,&gs->h,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsBool("-snes_ngs_secant_mat_coloring","Use the Jacobian coloring for the secant GS","",gs->secant_mat,&gs->secant_mat,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-snes_ngs_secant_mat_coloring","Use the graph coloring of the Jacobian for the secant GS","",gs->secant_mat,&gs->secant_mat,&flg);CHKERRQ(ierr);
 
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESView_NGS"
 PetscErrorCode SNESView_NGS(SNES snes, PetscViewer viewer)
 {
+  PetscErrorCode ierr;
+  PetscErrorCode (*f)(SNES,Vec,Vec,void*);
+  SNES_NGS       *gs = (SNES_NGS*)snes->data;
+  PetscBool      iascii;
+
   PetscFunctionBegin;
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
+  if (iascii) {
+    ierr = DMSNESGetNGS(snes->dm,&f,NULL);CHKERRQ(ierr);
+    if (f == SNESComputeNGSDefaultSecant) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  Use finite difference secant approximation with coloring with h = %g \n",(double)gs->h);CHKERRQ(ierr);
+    }
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSolve_NGS"
 PetscErrorCode SNESSolve_NGS(SNES snes)
 {
   Vec              F;
@@ -247,9 +233,7 @@ PetscErrorCode SNESSolve_NGS(SNES snes)
 
   PetscFunctionBegin;
 
-  if (snes->xl || snes->xu || snes->ops->computevariablebounds) {
-    SETERRQ1(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_WRONGSTATE, "SNES solver %s does not support bounds", ((PetscObject)snes)->type_name);
-  }
+  if (snes->xl || snes->xu || snes->ops->computevariablebounds) SETERRQ1(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_WRONGSTATE, "SNES solver %s does not support bounds", ((PetscObject)snes)->type_name);
 
   ierr = PetscCitationsRegister(SNESCitation,&SNEScite);CHKERRQ(ierr);
   X = snes->vec_sol;
@@ -324,19 +308,35 @@ PetscErrorCode SNESSolve_NGS(SNES snes)
 }
 
 /*MC
-  SNESNGS - Just calls the user-provided solution routine provided with SNESSetNGS()
+  SNESNGS - Either calls the user-provided solution routine provided with SNESSetNGS() or does a finite difference secant approximation
+            using coloring.
 
    Level: advanced
 
+  Options Database:
++   -snes_ngs_sweeps - Number of sweeps of GS to apply
+.    -snes_ngs_atol - Absolute residual tolerance for GS iteration
+.    -snes_ngs_rtol - Relative residual tolerance for GS iteration
+.    -snes_ngs_stol - Absolute update tolerance for GS iteration
+.    -snes_ngs_max_it - Maximum number of sweeps of GS to apply
+.    -snes_ngs_secant - Use pointwise secant local Jacobian approximation with coloring instead of user provided Gauss-Seidel routine, this is
+                        used by default if no user provided Gauss-Seidel routine is available. Requires either that a DM that can compute a coloring
+                        is available or a Jacobian sparse matrix is provided (from which to get the coloring).
+.    -snes_ngs_secant_h - Differencing parameter for secant approximation
+-    -snes_ngs_secant_mat_coloring - Use the graph coloring of the Jacobian for the secant GS even if a DM is available.
+
+
   Notes:
-  the Gauss-Seidel smoother is inherited through composition.  If a solver has been created with SNESGetPC(), it will have
+  the Gauss-Seidel smoother is inherited through composition.  If a solver has been created with SNESGetNPC(), it will have
   its parent's Gauss-Seidel routine associated with it.
 
-.seealso: SNESCreate(), SNES, SNESSetType(), SNESSetNGS(), SNESType (for list of available types)
+   References:
+.  1. - Peter R. Brune, Matthew G. Knepley, Barry F. Smith, and Xuemin Tu, "Composing Scalable Nonlinear Algebraic Solvers",
+   SIAM Review, 57(4), 2015
+
+.seealso: SNESCreate(), SNES, SNESSetType(), SNESSetNGS(), SNESType (for list of available types), SNESNGSSetSweeps(), SNESNGSSetTolerances()
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESCreate_NGS"
 PETSC_EXTERN PetscErrorCode SNESCreate_NGS(SNES snes)
 {
   SNES_NGS        *gs;
@@ -351,7 +351,9 @@ PETSC_EXTERN PetscErrorCode SNESCreate_NGS(SNES snes)
   snes->ops->reset          = SNESReset_NGS;
 
   snes->usesksp = PETSC_FALSE;
-  snes->usespc  = PETSC_FALSE;
+  snes->usesnpc = PETSC_FALSE;
+
+  snes->alwayscomputesfinalresidual = PETSC_FALSE;
 
   if (!snes->tolerancesset) {
     snes->max_its   = 10000;
