@@ -133,8 +133,7 @@ PetscErrorCode SNESMSRegisterDestroy(void)
 
 /*@C
   SNESMSInitializePackage - This function initializes everything in the SNESMS package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to SNESCreate_MS()
-  when using static libraries.
+  from SNESInitializePackage().
 
   Level: developer
 
@@ -293,6 +292,7 @@ static PetscErrorCode SNESSolve_MS(SNES snes)
 
   if (snes->jacobian) {         /* This method does not require a Jacobian, but it is usually preconditioned by PBJacobi */
     ierr = SNESComputeJacobian(snes,snes->vec_sol,snes->jacobian,snes->jacobian_pre);CHKERRQ(ierr);
+    SNESCheckJacobianDomainerror(snes);
   }
   if (ms->norms) {
     ierr = VecNorm(F,NORM_2,&fnorm);CHKERRQ(ierr); /* fnorm <- ||F||  */
