@@ -555,7 +555,6 @@ static PetscErrorCode SNESNASMSetComputeFinalJacobian_NASM(SNES snes,PetscBool f
 
   PetscFunctionBegin;
   nasm->finaljacobian = flg;
-  if (flg) snes->usesksp = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
 
@@ -855,7 +854,10 @@ static PetscErrorCode SNESSolve_NASM(SNES snes)
     /* Call general purpose update function */
     if (snes->ops->update) {ierr = (*snes->ops->update)(snes, snes->iter);CHKERRQ(ierr);}
   }
-  if (nasm->finaljacobian) {ierr = SNESNASMComputeFinalJacobian_Private(snes,X);CHKERRQ(ierr);}
+  if (nasm->finaljacobian) {
+    ierr = SNESNASMComputeFinalJacobian_Private(snes,X);CHKERRQ(ierr);
+    SNESCheckJacobianDomainerror(snes);
+  }
   if (normschedule == SNES_NORM_ALWAYS) {
     if (i == snes->max_its) {
       ierr = PetscInfo1(snes,"Maximum number of iterations has been reached: %D\n",snes->max_its);CHKERRQ(ierr);
@@ -880,6 +882,13 @@ static PetscErrorCode SNESSolve_NASM(SNES snes)
 
    Level: advanced
 
+   Developer Note: This is a non-Newton based nonlinear solver that does not directly require a Jacobian; hence the flag snes->usesksp is set to
+       false and SNESView() and -snes_view do not display a KSP object. However the flag nasm->finaljacobian is set (for example if
+       NASM is used as a nonlinear preconditioner for  KSPASPIN) then SNESSetUpMatrices() is called to generate the Jacobian (needed by KSPASPIN)
+       and this utilizes the KSP for storing the matrices, but the KSP is never used for solving a linear system. Note that when SNESNASM is
+       used by SNESASPIN they share the same Jacobian matrices because SNESSetUp() (called on the outer SNES KSPASPIN) causes the inner SNES
+       object (in this case SNESNASM) to inherit the outer Jacobian matrices.
+
    References:
 .  1. - Peter R. Brune, Matthew G. Knepley, Barry F. Smith, and Xuemin Tu, "Composing Scalable Nonlinear Algebraic Solvers",
    SIAM Review, 57(4), 2015
