@@ -30,8 +30,8 @@ PetscErrorCode  PetscDrawFinalizePackage(void)
 
 /*@C
   PetscInitializeDrawPackage - This function initializes everything in the PetscDraw package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the call to PetscInitialize()
-  when using static libraries.
+  from PetscDLLibraryRegister_petsc() when using dynamic libraries, and on the call to PetscInitialize()
+  when using shared or static libraries.
 
   Level: developer
 
