@@ -186,10 +186,15 @@ PetscErrorCode PetscViewerFlush_ASCII(PetscViewer viewer)
 /*@C
     PetscViewerASCIIGetPointer - Extracts the file pointer from an ASCII PetscViewer.
 
-    Not Collective
+    Not Collective, depending on the viewer the value may be meaningless except for process 0 of the viewer
 
-+   viewer - PetscViewer context, obtained from PetscViewerASCIIOpen()
--   fd - file pointer
+    Input Parameter:
+.    viewer - PetscViewer context, obtained from PetscViewerASCIIOpen()
+
+    Output Parameter:
+.    fd - file pointer
+
+    Notes: for the standard PETSCVIEWERASCII the value is valid only on process 0 of the viewer
 
     Level: intermediate
 
@@ -381,13 +386,16 @@ PetscErrorCode  PetscViewerASCIISubtractTab(PetscViewer viewer,PetscInt tabs)
 .    viewer - obtained with PetscViewerASCIIOpen()
 
     Level: intermediate
+    
+    Notes:
+    See documentation of PetscViewerASCIISynchronizedPrintf() for more details how the synchronized output should be done properly.
 
   Concepts: PetscViewerASCII^formating
   Concepts: tab^setting
 
-.seealso: PetscViewerASCIIPopSynchronized(), PetscPrintf(), PetscSynchronizedPrintf(), PetscViewerASCIIPrintf(),
-          PetscViewerASCIIPopTab(), PetscViewerASCIISynchronizedPrintf(), PetscViewerASCIIOpen(),
-          PetscViewerCreate(), PetscViewerDestroy(), PetscViewerSetType(), PetscViewerASCIIGetPointer()
+.seealso: PetscViewerASCIISynchronizedPrintf(), PetscViewerFlush(), PetscViewerASCIIPopSynchronized(),
+          PetscSynchronizedPrintf(), PetscViewerASCIIPrintf(), PetscViewerASCIIOpen(),
+          PetscViewerCreate(), PetscViewerDestroy(), PetscViewerSetType()
 @*/
 PetscErrorCode  PetscViewerASCIIPushSynchronized(PetscViewer viewer)
 {
@@ -411,13 +419,16 @@ PetscErrorCode  PetscViewerASCIIPushSynchronized(PetscViewer viewer)
 .    viewer - obtained with PetscViewerASCIIOpen()
 
     Level: intermediate
+    
+    Notes:
+    See documentation of PetscViewerASCIISynchronizedPrintf() for more details how the synchronized output should be done properly.
 
   Concepts: PetscViewerASCII^formating
   Concepts: tab^setting
 
-.seealso: PetscViewerASCIIPushSynchronized(), PetscPrintf(), PetscSynchronizedPrintf(), PetscViewerASCIIPrintf(),
-          PetscViewerASCIIPopTab(), PetscViewerASCIISynchronizedPrintf(), PetscViewerASCIIOpen(),
-          PetscViewerCreate(), PetscViewerDestroy(), PetscViewerSetType(), PetscViewerASCIIGetPointer()
+.seealso: PetscViewerASCIIPushSynchronized(), PetscViewerASCIISynchronizedPrintf(), PetscViewerFlush(),
+          PetscSynchronizedPrintf(), PetscViewerASCIIPrintf(), PetscViewerASCIIOpen(),
+          PetscViewerCreate(), PetscViewerDestroy(), PetscViewerSetType()
 @*/
 PetscErrorCode  PetscViewerASCIIPopSynchronized(PetscViewer viewer)
 {
@@ -891,15 +902,29 @@ PETSC_EXTERN PetscErrorCode PetscViewerCreate_ASCII(PetscViewer viewer)
     Level: intermediate
 
     Notes:
-    You must have previously called PetscViewerASCIISynchronizeAllow() to allow this routine to be called.
+    You must have previously called PetscViewerASCIIPushSynchronized() to allow this routine to be called.
+    Then you can do multiple independent calls to this routine.
+    The actual synchronized print is then done using PetscViewerFlush().
+    PetscViewerASCIIPopSynchronized() should be then called if we are already done with the synchronized output
+    to conclude the "synchronized session".
+    So the typical calling sequence looks like
+$ PetscViewerASCIIPushSynchronized(viewer);
+$ PetscViewerASCIISynchronizedPrintf(viewer, ...);
+$ PetscViewerASCIISynchronizedPrintf(viewer, ...);
+$ ...
+$ PetscViewerFlush(viewer);
+$ PetscViewerASCIISynchronizedPrintf(viewer, ...);
+$ PetscViewerASCIISynchronizedPrintf(viewer, ...);
+$ ...
+$ PetscViewerFlush(viewer);
+$ PetscViewerASCIIPopSynchronized(viewer);    
 
     Fortran Note:
       Can only print a single character* string
 
-.seealso: PetscSynchronizedPrintf(), PetscSynchronizedFlush(), PetscFPrintf(),
-          PetscFOpen(), PetscViewerFlush(), PetscViewerASCIIGetPointer(), PetscViewerDestroy(), PetscViewerASCIIOpen(),
-          PetscViewerASCIIPrintf(), PetscViewerASCIIPushSynchronized()
-
+.seealso: PetscViewerASCIIPushSynchronized(), PetscViewerFlush(), PetscViewerASCIIPopSynchronized(),
+          PetscSynchronizedPrintf(), PetscViewerASCIIPrintf(), PetscViewerASCIIOpen(),
+          PetscViewerCreate(), PetscViewerDestroy(), PetscViewerSetType()
 @*/
 PetscErrorCode  PetscViewerASCIISynchronizedPrintf(PetscViewer viewer,const char format[],...)
 {
@@ -1007,9 +1032,9 @@ PetscErrorCode  PetscViewerASCIISynchronizedPrintf(PetscViewer viewer,const char
 }
 
 /*@C
-   PetscViewerASCIIRead - Reads from am ASCII file
+   PetscViewerASCIIRead - Reads from a ASCII file
 
-   Collective on MPI_Comm
+   Only process 0 in the PetscViewer may call this
 
    Input Parameters:
 +  viewer - the ascii viewer
@@ -1024,7 +1049,7 @@ PetscErrorCode  PetscViewerASCIISynchronizedPrintf(PetscViewer viewer,const char
 
    Concepts: ascii files
 
-.seealso: PetscViewerASCIIOpen(), PetscViewerPushFormat(), PetscViewerDestroy(),
+.seealso: PetscViewerASCIIOpen(), PetscViewerPushFormat(), PetscViewerDestroy(), PetscViewerCreate(), PetscViewerFileSetMode(), PetscViewerFileSetName()
           VecView(), MatView(), VecLoad(), MatLoad(), PetscViewerBinaryGetDescriptor(),
           PetscViewerBinaryGetInfoPointer(), PetscFileMode, PetscViewer, PetscViewerBinaryRead()
 @*/
@@ -1034,14 +1059,20 @@ PetscErrorCode PetscViewerASCIIRead(PetscViewer viewer,void *data,PetscInt num,P
   FILE              *fd = vascii->fd;
   PetscInt           i;
   int                ret = 0;
+  PetscMPIInt        rank;
+  PetscErrorCode     ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)viewer),&rank);CHKERRQ(ierr);
+  if (rank) SETERRQ(PETSC_COMM_SELF, PETSC_ERR_ARG_WRONG,"Can only be called from process 0 in the PetscViewer");
   for (i=0; i<num; i++) {
     if (dtype == PETSC_CHAR)         ret = fscanf(fd, "%c",  &(((char*)data)[i]));
     else if (dtype == PETSC_STRING)  ret = fscanf(fd, "%s",  &(((char*)data)[i]));
     else if (dtype == PETSC_INT)     ret = fscanf(fd, "%" PetscInt_FMT,  &(((PetscInt*)data)[i]));
     else if (dtype == PETSC_ENUM)    ret = fscanf(fd, "%d",  &(((int*)data)[i]));
+    else if (dtype == PETSC_INT64)   ret = fscanf(fd, "%" PetscInt64_FMT,  &(((PetscInt64*)data)[i]));
+    else if (dtype == PETSC_LONG)    ret = fscanf(fd, "%ld", &(((long*)data)[i]));
     else if (dtype == PETSC_FLOAT)   ret = fscanf(fd, "%f",  &(((float*)data)[i]));
     else if (dtype == PETSC_DOUBLE)  ret = fscanf(fd, "%lg", &(((double*)data)[i]));
 #if defined(PETSC_USE_REAL___FLOAT128)
