@@ -28,8 +28,8 @@ PetscErrorCode  PetscSysFinalizePackage(void)
 
 /*@C
   PetscSysInitializePackage - This function initializes everything in the main Petsc package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the call to PetscInitialize()
-  when using static libraries.
+  from PetscDLLibraryRegister_petsc() when using dynamic libraries, and on the call to PetscInitialize()
+  when using shared or static libraries.
 
   Level: developer
 
