@@ -15,8 +15,6 @@ struct _n_PetscDLLibrary {
   char           libname[PETSC_MAX_PATH_LEN];
 };
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDLLibraryPrintPath"
 PetscErrorCode  PetscDLLibraryPrintPath(PetscDLLibrary libs)
 {
   PetscFunctionBegin;
@@ -27,8 +25,6 @@ PetscErrorCode  PetscDLLibraryPrintPath(PetscDLLibrary libs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDLLibraryRetrieve"
 /*@C
    PetscDLLibraryRetrieve - Copies a PETSc dynamic library from a remote location
      (if it is remote), indicates if it exits and its local name.
@@ -83,8 +79,8 @@ PetscErrorCode  PetscDLLibraryRetrieve(MPI_Comm comm,const char libname[],char *
   ierr = PetscFileRetrieve(comm,par2,lname,llen,found);CHKERRQ(ierr);
   if (!(*found)) {
     /* see if library name does already not have suffix attached */
-    ierr = PetscStrcpy(suffix,".");CHKERRQ(ierr);
-    ierr = PetscStrcat(suffix,PETSC_SLSUFFIX);CHKERRQ(ierr);
+    ierr = PetscStrncpy(suffix,".",sizeof(suffix));CHKERRQ(ierr);
+    ierr = PetscStrlcat(suffix,PETSC_SLSUFFIX,sizeof(suffix));CHKERRQ(ierr);
     ierr = PetscStrrstr(par2,suffix,&so);CHKERRQ(ierr);
     /* and attach the suffix if it is not there */
     if (!so) { ierr = PetscStrcat(par2,suffix);CHKERRQ(ierr); }
@@ -100,9 +96,13 @@ PetscErrorCode  PetscDLLibraryRetrieve(MPI_Comm comm,const char libname[],char *
   PetscFunctionReturn(0);
 }
 
+  /*
+     Some compilers when used with -std=c89 don't produce a usable PETSC_FUNCTION_NAME. Since this name is needed in PetscMallocDump()
+     to avoid reporting the memory allocations in the function as not freed we hardwire the value here.
+  */
+#undef    PETSC_FUNCTION_NAME
+#define   PETSC_FUNCTION_NAME "PetscDLLibraryOpen"
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDLLibraryOpen"
 /*@C
    PetscDLLibraryOpen - Opens a PETSc dynamic link library
 
@@ -136,7 +136,6 @@ PetscErrorCode  PetscDLLibraryOpen(MPI_Comm comm,const char path[],PetscDLLibrar
   char           *basename,registername[128];
   PetscDLHandle  handle;
   PetscErrorCode (*func)(void) = NULL;
-  size_t         len;
 
   PetscFunctionBegin;
   PetscValidCharPointer(path,2);
@@ -157,8 +156,8 @@ PetscErrorCode  PetscDLLibraryOpen(MPI_Comm comm,const char path[],PetscDLLibrar
 
   /* copy path and setup shared library suffix  */
   ierr = PetscStrncpy(libname,path,PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
-  ierr = PetscStrcpy(suffix,".");CHKERRQ(ierr);
-  ierr = PetscStrcat(suffix,PETSC_SLSUFFIX);CHKERRQ(ierr);
+  ierr = PetscStrncpy(suffix,".",sizeof(suffix));CHKERRQ(ierr);
+  ierr = PetscStrlcat(suffix,PETSC_SLSUFFIX,sizeof(suffix));CHKERRQ(ierr);
   /* remove wrong suffixes from libname */
   ierr = PetscStrrstr(libname,".gz",&s);CHKERRQ(ierr);
   if (s && s[3] == 0) s[0] = 0;
@@ -181,9 +180,8 @@ PetscErrorCode  PetscDLLibraryOpen(MPI_Comm comm,const char path[],PetscDLLibrar
     ierr = PetscInfo1(0,"Dynamic library %s does not have lib prefix\n",libname);CHKERRQ(ierr);
   }
   for (s=basename; *s; s++) if (*s == '-') *s = '_';
-  ierr = PetscStrlen(basename,&len);CHKERRQ(ierr);
-  ierr = PetscStrcpy(registername,"PetscDLLibraryRegister_");CHKERRQ(ierr);
-  ierr = PetscStrncat(registername,basename,len);CHKERRQ(ierr);
+  ierr = PetscStrncpy(registername,"PetscDLLibraryRegister_",sizeof(registername));CHKERRQ(ierr);
+  ierr = PetscStrlcat(registername,basename,sizeof(registername));CHKERRQ(ierr);
   ierr = PetscDLSym(handle,registername,(void**)&func);CHKERRQ(ierr);
   if (func) {
     ierr = PetscInfo1(0,"Loading registered routines from %s\n",libname);CHKERRQ(ierr);
@@ -199,8 +197,13 @@ PetscErrorCode  PetscDLLibraryOpen(MPI_Comm comm,const char path[],PetscDLLibrar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDLLibrarySym"
+#undef    PETSC_FUNCTION_NAME
+#if defined(__cplusplus)
+#  define PETSC_FUNCTION_NAME PETSC_FUNCTION_NAME_CXX
+#else
+#  define PETSC_FUNCTION_NAME PETSC_FUNCTION_NAME_C
+#endif
+
 /*@C
    PetscDLLibrarySym - Load a symbol from the dynamic link libraries.
 
@@ -217,7 +220,8 @@ PetscErrorCode  PetscDLLibraryOpen(MPI_Comm comm,const char path[],PetscDLLibrar
 
    Level: developer
 
-   Notes: Symbol can be of the form
+   Notes:
+    Symbol can be of the form
         [/path/libname[.so.1.0]:]functionname[()] where items in [] denote optional
 
         Will attempt to (retrieve and) open the library if it is not yet been opened.
@@ -255,8 +259,8 @@ PetscErrorCode  PetscDLLibrarySym(MPI_Comm comm,PetscDLLibrary *outlist,const ch
   if (path && path[0] != '\0') {
     /* copy path and remove suffix from libname */
     ierr = PetscStrncpy(libname,path,PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
-    ierr = PetscStrcpy(suffix,".");CHKERRQ(ierr);
-    ierr = PetscStrcat(suffix,PETSC_SLSUFFIX);CHKERRQ(ierr);
+    ierr = PetscStrncpy(suffix,".",sizeof(suffix));CHKERRQ(ierr);
+    ierr = PetscStrlcat(suffix,PETSC_SLSUFFIX,sizeof(suffix));CHKERRQ(ierr);
     ierr = PetscStrrstr(libname,suffix,&s);CHKERRQ(ierr);
     if (s) s[0] = 0;
     /* Look if library is already opened and in path */
@@ -273,7 +277,7 @@ PetscErrorCode  PetscDLLibrarySym(MPI_Comm comm,PetscDLLibrary *outlist,const ch
     ierr = PetscDLLibraryOpen(comm,path,&nlist);CHKERRQ(ierr);
     ierr = PetscInfo1(0,"Appending %s to dynamic library search path\n",path);CHKERRQ(ierr);
     if (prev) prev->next = nlist;
-    else      *outlist   = nlist;
+    else {if (outlist) *outlist   = nlist;}
 
 done:;
     ierr = PetscDLSym(nlist->handle,symbol,value);CHKERRQ(ierr);
@@ -308,8 +312,6 @@ done:;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDLLibraryAppend"
 /*@C
      PetscDLLibraryAppend - Appends another dynamic link library to the seach list, to the end
                 of the search path.
@@ -325,7 +327,8 @@ done:;
 
      Level: developer
 
-     Notes: if library is already in path will not add it.
+     Notes:
+    if library is already in path will not add it.
 
   If the library has the symbol PetscDLLibraryRegister_basename() in it then that function is automatically run
       when the library is opened.
@@ -349,22 +352,22 @@ PetscErrorCode  PetscDLLibraryAppend(MPI_Comm comm,PetscDLLibrary *outlist,const
   ierr = PetscTestDirectory(path,'r',&dir);CHKERRQ(ierr);
   if (dir) {
     ierr = PetscInfo1(0,"Checking directory %s for dynamic libraries\n",path);CHKERRQ(ierr);
-    ierr = PetscStrcpy(program,path);CHKERRQ(ierr);
+    ierr = PetscStrncpy(program,path,sizeof(program));CHKERRQ(ierr);
     ierr = PetscStrlen(program,&len);CHKERRQ(ierr);
     if (program[len-1] == '/') {
-      ierr = PetscStrcat(program,"*.");CHKERRQ(ierr);
+      ierr = PetscStrlcat(program,"*.",sizeof(program));CHKERRQ(ierr);
     } else {
-      ierr = PetscStrcat(program,"/*.");CHKERRQ(ierr);
+      ierr = PetscStrlcat(program,"/*.",sizeof(program));CHKERRQ(ierr);
     }
-    ierr = PetscStrcat(program,PETSC_SLSUFFIX);CHKERRQ(ierr);
+    ierr = PetscStrlcat(program,PETSC_SLSUFFIX,sizeof(program));CHKERRQ(ierr);
 
     ierr = PetscLs(comm,program,found,8*PETSC_MAX_PATH_LEN,&dir);CHKERRQ(ierr);
     if (!dir) PetscFunctionReturn(0);
   } else {
     ierr = PetscStrncpy(found,path,PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
   }
-  ierr = PetscStrcpy(suffix,".");CHKERRQ(ierr);
-  ierr = PetscStrcat(suffix,PETSC_SLSUFFIX);CHKERRQ(ierr);
+  ierr = PetscStrncpy(suffix,".",sizeof(suffix));CHKERRQ(ierr);
+  ierr = PetscStrlcat(suffix,PETSC_SLSUFFIX,sizeof(suffix));CHKERRQ(ierr);
 
   ierr = PetscTokenCreate(found,'\n',&token);CHKERRQ(ierr);
   ierr = PetscTokenFind(token,&libname);CHKERRQ(ierr);
@@ -396,8 +399,6 @@ PetscErrorCode  PetscDLLibraryAppend(MPI_Comm comm,PetscDLLibrary *outlist,const
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDLLibraryPrepend"
 /*@C
      PetscDLLibraryPrepend - Add another dynamic library to search for symbols to the beginning of
                  the search path.
@@ -413,7 +414,8 @@ PetscErrorCode  PetscDLLibraryAppend(MPI_Comm comm,PetscDLLibrary *outlist,const
 
      Level: developer
 
-     Notes: If library is already in path will remove old reference.
+     Notes:
+    If library is already in path will remove old reference.
 
 @*/
 PetscErrorCode  PetscDLLibraryPrepend(MPI_Comm comm,PetscDLLibrary *outlist,const char path[])
@@ -433,14 +435,14 @@ PetscErrorCode  PetscDLLibraryPrepend(MPI_Comm comm,PetscDLLibrary *outlist,cons
   ierr = PetscTestDirectory(path,'r',&dir);CHKERRQ(ierr);
   if (dir) {
     ierr = PetscInfo1(0,"Checking directory %s for dynamic libraries\n",path);CHKERRQ(ierr);
-    ierr = PetscStrcpy(program,path);CHKERRQ(ierr);
+    ierr = PetscStrncpy(program,path,sizeof(program));CHKERRQ(ierr);
     ierr = PetscStrlen(program,&len);CHKERRQ(ierr);
     if (program[len-1] == '/') {
-      ierr = PetscStrcat(program,"*.");CHKERRQ(ierr);
+      ierr = PetscStrlcat(program,"*.",sizeof(program));CHKERRQ(ierr);
     } else {
-      ierr = PetscStrcat(program,"/*.");CHKERRQ(ierr);
+      ierr = PetscStrlcat(program,"/*.",sizeof(program));CHKERRQ(ierr);
     }
-    ierr = PetscStrcat(program,PETSC_SLSUFFIX);CHKERRQ(ierr);
+    ierr = PetscStrlcat(program,PETSC_SLSUFFIX,sizeof(program));CHKERRQ(ierr);
 
     ierr = PetscLs(comm,program,found,8*PETSC_MAX_PATH_LEN,&dir);CHKERRQ(ierr);
     if (!dir) PetscFunctionReturn(0);
@@ -448,8 +450,8 @@ PetscErrorCode  PetscDLLibraryPrepend(MPI_Comm comm,PetscDLLibrary *outlist,cons
     ierr = PetscStrncpy(found,path,PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
   }
 
-  ierr = PetscStrcpy(suffix,".");CHKERRQ(ierr);
-  ierr = PetscStrcat(suffix,PETSC_SLSUFFIX);CHKERRQ(ierr);
+  ierr = PetscStrncpy(suffix,".",sizeof(suffix));CHKERRQ(ierr);
+  ierr = PetscStrlcat(suffix,PETSC_SLSUFFIX,sizeof(suffix));CHKERRQ(ierr);
 
   ierr = PetscTokenCreate(found,'\n',&token);CHKERRQ(ierr);
   ierr = PetscTokenFind(token,&libname);CHKERRQ(ierr);
@@ -488,8 +490,6 @@ PetscErrorCode  PetscDLLibraryPrepend(MPI_Comm comm,PetscDLLibrary *outlist,cons
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDLLibraryClose"
 /*@C
      PetscDLLibraryClose - Destroys the search path of dynamic libraries and closes the libraries.
 
