@@ -15,8 +15,6 @@ static char      PetscDebugger[PETSC_MAX_PATH_LEN];
 static char      DebugTerminal[PETSC_MAX_PATH_LEN];
 static PetscBool Xterm = PETSC_TRUE;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSetDebugTerminal"
 /*@C
    PetscSetDebugTerminal - Sets the terminal to use (instead of xterm) for debugging.
 
@@ -24,7 +22,7 @@ static PetscBool Xterm = PETSC_TRUE;
 
    Input Parameters:
 +  terminal - name of terminal and any flags required to execute a program.
-              For example "xterm -e", "urxvt -e".
+              For example "xterm -e", "urxvt -e", "gnome-terminal -x".
 
    Options Database Keys:
    -debug_terminal terminal - use this terminal instead of xterm
@@ -34,7 +32,7 @@ static PetscBool Xterm = PETSC_TRUE;
    Notes:
    You can start the debugger for all processes in the same GNU screen session.
 
-     mpirun -n 4 ./myapp -start_in_debugger -debug_terminal "screen -X -S debug screen"
+     mpiexec -n 4 ./myapp -start_in_debugger -debug_terminal "screen -X -S debug screen"
 
    will open 4 windows in the session named "debug".
 
@@ -54,8 +52,6 @@ PetscErrorCode  PetscSetDebugTerminal(const char terminal[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSetDebugger"
 /*@C
    PetscSetDebugger - Sets options associated with the debugger.
 
@@ -92,8 +88,6 @@ PetscErrorCode  PetscSetDebugger(const char debugger[],PetscBool xterm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSetDefaultDebugger"
 /*@C
     PetscSetDefaultDebugger - Causes PETSc to use its default  debugger.
 
@@ -123,8 +117,6 @@ PetscErrorCode  PetscSetDefaultDebugger(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscCheckDebugger_Private"
 static PetscErrorCode PetscCheckDebugger_Private(const char defaultDbg[], const char string[], const char *debugger[])
 {
   PetscBool      exists;
@@ -141,8 +133,6 @@ static PetscErrorCode PetscCheckDebugger_Private(const char defaultDbg[], const
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSetDebuggerFromString"
 /*@C
     PetscSetDebuggerFromString - Set the complete path for the
        debugger for PETSc to use.
@@ -184,8 +174,6 @@ PetscErrorCode  PetscSetDebuggerFromString(const char *string)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscAttachDebugger"
 /*@
    PetscAttachDebugger - Attaches the debugger to the running process.
 
@@ -195,7 +183,8 @@ PetscErrorCode  PetscSetDebuggerFromString(const char *string)
 
    Concepts: debugger^starting from program
 
-   Developer Notes: Since this can be called by the error handler should it be calling SETERRQ() and CHKERRQ()?
+   Developer Notes:
+    Since this can be called by the error handler should it be calling SETERRQ() and CHKERRQ()?
 
 .seealso: PetscSetDebugger()
 @*/
@@ -268,7 +257,7 @@ PetscErrorCode  PetscAttachDebugger(void)
     if (isxxgdb || isups || isddd) {
       args[1] = program; args[2] = pid; args[3] = "-display";
       args[0] = PetscDebugger; args[4] = display; args[5] = 0;
-      (*PetscErrorPrintf)("PETSC: Attaching %s to %s %s on %s\n",args[0],args[1],pid,hostname);
+      printf("PETSC: Attaching %s to %s %s on %s\n",args[0],args[1],pid,hostname);
       if (execvp(args[0],(char**)args)  < 0) {
         perror("Unable to start debugger");
         exit(0);
@@ -276,7 +265,7 @@ PetscErrorCode  PetscAttachDebugger(void)
     } else if (iskdbg) {
       args[1] = "-p"; args[2] = pid; args[3] = program;  args[4] = "-display";
       args[0] = PetscDebugger; args[5] = display; args[6] = 0;
-      (*PetscErrorPrintf)("PETSC: Attaching %s to %s %s on %s\n",args[0],args[3],pid,hostname);
+      printf("PETSC: Attaching %s to %s %s on %s\n",args[0],args[3],pid,hostname);
       if (execvp(args[0],(char**)args)  < 0) {
         perror("Unable to start debugger");
         exit(0);
@@ -284,7 +273,7 @@ PetscErrorCode  PetscAttachDebugger(void)
     } else if (isxldb) {
       args[1] = "-a"; args[2] = pid; args[3] = program;  args[4] = "-display";
       args[0] = PetscDebugger; args[5] = display; args[6] = 0;
-      (*PetscErrorPrintf)("PETSC: Attaching %s to %s %s on %s\n",args[0],args[1],pid,hostname);
+      printf("PETSC: Attaching %s to %s %s on %s\n",args[0],args[1],pid,hostname);
       if (execvp(args[0],(char**)args)  < 0) {
         perror("Unable to start debugger");
         exit(0);
@@ -292,7 +281,7 @@ PetscErrorCode  PetscAttachDebugger(void)
     } else if (isworkshop) {
       args[1] = "-s"; args[2] = pid; args[3] = "-D"; args[4] = "-";
       args[0] = PetscDebugger; args[5] = pid; args[6] = "-display"; args[7] = display; args[8] = 0;
-      (*PetscErrorPrintf)("PETSC: Attaching %s to %s on %s\n",args[0],pid,hostname);
+      printf("PETSC: Attaching %s to %s on %s\n",args[0],pid,hostname);
       if (execvp(args[0],(char**)args)  < 0) {
         perror("Unable to start debugger");
         exit(0);
@@ -303,6 +292,7 @@ PetscErrorCode  PetscAttachDebugger(void)
         PetscBool cmp;
         char      *tmp,*tmp1;
         ierr = PetscStrncmp(DebugTerminal,"screen",6,&cmp);CHKERRQ(ierr);
+        if (!cmp) {ierr = PetscStrncmp(DebugTerminal,"gnome-terminal",6,&cmp);CHKERRQ(ierr);}
         if (cmp) display[0] = 0; /* when using screen, we never pass -display */
         args[j++] = tmp = DebugTerminal;
         if (display[0]) {
@@ -353,19 +343,22 @@ PetscErrorCode  PetscAttachDebugger(void)
         args[j++] = "-pid";
         args[j++] = pid;
         args[j++] = program;
+#else
+        args[j++] = program;
+        args[j++] = pid;
 #endif
         args[j++] = 0;
       }
       if (Xterm) {
-        if (display[0]) (*PetscErrorPrintf)("PETSC: Attaching %s to %s of pid %s on display %s on machine %s\n",PetscDebugger,program,pid,display,hostname);
-        else            (*PetscErrorPrintf)("PETSC: Attaching %s to %s on pid %s on %s\n",PetscDebugger,program,pid,hostname);
+        if (display[0]) printf("PETSC: Attaching %s to %s of pid %s on display %s on machine %s\n",PetscDebugger,program,pid,display,hostname);
+        else            printf("PETSC: Attaching %s to %s on pid %s on %s\n",PetscDebugger,program,pid,hostname);
 
         if (execvp(args[0],(char**)args)  < 0) {
           perror("Unable to start debugger in xterm");
           exit(0);
         }
       } else {
-        (*PetscErrorPrintf)("PETSC: Attaching %s to %s of pid %s on %s\n",PetscDebugger,program,pid,hostname);
+        printf("PETSC: Attaching %s to %s of pid %s on %s\n",PetscDebugger,program,pid,hostname);
         if (execvp(args[0],(char**)args)  < 0) {
           perror("Unable to start debugger");
           exit(0);
@@ -374,7 +367,7 @@ PetscErrorCode  PetscAttachDebugger(void)
     }
   } else {   /* I am the child, continue with user code */
     sleeptime = 10; /* default to sleep waiting for debugger */
-    ierr = PetscOptionsGetReal(NULL,"-debugger_pause",&sleeptime,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsGetReal(NULL,NULL,"-debugger_pause",&sleeptime,NULL);CHKERRQ(ierr);
     if (sleeptime < 0) sleeptime = -sleeptime;
 #if defined(PETSC_NEED_DEBUGGER_NO_SLEEP)
     /*
@@ -401,8 +394,6 @@ PetscErrorCode  PetscAttachDebugger(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscAttachDebuggerErrorHandler"
 /*@C
    PetscAttachDebuggerErrorHandler - Error handler that attaches
    a debugger to a running process when an error is detected.
@@ -413,7 +404,6 @@ PetscErrorCode  PetscAttachDebugger(void)
    Input Parameters:
 +  comm - communicator over which error occurred
 .  line - the line number of the error (indicated by __LINE__)
-.  fun - function where error occured (indicated by __FUNCT__)
 .  file - the file in which the error was detected (indicated by __FILE__)
 .  message - an error text string, usually just printed to the screen
 .  number - the generic error number
@@ -464,8 +454,6 @@ PetscErrorCode  PetscAttachDebuggerErrorHandler(MPI_Comm comm,int line,const cha
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStopForDebugger"
 /*@C
    PetscStopForDebugger - Prints a message to the screen indicating how to
          attach to the process with the debugger and then waits for the
@@ -475,9 +463,11 @@ PetscErrorCode  PetscAttachDebuggerErrorHandler(MPI_Comm comm,int line,const cha
 
    Level: developer
 
-   Notes: This is likely never needed since PetscAttachDebugger() is easier to use and seems to always work.
+   Notes:
+    This is likely never needed since PetscAttachDebugger() is easier to use and seems to always work.
 
-   Developer Notes: Since this can be called by the error handler, should it be calling SETERRQ() and CHKERRQ()?
+   Developer Notes:
+    Since this can be called by the error handler, should it be calling SETERRQ() and CHKERRQ()?
 
    Concepts: debugger^waiting for attachment
 
@@ -527,20 +517,20 @@ PetscErrorCode  PetscStopForDebugger(void)
   ierr = PetscStrcmp(PetscDebugger,"dbx",&isdbx);CHKERRQ(ierr);
   ierr = PetscStrcmp(PetscDebugger,"lldb",&islldb);CHKERRQ(ierr);
 
-  if (isxxgdb || isups || isddd || iskdbg) (*PetscErrorPrintf)("[%d]%s>>%s %s %d\n",rank,hostname,PetscDebugger,program,ppid);
-  else if (isxldb) (*PetscErrorPrintf)("[%d]%s>>%s -a %d %s\n",rank,hostname,PetscDebugger,ppid,program);
-  else if (islldb) (*PetscErrorPrintf)("[%d]%s>>%s -p %d\n",rank,hostname,PetscDebugger,ppid);
+  if (isxxgdb || isups || isddd || iskdbg) printf("[%d]%s>>%s %s %d\n",rank,hostname,PetscDebugger,program,ppid);
+  else if (isxldb) printf("[%d]%s>>%s -a %d %s\n",rank,hostname,PetscDebugger,ppid,program);
+  else if (islldb) printf("[%d]%s>>%s -p %d\n",rank,hostname,PetscDebugger,ppid);
   else if (isdbx) {
 #if defined(PETSC_USE_P_FOR_DEBUGGER)
-     (*PetscErrorPrintf)("[%d]%s>>%s -p %d %s\n",rank,hostname,PetscDebugger,ppid,program);
+     printf("[%d]%s>>%s -p %d %s\n",rank,hostname,PetscDebugger,ppid,program);
 #elif defined(PETSC_USE_LARGEP_FOR_DEBUGGER)
-     (*PetscErrorPrintf)("[%d]%s>>%s -l ALL -P %d %s\n",rank,hostname,PetscDebugger,ppid,program);
+     printf("[%d]%s>>%s -l ALL -P %d %s\n",rank,hostname,PetscDebugger,ppid,program);
 #elif defined(PETSC_USE_A_FOR_DEBUGGER)
-     (*PetscErrorPrintf)("[%d]%s>>%s -a %d\n",rank,hostname,PetscDebugger,ppid);
+     printf("[%d]%s>>%s -a %d\n",rank,hostname,PetscDebugger,ppid);
 #elif defined(PETSC_USE_PID_FOR_DEBUGGER)
-     (*PetscErrorPrintf)("[%d]%s>>%s -pid %d %s\n",rank,hostname,PetscDebugger,ppid,program);
+     printf("[%d]%s>>%s -pid %d %s\n",rank,hostname,PetscDebugger,ppid,program);
 #else
-     (*PetscErrorPrintf)("[%d]%s>>%s %s %d\n",rank,hostname,PetscDebugger,program,ppid);
+     printf("[%d]%s>>%s %s %d\n",rank,hostname,PetscDebugger,program,ppid);
 #endif
   }
 #endif /* PETSC_CANNOT_START_DEBUGGER */
@@ -548,7 +538,7 @@ PetscErrorCode  PetscStopForDebugger(void)
   fflush(stdout); /* ignore error because may already be in error handler */
 
   sleeptime = 25; /* default to sleep waiting for debugger */
-  PetscOptionsGetInt(NULL,"-debugger_pause",&sleeptime,NULL); /* ignore error because may already be in error handler */
+  PetscOptionsGetInt(NULL,NULL,"-debugger_pause",&sleeptime,NULL); /* ignore error because may already be in error handler */
   if (sleeptime < 0) sleeptime = -sleeptime;
 #if defined(PETSC_NEED_DEBUGGER_NO_SLEEP)
   /*
