@@ -3,8 +3,6 @@ static char help[] = "Tests catching of floating point exceptions.\n\n";
 
 #include <petscsys.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "CreateError"
 int CreateError(PetscReal x)
 {
   PetscErrorCode ierr;
@@ -15,12 +13,10 @@ int CreateError(PetscReal x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   PetscErrorCode ierr;
-  PetscInitialize(&argc,&argv,(char*)0,help);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr = PetscPrintf(PETSC_COMM_SELF,"This is a contrived example to test floating pointing\n");CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_SELF,"It is not a true error.\n");CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_SELF,"Run with -fp_trap to catch the floating point error\n");CHKERRQ(ierr);
@@ -28,3 +24,18 @@ int main(int argc,char **argv)
   return 0;
 }
 
+
+/*
+
+    Because this example may produce different output on different machines we filter out everything.
+    This makes the test uneffective but currently we don't have a good way to know which machines should handle
+    the floating point exceptions properly.
+
+*/
+/*TEST
+
+   test:
+      args: -fp_trap
+      filter:  Error: true
+
+TEST*/
