@@ -76,6 +76,7 @@
       PetscEnum PETSC_OBJECT
       PetscEnum PETSC_FUNCTION
       PetscEnum PETSC_STRING
+      PetscEnum PETSC___FP16
       PetscEnum PETSC_STRUCT
       PetscEnum PETSC_DATATYPE_UNKNOWN
 
@@ -88,13 +89,14 @@
 #endif
 #define PETSC_FORTRANADDR PETSC_LONG
 
-      parameter (PETSC_INT=0,PETSC_DOUBLE=1,PETSC_COMPLEX=2)
+      parameter (PETSC_DATATYPE_UNKNOWN=0)
+      parameter (PETSC_DOUBLE=1,PETSC_COMPLEX=2)
       parameter (PETSC_LONG=3,PETSC_SHORT=4,PETSC_FLOAT=5)
       parameter (PETSC_CHAR=6,PETSC_BIT_LOGICAL=7,PETSC_ENUM=8)
       parameter (PETSC_BOOL=9,PETSC___FLOAT128=10)
       parameter (PETSC_OBJECT=11,PETSC_FUNCTION=12)
-      parameter (PETSC_STRING=12,PETSC_STRUCT=13)
-      parameter (PETSC_DATATYPE_UNKNOWN=14)
+      parameter (PETSC_STRING=13,PETSC___FP16=14,PETSC_STRUCT=15)
+      parameter (PETSC_INT=16)
 !
 !
 !
