@@ -7,7 +7,7 @@
 #define petscfixfilename_          petscfixfilename
 #endif
 
-PETSC_EXTERN void PETSC_STDCALL petscfixfilename_(CHAR filein PETSC_MIXED_LEN(len1),CHAR fileout PETSC_MIXED_LEN(len2),
+PETSC_EXTERN void PETSC_STDCALL petscfixfilename_(char* filein PETSC_MIXED_LEN(len1),char* fileout PETSC_MIXED_LEN(len2),
                                      PetscErrorCode *ierr PETSC_END_LEN(len1) PETSC_END_LEN(len2))
 {
   PetscInt i,n;
