@@ -17,8 +17,6 @@
 const char *const PetscFileModes[] = {"READ","WRITE","APPEND","UPDATE","APPEND_UPDATE","PetscFileMode","PETSC_FILE_",0};
 
 /* --------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "PetscByteSwapEnum"
 /*
   PetscByteSwapEnum - Swap bytes in a  PETSc Enum
 
@@ -38,8 +36,6 @@ PetscErrorCode  PetscByteSwapEnum(PetscEnum *buff,PetscInt n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscByteSwapBool"
 /*
   PetscByteSwapBool - Swap bytes in a  PETSc Bool
 
@@ -59,8 +55,6 @@ PetscErrorCode  PetscByteSwapBool(PetscBool *buff,PetscInt n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscByteSwapInt"
 /*
   PetscByteSwapInt - Swap bytes in a  PETSc integer (which may be 32 or 64 bits)
 
@@ -78,9 +72,27 @@ PetscErrorCode  PetscByteSwapInt(PetscInt *buff,PetscInt n)
   }
   PetscFunctionReturn(0);
 }
+
+/*
+  PetscByteSwapInt64 - Swap bytes in a  PETSc integer (64 bits)
+
+*/
+PetscErrorCode  PetscByteSwapInt64(PetscInt64 *buff,PetscInt n)
+{
+  PetscInt   i,j;
+  PetscInt64 tmp = 0;
+  char       *ptr1,*ptr2 = (char*)&tmp;
+
+  PetscFunctionBegin;
+  for (j=0; j<n; j++) {
+    ptr1 = (char*)(buff + j);
+    for (i=0; i<(PetscInt)sizeof(PetscInt64); i++) ptr2[i] = ptr1[sizeof(PetscInt64)-1-i];
+    for (i=0; i<(PetscInt)sizeof(PetscInt64); i++) ptr1[i] = ptr2[i];
+  }
+  PetscFunctionReturn(0);
+}
+
 /* --------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "PetscByteSwapShort"
 /*
   PetscByteSwapShort - Swap bytes in a short
 */
@@ -98,12 +110,45 @@ PetscErrorCode  PetscByteSwapShort(short *buff,PetscInt n)
   }
   PetscFunctionReturn(0);
 }
-/* --------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "PetscByteSwapScalar"
 /*
-  PetscByteSwapScalar - Swap bytes in a double
-  Complex is dealt with as if array of double twice as long.
+  PetscByteSwapLong - Swap bytes in a long
+*/
+PetscErrorCode  PetscByteSwapLong(long *buff,PetscInt n)
+{
+  PetscInt i,j;
+  long     tmp;
+  char     *ptr1,*ptr2 = (char*)&tmp;
+
+  PetscFunctionBegin;
+  for (j=0; j<n; j++) {
+    ptr1 = (char*)(buff + j);
+    for (i=0; i<(PetscInt) sizeof(long); i++) ptr2[i] = ptr1[sizeof(long)-1-i];
+    for (i=0; i<(PetscInt) sizeof(long); i++) ptr1[i] = ptr2[i];
+  }
+  PetscFunctionReturn(0);
+}
+/* --------------------------------------------------------- */
+/*
+  PetscByteSwapReal - Swap bytes in a PetscReal
+*/
+PetscErrorCode  PetscByteSwapReal(PetscReal *buff,PetscInt n)
+{
+  PetscInt  i,j;
+  PetscReal tmp,*buff1 = (PetscReal*)buff;
+  char      *ptr1,*ptr2 = (char*)&tmp;
+
+  PetscFunctionBegin;
+  for (j=0; j<n; j++) {
+    ptr1 = (char*)(buff1 + j);
+    for (i=0; i<(PetscInt) sizeof(PetscReal); i++) ptr2[i] = ptr1[sizeof(PetscReal)-1-i];
+    for (i=0; i<(PetscInt) sizeof(PetscReal); i++) ptr1[i] = ptr2[i];
+  }
+  PetscFunctionReturn(0);
+}
+/* --------------------------------------------------------- */
+/*
+  PetscByteSwapScalar - Swap bytes in a PetscScalar
+  The complex case is dealt with with an array of PetscReal, twice as long.
 */
 PetscErrorCode  PetscByteSwapScalar(PetscScalar *buff,PetscInt n)
 {
@@ -123,8 +168,6 @@ PetscErrorCode  PetscByteSwapScalar(PetscScalar *buff,PetscInt n)
   PetscFunctionReturn(0);
 }
 /* --------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "PetscByteSwapDouble"
 /*
   PetscByteSwapDouble - Swap bytes in a double
 */
@@ -143,8 +186,6 @@ PetscErrorCode  PetscByteSwapDouble(double *buff,PetscInt n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscByteSwapFloat"
 /*
   PetscByteSwapFloat - Swap bytes in a float
 */
@@ -163,8 +204,6 @@ PetscErrorCode PetscByteSwapFloat(float *buff,PetscInt n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscByteSwap"
 PetscErrorCode PetscByteSwap(void *data,PetscDataType pdtype,PetscInt count)
 {
   PetscErrorCode ierr;
@@ -174,15 +213,16 @@ PetscErrorCode PetscByteSwap(void *data,PetscDataType pdtype,PetscInt count)
   else if (pdtype == PETSC_ENUM)   {ierr = PetscByteSwapEnum((PetscEnum*)data,count);CHKERRQ(ierr);}
   else if (pdtype == PETSC_BOOL)   {ierr = PetscByteSwapBool((PetscBool*)data,count);CHKERRQ(ierr);}
   else if (pdtype == PETSC_SCALAR) {ierr = PetscByteSwapScalar((PetscScalar*)data,count);CHKERRQ(ierr);}
+  else if (pdtype == PETSC_REAL)   {ierr = PetscByteSwapReal((PetscReal*)data,count);CHKERRQ(ierr);}
+  else if (pdtype == PETSC_INT64)  {ierr = PetscByteSwapInt64((PetscInt64*)data,count);CHKERRQ(ierr);}
   else if (pdtype == PETSC_DOUBLE) {ierr = PetscByteSwapDouble((double*)data,count);CHKERRQ(ierr);}
   else if (pdtype == PETSC_FLOAT)  {ierr = PetscByteSwapFloat((float*)data,count);CHKERRQ(ierr);}
   else if (pdtype == PETSC_SHORT)  {ierr = PetscByteSwapShort((short*)data,count);CHKERRQ(ierr);}
+  else if (pdtype == PETSC_LONG)   {ierr = PetscByteSwapLong((long*)data,count);CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
 /* --------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "PetscBinaryRead"
 /*@
    PetscBinaryRead - Reads from a binary file.
 
@@ -248,19 +288,22 @@ PetscErrorCode  PetscBinaryRead(int fd,void *p,PetscInt n,PetscDataType type)
 
   if (type == PETSC_INT)          m *= sizeof(PetscInt);
   else if (type == PETSC_SCALAR)  m *= sizeof(PetscScalar);
+  else if (type == PETSC_REAL)    m *= sizeof(PetscReal);
   else if (type == PETSC_DOUBLE)  m *= sizeof(double);
   else if (type == PETSC_FLOAT)   m *= sizeof(float);
   else if (type == PETSC_SHORT)   m *= sizeof(short);
+  else if (type == PETSC_LONG)    m *= sizeof(long);
   else if (type == PETSC_CHAR)    m *= sizeof(char);
   else if (type == PETSC_ENUM)    m *= sizeof(PetscEnum);
-  else if (type == PETSC_BOOL)   m *= sizeof(PetscBool);
+  else if (type == PETSC_BOOL)    m *= sizeof(PetscBool);
+  else if (type == PETSC_INT64)   m *= sizeof(PetscInt64);
   else if (type == PETSC_BIT_LOGICAL) m  = PetscBTLength(m)*sizeof(char);
   else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Unknown type");
 
 #if defined(PETSC_USE_REAL___FLOAT128)
-  ierr = PetscOptionsGetBool(NULL,"-binary_read_double",&readdouble,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-binary_read_double",&readdouble,NULL);CHKERRQ(ierr);
   /* If using __float128 precision we still read in doubles from file */
-  if (type == PETSC_SCALAR && readdouble) {
+  if ((type == PETSC_SCALAR || type == PETSC_REAL) && readdouble) {
     m    = m/2;
     ierr = PetscMalloc1(n,&ppp);CHKERRQ(ierr);
     pp   = (char*)ppp;
@@ -278,7 +321,7 @@ PetscErrorCode  PetscBinaryRead(int fd,void *p,PetscInt n,PetscDataType type)
   }
 
 #if defined(PETSC_USE_REAL___FLOAT128)
-  if (type == PETSC_SCALAR && readdouble) {
+  if ((type == PETSC_SCALAR || type == PETSC_REAL) && readdouble) {
     PetscScalar *pv = (PetscScalar*) p;
     PetscInt    i;
 #if !defined(PETSC_WORDS_BIGENDIAN)
@@ -305,8 +348,6 @@ PetscErrorCode  PetscBinaryRead(int fd,void *p,PetscInt n,PetscDataType type)
   PetscFunctionReturn(0);
 }
 /* --------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "PetscBinaryWrite"
 /*@
    PetscBinaryWrite - Writes to a binary file.
 
@@ -329,6 +370,8 @@ PetscErrorCode  PetscBinaryRead(int fd,void *p,PetscInt n,PetscDataType type)
    file as 64 bit integers, this means they can only be read back in when the option --with-64bit-indices
    is used.
 
+   If running with __float128 precision the output is in __float128 unless one uses the -binary_write_double option
+
    The Buffer p should be read-write buffer, and not static data.
    This way, byte-swapping is done in-place, and then the buffer is
    written to the file.
@@ -357,6 +400,13 @@ PetscErrorCode  PetscBinaryWrite(int fd,void *p,PetscInt n,PetscDataType type,Pe
   void           *ptmp = p;
 #endif
   char           *fname = NULL;
+#if defined(PETSC_USE_REAL___FLOAT128)
+  PetscBool      writedouble = PETSC_FALSE;
+  double         *ppp;
+  PetscReal      *pv;
+  PetscInt       i;
+#endif
+  PetscDataType  wtype = type;
 
   PetscFunctionBegin;
   if (n < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Trying to write a negative amount of data %D",n);
@@ -376,26 +426,44 @@ PetscErrorCode  PetscBinaryWrite(int fd,void *p,PetscInt n,PetscDataType type,Pe
 #else
     ierr = PetscStrncpy(fname,"",m);CHKERRQ(ierr);
 #endif
-    type = PETSC_CHAR;
-    pp   = (char*)fname;
+    wtype = PETSC_CHAR;
+    pp    = (char*)fname;
 #if !defined(PETSC_WORDS_BIGENDIAN)
-    ptmp = (void*)fname;
+    ptmp  = (void*)fname;
 #endif
   }
 
-  if (type == PETSC_INT)          m *= sizeof(PetscInt);
-  else if (type == PETSC_SCALAR)  m *= sizeof(PetscScalar);
-  else if (type == PETSC_DOUBLE)  m *= sizeof(double);
-  else if (type == PETSC_FLOAT)   m *= sizeof(float);
-  else if (type == PETSC_SHORT)   m *= sizeof(short);
-  else if (type == PETSC_CHAR)    m *= sizeof(char);
-  else if (type == PETSC_ENUM)    m *= sizeof(PetscEnum);
-  else if (type == PETSC_BOOL)   m *= sizeof(PetscBool);
-  else if (type == PETSC_BIT_LOGICAL) m = PetscBTLength(m)*sizeof(char);
+#if defined(PETSC_USE_REAL___FLOAT128)
+  ierr = PetscOptionsGetBool(NULL,NULL,"-binary_write_double",&writedouble,NULL);CHKERRQ(ierr);
+  /* If using __float128 precision we still write in doubles to file */
+  if ((type == PETSC_SCALAR || type == PETSC_REAL) && writedouble) {
+    wtype = PETSC_DOUBLE;
+    ierr = PetscMalloc1(n,&ppp);CHKERRQ(ierr);
+    pv = (PetscReal*)pp;
+    for (i=0; i<n; i++) {
+      ppp[i] = (double) pv[i];
+    }
+    pp   = (char*)ppp;
+    ptmp = (char*)ppp;
+  }
+#endif
+
+  if (wtype == PETSC_INT)          m *= sizeof(PetscInt);
+  else if (wtype == PETSC_SCALAR)  m *= sizeof(PetscScalar);
+  else if (wtype == PETSC_REAL)    m *= sizeof(PetscReal);
+  else if (wtype == PETSC_DOUBLE)  m *= sizeof(double);
+  else if (wtype == PETSC_FLOAT)   m *= sizeof(float);
+  else if (wtype == PETSC_SHORT)   m *= sizeof(short);
+  else if (wtype == PETSC_LONG)    m *= sizeof(long);
+  else if (wtype == PETSC_CHAR)    m *= sizeof(char);
+  else if (wtype == PETSC_ENUM)    m *= sizeof(PetscEnum);
+  else if (wtype == PETSC_BOOL)    m *= sizeof(PetscBool);
+  else if (wtype == PETSC_INT64)   m *= sizeof(PetscInt64);
+  else if (wtype == PETSC_BIT_LOGICAL) m = PetscBTLength(m)*sizeof(char);
   else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Unknown type");
 
 #if !defined(PETSC_WORDS_BIGENDIAN)
-  ierr = PetscByteSwap(ptmp,type,n);CHKERRQ(ierr);
+  ierr = PetscByteSwap(ptmp,wtype,n);CHKERRQ(ierr);
 #endif
 
   while (m) {
@@ -409,17 +477,20 @@ PetscErrorCode  PetscBinaryWrite(int fd,void *p,PetscInt n,PetscDataType type,Pe
 
 #if !defined(PETSC_WORDS_BIGENDIAN)
   if (!istemp) {
-    ierr = PetscByteSwap(ptmp,type,n);CHKERRQ(ierr);
+    ierr = PetscByteSwap(ptmp,wtype,n);CHKERRQ(ierr);
   }
 #endif
   if (type == PETSC_FUNCTION) {
     free(fname);
   }
+#if defined(PETSC_USE_REAL___FLOAT128)
+  if ((type == PETSC_SCALAR || type == PETSC_REAL) && writedouble) {
+    ierr = PetscFree(ppp);CHKERRQ(ierr);
+  }
+#endif
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscBinaryOpen"
 /*@C
    PetscBinaryOpen - Opens a PETSc binary file.
 
@@ -437,7 +508,8 @@ PetscErrorCode  PetscBinaryWrite(int fd,void *p,PetscInt n,PetscDataType type,Pe
   Concepts: files^opening binary
   Concepts: binary files^opening
 
-   Notes: Files access with PetscBinaryRead() and PetscBinaryWrite() are ALWAYS written in
+   Notes:
+    Files access with PetscBinaryRead() and PetscBinaryWrite() are ALWAYS written in
    big-endian format. This means the file can be accessed using PetscBinaryOpen() and
    PetscBinaryRead() and PetscBinaryWrite() on any machine.
 
@@ -468,8 +540,6 @@ PetscErrorCode  PetscBinaryOpen(const char name[],PetscFileMode mode,int *fd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscBinaryClose"
 /*@
    PetscBinaryClose - Closes a PETSc binary file.
 
@@ -491,9 +561,7 @@ PetscErrorCode  PetscBinaryClose(int fd)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscBinarySeek"
-/*@
+/*@C
    PetscBinarySeek - Moves the file pointer on a PETSc binary file.
 
    Not Collective
@@ -542,8 +610,6 @@ PetscErrorCode  PetscBinarySeek(int fd,off_t off,PetscBinarySeekType whence,off_
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscBinarySynchronizedRead"
 /*@C
    PetscBinarySynchronizedRead - Reads from a binary file.
 
@@ -558,11 +624,6 @@ PetscErrorCode  PetscBinarySeek(int fd,off_t off,PetscBinarySeekType whence,off_
    Output Parameters:
 .  p - the buffer
 
-   Options Database Key:
-.   -binary_longints - indicates the file was generated on a Cray vector
-         machine (not the T3E/D) and the ints are stored as 64 bit
-         quantities, otherwise they are stored as 32 bit
-
    Level: developer
 
    Notes:
@@ -581,7 +642,7 @@ PetscErrorCode  PetscBinarySeek(int fd,off_t off,PetscBinarySeekType whence,off_
 @*/
 PetscErrorCode  PetscBinarySynchronizedRead(MPI_Comm comm,int fd,void *p,PetscInt n,PetscDataType type)
 {
-  PetscErrorCode ierr;
+  PetscErrorCode ierr,ierrp=0;
   PetscMPIInt    rank;
   MPI_Datatype   mtype;
   char           *fname = NULL;
@@ -599,8 +660,10 @@ PetscErrorCode  PetscBinarySynchronizedRead(MPI_Comm comm,int fd,void *p,PetscIn
 
   ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
   if (!rank) {
-    ierr = PetscBinaryRead(fd,p,n,type);CHKERRQ(ierr);
+    ierrp = PetscBinaryRead(fd,p,n,type);
   }
+  ierr = MPI_Bcast(&ierrp,1,MPI_INT,0,comm);CHKERRQ(ierr);
+  CHKERRQ(ierrp);
   ierr = PetscDataTypeToMPIDataType(type,&mtype);CHKERRQ(ierr);
   ierr = MPI_Bcast(p,n,mtype,0,comm);CHKERRQ(ierr);
 
@@ -615,8 +678,6 @@ PetscErrorCode  PetscBinarySynchronizedRead(MPI_Comm comm,int fd,void *p,PetscIn
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscBinarySynchronizedWrite"
 /*@C
    PetscBinarySynchronizedWrite - writes to a binary file.
 
@@ -640,7 +701,8 @@ PetscErrorCode  PetscBinarySynchronizedRead(MPI_Comm comm,int fd,void *p,PetscIn
    they are stored in the machine as 32 or 64, this means the same
    binary file may be read on any machine.
 
-   Notes: because byte-swapping may be done on the values in data it cannot be declared const
+   Notes:
+    because byte-swapping may be done on the values in data it cannot be declared const
 
    WARNING: This is NOT like PetscSynchronizedFPrintf()! This routine ignores calls on all but process 0,
    while PetscSynchronizedFPrintf() has all processes print their strings in order.
@@ -664,8 +726,6 @@ PetscErrorCode  PetscBinarySynchronizedWrite(MPI_Comm comm,int fd,void *p,PetscI
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscBinarySynchronizedSeek"
 /*@C
    PetscBinarySynchronizedSeek - Moves the file pointer on a PETSc binary file.
 
@@ -765,8 +825,6 @@ PetscMPIInt PetscDataRep_write_conv_fn(void *userbuf, MPI_Datatype datatype,Pets
 }
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "MPIU_File_write_all"
 PetscErrorCode MPIU_File_write_all(MPI_File fd,void *data,PetscMPIInt cnt,MPI_Datatype dtype,MPI_Status *status)
 {
   PetscErrorCode ierr;
@@ -780,8 +838,6 @@ PetscErrorCode MPIU_File_write_all(MPI_File fd,void *data,PetscMPIInt cnt,MPI_Da
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MPIU_File_read_all"
 PetscErrorCode MPIU_File_read_all(MPI_File fd,void *data,PetscMPIInt cnt,MPI_Datatype dtype,MPI_Status *status)
 {
   PetscErrorCode ierr;
