@@ -932,6 +932,40 @@ PetscErrorCode  PetscLogEventDeactivateClass(PetscClassId classid)
   PetscFunctionReturn(0);
 }
 
+/*MC
+   PetscLogEventSync - Synchronizes the beginning of a user event.
+
+   Synopsis:
+   #include <petsclog.h>
+   PetscErrorCode PetscLogEventSync(int e,MPI_Comm comm)
+
+   Collective
+
+   Input Parameters:
++  e - integer associated with the event obtained from PetscLogEventRegister()
+-  comm - an MPI communicator
+
+   Usage:
+.vb
+     PetscLogEvent USER_EVENT;
+     PetscLogEventRegister("User event",0,&USER_EVENT);
+     PetscLogEventSync(USER_EVENT,PETSC_COMM_WORLD);
+     PetscLogEventBegin(USER_EVENT,0,0,0,0);
+        [code segment to monitor]
+     PetscLogEventEnd(USER_EVENT,0,0,0,0);
+.ve
+
+   Notes:
+   This routine should be called only if there is not a
+   PetscObject available to pass to PetscLogEventBegin().
+
+   Level: developer
+
+.seealso: PetscLogEventRegister(), PetscLogEventBegin(), PetscLogEventEnd()
+
+.keywords: log, event, synchronization
+M*/
+
 /*MC
    PetscLogEventBegin - Logs the beginning of a user event.
 
@@ -1192,9 +1226,26 @@ PetscErrorCode  PetscLogView_Detailed(PetscViewer viewer)
     ierr = MPIU_Allreduce(&stageLog->stageInfo[stage].eventLog->numEvents, &numEvents, 1, MPI_INT, MPI_MAX, comm);CHKERRQ(ierr);
     for (event = 0; event < numEvents; event++) {
       eventInfo = &stageLog->stageInfo[stage].eventLog->eventInfo[event];
-      ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Stages[\"%s\"][\"%s\"][%d] = {\"count\" : %D, \"time\" : %g, \"syncTime\" : %g, \"numMessages\" : %g, \"messageLength\" : %g, \"numReductions\" : %g, \"flop\" : %g}\n",
+      ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Stages[\"%s\"][\"%s\"][%d] = {\"count\" : %D, \"time\" : %g, \"syncTime\" : %g, \"numMessages\" : %g, \"messageLength\" : %g, \"numReductions\" : %g, \"flop\" : %g",
                                                 stageLog->stageInfo[stage].name,stageLog->eventLog->eventInfo[event].name,rank,
                                                 eventInfo->count,eventInfo->time,eventInfo->syncTime,eventInfo->numMessages,eventInfo->messageLength,eventInfo->numReductions,eventInfo->flops);CHKERRQ(ierr);
+      if (eventInfo->dof[0] >= 0.) {
+        PetscInt d, e;
+
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer, ", \"dof\" : [");CHKERRQ(ierr);
+        for (d = 0; d < 8; ++d) {
+          if (d > 0) {ierr = PetscViewerASCIISynchronizedPrintf(viewer, ", ");CHKERRQ(ierr);}
+          ierr = PetscViewerASCIISynchronizedPrintf(viewer, "%g", eventInfo->dof[d]);CHKERRQ(ierr);
+        }
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer, "]");CHKERRQ(ierr);
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer, ", \"error\" : [");CHKERRQ(ierr);
+        for (e = 0; e < 8; ++e) {
+          if (e > 0) {ierr = PetscViewerASCIISynchronizedPrintf(viewer, ", ");CHKERRQ(ierr);}
+          ierr = PetscViewerASCIISynchronizedPrintf(viewer, "%g", eventInfo->errors[e]);CHKERRQ(ierr);
+        }
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer, "]");CHKERRQ(ierr);
+      }
+      ierr = PetscViewerASCIISynchronizedPrintf(viewer,"}\n");CHKERRQ(ierr);
     }
   }
   ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
@@ -1701,6 +1752,8 @@ $    Google Chrome - you must start up Chrome with the option --allow-file-acces
 $    Safari - see http://ccm.net/faq/36342-safari-how-to-enable-local-file-access
   or one can use the package http://xmlsoft.org/XSLT/xsltproc2.html to translate the xml file to html and then open it with 
   your browser.
+  Alternatively, use the script ${PETSC_DIR}/lib/petsc/bin/petsc-performance-view to automatically open a new browser
+  window and render the XML log file contents.
 
   The nested XML format was kindly donated by Koos Huijssen and Christiaan M. Klaij  MARITIME  RESEARCH  INSTITUTE  NETHERLANDS
 
@@ -1760,7 +1813,7 @@ PetscErrorCode PetscLogViewFromOptions(void)
   PetscViewerFormat format;
 
   PetscFunctionBegin;
-  ierr   = PetscOptionsGetViewer(PETSC_COMM_WORLD,NULL,"-log_view",&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr   = PetscOptionsGetViewer(PETSC_COMM_WORLD,NULL,NULL,"-log_view",&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = PetscLogView(viewer);CHKERRQ(ierr);
