@@ -145,6 +145,22 @@ PetscErrorCode PetscEventPerfInfoClear(PetscEventPerfInfo *eventInfo)
   eventInfo->time2         = 0.0;
   eventInfo->timeTmp       = 0.0;
   eventInfo->syncTime      = 0.0;
+  eventInfo->dof[0]        = -1.0;
+  eventInfo->dof[1]        = -1.0;
+  eventInfo->dof[2]        = -1.0;
+  eventInfo->dof[3]        = -1.0;
+  eventInfo->dof[4]        = -1.0;
+  eventInfo->dof[5]        = -1.0;
+  eventInfo->dof[6]        = -1.0;
+  eventInfo->dof[7]        = -1.0;
+  eventInfo->errors[0]     = -1.0;
+  eventInfo->errors[1]     = -1.0;
+  eventInfo->errors[2]     = -1.0;
+  eventInfo->errors[3]     = -1.0;
+  eventInfo->errors[4]     = -1.0;
+  eventInfo->errors[5]     = -1.0;
+  eventInfo->errors[6]     = -1.0;
+  eventInfo->errors[7]     = -1.0;
   eventInfo->numMessages   = 0.0;
   eventInfo->messageLength = 0.0;
   eventInfo->numReductions = 0.0;
@@ -608,6 +624,7 @@ PetscErrorCode PetscLogEventSynchronize(PetscLogEvent event,MPI_Comm comm)
   if (!eventRegLog->eventInfo[event].collective) PetscFunctionReturn(0);
   ierr = PetscStageLogGetCurrent(stageLog,&stage);CHKERRQ(ierr);
   ierr = PetscStageLogGetEventPerfLog(stageLog,stage,&eventLog);CHKERRQ(ierr);
+  if (eventLog->eventInfo[event].depth > 0) PetscFunctionReturn(0);
 
   PetscTimeSubtract(&time);
   ierr = MPI_Barrier(comm);CHKERRQ(ierr);
@@ -627,11 +644,12 @@ PetscErrorCode PetscLogEventBeginDefault(PetscLogEvent event,int t,PetscObject o
   ierr = PetscLogGetStageLog(&stageLog);CHKERRQ(ierr);
   ierr = PetscStageLogGetCurrent(stageLog,&stage);CHKERRQ(ierr);
   ierr = PetscStageLogGetEventPerfLog(stageLog,stage,&eventLog);CHKERRQ(ierr);
+  /* Synchronization */
+  ierr = PetscLogEventSynchronize(event,PetscObjectComm(o1));CHKERRQ(ierr);
   /* Check for double counting */
   eventLog->eventInfo[event].depth++;
   if (eventLog->eventInfo[event].depth > 1) PetscFunctionReturn(0);
   /* Log the performance info */
-  ierr = PetscLogEventSynchronize(event,PetscObjectComm(o1));CHKERRQ(ierr);
   eventLog->eventInfo[event].count++;
   eventLog->eventInfo[event].timeTmp = 0.0;
   PetscTimeSubtract(&eventLog->eventInfo[event].timeTmp);
@@ -859,3 +877,76 @@ PetscErrorCode PetscLogEventEndTrace(PetscLogEvent event,int t,PetscObject o1,Pe
   if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");
   PetscFunctionReturn(0);
 }
+
+/*@C
+  PetscLogEventSetDof - Set the nth number of degrees of freedom associated with this event
+
+  Not Collective
+
+  Input Parameters:
++ event - The event id to log
+. n     - The dof index, in [0, 8)
+- dof   - The number of dofs
+
+  Database Options:
+. -log_view - Activates log summary
+
+  Note: This is to enable logging of convergence
+
+  Level: developer
+
+.keywords: log, visible, event
+.seealso: PetscLogEventSetError(), PetscEventRegLogRegister(), PetscStageLogGetEventLog()
+@*/
+PetscErrorCode PetscLogEventSetDof(PetscLogEvent event, PetscInt n, PetscLogDouble dof)
+{
+  PetscStageLog     stageLog;
+  PetscEventPerfLog eventLog = NULL;
+  int               stage;
+  PetscErrorCode    ierr;
+
+  PetscFunctionBegin;
+  if ((n < 0) || (n > 7)) SETERRQ1(PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Error index %D is not in [0, 8)", n);
+  ierr = PetscLogGetStageLog(&stageLog);CHKERRQ(ierr);
+  ierr = PetscStageLogGetCurrent(stageLog,&stage);CHKERRQ(ierr);
+  ierr = PetscStageLogGetEventPerfLog(stageLog,stage,&eventLog);CHKERRQ(ierr);
+  eventLog->eventInfo[event].dof[n] = dof;
+  PetscFunctionReturn(0);
+}
+
+/*@C
+  PetscLogEventSetError - Set the nth error associated with this event
+
+  Not Collective
+
+  Input Parameters:
++ event - The event id to log
+. n     - The error index, in [0, 8)
+. error - The error
+
+  Database Options:
+. -log_view - Activates log summary
+
+  Note: This is to enable logging of convergence, and enable users to interpret the errors as they wish. For example,
+  as different norms, or as errors for different fields
+
+  Level: developer
+
+.keywords: log, visible, event
+.seealso: PetscLogEventSetDof(), PetscEventRegLogRegister(), PetscStageLogGetEventLog()
+@*/
+PetscErrorCode PetscLogEventSetError(PetscLogEvent event, PetscInt n, PetscLogDouble error)
+{
+  PetscStageLog     stageLog;
+  PetscEventPerfLog eventLog = NULL;
+  int               stage;
+  PetscErrorCode    ierr;
+
+  PetscFunctionBegin;
+  if ((n < 0) || (n > 7)) SETERRQ1(PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Error index %D is not in [0, 8)", n);
+  ierr = PetscLogGetStageLog(&stageLog);CHKERRQ(ierr);
+  ierr = PetscStageLogGetCurrent(stageLog,&stage);CHKERRQ(ierr);
+  ierr = PetscStageLogGetEventPerfLog(stageLog,stage,&eventLog);CHKERRQ(ierr);
+  eventLog->eventInfo[event].errors[n] = error;
+  PetscFunctionReturn(0);
+}
