@@ -1,4 +1,4 @@
-
+#define PETSC_DESIRE_FEATURE_TEST_MACROS /* for getpagesize() with c89 */
 #include <petscsys.h>           /*I "petscsys.h" I*/
 #if defined(PETSC_HAVE_PWD_H)
 #include <pwd.h>
@@ -28,8 +28,6 @@
 #include <fcntl.h>
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMemoryGetCurrentUsage"
 /*@
    PetscMemoryGetCurrentUsage - Returns the current resident set size (memory used)
    for the program.
@@ -40,7 +38,7 @@
 .   mem - memory usage in bytes
 
    Options Database Key:
-.  -memory_info - Print memory usage at end of run
+.  -memory_view - Print memory usage at end of run
 .  -malloc_log - Activate logging of memory usage
 
    Level: intermediate
@@ -52,7 +50,7 @@
    usage than PetscMallocGetCurrentUsage() for codes that employ Fortran with
    hardwired arrays.
 
-.seealso: PetscMallocGetMaximumUsage(), PetscMemoryGetMaximumUsage(), PetscMallocGetCurrentUsage()
+.seealso: PetscMallocGetMaximumUsage(), PetscMemoryGetMaximumUsage(), PetscMallocGetCurrentUsage(), PetscMemorySetGetMaximumUsage(), PetscMemoryView()
 
    Concepts: resident set size
    Concepts: memory usage
@@ -113,11 +111,12 @@ PetscErrorCode  PetscMemoryGetCurrentUsage(PetscLogDouble *mem)
   PetscFunctionReturn(0);
 }
 
+PETSC_INTERN PetscBool      PetscMemoryCollectMaximumUsage;
+PETSC_INTERN PetscLogDouble PetscMemoryMaximumUsage;
+
 PetscBool      PetscMemoryCollectMaximumUsage = PETSC_FALSE;
 PetscLogDouble PetscMemoryMaximumUsage        = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMemoryGetMaximumUsage"
 /*@
    PetscMemoryGetMaximumUsage - Returns the maximum resident set size (memory used)
    for the program.
@@ -128,7 +127,7 @@ PetscLogDouble PetscMemoryMaximumUsage        = 0;
 .   mem - memory usage in bytes
 
    Options Database Key:
-.  -memory_info - Print memory usage at end of run
+.  -memory_view - Print memory usage at end of run
 .  -malloc_log - Activate logging of memory usage
 
    Level: intermediate
@@ -155,16 +154,14 @@ PetscErrorCode  PetscMemoryGetMaximumUsage(PetscLogDouble *mem)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMemorySetGetMaximumUsage"
-/*@C
+/*@
    PetscMemorySetGetMaximumUsage - Tells PETSc to monitor the maximum memory usage so that
        PetscMemoryGetMaximumUsage() will work.
 
    Not Collective
 
    Options Database Key:
-.  -memory_info - Print memory usage at end of run
+.  -memory_view - Print memory usage at end of run
 .  -malloc_log - Activate logging of memory usage
 
    Level: intermediate
