@@ -4,10 +4,36 @@
 */
 #include <petsc/private/petscimpl.h>  /*I   "petscsys.h"    I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectSetOptions"
 /*@C
-   PetscObjectSetOptions - Sets the options database used by the object
+   PetscObjectGetOptions - Gets the options database used by the object. Call immediately after creating the object.
+
+   Collective on PetscObject
+
+   Input Parameter:
+.  obj - any PETSc object, for example a Vec, Mat or KSP.
+
+   Output Parameter:
+.  options - the options database
+
+   Notes:
+    if this is not called the object will use the default options database
+
+  Level: advanced
+
+.seealso: PetscOptionsCreate(), PetscOptionsDestroy(), PetscObjectSetOptionsPrefix(), PetscObjectAppendOptionsPrefix(), PetscObjectPrependOptionsPrefix(),
+          PetscObjectGetOptionsPrefix(), PetscObjectSetOptions()
+
+@*/
+PetscErrorCode  PetscObjectGetOptions(PetscObject obj,PetscOptions *options)
+{
+  PetscFunctionBegin;
+  PetscValidHeader(obj,1);
+  *options = obj->options;
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   PetscObjectSetOptions - Sets the options database used by the object. Call immediately after creating the object.
 
    Collective on PetscObject
 
@@ -15,11 +41,13 @@
 +  obj - any PETSc object, for example a Vec, Mat or KSP.
 -  options - the options database, use NULL for default
 
-   Notes: if this is not called the object will use the default options database
+   Notes:
+    if this is not called the object will use the default options database
 
   Level: advanced
 
-.seealso: PetscOptionsCreate(), PetscOptionsDestroy()
+.seealso: PetscOptionsCreate(), PetscOptionsDestroy(), PetscObjectSetOptionsPrefix(), PetscObjectAppendOptionsPrefix(), PetscObjectPrependOptionsPrefix(),
+          PetscObjectGetOptionsPrefix(), PetscObjectGetOptions()
 
 @*/
 PetscErrorCode  PetscObjectSetOptions(PetscObject obj,PetscOptions options)
@@ -30,9 +58,7 @@ PetscErrorCode  PetscObjectSetOptions(PetscObject obj,PetscOptions options)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectSetOptionsPrefix"
-/*
+/*@C
    PetscObjectSetOptionsPrefix - Sets the prefix used for searching for all
    options of PetscObjectType in the database.
 
@@ -49,7 +75,12 @@ PetscErrorCode  PetscObjectSetOptions(PetscObject obj,PetscOptions options)
 
    Concepts: prefix^setting
 
-*/
+  Level: advanced
+
+.seealso: PetscOptionsCreate(), PetscOptionsDestroy(), PetscObjectAppendOptionsPrefix(), PetscObjectPrependOptionsPrefix(),
+          PetscObjectGetOptionsPrefix(), TSSetOptionsPrefix(), SNESSetOptionsPrefix(), KSPSetOptionsPrefix()
+
+@*/
 PetscErrorCode  PetscObjectSetOptionsPrefix(PetscObject obj,const char prefix[])
 {
   PetscErrorCode ierr;
@@ -68,9 +99,7 @@ PetscErrorCode  PetscObjectSetOptionsPrefix(PetscObject obj,const char prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectAppendOptionsPrefix"
-/*
+/*@C
    PetscObjectAppendOptionsPrefix - Sets the prefix used for searching for all
    options of PetscObjectType in the database.
 
@@ -85,7 +114,12 @@ PetscErrorCode  PetscObjectSetOptionsPrefix(PetscObject obj,const char prefix[])
 
    Concepts: prefix^setting
 
-*/
+  Level: advanced
+
+.seealso: PetscOptionsCreate(), PetscOptionsDestroy(), PetscObjectSetOptionsPrefix(), PetscObjectPrependOptionsPrefix(),
+          PetscObjectGetOptionsPrefix(), TSAppendOptionsPrefix(), SNESAppendOptionsPrefix(), KSPAppendOptionsPrefix()
+
+@*/
 PetscErrorCode  PetscObjectAppendOptionsPrefix(PetscObject obj,const char prefix[])
 {
   char           *buf = obj->prefix;
@@ -110,9 +144,7 @@ PetscErrorCode  PetscObjectAppendOptionsPrefix(PetscObject obj,const char prefix
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectGetOptionsPrefix"
-/*
+/*@C
    PetscObjectGetOptionsPrefix - Gets the prefix of the PetscObject.
 
    Input Parameters:
@@ -123,7 +155,12 @@ PetscErrorCode  PetscObjectAppendOptionsPrefix(PetscObject obj,const char prefix
 
    Concepts: prefix^getting
 
-*/
+  Level: advanced
+
+.seealso: PetscOptionsCreate(), PetscOptionsDestroy(), PetscObjectSetOptionsPrefix(), PetscObjectAppendOptionsPrefix(), PetscObjectPrependOptionsPrefix(),
+          TSGetOptionsPrefix(), SNESGetOptionsPrefix(), KSPGetOptionsPrefix()
+
+@*/
 PetscErrorCode  PetscObjectGetOptionsPrefix(PetscObject obj,const char *prefix[])
 {
   PetscFunctionBegin;
@@ -133,9 +170,7 @@ PetscErrorCode  PetscObjectGetOptionsPrefix(PetscObject obj,const char *prefix[]
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectPrependOptionsPrefix"
-/*
+/*@C
    PetscObjectPrependOptionsPrefix - Sets the prefix used for searching for all
    options of PetscObjectType in the database.
 
@@ -150,7 +185,12 @@ PetscErrorCode  PetscObjectGetOptionsPrefix(PetscObject obj,const char *prefix[]
 
    Concepts: prefix^setting
 
-*/
+  Level: advanced
+
+.seealso: PetscOptionsCreate(), PetscOptionsDestroy(), PetscObjectSetOptionsPrefix(), PetscObjectAppendOptionsPrefix(),
+          PetscObjectGetOptionsPrefix(), TSPrependOptionsPrefix(), SNESPrependOptionsPrefix(), KSPPrependOptionsPrefix()
+
+@*/
 PetscErrorCode  PetscObjectPrependOptionsPrefix(PetscObject obj,const char prefix[])
 {
   char           *buf;
