@@ -99,8 +99,6 @@ PetscErrorCode  PetscMemmove(void *a,void *b,size_t n)
 #include <petsc/private/petscimpl.h>
 #include <hwloc.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscProcessPlacementView"
 /*@C
      PetscProcessPlacementView - display the MPI process placement by core
 
@@ -123,7 +121,7 @@ PetscErrorCode PetscProcessPlacementView(PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
-  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (!isascii) SETERRQ(PetscObjectComm((PetscObject)viewer),PETSC_ERR_SUP,"Only ASCII viewer is supported");
 
   ierr = MPI_Comm_rank(MPI_COMM_WORLD,&rank);CHKERRQ(ierr);
