@@ -1,8 +1,6 @@
 
 #include <petscsys.h>           /*I    "petscsys.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSplitOwnershipBlock"
 /*@
     PetscSplitOwnershipBlock - Given a global (or local) length determines a local
         (or global) length via a simple formula. Splits so each processors local size
@@ -37,7 +35,7 @@ PetscErrorCode  PetscSplitOwnershipBlock(MPI_Comm comm,PetscInt bs,PetscInt *n,P
 
   if (*N == PETSC_DECIDE) {
     if (*n % bs != 0) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"local size %D not divisible by block size %D",*n,bs);
-    ierr = MPI_Allreduce(n,N,1,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(n,N,1,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
   } else if (*n == PETSC_DECIDE) {
     PetscInt Nbs = *N/bs;
     ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
@@ -48,13 +46,11 @@ PetscErrorCode  PetscSplitOwnershipBlock(MPI_Comm comm,PetscInt bs,PetscInt *n,P
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSplitOwnership"
 /*@
     PetscSplitOwnership - Given a global (or local) length determines a local
         (or global) length via a simple formula
 
-   Collective on MPI_Comm (if N is PETSC_DECIDE)
+   Collective on MPI_Comm (if n or N is PETSC_DECIDE)
 
    Input Parameters:
 +    comm - MPI communicator that shares the object being divided
@@ -66,8 +62,9 @@ PetscErrorCode  PetscSplitOwnershipBlock(MPI_Comm comm,PetscInt bs,PetscInt *n,P
    Notes:
      n and N cannot be both PETSC_DECIDE
 
-     If one processor calls this with N of PETSC_DECIDE then all processors
-     must, otherwise the program will hang.
+     If one processor calls this with n or N of PETSC_DECIDE then all processors
+     must. Otherwise, an error is thrown in debug mode while the program will hang
+     in optimized (i.e. configured --with-debugging=0) mode.
 
 .seealso: PetscSplitOwnershipBlock()
 
@@ -79,9 +76,20 @@ PetscErrorCode  PetscSplitOwnership(MPI_Comm comm,PetscInt *n,PetscInt *N)
 
   PetscFunctionBegin;
   if (*N == PETSC_DECIDE && *n == PETSC_DECIDE) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Both n and N cannot be PETSC_DECIDE\n  likely a call to VecSetSizes() or MatSetSizes() is wrong.\nSee http://www.mcs.anl.gov/petsc/documentation/faq.html#split");
+#if defined(PETSC_USE_DEBUG)
+  {
+    PetscMPIInt l[2],g[2];
+    l[0] = (*n == PETSC_DECIDE) ? 1 : 0;
+    l[1] = (*N == PETSC_DECIDE) ? 1 : 0;
+    ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(l,g,2,MPI_INT,MPI_SUM,comm);CHKERRQ(ierr);
+    if (g[0] && g[0] != size) SETERRQ(comm,PETSC_ERR_ARG_INCOMP,"All processes must supply PETSC_DECIDE for local size");
+    if (g[1] && g[1] != size) SETERRQ(comm,PETSC_ERR_ARG_INCOMP,"All processes must supply PETSC_DECIDE for global size");
+  }
+#endif
 
   if (*N == PETSC_DECIDE) {
-    ierr = MPI_Allreduce(n,N,1,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(n,N,1,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
   } else if (*n == PETSC_DECIDE) {
     ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
     ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
@@ -89,7 +97,7 @@ PetscErrorCode  PetscSplitOwnership(MPI_Comm comm,PetscInt *n,PetscInt *N)
 #if defined(PETSC_USE_DEBUG)
   } else {
     PetscInt tmp;
-    ierr = MPI_Allreduce(n,&tmp,1,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(n,&tmp,1,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
     if (tmp != *N) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Sum of local lengths %D does not equal global length %D, my local length %D\n  likely a call to VecSetSizes() or MatSetSizes() is wrong.\nSee http://www.mcs.anl.gov/petsc/documentation/faq.html#split",tmp,*N,*n);
 #endif
   }
