@@ -1,5 +1,6 @@
 #include <petsc/private/taoimpl.h> /*I "petsctao.h" I*/
 
+
 /*@C
    TaoSetHessianRoutine - Sets the function to compute the Hessian as well as the location to store the matrix.
 
@@ -231,13 +232,13 @@ PetscErrorCode TaoComputeHessian(Tao tao, Vec X, Mat H, Mat Hpre)
   PetscCheckSameComm(tao,1,X,2);
   if (!tao->ops->computehessian) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Must call TaoSetHessianRoutine() first");
   ++tao->nhess;
-  ierr = VecLockPush(X);CHKERRQ(ierr);
+  ierr = VecLockReadPush(X);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(TAO_HessianEval,tao,X,H,Hpre);CHKERRQ(ierr);
   PetscStackPush("Tao user Hessian function");
   ierr = (*tao->ops->computehessian)(tao,X,H,Hpre,tao->user_hessP);CHKERRQ(ierr);
   PetscStackPop;
   ierr = PetscLogEventEnd(TAO_HessianEval,tao,X,H,Hpre);CHKERRQ(ierr);
-  ierr = VecLockPop(X);CHKERRQ(ierr);
+  ierr = VecLockReadPop(X);CHKERRQ(ierr);
 
   ierr = TaoTestHessian(tao);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -279,13 +280,59 @@ PetscErrorCode TaoComputeJacobian(Tao tao, Vec X, Mat J, Mat Jpre)
   PetscCheckSameComm(tao,1,X,2);
   if (!tao->ops->computejacobian) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Must call TaoSetJacobian() first");
   ++tao->njac;
-  ierr = VecLockPush(X);CHKERRQ(ierr);
+  ierr = VecLockReadPush(X);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(TAO_JacobianEval,tao,X,J,Jpre);CHKERRQ(ierr);
   PetscStackPush("Tao user Jacobian function");
   ierr = (*tao->ops->computejacobian)(tao,X,J,Jpre,tao->user_jacP);CHKERRQ(ierr);
   PetscStackPop;
   ierr = PetscLogEventEnd(TAO_JacobianEval,tao,X,J,Jpre);CHKERRQ(ierr);
-  ierr = VecLockPop(X);CHKERRQ(ierr);
+  ierr = VecLockReadPop(X);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   TaoComputeResidualJacobian - Computes the least-squares residual Jacobian matrix that has been
+   set with TaoSetJacobianResidual().
+
+   Collective on Tao
+
+   Input Parameters:
++  tao - the Tao solver context
+-  X   - input vector
+
+   Output Parameters:
++  J    - Jacobian matrix
+-  Jpre - Preconditioning matrix
+
+   Notes:
+   Most users should not need to explicitly call this routine, as it
+   is used internally within the minimization solvers.
+
+   TaoComputeResidualJacobian() is typically used within least-squares
+   implementations, so most users would not generally call this routine
+   themselves.
+
+   Level: developer
+
+.seealso: TaoComputeResidual(), TaoSetJacobianResidual()
+@*/
+PetscErrorCode TaoComputeResidualJacobian(Tao tao, Vec X, Mat J, Mat Jpre)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(tao,TAO_CLASSID,1);
+  PetscValidHeaderSpecific(X, VEC_CLASSID,2);
+  PetscCheckSameComm(tao,1,X,2);
+  if (!tao->ops->computeresidualjacobian) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Must call TaoSetResidualJacobian() first");
+  ++tao->njac;
+  ierr = VecLockReadPush(X);CHKERRQ(ierr);
+  ierr = PetscLogEventBegin(TAO_JacobianEval,tao,X,J,Jpre);CHKERRQ(ierr);
+  PetscStackPush("Tao user least-squares residual Jacobian function");
+  ierr = (*tao->ops->computeresidualjacobian)(tao,X,J,Jpre,tao->user_lsjacP);CHKERRQ(ierr);
+  PetscStackPop;
+  ierr = PetscLogEventEnd(TAO_JacobianEval,tao,X,J,Jpre);CHKERRQ(ierr);
+  ierr = VecLockReadPop(X);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -325,13 +372,13 @@ PetscErrorCode TaoComputeJacobianState(Tao tao, Vec X, Mat J, Mat Jpre, Mat Jinv
   PetscCheckSameComm(tao,1,X,2);
   if (!tao->ops->computejacobianstate) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Must call TaoSetJacobianState() first");
   ++tao->njac_state;
-  ierr = VecLockPush(X);CHKERRQ(ierr);
+  ierr = VecLockReadPush(X);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(TAO_JacobianEval,tao,X,J,Jpre);CHKERRQ(ierr);
   PetscStackPush("Tao user Jacobian(state) function");
   ierr = (*tao->ops->computejacobianstate)(tao,X,J,Jpre,Jinv,tao->user_jac_stateP);CHKERRQ(ierr);
   PetscStackPop;
   ierr = PetscLogEventEnd(TAO_JacobianEval,tao,X,J,Jpre);CHKERRQ(ierr);
-  ierr = VecLockPop(X);CHKERRQ(ierr);
+  ierr = VecLockReadPop(X);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -370,13 +417,13 @@ PetscErrorCode TaoComputeJacobianDesign(Tao tao, Vec X, Mat J)
   PetscCheckSameComm(tao,1,X,2);
   if (!tao->ops->computejacobiandesign) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Must call TaoSetJacobianDesign() first");
   ++tao->njac_design;
-  ierr = VecLockPush(X);CHKERRQ(ierr);
+  ierr = VecLockReadPush(X);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(TAO_JacobianEval,tao,X,J,NULL);CHKERRQ(ierr);
   PetscStackPush("Tao user Jacobian(design) function");
   ierr = (*tao->ops->computejacobiandesign)(tao,X,J,tao->user_jac_designP);CHKERRQ(ierr);
   PetscStackPop;
   ierr = PetscLogEventEnd(TAO_JacobianEval,tao,X,J,NULL);CHKERRQ(ierr);
-  ierr = VecLockPop(X);CHKERRQ(ierr);
+  ierr = VecLockReadPop(X);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -437,6 +484,64 @@ PetscErrorCode TaoSetJacobianRoutine(Tao tao, Mat J, Mat Jpre, PetscErrorCode (*
   PetscFunctionReturn(0);
 }
 
+/*@C
+   TaoSetJacobianResidualRoutine - Sets the function to compute the least-squares residual Jacobian as well as the 
+   location to store the matrix.
+
+   Logically collective on Tao
+
+   Input Parameters:
++  tao  - the Tao context
+.  J    - Matrix used for the jacobian
+.  Jpre - Matrix that will be used operated on by preconditioner, can be same as J
+.  func - Jacobian evaluation routine
+-  ctx  - [optional] user-defined context for private data for the
+          Jacobian evaluation routine (may be NULL)
+
+   Calling sequence of func:
+$    func(Tao tao,Vec x,Mat J,Mat Jpre,void *ctx);
+
++  tao  - the Tao  context
+.  x    - input vector
+.  J    - Jacobian matrix
+.  Jpre - preconditioning matrix, usually the same as J
+-  ctx  - [optional] user-defined Jacobian context
+
+   Level: intermediate
+@*/
+PetscErrorCode TaoSetJacobianResidualRoutine(Tao tao, Mat J, Mat Jpre, PetscErrorCode (*func)(Tao, Vec, Mat, Mat, void*), void *ctx)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(tao,TAO_CLASSID,1);
+  if (J) {
+    PetscValidHeaderSpecific(J,MAT_CLASSID,2);
+    PetscCheckSameComm(tao,1,J,2);
+  }
+  if (Jpre) {
+    PetscValidHeaderSpecific(Jpre,MAT_CLASSID,3);
+    PetscCheckSameComm(tao,1,Jpre,3);
+  }
+  if (ctx) {
+    tao->user_lsjacP = ctx;
+  }
+  if (func) {
+    tao->ops->computeresidualjacobian = func;
+  }
+  if (J) {
+    ierr = PetscObjectReference((PetscObject)J);CHKERRQ(ierr);
+    ierr = MatDestroy(&tao->ls_jac);CHKERRQ(ierr);
+    tao->ls_jac = J;
+  }
+  if (Jpre) {
+    ierr = PetscObjectReference((PetscObject)Jpre);CHKERRQ(ierr);
+    ierr = MatDestroy(&tao->ls_jac_pre);CHKERRQ(ierr);
+    tao->ls_jac_pre=Jpre;
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@C
    TaoSetJacobianStateRoutine - Sets the function to compute the Jacobian
    (and its inverse) of the constraint function with respect to the state variables.
@@ -620,13 +725,13 @@ PetscErrorCode TaoComputeJacobianEquality(Tao tao, Vec X, Mat J, Mat Jpre)
   PetscCheckSameComm(tao,1,X,2);
   if (!tao->ops->computejacobianequality) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Must call TaoSetJacobianEquality() first");
   ++tao->njac_equality;
-  ierr = VecLockPush(X);CHKERRQ(ierr);
+  ierr = VecLockReadPush(X);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(TAO_JacobianEval,tao,X,J,Jpre);CHKERRQ(ierr);
   PetscStackPush("Tao user Jacobian(equality) function");
   ierr = (*tao->ops->computejacobianequality)(tao,X,J,Jpre,tao->user_jac_equalityP);CHKERRQ(ierr);
   PetscStackPop;
   ierr = PetscLogEventEnd(TAO_JacobianEval,tao,X,J,Jpre);CHKERRQ(ierr);
-  ierr = VecLockPop(X);CHKERRQ(ierr);
+  ierr = VecLockReadPop(X);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -662,13 +767,13 @@ PetscErrorCode TaoComputeJacobianInequality(Tao tao, Vec X, Mat J, Mat Jpre)
   PetscCheckSameComm(tao,1,X,2);
   if (!tao->ops->computejacobianinequality) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Must call TaoSetJacobianInequality() first");
   ++tao->njac_inequality;
-  ierr = VecLockPush(X);CHKERRQ(ierr);
+  ierr = VecLockReadPush(X);CHKERRQ(ierr);
   ierr = PetscLogEventBegin(TAO_JacobianEval,tao,X,J,Jpre);CHKERRQ(ierr);
   PetscStackPush("Tao user Jacobian(inequality) function");
   ierr = (*tao->ops->computejacobianinequality)(tao,X,J,Jpre,tao->user_jac_inequalityP);CHKERRQ(ierr);
   PetscStackPop;
   ierr = PetscLogEventEnd(TAO_JacobianEval,tao,X,J,Jpre);CHKERRQ(ierr);
-  ierr = VecLockPop(X);CHKERRQ(ierr);
+  ierr = VecLockReadPop(X);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
