@@ -8,13 +8,8 @@
 
 */
 
-/*T
-   requires: !single
-T*/
-
 #include <petsctao.h>
 
-
 /*
 Description:   These data are the result of a NIST study involving
                ultrasonic calibration.  The response variable is
@@ -32,7 +27,7 @@ static char help[]="Finds the nonlinear least-squares solution to the model \n\
    Concepts: TAO^Solving a system of nonlinear equations, nonlinear least squares
    Routines: TaoCreate();
    Routines: TaoSetType();
-   Routines: TaoSetSeparableObjectiveRoutine();
+   Routines: TaoSetResidualRoutine();
    Routines: TaoSetMonitor();
    Routines: TaoSetInitialVector();
    Routines: TaoSetFromOptions();
@@ -72,7 +67,7 @@ int main(int argc,char **argv)
   AppCtx         user;               /* user-defined work context */
 
    /* Initialize TAO and PETSc */
-  ierr = PetscInitialize(&argc,&argv,(char *)0,help);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&argv,(char *)0,help);if (ierr) return ierr;
   MPI_Comm_size(MPI_COMM_WORLD,&user.size);
   MPI_Comm_rank(MPI_COMM_WORLD,&user.rank);
   ierr = InitializeData(&user);CHKERRQ(ierr);
@@ -92,7 +87,7 @@ int main(int argc,char **argv)
     /* Set the function and Jacobian routines. */
     ierr = FormStartingPoint(x);CHKERRQ(ierr);
     ierr = TaoSetInitialVector(tao,x);CHKERRQ(ierr);
-    ierr = TaoSetSeparableObjectiveRoutine(tao,f,EvaluateFunction,(void*)&user);CHKERRQ(ierr);
+    ierr = TaoSetResidualRoutine(tao,f,EvaluateFunction,(void*)&user);CHKERRQ(ierr);
 
     /* Check for any TAO command line arguments */
     ierr = TaoSetFromOptions(tao);CHKERRQ(ierr);
@@ -459,7 +454,6 @@ PetscErrorCode StopWorkers(AppCtx *user)
   PetscFunctionReturn(0);
 }
 
-
 /*TEST
 
    build:
