@@ -12,6 +12,7 @@ PETSC_EXTERN PetscErrorCode TSAdaptCreate_Basic(TSAdapt);
 PETSC_EXTERN PetscErrorCode TSAdaptCreate_DSP(TSAdapt);
 PETSC_EXTERN PetscErrorCode TSAdaptCreate_CFL(TSAdapt);
 PETSC_EXTERN PetscErrorCode TSAdaptCreate_GLEE(TSAdapt);
+PETSC_EXTERN PetscErrorCode TSAdaptCreate_History(TSAdapt);
 
 /*@C
    TSAdaptRegister -  adds a TSAdapt implementation
@@ -69,11 +70,12 @@ PetscErrorCode  TSAdaptRegisterAll(void)
   PetscFunctionBegin;
   if (TSAdaptRegisterAllCalled) PetscFunctionReturn(0);
   TSAdaptRegisterAllCalled = PETSC_TRUE;
-  ierr = TSAdaptRegister(TSADAPTNONE, TSAdaptCreate_None);CHKERRQ(ierr);
-  ierr = TSAdaptRegister(TSADAPTBASIC,TSAdaptCreate_Basic);CHKERRQ(ierr);
-  ierr = TSAdaptRegister(TSADAPTDSP,  TSAdaptCreate_DSP);CHKERRQ(ierr);
-  ierr = TSAdaptRegister(TSADAPTCFL,  TSAdaptCreate_CFL);CHKERRQ(ierr);
-  ierr = TSAdaptRegister(TSADAPTGLEE, TSAdaptCreate_GLEE);CHKERRQ(ierr);
+  ierr = TSAdaptRegister(TSADAPTNONE,   TSAdaptCreate_None);CHKERRQ(ierr);
+  ierr = TSAdaptRegister(TSADAPTBASIC,  TSAdaptCreate_Basic);CHKERRQ(ierr);
+  ierr = TSAdaptRegister(TSADAPTDSP,    TSAdaptCreate_DSP);CHKERRQ(ierr);
+  ierr = TSAdaptRegister(TSADAPTCFL,    TSAdaptCreate_CFL);CHKERRQ(ierr);
+  ierr = TSAdaptRegister(TSADAPTGLEE,   TSAdaptCreate_GLEE);CHKERRQ(ierr);
+  ierr = TSAdaptRegister(TSADAPTHISTORY,TSAdaptCreate_History);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -99,8 +101,7 @@ PetscErrorCode  TSAdaptFinalizePackage(void)
 
 /*@C
   TSAdaptInitializePackage - This function initializes everything in the TSAdapt package. It is
-  called from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to
-  TSAdaptCreate() when using static libraries.
+  called from TSInitializePackage().
 
   Level: developer
 
@@ -603,7 +604,8 @@ PetscErrorCode TSAdaptGetStepLimits(TSAdapt adapt,PetscReal *hmin,PetscReal *hma
 .  -ts_adapt_dt_min <min> - minimum timestep to use
 .  -ts_adapt_dt_max <max> - maximum timestep to use
 .  -ts_adapt_scale_solve_failed <scale> - scale timestep by this factor if a solve fails
--  -ts_adapt_wnormtype <2 or infinity> - type of norm for computing error estimates
+.  -ts_adapt_wnormtype <2 or infinity> - type of norm for computing error estimates
+-  -ts_adapt_time_step_increase_delay - number of timesteps to delay increasing the time step after it has been decreased due to failed solver
 
    Level: advanced
 
@@ -656,6 +658,8 @@ PetscErrorCode  TSAdaptSetFromOptions(PetscOptionItems *PetscOptionsObject,TSAda
   ierr = PetscOptionsEnum("-ts_adapt_wnormtype","Type of norm computed for error estimation","",NormTypes,(PetscEnum)adapt->wnormtype,(PetscEnum*)&adapt->wnormtype,NULL);CHKERRQ(ierr);
   if (adapt->wnormtype != NORM_2 && adapt->wnormtype != NORM_INFINITY) SETERRQ(PetscObjectComm((PetscObject)adapt),PETSC_ERR_SUP,"Only 2-norm and infinite norm supported");
 
+  ierr = PetscOptionsInt("-ts_adapt_time_step_increase_delay","Number of timesteps to delay increasing the time step after it has been decreased due to failed solver","TSAdaptSetTimeStepIncreaseDelay",adapt->timestepjustdecreased_delay,&adapt->timestepjustdecreased_delay,NULL);CHKERRQ(ierr);
+  
   ierr = PetscOptionsBool("-ts_adapt_monitor","Print choices made by adaptive controller","TSAdaptSetMonitor",adapt->monitor ? PETSC_TRUE : PETSC_FALSE,&flg,&set);CHKERRQ(ierr);
   if (set) {ierr = TSAdaptSetMonitor(adapt,flg);CHKERRQ(ierr);}
 
@@ -819,9 +823,10 @@ PetscErrorCode TSAdaptChoose(TSAdapt adapt,TS ts,PetscReal h,PetscInt *next_sc,P
     /* Increase/reduce step size if end time of next step is close to or overshoots max time */
     PetscReal t = ts->ptime + ts->time_step, h = *next_h;
     PetscReal tend = t + h, tmax = ts->max_time, hmax = tmax - t;
-    PetscReal a = (PetscReal)1.01; /* allow 1% step size increase */
+    PetscReal a = (PetscReal)(1.0 + adapt->matchstepfac[0]);
+    PetscReal b = adapt->matchstepfac[1];
     if (t < tmax && tend > tmax) *next_h = hmax;
-    if (t < tmax && tend < tmax && h > hmax/2) *next_h = hmax/2;
+    if (t < tmax && tend < tmax && h*b > hmax) *next_h = hmax/2;
     if (t < tmax && tend < tmax && h*a > hmax) *next_h = hmax;
   }
 
@@ -838,6 +843,36 @@ PetscErrorCode TSAdaptChoose(TSAdapt adapt,TS ts,PetscReal h,PetscInt *next_sc,P
   PetscFunctionReturn(0);
 }
 
+/*@
+   TSAdaptSetTimeStepIncreaseDelay - The number of timesteps to wait after a decrease in the timestep due to failed solver
+                                     before increasing the time step.
+
+   Logicially Collective on TSAdapt
+
+   Input Arguments:
++  adapt - adaptive controller context
+-  cnt - the number of timesteps
+
+   Options Database Key:
+.  -ts_adapt_time_step_increase_delay cnt - number of steps to delay the increase
+
+   Notes: This is to prevent an adaptor from bouncing back and forth between two nearby timesteps. The default is 0.
+          The successful use of this option is problem dependent
+
+   Developer Note: there is no theory to support this option
+
+   Level: advanced
+
+.seealso:
+@*/
+PetscErrorCode TSAdaptSetTimeStepIncreaseDelay(TSAdapt adapt,PetscInt cnt)
+{
+  PetscFunctionBegin;
+  adapt->timestepjustdecreased_delay = cnt;
+  PetscFunctionReturn(0);
+}
+
+  
 /*@
    TSAdaptCheckStage - checks whether to accept a stage, (e.g. reject and change time step size if nonlinear solve fails)
 
@@ -891,7 +926,7 @@ PetscErrorCode TSAdaptCheckStage(TSAdapt adapt,TS ts,PetscReal t,Vec Y,PetscBool
     ierr = TSGetTimeStep(ts,&dt);CHKERRQ(ierr);
     new_dt = dt * adapt->scale_solve_failed;
     ierr = TSSetTimeStep(ts,new_dt);CHKERRQ(ierr);
-    adapt->timestepjustincreased += 4;
+    adapt->timestepjustdecreased += adapt->timestepjustdecreased_delay;
     if (adapt->monitor) {
       ierr = PetscViewerASCIIAddTab(adapt->monitor,((PetscObject)adapt)->tablevel);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPrintf(adapt->monitor,"    TSAdapt %s step %3D stage rejected (%s) t=%-11g+%10.3e retrying with dt=%-10.3e\n",((PetscObject)adapt)->type_name,ts->steps,SNESConvergedReasons[snesreason],(double)ts->ptime,(double)dt,(double)new_dt);CHKERRQ(ierr);
@@ -940,7 +975,12 @@ PetscErrorCode  TSAdaptCreate(MPI_Comm comm,TSAdapt *inadapt)
   adapt->dt_min             = 1e-20;
   adapt->dt_max             = 1e+20;
   adapt->scale_solve_failed = 0.25;
+  /* these two safety factors are not public, and they are used only in the TS_EXACTFINALTIME_MATCHSTEP case
+     to prevent from situations were unreasonably small time steps are taken in order to match the final time */
+  adapt->matchstepfac[0]    = 0.01; /* allow 1% step size increase in the last step */
+  adapt->matchstepfac[1]    = 2.0;  /* halve last step if it is greater than what remains divided this factor */
   adapt->wnormtype          = NORM_2;
+  adapt->timestepjustdecreased_delay = 0;
 
   *inadapt = adapt;
   PetscFunctionReturn(0);
