@@ -14,14 +14,12 @@ PetscFunctionList CharacteristicList              = NULL;
 PetscBool         CharacteristicRegisterAllCalled = PETSC_FALSE;
 
 PetscErrorCode DMDAGetNeighborsRank(DM, PetscMPIInt []);
-PetscInt       DMDAGetNeighborRelative(DM, PassiveReal, PassiveReal);
+PetscInt       DMDAGetNeighborRelative(DM, PetscReal, PetscReal);
 PetscErrorCode DMDAMapToPeriodicDomain(DM, PetscScalar []);
 
 PetscErrorCode CharacteristicHeapSort(Characteristic, Queue, PetscInt);
 PetscErrorCode CharacteristicSiftDown(Characteristic, Queue, PetscInt, PetscInt);
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicView"
 PetscErrorCode CharacteristicView(Characteristic c, PetscViewer viewer)
 {
   PetscBool      iascii;
@@ -44,8 +42,6 @@ PetscErrorCode CharacteristicView(Characteristic c, PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicDestroy"
 PetscErrorCode CharacteristicDestroy(Characteristic *c)
 {
   PetscErrorCode ierr;
@@ -73,8 +69,6 @@ PetscErrorCode CharacteristicDestroy(Characteristic *c)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicCreate"
 PetscErrorCode CharacteristicCreate(MPI_Comm comm, Characteristic *c)
 {
   Characteristic newC;
@@ -85,7 +79,7 @@ PetscErrorCode CharacteristicCreate(MPI_Comm comm, Characteristic *c)
   *c = NULL;
   ierr = CharacteristicInitializePackage();CHKERRQ(ierr);
 
-  ierr = PetscHeaderCreate(newC, CHARACTERISTIC_CLASSID, "Characteristic", "Characteristic", "SemiLagrange", comm, CharacteristicDestroy, CharacteristicView);CHKERRQ(ierr);
+  ierr = PetscHeaderCreate(newC, CHARACTERISTIC_CLASSID, "Characteristic", "Characteristic", "Characteristic", comm, CharacteristicDestroy, CharacteristicView);CHKERRQ(ierr);
   *c   = newC;
 
   newC->structured          = PETSC_TRUE;
@@ -126,8 +120,6 @@ PetscErrorCode CharacteristicCreate(MPI_Comm comm, Characteristic *c)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicSetType"
 /*@C
    CharacteristicSetType - Builds Characteristic for a particular solver.
 
@@ -190,8 +182,6 @@ PetscErrorCode CharacteristicSetType(Characteristic c, CharacteristicType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicSetUp"
 /*@
    CharacteristicSetUp - Sets up the internal data structures for the
    later use of an iterative solver.
@@ -229,8 +219,6 @@ PetscErrorCode CharacteristicSetUp(Characteristic c)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicRegister"
 /*@C
   CharacteristicRegister -  Adds a solver to the method of characteristics package.
 
@@ -269,12 +257,11 @@ PetscErrorCode CharacteristicRegister(const char sname[],PetscErrorCode (*functi
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = CharacteristicInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&CharacteristicList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicSetVelocityInterpolation"
 PetscErrorCode CharacteristicSetVelocityInterpolation(Characteristic c, DM da, Vec v, Vec vOld, PetscInt numComponents, PetscInt components[], PetscErrorCode (*interp)(Vec, PetscReal[], PetscInt, PetscInt[], PetscScalar[], void*), void *ctx)
 {
   PetscFunctionBegin;
@@ -288,8 +275,6 @@ PetscErrorCode CharacteristicSetVelocityInterpolation(Characteristic c, DM da, V
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicSetVelocityInterpolationLocal"
 PetscErrorCode CharacteristicSetVelocityInterpolationLocal(Characteristic c, DM da, Vec v, Vec vOld, PetscInt numComponents, PetscInt components[], PetscErrorCode (*interp)(void*, PetscReal [], PetscInt, PetscInt[], PetscScalar[], void*), void *ctx)
 {
   PetscFunctionBegin;
@@ -303,8 +288,6 @@ PetscErrorCode CharacteristicSetVelocityInterpolationLocal(Characteristic c, DM
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicSetFieldInterpolation"
 PetscErrorCode CharacteristicSetFieldInterpolation(Characteristic c, DM da, Vec v, PetscInt numComponents, PetscInt components[], PetscErrorCode (*interp)(Vec, PetscReal[], PetscInt, PetscInt[], PetscScalar[], void*), void *ctx)
 {
   PetscFunctionBegin;
@@ -320,8 +303,6 @@ PetscErrorCode CharacteristicSetFieldInterpolation(Characteristic c, DM da, Vec
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicSetFieldInterpolationLocal"
 PetscErrorCode CharacteristicSetFieldInterpolationLocal(Characteristic c, DM da, Vec v, PetscInt numComponents, PetscInt components[], PetscErrorCode (*interp)(void*, PetscReal[], PetscInt, PetscInt[], PetscScalar [], void*), void *ctx)
 {
   PetscFunctionBegin;
@@ -337,8 +318,6 @@ PetscErrorCode CharacteristicSetFieldInterpolationLocal(Characteristic c, DM da,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicSolve"
 PetscErrorCode CharacteristicSolve(Characteristic c, PetscReal dt, Vec solution)
 {
   CharacteristicPointDA2D Qi;
@@ -350,9 +329,9 @@ PetscErrorCode CharacteristicSolve(Characteristic c, PetscReal dt, Vec solution)
   void                    *velocityArray;
   void                    *velocityArrayOld;
   void                    *fieldArray;
-  PassiveScalar           *interpIndices;
-  PassiveScalar           *velocityValues, *velocityValuesOld;
-  PassiveScalar           *fieldValues;
+  PetscScalar             *interpIndices;
+  PetscScalar             *velocityValues, *velocityValuesOld;
+  PetscScalar             *fieldValues;
   PetscMPIInt             rank;
   PetscInt                dim;
   PetscMPIInt             neighbors[9];
@@ -399,8 +378,8 @@ PetscErrorCode CharacteristicSolve(Characteristic c, PetscReal dt, Vec solution)
     for (Qi.i = is; Qi.i < ie; Qi.i++) {
       interpIndices[0] = Qi.i;
       interpIndices[1] = Qi.j;
-      if (c->velocityInterpLocal) c->velocityInterpLocal(velocityArray, interpIndices, c->numVelocityComp, c->velocityComp, velocityValues, c->velocityCtx);
-      else c->velocityInterp(c->velocity, interpIndices, c->numVelocityComp, c->velocityComp, velocityValues, c->velocityCtx);
+      if (c->velocityInterpLocal) {ierr = c->velocityInterpLocal(velocityArray, interpIndices, c->numVelocityComp, c->velocityComp, velocityValues, c->velocityCtx);CHKERRQ(ierr);}
+      else {ierr = c->velocityInterp(c->velocity, interpIndices, c->numVelocityComp, c->velocityComp, velocityValues, c->velocityCtx);CHKERRQ(ierr);}
       Qi.x = Qi.i - velocityValues[0]*dt/2.0;
       Qi.y = Qi.j - velocityValues[1]*dt/2.0;
 
@@ -427,11 +406,11 @@ PetscErrorCode CharacteristicSolve(Characteristic c, PetscReal dt, Vec solution)
       interpIndices[0] = Qi.x;
       interpIndices[1] = Qi.y;
       if (c->velocityInterpLocal) {
-        c->velocityInterpLocal(velocityArray,    interpIndices, c->numVelocityComp, c->velocityComp, velocityValues,    c->velocityCtx);
-        c->velocityInterpLocal(velocityArrayOld, interpIndices, c->numVelocityComp, c->velocityComp, velocityValuesOld, c->velocityCtx);
+        ierr = c->velocityInterpLocal(velocityArray,    interpIndices, c->numVelocityComp, c->velocityComp, velocityValues, c->velocityCtx);CHKERRQ(ierr);
+        ierr = c->velocityInterpLocal(velocityArrayOld, interpIndices, c->numVelocityComp, c->velocityComp, velocityValuesOld, c->velocityCtx);CHKERRQ(ierr);
       } else {
-        c->velocityInterp(c->velocity,    interpIndices, c->numVelocityComp, c->velocityComp, velocityValues,    c->velocityCtx);
-        c->velocityInterp(c->velocityOld, interpIndices, c->numVelocityComp, c->velocityComp, velocityValuesOld, c->velocityCtx);
+        ierr = c->velocityInterp(c->velocity,    interpIndices, c->numVelocityComp, c->velocityComp, velocityValues, c->velocityCtx);CHKERRQ(ierr);
+        ierr = c->velocityInterp(c->velocityOld, interpIndices, c->numVelocityComp, c->velocityComp, velocityValuesOld, c->velocityCtx);CHKERRQ(ierr);
       }
       Qi.x = 0.5*(velocityValues[0] + velocityValuesOld[0]);
       Qi.y = 0.5*(velocityValues[1] + velocityValuesOld[1]);
@@ -453,11 +432,11 @@ PetscErrorCode CharacteristicSolve(Characteristic c, PetscReal dt, Vec solution)
     interpIndices[0] = Qi.x;
     interpIndices[1] = Qi.y;
     if (c->velocityInterpLocal) {
-      c->velocityInterpLocal(velocityArray,    interpIndices, c->numVelocityComp, c->velocityComp, velocityValues,    c->velocityCtx);
-      c->velocityInterpLocal(velocityArrayOld, interpIndices, c->numVelocityComp, c->velocityComp, velocityValuesOld, c->velocityCtx);
+      ierr = c->velocityInterpLocal(velocityArray,    interpIndices, c->numVelocityComp, c->velocityComp, velocityValues,    c->velocityCtx);CHKERRQ(ierr);
+      ierr = c->velocityInterpLocal(velocityArrayOld, interpIndices, c->numVelocityComp, c->velocityComp, velocityValuesOld, c->velocityCtx);CHKERRQ(ierr);
     } else {
-      c->velocityInterp(c->velocity,    interpIndices, c->numVelocityComp, c->velocityComp, velocityValues,    c->velocityCtx);
-      c->velocityInterp(c->velocityOld, interpIndices, c->numVelocityComp, c->velocityComp, velocityValuesOld, c->velocityCtx);
+      ierr = c->velocityInterp(c->velocity,    interpIndices, c->numVelocityComp, c->velocityComp, velocityValues,    c->velocityCtx);CHKERRQ(ierr);
+      ierr = c->velocityInterp(c->velocityOld, interpIndices, c->numVelocityComp, c->velocityComp, velocityValuesOld, c->velocityCtx);CHKERRQ(ierr);
     }
     Qi.x = 0.5*(velocityValues[0] + velocityValuesOld[0]);
     Qi.y = 0.5*(velocityValues[1] + velocityValuesOld[1]);
@@ -514,8 +493,8 @@ PetscErrorCode CharacteristicSolve(Characteristic c, PetscReal dt, Vec solution)
     if (c->neighbors[c->queue[n].proc] == rank) {
       interpIndices[0] = c->queue[n].x;
       interpIndices[1] = c->queue[n].y;
-      if (c->fieldInterpLocal) c->fieldInterpLocal(fieldArray, interpIndices, c->numFieldComp, c->fieldComp, fieldValues, c->fieldCtx);
-      else c->fieldInterp(c->field, interpIndices, c->numFieldComp, c->fieldComp, fieldValues, c->fieldCtx);
+      if (c->fieldInterpLocal) {ierr = c->fieldInterpLocal(fieldArray, interpIndices, c->numFieldComp, c->fieldComp, fieldValues, c->fieldCtx);CHKERRQ(ierr);}
+      else {ierr = c->fieldInterp(c->field, interpIndices, c->numFieldComp, c->fieldComp, fieldValues, c->fieldCtx);CHKERRQ(ierr);}
       for (comp = 0; comp < c->numFieldComp; comp++) c->queue[n].field[comp] = fieldValues[comp];
     }
   }
@@ -539,8 +518,8 @@ PetscErrorCode CharacteristicSolve(Characteristic c, PetscReal dt, Vec solution)
       if ((im < (PetscScalar) is - 1.) || (im > (PetscScalar) ie) || (jm < (PetscScalar)  js - 1.) || (jm > (PetscScalar) je)) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_LIB, "Nonlocal point: (%g,%g)", im, jm);
     }
 
-    if (c->fieldInterpLocal) c->fieldInterpLocal(fieldArray, interpIndices, c->numFieldComp, c->fieldComp, fieldValues, c->fieldCtx);
-    else c->fieldInterp(c->field, interpIndices, c->numFieldComp, c->fieldComp, fieldValues, c->fieldCtx);
+    if (c->fieldInterpLocal) {ierr = c->fieldInterpLocal(fieldArray, interpIndices, c->numFieldComp, c->fieldComp, fieldValues, c->fieldCtx);CHKERRQ(ierr);}
+    else {ierr = c->fieldInterp(c->field, interpIndices, c->numFieldComp, c->fieldComp, fieldValues, c->fieldCtx);CHKERRQ(ierr);}
     for (comp = 0; comp < c->numFieldComp; comp++) c->queueRemote[n].field[comp] = fieldValues[comp];
   }
   ierr = PetscLogEventEnd(CHARACTERISTIC_FullTimeRemote,0,0,0,0);CHKERRQ(ierr);
@@ -574,8 +553,6 @@ PetscErrorCode CharacteristicSolve(Characteristic c, PetscReal dt, Vec solution)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicSetNeighbors"
 PetscErrorCode CharacteristicSetNeighbors(Characteristic c, PetscInt numNeighbors, PetscMPIInt neighbors[])
 {
   PetscErrorCode ierr;
@@ -588,8 +565,6 @@ PetscErrorCode CharacteristicSetNeighbors(Characteristic c, PetscInt numNeighbor
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicAddPoint"
 PetscErrorCode CharacteristicAddPoint(Characteristic c, CharacteristicPointDA2D *point)
 {
   PetscFunctionBegin;
@@ -598,8 +573,6 @@ PetscErrorCode CharacteristicAddPoint(Characteristic c, CharacteristicPointDA2D
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicSendCoordinatesBegin"
 int CharacteristicSendCoordinatesBegin(Characteristic c)
 {
   PetscMPIInt    rank, tag = 121;
@@ -633,7 +606,7 @@ int CharacteristicSendCoordinatesBegin(Characteristic c)
   c->needCount[0] = 0;
   /* HACK END */
   if (c->queueRemoteMax) {
-    ierr = PetscMalloc(sizeof(CharacteristicPointDA2D) * c->queueRemoteMax, &c->queueRemote);CHKERRQ(ierr);
+    ierr = PetscMalloc1(c->queueRemoteMax, &c->queueRemote);CHKERRQ(ierr);
   } else c->queueRemote = NULL;
   c->queueRemoteSize = c->queueRemoteMax;
 
@@ -649,8 +622,6 @@ int CharacteristicSendCoordinatesBegin(Characteristic c)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicSendCoordinatesEnd"
 PetscErrorCode CharacteristicSendCoordinatesEnd(Characteristic c)
 {
 #if 0
@@ -670,8 +641,6 @@ PetscErrorCode CharacteristicSendCoordinatesEnd(Characteristic c)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicGetValuesBegin"
 PetscErrorCode CharacteristicGetValuesBegin(Characteristic c)
 {
   PetscMPIInt    tag = 121;
@@ -689,8 +658,6 @@ PetscErrorCode CharacteristicGetValuesBegin(Characteristic c)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicGetValuesEnd"
 PetscErrorCode CharacteristicGetValuesEnd(Characteristic c)
 {
   PetscErrorCode ierr;
@@ -703,8 +670,6 @@ PetscErrorCode CharacteristicGetValuesEnd(Characteristic c)
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicHeapSort"
 /*
   Based on code from http://linux.wku.edu/~lamonml/algor/sort/heap.html
 */
@@ -717,7 +682,7 @@ PetscErrorCode CharacteristicHeapSort(Characteristic c, Queue queue, PetscInt si
 
   PetscFunctionBegin;
   if (0) { /* Check the order of the queue before sorting */
-    PetscInfo(NULL, "Before Heap sort\n");
+    ierr = PetscInfo(NULL, "Before Heap sort\n");CHKERRQ(ierr);
     for (n=0; n<size; n++) {
       ierr = PetscInfo2(NULL,"%d %d\n",n,queue[n].proc);CHKERRQ(ierr);
     }
@@ -743,8 +708,6 @@ PetscErrorCode CharacteristicHeapSort(Characteristic c, Queue queue, PetscInt si
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicSiftDown"
 /*
   Based on code from http://linux.wku.edu/~lamonml/algor/sort/heap.html
 */
@@ -771,8 +734,6 @@ PetscErrorCode CharacteristicSiftDown(Characteristic c, Queue queue, PetscInt ro
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetNeighborsRank"
 /* [center, left, top-left, top, top-right, right, bottom-right, bottom, bottom-left] */
 PetscErrorCode DMDAGetNeighborsRank(DM da, PetscMPIInt neighbors[])
 {
@@ -835,8 +796,6 @@ PetscErrorCode DMDAGetNeighborsRank(DM da, PetscMPIInt neighbors[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetNeighborRelative"
 /*
   SUBDOMAIN NEIGHBORHOOD PROCESS MAP:
     2 | 3 | 4
@@ -846,15 +805,15 @@ PetscErrorCode DMDAGetNeighborsRank(DM da, PetscMPIInt neighbors[])
     8 | 7 | 6
       |   |
 */
-PetscInt DMDAGetNeighborRelative(DM da, PassiveReal ir, PassiveReal jr)
+PetscInt DMDAGetNeighborRelative(DM da, PetscReal ir, PetscReal jr)
 {
   DMDALocalInfo  info;
-  PassiveReal    is,ie,js,je;
+  PetscReal      is,ie,js,je;
   PetscErrorCode ierr;
 
   ierr = DMDAGetLocalInfo(da, &info);CHKERRQ(ierr);
-  is   = (PassiveReal) info.xs - 0.5; ie = (PassiveReal) info.xs + info.xm - 0.5;
-  js   = (PassiveReal) info.ys - 0.5; je = (PassiveReal) info.ys + info.ym - 0.5;
+  is   = (PetscReal) info.xs - 0.5; ie = (PetscReal) info.xs + info.xm - 0.5;
+  js   = (PetscReal) info.ys - 0.5; je = (PetscReal) info.ys + info.ym - 0.5;
 
   if (ir >= is && ir <= ie) { /* center column */
     if (jr >= js && jr <= je) return 0;
