@@ -14,7 +14,7 @@ Input arguments are\n\
 #include <petscts.h>
 
 /* special variable - max size of all arrays  */
-#define num_z 60
+#define num_z 10
 
 /*
    User-defined application context - contains data needed by the
@@ -41,8 +41,6 @@ extern PetscErrorCode femA(AppCtx*,PetscInt,PetscScalar*);
 extern PetscErrorCode rhs(AppCtx*,PetscScalar*, PetscInt, PetscScalar*,PetscReal);
 extern PetscErrorCode RHSfunction(TS,PetscReal,Vec,Vec,void*);
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   PetscInt       i,m,nz,steps,max_steps,k,nphase=1;
@@ -56,11 +54,11 @@ int main(int argc,char **argv)
   Vec            init_sol; /* ts solution vector */
   PetscMPIInt    size;
 
-  PetscInitialize(&argc,&argv,(char*)0,help);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
   if (size != 1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"This is a uniprocessor example only");
 
-  ierr = PetscOptionsGetInt(NULL,"-nphase",&nphase,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-nphase",&nphase,NULL);CHKERRQ(ierr);
   if (nphase > 3) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"nphase must be an integer between 1 and 3");
 
   /* initializations */
@@ -77,8 +75,8 @@ int main(int argc,char **argv)
   appctx.debug      = PETSC_FALSE;
   appctx.useAlhs    = PETSC_FALSE;
 
-  ierr = PetscOptionsHasName(NULL,"-debug",&appctx.debug);CHKERRQ(ierr);
-  ierr = PetscOptionsHasName(NULL,"-useAlhs",&appctx.useAlhs);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-debug",&appctx.debug);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-useAlhs",&appctx.useAlhs);CHKERRQ(ierr);
 
   /* create vector to hold ts solution */
   /*-----------------------------------*/
@@ -91,7 +89,7 @@ int main(int argc,char **argv)
 
   /* create LHS matrix Amat */
   /*------------------------*/
-  ierr = MatCreateSeqAIJ(PETSC_COMM_WORLD, m, m, 3, NULL, &appctx.Amat);
+  ierr = MatCreateSeqAIJ(PETSC_COMM_WORLD, m, m, 3, NULL, &appctx.Amat);CHKERRQ(ierr);
   ierr = MatSetFromOptions(appctx.Amat);CHKERRQ(ierr);
   ierr = MatSetUp(appctx.Amat);CHKERRQ(ierr);
   /* set space grid points - interio points only! */
@@ -141,7 +139,7 @@ int main(int argc,char **argv)
      * places.
      */
     ierr = TSSetIFunction(ts,NULL,TSComputeIFunctionLinear,&appctx);CHKERRQ(ierr);
-    ierr = TSSetIJacobian(ts,appctx.Amat,appctx.Amat,TSComputeIJacobianConstant,&appctx);
+    ierr = TSSetIJacobian(ts,appctx.Amat,appctx.Amat,TSComputeIJacobianConstant,&appctx);CHKERRQ(ierr);
   }
 
   /* use petsc to compute the jacobian by finite differences */
@@ -166,15 +164,18 @@ int main(int argc,char **argv)
   stepsz[0] = 1.0/(2.0*(nz-1)*(nz-1)); /* (mesh_size)^2/2.0 */
   ftime     = 0.0;
   for (k=0; k<nphase; k++) {
-    if (nphase > 1) {ierr = PetscPrintf(PETSC_COMM_WORLD,"Phase %D initial time %g, stepsz %g, duration: %g\n",k,(double)ftime,(double)stepsz[k],(double)((k+1)*T));}
-    ierr = TSSetInitialTimeStep(ts,ftime,stepsz[k]);CHKERRQ(ierr);
-    ierr = TSSetDuration(ts,max_steps,(k+1)*T);CHKERRQ(ierr);
+    if (nphase > 1) {ierr = PetscPrintf(PETSC_COMM_WORLD,"Phase %D initial time %g, stepsz %g, duration: %g\n",k,(double)ftime,(double)stepsz[k],(double)((k+1)*T));CHKERRQ(ierr);}
+    ierr = TSSetTime(ts,ftime);CHKERRQ(ierr);
+    ierr = TSSetTimeStep(ts,stepsz[k]);CHKERRQ(ierr);
+    ierr = TSSetMaxSteps(ts,max_steps);CHKERRQ(ierr);
+    ierr = TSSetMaxTime(ts,(k+1)*T);CHKERRQ(ierr);
+    ierr = TSSetExactFinalTime(ts,TS_EXACTFINALTIME_STEPOVER);CHKERRQ(ierr);
 
     /* loop over time steps */
     /*----------------------*/
     ierr = TSSolve(ts,init_sol);CHKERRQ(ierr);
     ierr = TSGetSolveTime(ts,&ftime);CHKERRQ(ierr);
-    ierr = TSGetTimeStepNumber(ts,&steps);CHKERRQ(ierr);
+    ierr = TSGetStepNumber(ts,&steps);CHKERRQ(ierr);
     stepsz[k+1] = stepsz[k]*1.5; /* change step size for the next phase */
   }
 
@@ -188,9 +189,9 @@ int main(int argc,char **argv)
   ierr = VecDestroy(&appctx.solution);CHKERRQ(ierr);
   ierr = PetscFree(z);CHKERRQ(ierr);
 
-  PetscFinalize();
-  return 0;
-  }
+  ierr = PetscFinalize();
+  return ierr;
+}
 
 /*------------------------------------------------------------------------
   Set exact solution
@@ -206,8 +207,6 @@ PetscScalar exact(PetscScalar z,PetscReal t)
   return val;
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "Monitor"
 /*
    Monitor - User-provided routine to monitor the solution computed at
    each timestep.  This example plots the solution and computes the
@@ -378,7 +377,7 @@ PetscErrorCode femBg(PetscScalar btri[][3],PetscScalar *f,PetscInt nz,PetscScala
     zquad[il][0] = zip;
     zquad[il][1] = (0.5)*(zip+zipq);
     zquad[il][2] = zipq;
-    dlen[il]     = fabs(dl);
+    dlen[il]     = PetscAbsScalar(dl);
     nli[il][0]   = ip;
     nli[il][1]   = ipq;
   }
@@ -460,7 +459,7 @@ PetscErrorCode femA(AppCtx *obj,PetscInt nz,PetscScalar *z)
     rquad[il][0] = zip;
     rquad[il][1] = (0.5)*(zip+zipq);
     rquad[il][2] = zipq;
-    dlen[il]     = fabs(dl);
+    dlen[il]     = PetscAbsScalar(dl);
     nli[il][0]   = ip;
     nli[il][1]   = ipq;
   } /*end for (il)*/
@@ -579,3 +578,21 @@ PetscErrorCode RHSfunction(TS ts,PetscReal t,Vec globalin,Vec globalout,void *ct
   }
   return 0;
 }
+
+/*TEST
+
+    build:
+      requires: !complex
+
+    test:
+      suffix: euler
+      output_file: output/ex3.out
+
+    test:
+      suffix: 2
+      args:   -useAlhs
+      output_file: output/ex3.out
+      TODO: Broken because SNESComputeJacobianDefault is incompatible with TSComputeIJacobianConstant
+
+TEST*/
+
