@@ -1579,7 +1579,7 @@ int main(int argc,char *argv[])
   PetscReal      ftime;
   PetscErrorCode ierr;
 
-  ierr = PetscInitialize(&argc,&argv,0,help);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&argv,0,help);if (ierr) return ierr;
   comm = PETSC_COMM_WORLD;
 
   ierr = THICreate(comm,&thi);CHKERRQ(ierr);
