@@ -3,6 +3,7 @@ static char help[] ="Solves a simple time-dependent linear PDE (the heat equatio
 Input parameters include:\n\
   -m <points>, where <points> = number of grid points\n\
   -time_dependent_rhs : Treat the problem as having a time-dependent right-hand side\n\
+  -use_ifunc          : Use IFunction/IJacobian interface\n\
   -debug              : Activate debugging printouts\n\
   -nox                : Deactivate x-window graphics\n\n";
 
@@ -67,8 +68,10 @@ typedef struct {
   PetscInt    m;                 /* total number of grid points */
   PetscReal   h;                 /* mesh width h = 1/(m-1) */
   PetscBool   debug;             /* flag (1 indicates activation of debugging printouts) */
-  PetscViewer viewer1,viewer2;  /* viewers for the solution and error */
-  PetscReal   norm_2,norm_max;  /* error norms */
+  PetscViewer viewer1,viewer2;   /* viewers for the solution and error */
+  PetscReal   norm_2,norm_max;   /* error norms */
+  Mat         A;                 /* RHS mat, used with IFunction interface */
+  PetscReal   oshift;            /* old shift applied, prevent to recompute the IJacobian */
 } AppCtx;
 
 /*
@@ -76,11 +79,11 @@ typedef struct {
 */
 extern PetscErrorCode InitialConditions(Vec,AppCtx*);
 extern PetscErrorCode RHSMatrixHeat(TS,PetscReal,Vec,Mat,Mat,void*);
+extern PetscErrorCode IFunctionHeat(TS,PetscReal,Vec,Vec,Vec,void*);
+extern PetscErrorCode IJacobianHeat(TS,PetscReal,Vec,Vec,PetscReal,Mat,Mat,void*);
 extern PetscErrorCode Monitor(TS,PetscInt,PetscReal,Vec,void*);
 extern PetscErrorCode ExactSolution(PetscReal,Vec,AppCtx*);
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   AppCtx         appctx;                 /* user-defined application context */
@@ -100,13 +103,13 @@ int main(int argc,char **argv)
      Initialize program and set problem parameters
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-  ierr = PetscInitialize(&argc,&argv,(char*)0,help);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
   if (size != 1) SETERRQ(PETSC_COMM_SELF,1,"This is a uniprocessor example only!");
 
   m    = 60;
-  ierr = PetscOptionsGetInt(NULL,"-m",&m,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsHasName(NULL,"-debug",&appctx.debug);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-m",&m,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-debug",&appctx.debug);CHKERRQ(ierr);
 
   appctx.m        = m;
   appctx.h        = 1.0/(m-1.0);
@@ -160,33 +163,48 @@ int main(int argc,char **argv)
   ierr = MatSetUp(A);CHKERRQ(ierr);
 
   flg  = PETSC_FALSE;
-  ierr = PetscOptionsGetBool(NULL,"-time_dependent_rhs",&flg,NULL);CHKERRQ(ierr);
-  if (flg) {
-    /*
-       For linear problems with a time-dependent f(u,t) in the equation
-       u_t = f(u,t), the user provides the discretized right-hand-side
-       as a time-dependent matrix.
-    */
-    ierr = TSSetRHSFunction(ts,NULL,TSComputeRHSFunctionLinear,&appctx);CHKERRQ(ierr);
-    ierr = TSSetRHSJacobian(ts,A,A,RHSMatrixHeat,&appctx);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-use_ifunc",&flg,NULL);CHKERRQ(ierr);
+  if (!flg) {
+    appctx.A = NULL;
+    ierr = PetscOptionsGetBool(NULL,NULL,"-time_dependent_rhs",&flg,NULL);CHKERRQ(ierr);
+    if (flg) {
+      /*
+         For linear problems with a time-dependent f(u,t) in the equation
+         u_t = f(u,t), the user provides the discretized right-hand-side
+         as a time-dependent matrix.
+      */
+      ierr = TSSetRHSFunction(ts,NULL,TSComputeRHSFunctionLinear,&appctx);CHKERRQ(ierr);
+      ierr = TSSetRHSJacobian(ts,A,A,RHSMatrixHeat,&appctx);CHKERRQ(ierr);
+    } else {
+      /*
+         For linear problems with a time-independent f(u) in the equation
+         u_t = f(u), the user provides the discretized right-hand-side
+         as a matrix only once, and then sets the special Jacobian evaluation
+         routine TSComputeRHSJacobianConstant() which will NOT recompute the Jacobian.
+      */
+      ierr = RHSMatrixHeat(ts,0.0,u,A,A,&appctx);CHKERRQ(ierr);
+      ierr = TSSetRHSFunction(ts,NULL,TSComputeRHSFunctionLinear,&appctx);CHKERRQ(ierr);
+      ierr = TSSetRHSJacobian(ts,A,A,TSComputeRHSJacobianConstant,&appctx);CHKERRQ(ierr);
+    }
   } else {
-    /*
-       For linear problems with a time-independent f(u) in the equation
-       u_t = f(u), the user provides the discretized right-hand-side
-       as a matrix only once, and then sets the special Jacobian evaluation
-       routine TSComputeRHSJacobianConstant() which will NOT recompute the Jacobian.
-    */
-    ierr = RHSMatrixHeat(ts,0.0,u,A,A,&appctx);CHKERRQ(ierr);
-    ierr = TSSetRHSFunction(ts,NULL,TSComputeRHSFunctionLinear,&appctx);CHKERRQ(ierr);
-    ierr = TSSetRHSJacobian(ts,A,A,TSComputeRHSJacobianConstant,&appctx);CHKERRQ(ierr);
-  }
+    Mat J;
 
+    ierr = RHSMatrixHeat(ts,0.0,u,A,A,&appctx);CHKERRQ(ierr);
+    ierr = MatDuplicate(A,MAT_DO_NOT_COPY_VALUES,&J);CHKERRQ(ierr);
+    ierr = TSSetIFunction(ts,NULL,IFunctionHeat,&appctx);CHKERRQ(ierr);
+    ierr = TSSetIJacobian(ts,J,J,IJacobianHeat,&appctx);CHKERRQ(ierr);
+    ierr = MatDestroy(&J);CHKERRQ(ierr);
+
+    ierr = PetscObjectReference((PetscObject)A);CHKERRQ(ierr);
+    appctx.A = A;
+    appctx.oshift = PETSC_MIN_REAL;
+  }
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Set solution vector and initial timestep
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
   dt   = appctx.h*appctx.h/2.0;
-  ierr = TSSetInitialTimeStep(ts,0.0,dt);CHKERRQ(ierr);
+  ierr = TSSetTimeStep(ts,dt);CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Customize timestepping solver:
@@ -194,11 +212,13 @@ int main(int argc,char **argv)
        - Set timestepping duration info
      Then set runtime options, which can override these defaults.
      For example,
-          -ts_max_steps <maxsteps> -ts_final_time <maxtime>
-     to override the defaults set by TSSetDuration().
+          -ts_max_steps <maxsteps> -ts_max_time <maxtime>
+     to override the defaults set by TSSetMaxSteps()/TSSetMaxTime().
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-  ierr = TSSetDuration(ts,time_steps_max,time_total_max);CHKERRQ(ierr);
+  ierr = TSSetMaxSteps(ts,time_steps_max);CHKERRQ(ierr);
+  ierr = TSSetMaxTime(ts,time_total_max);CHKERRQ(ierr);
+  ierr = TSSetExactFinalTime(ts,TS_EXACTFINALTIME_STEPOVER);CHKERRQ(ierr);
   ierr = TSSetFromOptions(ts);CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -214,7 +234,7 @@ int main(int argc,char **argv)
      Run the timestepping solver
   */
   ierr = TSSolve(ts,u);CHKERRQ(ierr);
-  ierr = TSGetTimeStepNumber(ts,&steps);CHKERRQ(ierr);
+  ierr = TSGetStepNumber(ts,&steps);CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      View timestepping solver info
@@ -234,19 +254,18 @@ int main(int argc,char **argv)
   ierr = PetscViewerDestroy(&appctx.viewer1);CHKERRQ(ierr);
   ierr = PetscViewerDestroy(&appctx.viewer2);CHKERRQ(ierr);
   ierr = VecDestroy(&appctx.solution);CHKERRQ(ierr);
+  ierr = MatDestroy(&appctx.A);CHKERRQ(ierr);
 
   /*
      Always call PetscFinalize() before exiting a program.  This routine
        - finalizes the PETSc libraries as well as MPI
        - provides summary and diagnostic information if certain runtime
-         options are chosen (e.g., -log_summary).
+         options are chosen (e.g., -log_view).
   */
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 /* --------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "InitialConditions"
 /*
    InitialConditions - Computes the solution at the initial time.
 
@@ -297,8 +316,6 @@ PetscErrorCode InitialConditions(Vec u,AppCtx *appctx)
   return 0;
 }
 /* --------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "ExactSolution"
 /*
    ExactSolution - Computes the exact solution at a given time.
 
@@ -337,8 +354,6 @@ PetscErrorCode ExactSolution(PetscReal t,Vec solution,AppCtx *appctx)
   return 0;
 }
 /* --------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "Monitor"
 /*
    Monitor - User-provided routine to monitor the solution computed at
    each timestep.  This example plots the solution and computes the
@@ -390,15 +405,17 @@ PetscErrorCode Monitor(TS ts,PetscInt step,PetscReal time,Vec u,void *ctx)
   ierr   = VecNorm(appctx->solution,NORM_2,&norm_2);CHKERRQ(ierr);
   norm_2 = PetscSqrtReal(appctx->h)*norm_2;
   ierr   = VecNorm(appctx->solution,NORM_MAX,&norm_max);CHKERRQ(ierr);
+  if (norm_2   < 1e-14) norm_2   = 0;
+  if (norm_max < 1e-14) norm_max = 0;
 
   ierr = TSGetTimeStep(ts,&dt);CHKERRQ(ierr);
-  ierr = PetscPrintf(PETSC_COMM_WORLD,"Timestep %3D: step size = %-11g, time = %-11g, 2-norm error = %-11g, max norm error = %-11g\n",step,(double)dt,(double)time,(double)norm_2,(double)norm_max);CHKERRQ(ierr);
+  ierr = PetscPrintf(PETSC_COMM_WORLD,"Timestep %3D: step size = %g, time = %g, 2-norm error = %g, max norm error = %g\n",step,(double)dt,(double)time,(double)norm_2,(double)norm_max);CHKERRQ(ierr);
 
   appctx->norm_2   += norm_2;
   appctx->norm_max += norm_max;
 
   dttol = .0001;
-  ierr  = PetscOptionsGetReal(NULL,"-dttol",&dttol,NULL);CHKERRQ(ierr);
+  ierr  = PetscOptionsGetReal(NULL,NULL,"-dttol",&dttol,NULL);CHKERRQ(ierr);
   if (dt < dttol) {
     dt  *= .999;
     ierr = TSSetTimeStep(ts,dt);CHKERRQ(ierr);
@@ -420,8 +437,6 @@ PetscErrorCode Monitor(TS ts,PetscInt step,PetscReal time,Vec u,void *ctx)
   return 0;
 }
 /* --------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "RHSMatrixHeat"
 /*
    RHSMatrixHeat - User-provided routine to compute the right-hand-side
    matrix for the heat equation.
@@ -497,3 +512,73 @@ PetscErrorCode RHSMatrixHeat(TS ts,PetscReal t,Vec X,Mat AA,Mat BB,void *ctx)
 
   return 0;
 }
+
+PetscErrorCode IFunctionHeat(TS ts,PetscReal t,Vec X,Vec Xdot,Vec r,void *ctx)
+{
+  AppCtx         *appctx = (AppCtx*)ctx;     /* user-defined application context */
+  PetscErrorCode ierr;
+
+  ierr = MatMult(appctx->A,X,r);CHKERRQ(ierr);
+  ierr = VecAYPX(r,-1.0,Xdot);CHKERRQ(ierr);
+  return 0;
+}
+
+PetscErrorCode IJacobianHeat(TS ts,PetscReal t,Vec X,Vec Xdot,PetscReal s,Mat A,Mat B,void *ctx)
+{
+  AppCtx         *appctx = (AppCtx*)ctx;     /* user-defined application context */
+  PetscErrorCode ierr;
+
+  if (appctx->oshift == s) return 0;
+  ierr = MatCopy(appctx->A,A,SAME_NONZERO_PATTERN);CHKERRQ(ierr);
+  ierr = MatScale(A,-1);CHKERRQ(ierr);
+  ierr = MatShift(A,s);CHKERRQ(ierr);
+  ierr = MatCopy(A,B,SAME_NONZERO_PATTERN);CHKERRQ(ierr);
+  appctx->oshift = s;
+  return 0;
+}
+
+/*TEST
+
+    test:
+      args: -nox -ts_type ssp -ts_dt 0.0005
+
+    test:
+      suffix: 2
+      args: -nox -ts_type ssp -ts_dt 0.0005 -time_dependent_rhs 1
+
+    test:
+      suffix: 3
+      args:  -nox -ts_type rosw -ts_max_steps 3 -ksp_converged_reason
+      filter: sed "s/ATOL/RTOL/g"
+      requires: !single
+
+    test:
+      suffix: 4
+      args: -nox -ts_type beuler -ts_max_steps 3 -ksp_converged_reason
+      filter: sed "s/ATOL/RTOL/g"
+
+    test:
+      suffix: 5
+      args: -nox -ts_type beuler -ts_max_steps 3 -ksp_converged_reason -time_dependent_rhs
+      filter: sed "s/ATOL/RTOL/g"
+
+    test:
+      requires: !single
+      suffix: pod_guess
+      args: -nox -ts_type beuler -use_ifunc -ts_dt 0.0005 -ksp_guess_type pod -pc_type none -ksp_converged_reason
+
+    test:
+      requires: !single
+      suffix: pod_guess_Ainner
+      args: -nox -ts_type beuler -use_ifunc -ts_dt 0.0005 -ksp_guess_type pod -ksp_guess_pod_Ainner -pc_type none -ksp_converged_reason
+
+    test:
+      requires: !single
+      suffix: fischer_guess
+      args: -nox -ts_type beuler -use_ifunc -ts_dt 0.0005 -ksp_guess_type fischer -pc_type none -ksp_converged_reason
+
+    test:
+      requires: !single
+      suffix: fischer_guess_2
+      args: -nox -ts_type beuler -use_ifunc -ts_dt 0.0005 -ksp_guess_type fischer -ksp_guess_fischer_model 2,10 -pc_type none -ksp_converged_reason
+TEST*/
