@@ -19,8 +19,8 @@ struct _RKTableau {
   char      *name;
   PetscInt   order;               /* Classical approximation order of the method i              */
   PetscInt   s;                   /* Number of stages                                           */
+  PetscInt   p;                   /* Interpolation order                                        */
   PetscBool  FSAL;                /* flag to indicate if tableau is FSAL                        */
-  PetscInt   pinterp;             /* Interpolation order                                        */
   PetscReal *A,*b,*c;             /* Tableau                                                    */
   PetscReal *bembed;              /* Embedded formula of order one less (order-1)               */
   PetscReal *binterp;             /* Dense output formula                                       */
@@ -39,7 +39,6 @@ typedef struct {
   Vec          *YdotRHS;         /* Function evaluations for the non-stiff part */
   Vec          *VecDeltaLam;     /* Increment of the adjoint sensitivity w.r.t IC at stage */
   Vec          *VecDeltaMu;      /* Increment of the adjoint sensitivity w.r.t P at stage */
-  Vec          *VecSensiTemp;    /* Vector to be timed with Jacobian transpose */
   Vec          VecCostIntegral0; /* backup for roll-backs due to events */
   PetscScalar  *work;            /* Scalar work */
   PetscReal    stage_time;
@@ -49,71 +48,108 @@ typedef struct {
 } TS_RK;
 
 /*MC
-     TSRK1 - First order forward Euler scheme.
+     TSRK1FE - First order forward Euler scheme.
 
      This method has one stage.
 
+     Options database:
+.     -ts_rk_type 1fe
+
      Level: advanced
 
-.seealso: TSRK
+.seealso: TSRK, TSRKType, TSRKSetType()
 M*/
 /*MC
      TSRK2A - Second order RK scheme.
 
      This method has two stages.
 
+     Options database:
+.     -ts_rk_type 2a
+
      Level: advanced
 
-.seealso: TSRK
+.seealso: TSRK, TSRKType, TSRKSetType()
 M*/
 /*MC
      TSRK3 - Third order RK scheme.
 
      This method has three stages.
 
+     Options database:
+.     -ts_rk_type 3
+
      Level: advanced
 
-.seealso: TSRK
+.seealso: TSRK, TSRKType, TSRKSetType()
 M*/
 /*MC
      TSRK3BS - Third order RK scheme of Bogacki-Shampine with 2nd order embedded method.
 
-     This method has four stages.
+     This method has four stages with the First Same As Last (FSAL) property.
+
+     Options database:
+.     -ts_rk_type 3bs
 
      Level: advanced
 
-.seealso: TSRK
+     References: https://doi.org/10.1016/0893-9659(89)90079-7
+
+.seealso: TSRK, TSRKType, TSRKSetType()
 M*/
 /*MC
      TSRK4 - Fourth order RK scheme.
 
      This is the classical Runge-Kutta method with four stages.
 
+     Options database:
+.     -ts_rk_type 4
+
      Level: advanced
 
-.seealso: TSRK
+.seealso: TSRK, TSRKType, TSRKSetType()
 M*/
 /*MC
      TSRK5F - Fifth order Fehlberg RK scheme with a 4th order embedded method.
 
      This method has six stages.
 
+     Options database:
+.     -ts_rk_type 5f
+
      Level: advanced
 
-.seealso: TSRK
+.seealso: TSRK, TSRKType, TSRKSetType()
 M*/
 /*MC
      TSRK5DP - Fifth order Dormand-Prince RK scheme with the 4th order embedded method.
 
-     This method has seven stages.
+     This method has seven stages with the First Same As Last (FSAL) property.
+
+     Options database:
+.     -ts_rk_type 5dp
 
      Level: advanced
 
-.seealso: TSRK
+     References: https://doi.org/10.1016/0771-050X(80)90013-3
+
+.seealso: TSRK, TSRKType, TSRKSetType()
+M*/
+/*MC
+     TSRK5BS - Fifth order Bogacki-Shampine RK scheme with 4th order embedded method.
+
+     This method has eight stages with the First Same As Last (FSAL) property.
+
+     Options database:
+.     -ts_rk_type 5bs
+
+     Level: advanced
+
+     References: https://doi.org/10.1016/0898-1221(96)00141-1
+
+.seealso: TSRK, TSRKType, TSRKSetType()
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRKRegisterAll"
 /*@C
   TSRKRegisterAll - Registers all of the Runge-Kutta explicit methods in TSRK
 
@@ -133,77 +169,91 @@ PetscErrorCode TSRKRegisterAll(void)
   if (TSRKRegisterAllCalled) PetscFunctionReturn(0);
   TSRKRegisterAllCalled = PETSC_TRUE;
 
+#define RC PetscRealConstant
   {
     const PetscReal
-      A[1][1] = {{0.0}},
-      b[1]    = {1.0};
-    ierr = TSRKRegister(TSRK1FE,1,1,&A[0][0],b,NULL,NULL,1,b);CHKERRQ(ierr);
+      A[1][1] = {{0}},
+      b[1]    = {RC(1.0)};
+    ierr = TSRKRegister(TSRK1FE,1,1,&A[0][0],b,NULL,NULL,0,NULL);CHKERRQ(ierr);
   }
   {
     const PetscReal
-      A[2][2]     = {{0.0,0.0},
-                    {1.0,0.0}},
-      b[2]        = {0.5,0.5},
-      bembed[2]   = {1.0,0};
-    ierr = TSRKRegister(TSRK2A,2,2,&A[0][0],b,NULL,bembed,1,b);CHKERRQ(ierr);
+      A[2][2]   = {{0,0},
+                   {RC(1.0),0}},
+      b[2]      =  {RC(0.5),RC(0.5)},
+      bembed[2] =  {RC(1.0),0};
+    ierr = TSRKRegister(TSRK2A,2,2,&A[0][0],b,NULL,bembed,0,NULL);CHKERRQ(ierr);
   }
   {
     const PetscReal
       A[3][3] = {{0,0,0},
-                 {2.0/3.0,0,0},
-                 {-1.0/3.0,1.0,0}},
-      b[3]    = {0.25,0.5,0.25};
-    ierr = TSRKRegister(TSRK3,3,3,&A[0][0],b,NULL,NULL,1,b);CHKERRQ(ierr);
+                 {RC(2.0)/RC(3.0),0,0},
+                 {RC(-1.0)/RC(3.0),RC(1.0),0}},
+      b[3]    =  {RC(0.25),RC(0.5),RC(0.25)};
+    ierr = TSRKRegister(TSRK3,3,3,&A[0][0],b,NULL,NULL,0,NULL);CHKERRQ(ierr);
+  }
+  {
+    const PetscReal
+      A[4][4]   = {{0,0,0,0},
+                   {RC(1.0)/RC(2.0),0,0,0},
+                   {0,RC(3.0)/RC(4.0),0,0},
+                   {RC(2.0)/RC(9.0),RC(1.0)/RC(3.0),RC(4.0)/RC(9.0),0}},
+      b[4]      =  {RC(2.0)/RC(9.0),RC(1.0)/RC(3.0),RC(4.0)/RC(9.0),0},
+      bembed[4] =  {RC(7.0)/RC(24.0),RC(1.0)/RC(4.0),RC(1.0)/RC(3.0),RC(1.0)/RC(8.0)};
+    ierr = TSRKRegister(TSRK3BS,3,4,&A[0][0],b,NULL,bembed,0,NULL);CHKERRQ(ierr);
   }
   {
     const PetscReal
       A[4][4] = {{0,0,0,0},
-                 {1.0/2.0,0,0,0},
-                 {0,3.0/4.0,0,0},
-                 {2.0/9.0,1.0/3.0,4.0/9.0,0}},
-      b[4]    = {2.0/9.0,1.0/3.0,4.0/9.0,0},
-      bembed[4] = {7.0/24.0,1.0/4.0,1.0/3.0,1.0/8.0};
-    ierr = TSRKRegister(TSRK3BS,3,4,&A[0][0],b,NULL,bembed,1,b);CHKERRQ(ierr);
-  }
-  {
-    const PetscReal
-      A[4][4] = {{0,0,0,0},
-                 {0.5,0,0,0},
-                 {0,0.5,0,0},
-                 {0,0,1.0,0}},
-      b[4]    = {1.0/6.0,1.0/3.0,1.0/3.0,1.0/6.0};
-    ierr = TSRKRegister(TSRK4,4,4,&A[0][0],b,NULL,NULL,1,b);CHKERRQ(ierr);
+                 {RC(0.5),0,0,0},
+                 {0,RC(0.5),0,0},
+                 {0,0,RC(1.0),0}},
+      b[4]    =  {RC(1.0)/RC(6.0),RC(1.0)/RC(3.0),RC(1.0)/RC(3.0),RC(1.0)/RC(6.0)};
+    ierr = TSRKRegister(TSRK4,4,4,&A[0][0],b,NULL,NULL,0,NULL);CHKERRQ(ierr);
   }
   {
     const PetscReal
       A[6][6]   = {{0,0,0,0,0,0},
-                   {0.25,0,0,0,0,0},
-                   {3.0/32.0,9.0/32.0,0,0,0,0},
-                   {1932.0/2197.0,-7200.0/2197.0,7296.0/2197.0,0,0,0},
-                   {439.0/216.0,-8.0,3680.0/513.0,-845.0/4104.0,0,0},
-                   {-8.0/27.0,2.0,-3544.0/2565.0,1859.0/4104.0,-11.0/40.0,0}},
-      b[6]      = {16.0/135.0,0,6656.0/12825.0,28561.0/56430.0,-9.0/50.0,2.0/55.0},
-      bembed[6] = {25.0/216.0,0,1408.0/2565.0,2197.0/4104.0,-1.0/5.0,0};
-    ierr = TSRKRegister(TSRK5F,5,6,&A[0][0],b,NULL,bembed,1,b);CHKERRQ(ierr);
+                   {RC(0.25),0,0,0,0,0},
+                   {RC(3.0)/RC(32.0),RC(9.0)/RC(32.0),0,0,0,0},
+                   {RC(1932.0)/RC(2197.0),RC(-7200.0)/RC(2197.0),RC(7296.0)/RC(2197.0),0,0,0},
+                   {RC(439.0)/RC(216.0),RC(-8.0),RC(3680.0)/RC(513.0),RC(-845.0)/RC(4104.0),0,0},
+                   {RC(-8.0)/RC(27.0),RC(2.0),RC(-3544.0)/RC(2565.0),RC(1859.0)/RC(4104.0),RC(-11.0)/RC(40.0),0}},
+      b[6]      =  {RC(16.0)/RC(135.0),0,RC(6656.0)/RC(12825.0),RC(28561.0)/RC(56430.0),RC(-9.0)/RC(50.0),RC(2.0)/RC(55.0)},
+      bembed[6] =  {RC(25.0)/RC(216.0),0,RC(1408.0)/RC(2565.0),RC(2197.0)/RC(4104.0),RC(-1.0)/RC(5.0),0};
+    ierr = TSRKRegister(TSRK5F,5,6,&A[0][0],b,NULL,bembed,0,NULL);CHKERRQ(ierr);
   }
   {
     const PetscReal
       A[7][7]   = {{0,0,0,0,0,0,0},
-                   {1.0/5.0,0,0,0,0,0,0},
-                   {3.0/40.0,9.0/40.0,0,0,0,0,0},
-                   {44.0/45.0,-56.0/15.0,32.0/9.0,0,0,0,0},
-                   {19372.0/6561.0,-25360.0/2187.0,64448.0/6561.0,-212.0/729.0,0,0,0},
-                   {9017.0/3168.0,-355.0/33.0,46732.0/5247.0,49.0/176.0,-5103.0/18656.0,0,0},
-                   {35.0/384.0,0,500.0/1113.0,125.0/192.0,-2187.0/6784.0,11.0/84.0,0}},
-      b[7]      = {35.0/384.0,0,500.0/1113.0,125.0/192.0,-2187.0/6784.0,11.0/84.0,0},
-      bembed[7] = {5179.0/57600.0,0,7571.0/16695.0,393.0/640.0,-92097.0/339200.0,187.0/2100.0,1.0/40.0};
-    ierr = TSRKRegister(TSRK5DP,5,7,&A[0][0],b,NULL,bembed,1,b);CHKERRQ(ierr);
+                   {RC(1.0)/RC(5.0),0,0,0,0,0,0},
+                   {RC(3.0)/RC(40.0),RC(9.0)/RC(40.0),0,0,0,0,0},
+                   {RC(44.0)/RC(45.0),RC(-56.0)/RC(15.0),RC(32.0)/RC(9.0),0,0,0,0},
+                   {RC(19372.0)/RC(6561.0),RC(-25360.0)/RC(2187.0),RC(64448.0)/RC(6561.0),RC(-212.0)/RC(729.0),0,0,0},
+                   {RC(9017.0)/RC(3168.0),RC(-355.0)/RC(33.0),RC(46732.0)/RC(5247.0),RC(49.0)/RC(176.0),RC(-5103.0)/RC(18656.0),0,0},
+                   {RC(35.0)/RC(384.0),0,RC(500.0)/RC(1113.0),RC(125.0)/RC(192.0),RC(-2187.0)/RC(6784.0),RC(11.0)/RC(84.0),0}},
+      b[7]      =  {RC(35.0)/RC(384.0),0,RC(500.0)/RC(1113.0),RC(125.0)/RC(192.0),RC(-2187.0)/RC(6784.0),RC(11.0)/RC(84.0),0},
+      bembed[7] =  {RC(5179.0)/RC(57600.0),0,RC(7571.0)/RC(16695.0),RC(393.0)/RC(640.0),RC(-92097.0)/RC(339200.0),RC(187.0)/RC(2100.0),RC(1.0)/RC(40.0)};
+    ierr = TSRKRegister(TSRK5DP,5,7,&A[0][0],b,NULL,bembed,0,NULL);CHKERRQ(ierr);
   }
+  {
+    const PetscReal
+      A[8][8]   = {{0,0,0,0,0,0,0,0},
+                   {RC(1.0)/RC(6.0),0,0,0,0,0,0,0},
+                   {RC(2.0)/RC(27.0),RC(4.0)/RC(27.0),0,0,0,0,0,0},
+                   {RC(183.0)/RC(1372.0),RC(-162.0)/RC(343.0),RC(1053.0)/RC(1372.0),0,0,0,0,0},
+                   {RC(68.0)/RC(297.0),RC(-4.0)/RC(11.0),RC(42.0)/RC(143.0),RC(1960.0)/RC(3861.0),0,0,0,0},
+                   {RC(597.0)/RC(22528.0),RC(81.0)/RC(352.0),RC(63099.0)/RC(585728.0),RC(58653.0)/RC(366080.0),RC(4617.0)/RC(20480.0),0,0,0},
+                   {RC(174197.0)/RC(959244.0),RC(-30942.0)/RC(79937.0),RC(8152137.0)/RC(19744439.0),RC(666106.0)/RC(1039181.0),RC(-29421.0)/RC(29068.0),RC(482048.0)/RC(414219.0),0,0},
+                   {RC(587.0)/RC(8064.0),0,RC(4440339.0)/RC(15491840.0),RC(24353.0)/RC(124800.0),RC(387.0)/RC(44800.0),RC(2152.0)/RC(5985.0),RC(7267.0)/RC(94080.0),0}},
+      b[8]      =  {RC(587.0)/RC(8064.0),0,RC(4440339.0)/RC(15491840.0),RC(24353.0)/RC(124800.0),RC(387.0)/RC(44800.0),RC(2152.0)/RC(5985.0),RC(7267.0)/RC(94080.0),0},
+      bembed[8] =  {RC(2479.0)/RC(34992.0),0,RC(123.0)/RC(416.0),RC(612941.0)/RC(3411720.0),RC(43.0)/RC(1440.0),RC(2272.0)/RC(6561.0),RC(79937.0)/RC(1113912.0),RC(3293.0)/RC(556956.0)};
+    ierr = TSRKRegister(TSRK5BS,5,8,&A[0][0],b,NULL,bembed,0,NULL);CHKERRQ(ierr);
+  }
+#undef RC
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRKRegisterDestroy"
 /*@C
    TSRKRegisterDestroy - Frees the list of schemes that were registered by TSRKRegister().
 
@@ -233,12 +283,9 @@ PetscErrorCode TSRKRegisterDestroy(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRKInitializePackage"
 /*@C
   TSRKInitializePackage - This function initializes everything in the TSRK package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to TSCreate_RK()
-  when using static libraries.
+  from TSInitializePackage().
 
   Level: developer
 
@@ -257,8 +304,6 @@ PetscErrorCode TSRKInitializePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRKFinalizePackage"
 /*@C
   TSRKFinalizePackage - This function destroys everything in the TSRK package. It is
   called from PetscFinalize().
@@ -278,8 +323,6 @@ PetscErrorCode TSRKFinalizePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRKRegister"
 /*@C
    TSRKRegister - register an RK scheme by providing the entries in the Butcher tableau and optionally embedded approximations and interpolation
 
@@ -293,8 +336,8 @@ PetscErrorCode TSRKFinalizePackage(void)
 .  b - step completion table (dimension s; NULL to use last row of A)
 .  c - abscissa (dimension s; NULL to use row sums of A)
 .  bembed - completion table for embedded method (dimension s; NULL if not available)
-.  pinterp - Order of the interpolation scheme, equal to the number of columns of binterp
--  binterp - Coefficients of the interpolation formula (dimension s*pinterp; NULL to reuse binterpt)
+.  p - Order of the interpolation scheme, equal to the number of columns of binterp
+-  binterp - Coefficients of the interpolation formula (dimension s*p; NULL to reuse b with p=1)
 
    Notes:
    Several RK methods are provided, this function is only needed to create new methods.
@@ -306,9 +349,8 @@ PetscErrorCode TSRKFinalizePackage(void)
 .seealso: TSRK
 @*/
 PetscErrorCode TSRKRegister(TSRKType name,PetscInt order,PetscInt s,
-                                 const PetscReal A[],const PetscReal b[],const PetscReal c[],
-                                 const PetscReal bembed[],
-                                 PetscInt pinterp,const PetscReal binterp[])
+                            const PetscReal A[],const PetscReal b[],const PetscReal c[],
+                            const PetscReal bembed[],PetscInt p,const PetscReal binterp[])
 {
   PetscErrorCode  ierr;
   RKTableauLink   link;
@@ -316,35 +358,44 @@ PetscErrorCode TSRKRegister(TSRKType name,PetscInt order,PetscInt s,
   PetscInt        i,j;
 
   PetscFunctionBegin;
-  ierr     = PetscMalloc(sizeof(*link),&link);CHKERRQ(ierr);
-  ierr     = PetscMemzero(link,sizeof(*link));CHKERRQ(ierr);
-  t        = &link->tab;
-  ierr     = PetscStrallocpy(name,&t->name);CHKERRQ(ierr);
+  PetscValidCharPointer(name,1);
+  PetscValidRealPointer(A,4);
+  if (b) PetscValidRealPointer(b,5);
+  if (c) PetscValidRealPointer(c,6);
+  if (bembed) PetscValidRealPointer(bembed,7);
+  if (binterp || p > 1) PetscValidRealPointer(binterp,9);
+
+  ierr = TSRKInitializePackage();CHKERRQ(ierr);
+  ierr = PetscNew(&link);CHKERRQ(ierr);
+  t = &link->tab;
+
+  ierr = PetscStrallocpy(name,&t->name);CHKERRQ(ierr);
   t->order = order;
-  t->s     = s;
-  ierr     = PetscMalloc3(s*s,&t->A,s,&t->b,s,&t->c);CHKERRQ(ierr);
-  ierr     = PetscMemcpy(t->A,A,s*s*sizeof(A[0]));CHKERRQ(ierr);
+  t->s = s;
+  ierr = PetscMalloc3(s*s,&t->A,s,&t->b,s,&t->c);CHKERRQ(ierr);
+  ierr = PetscMemcpy(t->A,A,s*s*sizeof(A[0]));CHKERRQ(ierr);
   if (b)  { ierr = PetscMemcpy(t->b,b,s*sizeof(b[0]));CHKERRQ(ierr); }
   else for (i=0; i<s; i++) t->b[i] = A[(s-1)*s+i];
   if (c)  { ierr = PetscMemcpy(t->c,c,s*sizeof(c[0]));CHKERRQ(ierr); }
   else for (i=0; i<s; i++) for (j=0,t->c[i]=0; j<s; j++) t->c[i] += A[i*s+j];
   t->FSAL = PETSC_TRUE;
   for (i=0; i<s; i++) if (t->A[(s-1)*s+i] != t->b[i]) t->FSAL = PETSC_FALSE;
+
   if (bembed) {
     ierr = PetscMalloc1(s,&t->bembed);CHKERRQ(ierr);
     ierr = PetscMemcpy(t->bembed,bembed,s*sizeof(bembed[0]));CHKERRQ(ierr);
   }
 
-  t->pinterp     = pinterp;
-  ierr           = PetscMalloc1(s*pinterp,&t->binterp);CHKERRQ(ierr);
-  ierr           = PetscMemcpy(t->binterp,binterp,s*pinterp*sizeof(binterp[0]));CHKERRQ(ierr);
-  link->next     = RKTableauList;
+  if (!binterp) { p = 1; binterp = t->b; }
+  t->p = p;
+  ierr = PetscMalloc1(s*p,&t->binterp);CHKERRQ(ierr);
+  ierr = PetscMemcpy(t->binterp,binterp,s*p*sizeof(binterp[0]));CHKERRQ(ierr);
+
+  link->next = RKTableauList;
   RKTableauList = link;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSEvaluateStep_RK"
 /*
  The step completion formula is
 
@@ -395,9 +446,6 @@ static PetscErrorCode TSEvaluateStep_RK(TS ts,PetscInt order,Vec X,PetscBool *do
       ierr = VecCopy(ts->vec_sol,X);CHKERRQ(ierr);
       for (j=0; j<s; j++) w[j] = h*(tab->bembed[j] - tab->b[j]);
       ierr = VecMAXPY(X,s,w,rk->YdotRHS);CHKERRQ(ierr);
-      if (ts->vec_costintegral && ts->costintegralfwd) {
-        ierr = VecCopy(rk->VecCostIntegral0,ts->vec_costintegral);CHKERRQ(ierr);
-      }
     }
     if (done) *done = PETSC_TRUE;
     PetscFunctionReturn(0);
@@ -408,8 +456,6 @@ unavailable:
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSForwardCostIntegral_RK"
 static PetscErrorCode TSForwardCostIntegral_RK(TS ts)
 {
   TS_RK           *rk = (TS_RK*)ts->data;
@@ -425,14 +471,12 @@ static PetscErrorCode TSForwardCostIntegral_RK(TS ts)
   ierr = VecCopy(ts->vec_costintegral,rk->VecCostIntegral0);CHKERRQ(ierr);
   for (i=s-1; i>=0; i--) {
     /* Evolve ts->vec_costintegral to compute integrals */
-    ierr = TSAdjointComputeCostIntegrand(ts,rk->ptime+rk->time_step*(1.0-c[i]),Y[i],ts->vec_costintegrand);CHKERRQ(ierr);
+    ierr = TSComputeCostIntegrand(ts,rk->ptime+rk->time_step*c[i],Y[i],ts->vec_costintegrand);CHKERRQ(ierr);
     ierr = VecAXPY(ts->vec_costintegral,rk->time_step*b[i],ts->vec_costintegrand);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdjointCostIntegral_RK"
 static PetscErrorCode TSAdjointCostIntegral_RK(TS ts)
 {
   TS_RK           *rk = (TS_RK*)ts->data;
@@ -446,14 +490,12 @@ static PetscErrorCode TSAdjointCostIntegral_RK(TS ts)
   PetscFunctionBegin;
   for (i=s-1; i>=0; i--) {
     /* Evolve ts->vec_costintegral to compute integrals */
-    ierr = TSAdjointComputeCostIntegrand(ts,ts->ptime-ts->time_step*(1.0-c[i]),Y[i],ts->vec_costintegrand);CHKERRQ(ierr);
+    ierr = TSComputeCostIntegrand(ts,ts->ptime+ts->time_step*(1.0-c[i]),Y[i],ts->vec_costintegrand);CHKERRQ(ierr);
     ierr = VecAXPY(ts->vec_costintegral,-ts->time_step*b[i],ts->vec_costintegrand);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRollBack_RK"
 static PetscErrorCode TSRollBack_RK(TS ts)
 {
   TS_RK           *rk = (TS_RK*)ts->data;
@@ -480,8 +522,6 @@ static PetscErrorCode TSRollBack_RK(TS ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSStep_RK"
 static PetscErrorCode TSStep_RK(TS ts)
 {
   TS_RK           *rk   = (TS_RK*)ts->data;
@@ -490,6 +530,7 @@ static PetscErrorCode TSStep_RK(TS ts)
   const PetscReal *A = tab->A,*c = tab->c;
   PetscScalar     *w = rk->work;
   Vec             *Y = rk->Y,*YdotRHS = rk->YdotRHS;
+  PetscBool        FSAL = tab->FSAL;
   TSAdapt          adapt;
   PetscInt         i,j;
   PetscInt         rejections = 0;
@@ -498,6 +539,8 @@ static PetscErrorCode TSStep_RK(TS ts)
   PetscErrorCode   ierr;
 
   PetscFunctionBegin;
+  if (ts->steprollback || ts->steprestart) FSAL = PETSC_FALSE;
+  if (FSAL) { ierr = VecCopy(YdotRHS[s-1],YdotRHS[0]);CHKERRQ(ierr); }
 
   rk->status = TS_STEP_INCOMPLETE;
   while (!ts->reason && rk->status != TS_STEP_COMPLETE) {
@@ -513,6 +556,7 @@ static PetscErrorCode TSStep_RK(TS ts)
       ierr = TSGetAdapt(ts,&adapt);CHKERRQ(ierr);
       ierr = TSAdaptCheckStage(adapt,ts,rk->stage_time,Y[i],&stageok);CHKERRQ(ierr);
       if (!stageok) goto reject_step;
+      if (FSAL && !i) continue;
       ierr = TSComputeRHSFunction(ts,t+h*c[i],Y[i],YdotRHS[i]);CHKERRQ(ierr);
     }
 
@@ -521,7 +565,7 @@ static PetscErrorCode TSStep_RK(TS ts)
     rk->status = TS_STEP_PENDING;
     ierr = TSGetAdapt(ts,&adapt);CHKERRQ(ierr);
     ierr = TSAdaptCandidatesClear(adapt);CHKERRQ(ierr);
-    ierr = TSAdaptCandidateAdd(adapt,tab->name,tab->order,1,tab->ccfl,1.*tab->s,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = TSAdaptCandidateAdd(adapt,tab->name,tab->order,1,tab->ccfl,(PetscReal)tab->s,PETSC_TRUE);CHKERRQ(ierr);
     ierr = TSAdaptChoose(adapt,ts,ts->time_step,NULL,&next_time_step,&accept);CHKERRQ(ierr);
     rk->status = accept ? TS_STEP_COMPLETE : TS_STEP_INCOMPLETE;
     if (!accept) { /* Roll back the current step */
@@ -549,8 +593,6 @@ static PetscErrorCode TSStep_RK(TS ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdjointSetUp_RK"
 static PetscErrorCode TSAdjointSetUp_RK(TS ts)
 {
   TS_RK         *rk  = (TS_RK*)ts->data;
@@ -564,12 +606,9 @@ static PetscErrorCode TSAdjointSetUp_RK(TS ts)
   if(ts->vecs_sensip) {
     ierr = VecDuplicateVecs(ts->vecs_sensip[0],s*ts->numcost,&rk->VecDeltaMu);CHKERRQ(ierr);
   }
-  ierr = VecDuplicateVecs(ts->vecs_sensi[0],ts->numcost,&rk->VecSensiTemp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdjointStep_RK"
 static PetscErrorCode TSAdjointStep_RK(TS ts)
 {
   TS_RK           *rk   = (TS_RK*)ts->data;
@@ -577,57 +616,72 @@ static PetscErrorCode TSAdjointStep_RK(TS ts)
   const PetscInt   s    = tab->s;
   const PetscReal *A = tab->A,*b = tab->b,*c = tab->c;
   PetscScalar     *w    = rk->work;
-  Vec             *Y    = rk->Y,*VecDeltaLam = rk->VecDeltaLam,*VecDeltaMu = rk->VecDeltaMu,*VecSensiTemp = rk->VecSensiTemp;
+  Vec             *Y    = rk->Y,*VecDeltaLam = rk->VecDeltaLam,*VecDeltaMu = rk->VecDeltaMu;
   PetscInt         i,j,nadj;
   PetscReal        t = ts->ptime;
   PetscErrorCode   ierr;
   PetscReal        h = ts->time_step;
-  Mat              J,Jp;
 
   PetscFunctionBegin;
   rk->status = TS_STEP_INCOMPLETE;
   for (i=s-1; i>=0; i--) {
-    rk->stage_time = t + h*(1.0-c[i]);
-    for (nadj=0; nadj<ts->numcost; nadj++) {
-      ierr = VecCopy(ts->vecs_sensi[nadj],VecSensiTemp[nadj]);CHKERRQ(ierr);
-      ierr = VecScale(VecSensiTemp[nadj],-h*b[i]);CHKERRQ(ierr);
-      for (j=i+1; j<s; j++) {
-        ierr = VecAXPY(VecSensiTemp[nadj],-h*A[j*s+i],VecDeltaLam[nadj*s+j]);CHKERRQ(ierr);
+    Mat       J;
+    PetscReal stage_time = t + h*(1.0-c[i]);
+    PetscBool zero = PETSC_FALSE;
+
+    ierr = TSGetRHSJacobian(ts,&J,NULL,NULL,NULL);CHKERRQ(ierr);
+    ierr = TSComputeRHSJacobian(ts,stage_time,Y[i],J,J);CHKERRQ(ierr);
+    if (ts->vec_costintegral) {
+      ierr = TSAdjointComputeDRDYFunction(ts,stage_time,Y[i],ts->vecs_drdy);CHKERRQ(ierr);
+    }
+    /* Stage values of mu */
+    if (ts->vecs_sensip) {
+      ierr = TSAdjointComputeRHSJacobian(ts,stage_time,Y[i],ts->Jacp);CHKERRQ(ierr);
+      if (ts->vec_costintegral) {
+        ierr = TSAdjointComputeDRDPFunction(ts,stage_time,Y[i],ts->vecs_drdp);CHKERRQ(ierr);
       }
     }
-    /* Stage values of lambda */
-    ierr = TSGetRHSJacobian(ts,&J,&Jp,NULL,NULL);CHKERRQ(ierr);
-    ierr = TSComputeRHSJacobian(ts,rk->stage_time,Y[i],J,Jp);CHKERRQ(ierr);
-    if (ts->vec_costintegral) {
-      ierr = TSAdjointComputeDRDYFunction(ts,rk->stage_time,Y[i],ts->vecs_drdy);CHKERRQ(ierr);
-    }
+
+    if (b[i] == 0 && i == s-1) zero = PETSC_TRUE;
     for (nadj=0; nadj<ts->numcost; nadj++) {
-      ierr = MatMultTranspose(J,VecSensiTemp[nadj],VecDeltaLam[nadj*s+i]);CHKERRQ(ierr);
+      DM  dm;
+      Vec VecSensiTemp;
+
+      ierr = TSGetDM(ts,&dm);CHKERRQ(ierr);
+      ierr = DMGetGlobalVector(dm,&VecSensiTemp);CHKERRQ(ierr);
+      /* Stage values of lambda */
+      if (!zero) {
+        ierr = VecCopy(ts->vecs_sensi[nadj],VecSensiTemp);CHKERRQ(ierr);
+        ierr = VecScale(VecSensiTemp,-h*b[i]);CHKERRQ(ierr);
+        for (j=i+1; j<s; j++) w[j-i-1]  = -h*A[j*s+i];
+        ierr = VecMAXPY(VecSensiTemp,s-i-1,w,&VecDeltaLam[nadj*s+i+1]);CHKERRQ(ierr);
+        ierr = MatMultTranspose(J,VecSensiTemp,VecDeltaLam[nadj*s+i]);CHKERRQ(ierr);
+      } else {
+        ierr = VecSet(VecDeltaLam[nadj*s+i],0);CHKERRQ(ierr);
+      }
       if (ts->vec_costintegral) {
         ierr = VecAXPY(VecDeltaLam[nadj*s+i],-h*b[i],ts->vecs_drdy[nadj]);CHKERRQ(ierr);
       }
-    }
 
-    /* Stage values of mu */
-    if(ts->vecs_sensip) {
-      ierr = TSAdjointComputeRHSJacobian(ts,rk->stage_time,Y[i],ts->Jacp);CHKERRQ(ierr);
-      if (ts->vec_costintegral) {
-        ierr = TSAdjointComputeDRDPFunction(ts,rk->stage_time,Y[i],ts->vecs_drdp);CHKERRQ(ierr);
-      }
-
-      for (nadj=0; nadj<ts->numcost; nadj++) {
-        ierr = MatMultTranspose(ts->Jacp,VecSensiTemp[nadj],VecDeltaMu[nadj*s+i]);CHKERRQ(ierr);
+      /* Stage values of mu */
+      if (ts->vecs_sensip) {
+        if (!zero) {
+          ierr = MatMultTranspose(ts->Jacp,VecSensiTemp,VecDeltaMu[nadj*s+i]);CHKERRQ(ierr);
+        } else {
+          ierr = VecSet(VecDeltaMu[nadj*s+i],0);CHKERRQ(ierr);
+        }
         if (ts->vec_costintegral) {
           ierr = VecAXPY(VecDeltaMu[nadj*s+i],-h*b[i],ts->vecs_drdp[nadj]);CHKERRQ(ierr);
         }
       }
+      ierr = DMRestoreGlobalVector(dm,&VecSensiTemp);CHKERRQ(ierr);
     }
   }
 
   for (j=0; j<s; j++) w[j] = 1.0;
   for (nadj=0; nadj<ts->numcost; nadj++) {
     ierr = VecMAXPY(ts->vecs_sensi[nadj],s,w,&VecDeltaLam[nadj*s]);CHKERRQ(ierr);
-    if(ts->vecs_sensip) {
+    if (ts->vecs_sensip) {
       ierr = VecMAXPY(ts->vecs_sensip[nadj],s,w,&VecDeltaMu[nadj*s]);CHKERRQ(ierr);
     }
   }
@@ -635,12 +689,10 @@ static PetscErrorCode TSAdjointStep_RK(TS ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSInterpolate_RK"
 static PetscErrorCode TSInterpolate_RK(TS ts,PetscReal itime,Vec X)
 {
   TS_RK           *rk = (TS_RK*)ts->data;
-  PetscInt         s  = rk->tableau->s,pinterp = rk->tableau->pinterp,i,j;
+  PetscInt         s  = rk->tableau->s,p = rk->tableau->p,i,j;
   PetscReal        h;
   PetscReal        tt,t;
   PetscScalar     *b;
@@ -664,9 +716,9 @@ static PetscErrorCode TSInterpolate_RK(TS ts,PetscReal itime,Vec X)
   }
   ierr = PetscMalloc1(s,&b);CHKERRQ(ierr);
   for (i=0; i<s; i++) b[i] = 0;
-  for (j=0,tt=t; j<pinterp; j++,tt*=t) {
+  for (j=0,tt=t; j<p; j++,tt*=t) {
     for (i=0; i<s; i++) {
-      b[i]  += h * B[i*pinterp+j] * tt;
+      b[i]  += h * B[i*p+j] * tt;
     }
   }
 
@@ -679,8 +731,6 @@ static PetscErrorCode TSInterpolate_RK(TS ts,PetscReal itime,Vec X)
 
 /*------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRKTableauReset"
 static PetscErrorCode TSRKTableauReset(TS ts)
 {
   TS_RK          *rk = (TS_RK*)ts->data;
@@ -697,8 +747,6 @@ static PetscErrorCode TSRKTableauReset(TS ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSReset_RK"
 static PetscErrorCode TSReset_RK(TS ts)
 {
   TS_RK         *rk = (TS_RK*)ts->data;
@@ -707,35 +755,15 @@ static PetscErrorCode TSReset_RK(TS ts)
   PetscFunctionBegin;
   ierr = TSRKTableauReset(ts);CHKERRQ(ierr);
   ierr = VecDestroy(&rk->VecCostIntegral0);CHKERRQ(ierr);
-  ierr = VecDestroyVecs(ts->numcost,&rk->VecSensiTemp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSDestroy_RK"
-static PetscErrorCode TSDestroy_RK(TS ts)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  ierr = TSReset_RK(ts);CHKERRQ(ierr);
-  ierr = PetscFree(ts->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)ts,"TSRKGetType_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)ts,"TSRKSetType_C",NULL);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-
-#undef __FUNCT__
-#define __FUNCT__ "DMCoarsenHook_TSRK"
 static PetscErrorCode DMCoarsenHook_TSRK(DM fine,DM coarse,void *ctx)
 {
   PetscFunctionBegin;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMRestrictHook_TSRK"
 static PetscErrorCode DMRestrictHook_TSRK(DM fine,Mat restrct,Vec rscale,Mat inject,DM coarse,void *ctx)
 {
   PetscFunctionBegin;
@@ -743,16 +771,12 @@ static PetscErrorCode DMRestrictHook_TSRK(DM fine,Mat restrct,Vec rscale,Mat inj
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSubDomainHook_TSRK"
 static PetscErrorCode DMSubDomainHook_TSRK(DM dm,DM subdm,void *ctx)
 {
   PetscFunctionBegin;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSubDomainRestrictHook_TSRK"
 static PetscErrorCode DMSubDomainRestrictHook_TSRK(DM dm,VecScatter gscat,VecScatter lscat,DM subdm,void *ctx)
 {
 
@@ -760,8 +784,6 @@ static PetscErrorCode DMSubDomainRestrictHook_TSRK(DM dm,VecScatter gscat,VecSca
   PetscFunctionReturn(0);
 }
 /*
-#undef __FUNCT__
-#define __FUNCT__ "RKSetAdjCoe"
 static PetscErrorCode RKSetAdjCoe(RKTableau tab)
 {
   PetscReal *A,*b;
@@ -787,8 +809,6 @@ static PetscErrorCode RKSetAdjCoe(RKTableau tab)
 }
 */
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRKTableauSetUp"
 static PetscErrorCode TSRKTableauSetUp(TS ts)
 {
   TS_RK         *rk  = (TS_RK*)ts->data;
@@ -803,8 +823,6 @@ static PetscErrorCode TSRKTableauSetUp(TS ts)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSetUp_RK"
 static PetscErrorCode TSSetUp_RK(TS ts)
 {
   TS_RK         *rk = (TS_RK*)ts->data;
@@ -812,23 +830,20 @@ static PetscErrorCode TSSetUp_RK(TS ts)
   DM             dm;
 
   PetscFunctionBegin;
+  ierr = TSCheckImplicitTerm(ts);CHKERRQ(ierr);
   ierr = TSRKTableauSetUp(ts);CHKERRQ(ierr);
   if (!rk->VecCostIntegral0 && ts->vec_costintegral && ts->costintegralfwd) { /* back up cost integral */
     ierr = VecDuplicate(ts->vec_costintegral,&rk->VecCostIntegral0);CHKERRQ(ierr);
   }
   ierr = TSGetDM(ts,&dm);CHKERRQ(ierr);
-  if (dm) {
-    ierr = DMCoarsenHookAdd(dm,DMCoarsenHook_TSRK,DMRestrictHook_TSRK,ts);CHKERRQ(ierr);
-    ierr = DMSubDomainHookAdd(dm,DMSubDomainHook_TSRK,DMSubDomainRestrictHook_TSRK,ts);CHKERRQ(ierr);
-  }
+  ierr = DMCoarsenHookAdd(dm,DMCoarsenHook_TSRK,DMRestrictHook_TSRK,ts);CHKERRQ(ierr);
+  ierr = DMSubDomainHookAdd(dm,DMSubDomainHook_TSRK,DMSubDomainRestrictHook_TSRK,ts);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 
 /*------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSetFromOptions_RK"
 static PetscErrorCode TSSetFromOptions_RK(PetscOptionItems *PetscOptionsObject,TS ts)
 {
   TS_RK         *rk = (TS_RK*)ts->data;
@@ -842,7 +857,7 @@ static PetscErrorCode TSSetFromOptions_RK(PetscOptionItems *PetscOptionsObject,T
     PetscBool      flg;
     const char   **namelist;
     for (link=RKTableauList,count=0; link; link=link->next,count++) ;
-    ierr = PetscMalloc1(count,&namelist);CHKERRQ(ierr);
+    ierr = PetscMalloc1(count,(char***)&namelist);CHKERRQ(ierr);
     for (link=RKTableauList,count=0; link; link=link->next,count++) namelist[count] = link->tab.name;
     ierr = PetscOptionsEList("-ts_rk_type","Family of RK method","TSRKSetType",(const char*const*)namelist,count,rk->tableau->name,&choice,&flg);CHKERRQ(ierr);
     if (flg) {ierr = TSRKSetType(ts,namelist[choice]);CHKERRQ(ierr);}
@@ -852,29 +867,6 @@ static PetscErrorCode TSSetFromOptions_RK(PetscOptionItems *PetscOptionsObject,T
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscFormatRealArray"
-static PetscErrorCode PetscFormatRealArray(char buf[],size_t len,const char *fmt,PetscInt n,const PetscReal x[])
-{
-  PetscErrorCode ierr;
-  PetscInt       i;
-  size_t         left,count;
-  char           *p;
-
-  PetscFunctionBegin;
-  for (i=0,p=buf,left=len; i<n; i++) {
-    ierr = PetscSNPrintfCount(p,left,fmt,&count,x[i]);CHKERRQ(ierr);
-    if (count >= left) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Insufficient space in buffer");
-    left -= count;
-    p    += count;
-    *p++  = ' ';
-  }
-  p[i ? 0 : -1] = 0;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "TSView_RK"
 static PetscErrorCode TSView_RK(TS ts,PetscViewer viewer)
 {
   TS_RK          *rk = (TS_RK*)ts->data;
@@ -888,17 +880,15 @@ static PetscErrorCode TSView_RK(TS ts,PetscViewer viewer)
     TSRKType  rktype;
     char      buf[512];
     ierr = TSRKGetType(ts,&rktype);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  RK %s\n",rktype);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  RK type %s\n",rktype);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  Order: %D\n",tab->order);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  FSAL property: %s\n",tab->FSAL ? "yes" : "no");CHKERRQ(ierr);
     ierr = PetscFormatRealArray(buf,sizeof(buf),"% 8.6f",tab->s,tab->c);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  Abscissa     c = %s\n",buf);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"FSAL: %s\n",tab->FSAL ? "yes" : "no");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  Abscissa c = %s\n",buf);CHKERRQ(ierr);
   }
-  if (ts->adapt) {ierr = TSAdaptView(ts->adapt,viewer);CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSLoad_RK"
 static PetscErrorCode TSLoad_RK(TS ts,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -910,8 +900,6 @@ static PetscErrorCode TSLoad_RK(TS ts,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRKSetType"
 /*@C
   TSRKSetType - Set the type of RK scheme
 
@@ -921,9 +909,12 @@ static PetscErrorCode TSLoad_RK(TS ts,PetscViewer viewer)
 +  ts - timestepping context
 -  rktype - type of RK-scheme
 
+  Options Database:
+.   -ts_rk_type - <1fe,2a,3,3bs,4,5f,5dp,5bs>
+
   Level: intermediate
 
-.seealso: TSRKGetType(), TSRK, TSRK2, TSRK3, TSRKPRSSP2, TSRK4, TSRK5
+.seealso: TSRKGetType(), TSRK, TSRKType, TSRK1FE, TSRK2A, TSRK3, TSRK3BS, TSRK4, TSRK5F, TSRK5DP, TSRK5BS
 @*/
 PetscErrorCode TSRKSetType(TS ts,TSRKType rktype)
 {
@@ -931,12 +922,11 @@ PetscErrorCode TSRKSetType(TS ts,TSRKType rktype)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ts,TS_CLASSID,1);
+  PetscValidCharPointer(rktype,2);
   ierr = PetscTryMethod(ts,"TSRKSetType_C",(TS,TSRKType),(ts,rktype));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRKGetType"
 /*@C
   TSRKGetType - Get the type of RK scheme
 
@@ -962,8 +952,6 @@ PetscErrorCode TSRKGetType(TS ts,TSRKType *rktype)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRKGetType_RK"
 static PetscErrorCode TSRKGetType_RK(TS ts,TSRKType *rktype)
 {
   TS_RK *rk = (TS_RK*)ts->data;
@@ -972,8 +960,6 @@ static PetscErrorCode TSRKGetType_RK(TS ts,TSRKType *rktype)
   *rktype = rk->tableau->name;
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__ "TSRKSetType_RK"
 static PetscErrorCode TSRKSetType_RK(TS ts,TSRKType rktype)
 {
   TS_RK          *rk = (TS_RK*)ts->data;
@@ -992,6 +978,7 @@ static PetscErrorCode TSRKSetType_RK(TS ts,TSRKType rktype)
       if (ts->setupcalled) {ierr = TSRKTableauReset(ts);CHKERRQ(ierr);}
       rk->tableau = &link->tab;
       if (ts->setupcalled) {ierr = TSRKTableauSetUp(ts);CHKERRQ(ierr);}
+      ts->default_adapt_type = rk->tableau->bembed ? TSADAPTBASIC : TSADAPTNONE;
       PetscFunctionReturn(0);
     }
   }
@@ -999,18 +986,31 @@ static PetscErrorCode TSRKSetType_RK(TS ts,TSRKType rktype)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSGetStages_RK"
 static PetscErrorCode  TSGetStages_RK(TS ts,PetscInt *ns,Vec **Y)
 {
-  TS_RK          *rk = (TS_RK*)ts->data;
+  TS_RK *rk = (TS_RK*)ts->data;
 
   PetscFunctionBegin;
-  *ns = rk->tableau->s;
-  if(Y) *Y  = rk->Y;
+  if (ns) *ns = rk->tableau->s;
+  if (Y)   *Y = rk->Y;
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode TSDestroy_RK(TS ts)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = TSReset_RK(ts);CHKERRQ(ierr);
+  if (ts->dm) {
+    ierr = DMCoarsenHookRemove(ts->dm,DMCoarsenHook_TSRK,DMRestrictHook_TSRK,ts);CHKERRQ(ierr);
+    ierr = DMSubDomainHookRemove(ts->dm,DMSubDomainHook_TSRK,DMSubDomainRestrictHook_TSRK,ts);CHKERRQ(ierr);
+  }
+  ierr = PetscFree(ts->data);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ts,"TSRKGetType_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ts,"TSRKSetType_C",NULL);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
 
 /* ------------------------------------------------------------ */
 /*MC
@@ -1020,7 +1020,7 @@ static PetscErrorCode  TSGetStages_RK(TS ts,PetscInt *ns,Vec **Y)
   using TSSetRHSFunction().
 
   Notes:
-  The default is TSRK3, it can be changed with TSRKSetType() or -ts_rk_type
+  The default is TSRK3BS, it can be changed with TSRKSetType() or -ts_rk_type
 
   Level: beginner
 
@@ -1028,8 +1028,6 @@ static PetscErrorCode  TSGetStages_RK(TS ts,PetscInt *ns,Vec **Y)
            TSRK4, TSRK5, TSRKPRSSP2, TSRKBPR3, TSRKType, TSRKRegister()
 
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "TSCreate_RK"
 PETSC_EXTERN PetscErrorCode TSCreate_RK(TS ts)
 {
   TS_RK          *rk;
