@@ -1313,8 +1313,7 @@ PetscErrorCode  TSGLLERegisterAll(void)
 
 /*@C
   TSGLLEInitializePackage - This function initializes everything in the TSGLLE package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to TSCreate_GLLE()
-  when using static libraries.
+  from TSInitializePackage().
 
   Level: developer
 
