@@ -108,8 +108,7 @@ PetscErrorCode TSBasicSymplecticRegisterDestroy(void)
 
 /*@C
   TSBasicSymplecticInitializePackage - This function initializes everything in the TSBasicSymplectic package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to TSCreate_BasicSymplectic()
-  when using static libraries.
+  from TSInitializePackage().
 
   Level: developer
 
