@@ -546,7 +546,7 @@ PetscErrorCode TSAdjointMonitorSetFromOptions(TS ts,const char name[],const char
   PetscBool         flg;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)ts),((PetscObject)ts)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)ts),((PetscObject) ts)->options,((PetscObject)ts)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     PetscViewerAndFormat *vf;
     ierr = PetscViewerAndFormatCreate(viewer,format,&vf);CHKERRQ(ierr);
@@ -904,11 +904,11 @@ PetscErrorCode TSAdjointMonitor(TS ts,PetscInt step,PetscReal ptime,Vec u,PetscI
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ts,TS_CLASSID,1);
   PetscValidHeaderSpecific(u,VEC_CLASSID,4);
-  ierr = VecLockPush(u);CHKERRQ(ierr);
+  ierr = VecLockReadPush(u);CHKERRQ(ierr);
   for (i=0; i<n; i++) {
     ierr = (*ts->adjointmonitor[i])(ts,step,ptime,u,numcost,lambda,mu,ts->adjointmonitorcontext[i]);CHKERRQ(ierr);
   }
-  ierr = VecLockPop(u);CHKERRQ(ierr);
+  ierr = VecLockReadPop(u);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
