@@ -4,8 +4,6 @@
 PetscFunctionList AOList              = NULL;
 PetscBool         AORegisterAllCalled = PETSC_FALSE;
 
-#undef __FUNCT__
-#define __FUNCT__ "AOSetType"
 /*@C
   AOSetType - Builds an application ordering for a particular implementation.
 
@@ -49,8 +47,6 @@ PetscErrorCode  AOSetType(AO ao, AOType method)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOGetType"
 /*@C
   AOGetType - Gets the AO type name (as a string) from the AO.
 
@@ -82,18 +78,26 @@ PetscErrorCode  AOGetType(AO ao, AOType *type)
 
 /*--------------------------------------------------------------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "AORegister"
 /*@C
-  AORegister - 
+  AORegister - Register  an application ordering method
+
+    Not Collective
+
+   Input Parameters:
++   sname - the name of the AO scheme
+-   function - the create routine for the application ordering method
 
   Level: advanced
+
+.seealso:   AOCreate(), AORegisterAll(), AOBASIC, AOADVANCED, AOMAPPING, AOMEMORYSCALABLE
+
 @*/
 PetscErrorCode  AORegister(const char sname[], PetscErrorCode (*function)(AO))
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = AOInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&AOList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
