@@ -25,7 +25,8 @@ PetscErrorCode  AOFinalizePackage(void)
 
 /*@C
   AOInitializePackage - This function initializes everything in the AO package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to AOCreate().
+  from PetscDLLibraryRegister_petscvec() when using dynamic libraries, and on the first call to AOCreate()
+  when using static or shared libraries.
 
   Level: developer
 
