@@ -655,6 +655,41 @@ PetscErrorCode PetscSFGetRanks(PetscSF sf,PetscInt *nranks,const PetscMPIInt **r
   PetscFunctionReturn(0);
 }
 
+/*@C
+   PetscSFGetLeafRanks - Get leaf ranks referencing roots on this process
+
+   Not Collective
+
+   Input Arguments:
+.  sf - star forest
+
+   Output Arguments:
++  niranks - number of leaf ranks referencing roots on this process
+.  iranks - array of ranks
+.  ioffset - offset in irootloc for each rank (length niranks+1)
+-  irootloc - concatenated array holding local indices of roots referenced by each leaf rank
+
+   Level: developer
+
+.seealso: PetscSFGetRanks()
+@*/
+PetscErrorCode PetscSFGetLeafRanks(PetscSF sf,PetscInt *niranks,const PetscMPIInt **iranks,const PetscInt **ioffset,const PetscInt **irootloc)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(sf,PETSCSF_CLASSID,1);
+  if (!sf->setupcalled) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Must call PetscSFSetUp() before obtaining ranks");
+  if (sf->ops->GetLeafRanks) {
+    ierr = (sf->ops->GetLeafRanks)(sf,niranks,iranks,ioffset,irootloc);CHKERRQ(ierr);
+  } else {
+    PetscSFType type;
+    ierr = PetscSFGetType(sf,&type);CHKERRQ(ierr);
+    SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"PetscSFGetLeafRanks() is not supported on this StarForest type: %s", type);
+  }
+  PetscFunctionReturn(0);
+}
+
 static PetscBool InList(PetscMPIInt needle,PetscMPIInt n,const PetscMPIInt *list) {
   PetscInt i;
   for (i=0; i<n; i++) {
@@ -845,7 +880,7 @@ PetscErrorCode PetscSFGetGroups(PetscSF sf,MPI_Group *incoming,MPI_Group *outgoi
    directly. Since multi satisfies the stronger condition that each entry in the global space has exactly one incoming
    edge, it is a candidate for future optimization that might involve its removal.
 
-.seealso: PetscSFSetGraph(), PetscSFGatherBegin(), PetscSFScatterBegin()
+.seealso: PetscSFSetGraph(), PetscSFGatherBegin(), PetscSFScatterBegin(), PetscSFComputeMultiRootOriginalNumbering()
 @*/
 PetscErrorCode PetscSFGetMultiSF(PetscSF sf,PetscSF *multi)
 {
@@ -927,8 +962,8 @@ PetscErrorCode PetscSFGetMultiSF(PetscSF sf,PetscSF *multi)
 
    Input Arguments:
 +  sf - original star forest
-.  nroots - number of roots to select on this process
--  selected - selected roots on this process
+.  nselected  - number of selected roots on this process
+-  selected   - indices of the selected roots on this process
 
    Output Arguments:
 .  newsf - new star forest
@@ -941,40 +976,51 @@ PetscErrorCode PetscSFGetMultiSF(PetscSF sf,PetscSF *multi)
 
 .seealso: PetscSFSetGraph(), PetscSFGetGraph()
 @*/
-PetscErrorCode PetscSFCreateEmbeddedSF(PetscSF sf,PetscInt nroots,const PetscInt *selected,PetscSF *newsf)
+PetscErrorCode PetscSFCreateEmbeddedSF(PetscSF sf,PetscInt nselected,const PetscInt *selected,PetscSF *newsf)
 {
-  PetscInt      *rootdata, *leafdata, *ilocal;
-  PetscSFNode   *iremote;
-  PetscInt       leafsize = 0, nleaves = 0, n, i;
-  PetscErrorCode ierr;
+  PetscInt          *rootdata,*leafdata,*new_ilocal;
+  PetscSFNode       *new_iremote;
+  const PetscInt    *ilocal;
+  const PetscSFNode *iremote;
+  PetscInt          nleaves,nroots,n,i,new_nleaves = 0;
+  PetscSF           tmpsf;
+  PetscErrorCode    ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(sf,PETSCSF_CLASSID,1);
   PetscSFCheckGraphSet(sf,1);
-  if (nroots) PetscValidPointer(selected,3);
+  if (nselected) PetscValidPointer(selected,3);
   PetscValidPointer(newsf,4);
-  if (sf->mine) for (i = 0; i < sf->nleaves; ++i) {leafsize = PetscMax(leafsize, sf->mine[i]+1);}
-  else leafsize = sf->nleaves;
-  ierr = PetscCalloc2(sf->nroots,&rootdata,leafsize,&leafdata);CHKERRQ(ierr);
-  for (i=0; i<nroots; ++i) rootdata[selected[i]] = 1;
-  ierr = PetscSFBcastBegin(sf,MPIU_INT,rootdata,leafdata);CHKERRQ(ierr);
-  ierr = PetscSFBcastEnd(sf,MPIU_INT,rootdata,leafdata);CHKERRQ(ierr);
-  for (i = 0; i < leafsize; ++i) nleaves += leafdata[i];
-  ierr = PetscMalloc1(nleaves,&ilocal);CHKERRQ(ierr);
-  ierr = PetscMalloc1(nleaves,&iremote);CHKERRQ(ierr);
-  for (i = 0, n = 0; i < sf->nleaves; ++i) {
-    const PetscInt lidx = sf->mine ? sf->mine[i] : i;
 
-    if (leafdata[lidx]) {
-      ilocal[n]        = lidx;
-      iremote[n].rank  = sf->remote[i].rank;
-      iremote[n].index = sf->remote[i].index;
+  /* Find out which leaves (not leaf data items) are still connected to roots in the embedded sf */
+  ierr = PetscSFGetGraph(sf,&nroots,&nleaves,&ilocal,&iremote);CHKERRQ(ierr);
+  ierr = PetscSFDuplicate(sf,PETSCSF_DUPLICATE_RANKS,&tmpsf);CHKERRQ(ierr);
+  ierr = PetscSFSetGraph(tmpsf,nroots,nleaves,NULL/*contiguous*/,PETSC_USE_POINTER,iremote,PETSC_USE_POINTER);CHKERRQ(ierr);
+  ierr = PetscCalloc2(nroots,&rootdata,nleaves,&leafdata);CHKERRQ(ierr);
+  for (i=0; i<nselected; ++i) {
+    if (selected[i] < 0 || selected[i] >= nroots) SETERRQ2(PetscObjectComm((PetscObject)sf),PETSC_ERR_ARG_OUTOFRANGE,"Root index %D is not in [0,%D)",selected[i],nroots);
+    rootdata[selected[i]] = 1;
+  }
+
+  ierr = PetscSFBcastBegin(tmpsf,MPIU_INT,rootdata,leafdata);CHKERRQ(ierr);
+  ierr = PetscSFBcastEnd(tmpsf,MPIU_INT,rootdata,leafdata);CHKERRQ(ierr);
+  ierr = PetscSFDestroy(&tmpsf);CHKERRQ(ierr);
+
+  /* Build newsf with leaves that are still connected */
+  for (i = 0; i < nleaves; ++i) new_nleaves += leafdata[i];
+  ierr = PetscMalloc1(new_nleaves,&new_ilocal);CHKERRQ(ierr);
+  ierr = PetscMalloc1(new_nleaves,&new_iremote);CHKERRQ(ierr);
+  for (i = 0, n = 0; i < nleaves; ++i) {
+    if (leafdata[i]) {
+      new_ilocal[n]        = sf->mine ? sf->mine[i] : i;
+      new_iremote[n].rank  = sf->remote[i].rank;
+      new_iremote[n].index = sf->remote[i].index;
       ++n;
     }
   }
-  if (n != nleaves) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_PLIB, "There is a size mismatch in the SF embedding, %d != %d", n, nleaves);
-  ierr = PetscSFDuplicate(sf,PETSCSF_DUPLICATE_RANKS,newsf);CHKERRQ(ierr);
-  ierr = PetscSFSetGraph(*newsf,sf->nroots,nleaves,ilocal,PETSC_OWN_POINTER,iremote,PETSC_OWN_POINTER);CHKERRQ(ierr);
+  if (n != new_nleaves) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"There is a size mismatch in the SF embedding, %d != %d",n,new_nleaves);
+  ierr = PetscSFDuplicate(sf,PETSCSF_DUPLICATE_CONFONLY,newsf);CHKERRQ(ierr);
+  ierr = PetscSFSetGraph(*newsf,nroots,new_nleaves,new_ilocal,PETSC_OWN_POINTER,new_iremote,PETSC_OWN_POINTER);CHKERRQ(ierr);
   ierr = PetscFree2(rootdata,leafdata);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -1082,6 +1128,68 @@ PetscErrorCode PetscSFBcastEnd(PetscSF sf,MPI_Datatype unit,const void *rootdata
   PetscFunctionReturn(0);
 }
 
+/*@C
+   PetscSFBcastAndOpBegin - begin pointwise broadcast with root value being reduced to leaf value, to be concluded with call to PetscSFBcastAndOpEnd()
+
+   Collective on PetscSF
+
+   Input Arguments:
++  sf - star forest on which to communicate
+.  unit - data type associated with each node
+.  rootdata - buffer to broadcast
+-  op - operation to use for reduction
+
+   Output Arguments:
+.  leafdata - buffer to be reduced with values from each leaf's respective root
+
+   Level: intermediate
+
+.seealso: PetscSFBcastAndOpEnd(), PetscSFBcastBegin(), PetscSFBcastEnd()
+@*/
+PetscErrorCode PetscSFBcastAndOpBegin(PetscSF sf,MPI_Datatype unit,const void *rootdata,void *leafdata,MPI_Op op)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(sf,PETSCSF_CLASSID,1);
+  ierr = PetscSFSetUp(sf);CHKERRQ(ierr);
+  ierr = PetscLogEventBegin(PETSCSF_BcastAndOpBegin,sf,0,0,0);CHKERRQ(ierr);
+  ierr = (*sf->ops->BcastAndOpBegin)(sf,unit,rootdata,leafdata,op);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(PETSCSF_BcastAndOpBegin,sf,0,0,0);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   PetscSFBcastAndOpEnd - end a broadcast & reduce operation started with PetscSFBcastAndOpBegin()
+
+   Collective
+
+   Input Arguments:
++  sf - star forest
+.  unit - data type
+.  rootdata - buffer to broadcast
+-  op - operation to use for reduction
+
+   Output Arguments:
+.  leafdata - buffer to be reduced with values from each leaf's respective root
+
+   Level: intermediate
+
+.seealso: PetscSFSetGraph(), PetscSFReduceEnd()
+@*/
+PetscErrorCode PetscSFBcastAndOpEnd(PetscSF sf,MPI_Datatype unit,const void *rootdata,void *leafdata,MPI_Op op)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(sf,PETSCSF_CLASSID,1);
+  ierr = PetscSFSetUp(sf);CHKERRQ(ierr);
+  ierr = PetscLogEventBegin(PETSCSF_BcastAndOpEnd,sf,0,0,0);CHKERRQ(ierr);
+  ierr = (*sf->ops->BcastAndOpEnd)(sf,unit,rootdata,leafdata,op);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(PETSCSF_BcastAndOpEnd,sf,0,0,0);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@C
    PetscSFReduceBegin - begin reduction of leafdata into rootdata, to be completed with call to PetscSFReduceEnd()
 
@@ -1213,6 +1321,49 @@ PetscErrorCode PetscSFComputeDegreeEnd(PetscSF sf,const PetscInt **degree)
   PetscFunctionReturn(0);
 }
 
+
+/*@C
+   PetscSFComputeMultiRootOriginalNumbering - Returns original numbering of multi-roots (roots of multi-SF returned by PetscSFGetMultiSF()). Each multi-root is assigned index of its original root.
+
+   Collective
+
+   Input Arguments:
++  sf - star forest
+-  degree - degree of each root vertex, computed with PetscSFComputeDegreeBegin()/PetscSFComputeDegreeEnd()
+
+   Output Arguments:
+.  mRootsOrigNumbering - original indices of multi-roots; length of the array is equal to the number of multi-roots (roots of multi-SF)
+
+   Level: developer
+
+.seealso: PetscSFComputeDegreeBegin(), PetscSFComputeDegreeEnd(), PetscSFGetMultiSF()
+@*/
+PetscErrorCode PetscSFComputeMultiRootOriginalNumbering(PetscSF sf, const PetscInt degree[], PetscInt *mRootsOrigNumbering[])
+{
+  PetscSF             msf;
+  PetscInt            i, j, k, nroots, nmroots;
+  PetscErrorCode      ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(sf,PETSCSF_CLASSID,1);
+  ierr = PetscSFGetGraph(sf, &nroots, NULL, NULL, NULL);CHKERRQ(ierr);
+  if (nroots) PetscValidIntPointer(degree,2);
+  PetscValidPointer(mRootsOrigNumbering,3);
+  ierr = PetscSFGetMultiSF(sf,&msf);CHKERRQ(ierr);
+  ierr = PetscSFGetGraph(msf, &nmroots, NULL, NULL, NULL);CHKERRQ(ierr);
+  ierr = PetscMalloc1(nmroots, mRootsOrigNumbering);CHKERRQ(ierr);
+  for (i=0,j=0,k=0; i<nroots; i++) {
+    if (!degree[i]) continue;
+    for (j=0; j<degree[i]; j++,k++) {
+      (*mRootsOrigNumbering)[k] = i;
+    }
+  }
+#if defined(PETSC_USE_DEBUG)
+  if (PetscUnlikely(k != nmroots)) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"sanity check fail");
+#endif
+  PetscFunctionReturn(0);
+}
+
 /*@C
    PetscSFFetchAndOpBegin - begin operation that fetches values from root and updates atomically by applying operation using my leaf value, to be completed with PetscSFFetchAndOpEnd()
 
