@@ -6,8 +6,6 @@
 #include <petscis.h> /*I "petscis.h" I*/
 #include <petscsf.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutCreate"
 /*@
   PetscLayoutCreate - Allocates PetscLayout space and sets the map contents to the default.
 
@@ -53,14 +51,12 @@ PetscErrorCode PetscLayoutCreate(MPI_Comm comm,PetscLayout *map)
   (*map)->bs     = -1;
   (*map)->n      = -1;
   (*map)->N      = -1;
-  (*map)->range  = 0;
+  (*map)->range  = NULL;
   (*map)->rstart = 0;
   (*map)->rend   = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutDestroy"
 /*@
   PetscLayoutDestroy - Frees a map object and frees its range if that exists.
 
@@ -94,8 +90,35 @@ PetscErrorCode PetscLayoutDestroy(PetscLayout *map)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutSetUp"
+static PetscErrorCode PetscLayoutSetUp_SizesFromRanges_Private(PetscLayout map)
+{
+  PetscMPIInt    rank,size;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = MPI_Comm_size(map->comm, &size);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(map->comm, &rank);CHKERRQ(ierr);
+  map->rstart = map->range[rank];
+  map->rend   = map->range[rank+1];
+  map->n      = map->rend - map->rstart;
+  map->N      = map->range[size];
+#if defined(PETSC_USE_DEBUG)
+  /* just check that n, N and bs are consistent */
+  {
+    PetscInt tmp;
+    ierr = MPIU_Allreduce(&map->n,&tmp,1,MPIU_INT,MPI_SUM,map->comm);CHKERRQ(ierr);
+    if (tmp != map->N) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Sum of local lengths %D does not equal global length %D, my local length %D.\nThe provided PetscLayout is wrong.",tmp,map->N,map->n);
+  }
+  if (map->bs > 1) {
+    if (map->n % map->bs) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Local size %D must be divisible by blocksize %D",map->n,map->bs);
+  }
+  if (map->bs > 1) {
+    if (map->N % map->bs) SETERRQ2(map->comm,PETSC_ERR_PLIB,"Global size %D must be divisible by blocksize %D",map->N,map->bs);
+  }
+#endif
+  PetscFunctionReturn(0);
+}
+
 /*@
   PetscLayoutSetUp - given a map where you have set either the global or local
                      size sets up the map so that it may be used.
@@ -107,13 +130,15 @@ PetscErrorCode PetscLayoutDestroy(PetscLayout *map)
 
   Level: developer
 
-  Notes: Typical calling sequence
+  Notes:
+    Typical calling sequence
 $ PetscLayoutCreate(MPI_Comm,PetscLayout *);
 $ PetscLayoutSetBlockSize(PetscLayout,1);
 $ PetscLayoutSetSize(PetscLayout,n) or PetscLayoutSetLocalSize(PetscLayout,N); or both
 $ PetscLayoutSetUp(PetscLayout);
 $ PetscLayoutGetSize(PetscLayout,PetscInt *);
 
+  If range exists, and local size is not set, everything gets computed from the range.
 
   If the local size, global size are already set and range exists then this does nothing.
 
@@ -128,12 +153,16 @@ PetscErrorCode PetscLayoutSetUp(PetscLayout map)
 
   PetscFunctionBegin;
   if ((map->n >= 0) && (map->N >= 0) && (map->range)) PetscFunctionReturn(0);
+  if (map->range && map->n < 0) {
+    ierr = PetscLayoutSetUp_SizesFromRanges_Private(map);CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
 
   if (map->n > 0 && map->bs > 1) {
-    if (map->n % map->bs) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Local matrix size %D must be divisible by blocksize %D",map->n,map->bs);
+    if (map->n % map->bs) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Local size %D must be divisible by blocksize %D",map->n,map->bs);
   }
   if (map->N > 0 && map->bs > 1) {
-    if (map->N % map->bs) SETERRQ2(map->comm,PETSC_ERR_PLIB,"Global matrix size %D must be divisible by blocksize %D",map->N,map->bs);
+    if (map->N % map->bs) SETERRQ2(map->comm,PETSC_ERR_PLIB,"Global size %D must be divisible by blocksize %D",map->N,map->bs);
   }
 
   ierr = MPI_Comm_size(map->comm, &size);CHKERRQ(ierr);
@@ -156,8 +185,6 @@ PetscErrorCode PetscLayoutSetUp(PetscLayout map)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutDuplicate"
 /*@
   PetscLayoutDuplicate - creates a new PetscLayout with the same information as a given one. If the PetscLayout already exists it is destroyed first.
 
@@ -171,7 +198,8 @@ PetscErrorCode PetscLayoutSetUp(PetscLayout map)
 
   Level: developer
 
-  Notes: PetscLayoutSetUp() does not need to be called on the resulting PetscLayout
+  Notes:
+    PetscLayoutSetUp() does not need to be called on the resulting PetscLayout
 
 .seealso: PetscLayoutCreate(), PetscLayoutDestroy(), PetscLayoutSetUp(), PetscLayoutReference()
 @*/
@@ -193,8 +221,6 @@ PetscErrorCode PetscLayoutDuplicate(PetscLayout in,PetscLayout *out)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutReference"
 /*@
   PetscLayoutReference - Causes a PETSc Vec or Mat to share a PetscLayout with one that already exists. Used by Vec/MatDuplicate_XXX()
 
@@ -208,7 +234,8 @@ PetscErrorCode PetscLayoutDuplicate(PetscLayout in,PetscLayout *out)
 
   Level: developer
 
-  Notes: PetscLayoutSetUp() does not need to be called on the resulting PetscLayout
+  Notes:
+    PetscLayoutSetUp() does not need to be called on the resulting PetscLayout
 
   If the out location already contains a PetscLayout it is destroyed
 
@@ -225,8 +252,6 @@ PetscErrorCode PetscLayoutReference(PetscLayout in,PetscLayout *out)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutSetISLocalToGlobalMapping"
 /*@
   PetscLayoutSetISLocalToGlobalMapping - sets a ISLocalGlobalMapping into a PetscLayout
 
@@ -239,7 +264,8 @@ PetscErrorCode PetscLayoutReference(PetscLayout in,PetscLayout *out)
 
   Level: developer
 
-  Notes: PetscLayoutSetUp() does not need to be called on the resulting PetscLayout
+  Notes:
+    PetscLayoutSetUp() does not need to be called on the resulting PetscLayout
 
   If the ltog location already contains a PetscLayout it is destroyed
 
@@ -259,8 +285,6 @@ PetscErrorCode PetscLayoutSetISLocalToGlobalMapping(PetscLayout in,ISLocalToGlob
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutSetLocalSize"
 /*@
   PetscLayoutSetLocalSize - Sets the local size for a PetscLayout object.
 
@@ -309,8 +333,6 @@ PetscErrorCode PetscLayoutSetLocalSize(PetscLayout map,PetscInt n)
           PetscLayoutGetRange(), PetscLayoutGetRanges(), PetscLayoutSetBlockSize(), PetscLayoutGetBlockSize()
 
 @*/
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutGetLocalSize"
 PetscErrorCode  PetscLayoutGetLocalSize(PetscLayout map,PetscInt *n)
 {
   PetscFunctionBegin;
@@ -318,8 +340,6 @@ PetscErrorCode  PetscLayoutGetLocalSize(PetscLayout map,PetscInt *n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutSetSize"
 /*@
   PetscLayoutSetSize - Sets the global size for a PetscLayout object.
 
@@ -344,8 +364,6 @@ PetscErrorCode PetscLayoutSetSize(PetscLayout map,PetscInt n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutGetSize"
 /*@
   PetscLayoutGetSize - Gets the global size for a PetscLayout object.
 
@@ -372,8 +390,6 @@ PetscErrorCode PetscLayoutGetSize(PetscLayout map,PetscInt *n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutSetBlockSize"
 /*@
   PetscLayoutSetBlockSize - Sets the block size for a PetscLayout object.
 
@@ -395,21 +411,20 @@ PetscErrorCode PetscLayoutSetBlockSize(PetscLayout map,PetscInt bs)
 {
   PetscFunctionBegin;
   if (bs < 0) PetscFunctionReturn(0);
-  if (map->n > 0 && map->n % bs) SETERRQ2(map->comm,PETSC_ERR_ARG_INCOMP,"Local size %D not compatible with block size %D",map->n,bs);
-  if (map->range && map->bs > 0 && map->bs != bs) SETERRQ2(map->comm,PETSC_ERR_ARG_INCOMP,"Cannot change block size %D to %D",map->bs,bs);
+  if (map->n > 0 && map->n % bs) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Local size %D not compatible with block size %D",map->n,bs);
   if (map->mapping) {
-    PetscInt       lbs;
+    PetscInt       obs;
     PetscErrorCode ierr;
 
-    ierr = ISLocalToGlobalMappingGetBlockSize(map->mapping,&lbs);CHKERRQ(ierr);
-    if (lbs != bs) SETERRQ2(map->comm,PETSC_ERR_PLIB,"Blocksize of localtoglobalmapping %D must match that of layout %D",lbs,bs);
+    ierr = ISLocalToGlobalMappingGetBlockSize(map->mapping,&obs);CHKERRQ(ierr);
+    if (obs > 1) {
+      ierr = ISLocalToGlobalMappingSetBlockSize(map->mapping,bs);CHKERRQ(ierr);
+    }
   }
   map->bs = bs;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutGetBlockSize"
 /*@
   PetscLayoutGetBlockSize - Gets the block size for a PetscLayout object.
 
@@ -436,8 +451,6 @@ PetscErrorCode PetscLayoutGetBlockSize(PetscLayout map,PetscInt *bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutGetRange"
 /*@
   PetscLayoutGetRange - gets the range of values owned by this process
 
@@ -490,8 +503,6 @@ PetscErrorCode PetscLayoutGetRange(PetscLayout map,PetscInt *rstart,PetscInt *re
           PetscLayoutGetSize(), PetscLayoutGetRange(), PetscLayoutSetBlockSize(), PetscLayoutGetSize(), PetscLayoutSetUp()
 
 @*/
-#undef __FUNCT__
-#define __FUNCT__ "PetscLayoutGetRanges"
 PetscErrorCode  PetscLayoutGetRanges(PetscLayout map,const PetscInt *range[])
 {
   PetscFunctionBegin;
@@ -499,8 +510,6 @@ PetscErrorCode  PetscLayoutGetRanges(PetscLayout map,const PetscInt *range[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFSetGraphLayout"
 /*@C
    PetscSFSetGraphLayout - Set a parallel star forest via global indices and a PetscLayout
 
@@ -511,10 +520,15 @@ PetscErrorCode  PetscLayoutGetRanges(PetscLayout map,const PetscInt *range[])
 .  layout - PetscLayout defining the global space
 .  nleaves - number of leaf vertices on the current process, each of these references a root on any process
 .  ilocal - locations of leaves in leafdata buffers, pass NULL for contiguous storage
+.  localmode - copy mode for ilocal
 -  iremote - remote locations of root vertices for each leaf on the current process
 
    Level: intermediate
 
+   Developers Note: Local indices which are the identity permutation in the range [0,nleaves) are discarded as they
+   encode contiguous storage. In such case, if localmode is PETSC_OWN_POINTER, the memory is deallocated as it is not
+   needed
+
 .seealso: PetscSFCreate(), PetscSFView(), PetscSFSetGraph(), PetscSFGetGraph()
 @*/
 PetscErrorCode PetscSFSetGraphLayout(PetscSF sf,PetscLayout layout,PetscInt nleaves,const PetscInt *ilocal,PetscCopyMode localmode,const PetscInt *iremote)
@@ -536,3 +550,36 @@ PetscErrorCode PetscSFSetGraphLayout(PetscSF sf,PetscLayout layout,PetscInt nlea
   PetscFunctionReturn(0);
 }
 
+/*@
+  PetscLayoutCompare - Compares two layouts
+
+  Not Collective
+
+  Input Parameters:
++ mapa - pointer to the first map
+- mapb - pointer to the second map
+
+  Output Parameters:
+. congruent - PETSC_TRUE if the two layouts are congruent, PETSC_FALSE otherwise
+
+  Level: beginner
+
+  Notes:
+
+.seealso: PetscLayoutCreate(), PetscLayoutSetLocalSize(), PetscLayoutGetLocalSize(), PetscLayoutGetBlockSize(),
+          PetscLayoutGetRange(), PetscLayoutGetRanges(), PetscLayoutSetSize(), PetscLayoutGetSize(), PetscLayoutSetUp()
+@*/
+PetscErrorCode PetscLayoutCompare(PetscLayout mapa,PetscLayout mapb,PetscBool *congruent)
+{
+  PetscErrorCode ierr;
+  PetscMPIInt    sizea,sizeb;
+
+  PetscFunctionBegin;
+  *congruent = PETSC_FALSE;
+  ierr = MPI_Comm_size(mapa->comm,&sizea);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(mapb->comm,&sizeb);CHKERRQ(ierr);
+  if (mapa->N == mapb->N && mapa->range && mapb->range && sizea == sizeb) {
+    ierr = PetscMemcmp(mapa->range,mapb->range,(sizea+1)*sizeof(PetscInt),congruent);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
