@@ -21,8 +21,6 @@ T*/
 */
 #include <petscvec.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   PetscMPIInt    rank,size;
@@ -32,7 +30,7 @@ int main(int argc,char **argv)
   PetscScalar    value,*array,*tarray=0;
   Vec            lx,gx,gxs;
 
-  PetscInitialize(&argc,&argv,(char*)0,help);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
   if (size != 2) SETERRQ(PETSC_COMM_SELF,1,"Must run example with two processors\n");
@@ -69,8 +67,8 @@ int main(int argc,char **argv)
      the local vector (lx) and the global vector (gx) share the same
      array for storing vector values.
   */
-  ierr = PetscOptionsHasName(NULL,"-allocate",&flg);CHKERRQ(ierr);
-  ierr = PetscOptionsHasName(NULL,"-vecmpisetghost",&flg2);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-allocate",&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-vecmpisetghost",&flg2);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscMalloc1(nlocal+nghost,&tarray);CHKERRQ(ierr);
     ierr = VecCreateGhostWithArray(PETSC_COMM_WORLD,nlocal,PETSC_DECIDE,nghost,ifrom,tarray,&gxs);CHKERRQ(ierr);
@@ -122,8 +120,24 @@ int main(int argc,char **argv)
   ierr = VecDestroy(&gx);CHKERRQ(ierr);
   if (flg) {ierr = PetscFree(tarray);CHKERRQ(ierr);}
   ierr = PetscFinalize();
-  return 0;
+  return ierr; 
 }
 
+/*TEST
+
+     test:
+       nsize: 2
+
+     test:
+       suffix: 2
+       nsize: 2
+       args: -allocate
+
+     test:
+       suffix: 3
+       nsize: 2
+       args: -vecmpisetghost
+
+TEST*/
 
 
