@@ -4,15 +4,15 @@
 */
 
 #include <../src/vec/vec/impls/dvecimpl.h>          /*I "petscvec.h" I*/
-#include <../src/vec/vec/impls/mpi/pvecimpl.h> /* For VecView_MPI_HDF5 */
+#include <../src/vec/vec/impls/mpi/pvecimpl.h>      /* For VecView_MPI_HDF5 */
+#include <petsc/private/glvisviewerimpl.h>
+#include <petsc/private/glvisvecimpl.h>
 #include <petscblaslapack.h>
 
 #if defined(PETSC_HAVE_HDF5)
 extern PetscErrorCode VecView_MPI_HDF5(Vec,PetscViewer);
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseMax_Seq"
 PetscErrorCode VecPointwiseMax_Seq(Vec win,Vec xin,Vec yin)
 {
   PetscErrorCode ierr;
@@ -33,8 +33,6 @@ PetscErrorCode VecPointwiseMax_Seq(Vec win,Vec xin,Vec yin)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseMin_Seq"
 PetscErrorCode VecPointwiseMin_Seq(Vec win,Vec xin,Vec yin)
 {
   PetscErrorCode ierr;
@@ -55,8 +53,6 @@ PetscErrorCode VecPointwiseMin_Seq(Vec win,Vec xin,Vec yin)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseMaxAbs_Seq"
 PetscErrorCode VecPointwiseMaxAbs_Seq(Vec win,Vec xin,Vec yin)
 {
   PetscErrorCode ierr;
@@ -79,8 +75,6 @@ PetscErrorCode VecPointwiseMaxAbs_Seq(Vec win,Vec xin,Vec yin)
 
 #include <../src/vec/vec/impls/seq/ftn-kernels/fxtimesy.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseMult_Seq"
 PetscErrorCode VecPointwiseMult_Seq(Vec win,Vec xin,Vec yin)
 {
   PetscErrorCode ierr;
@@ -109,8 +103,6 @@ PetscErrorCode VecPointwiseMult_Seq(Vec win,Vec xin,Vec yin)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseDivide_Seq"
 PetscErrorCode VecPointwiseDivide_Seq(Vec win,Vec xin,Vec yin)
 {
   PetscErrorCode ierr;
@@ -122,7 +114,10 @@ PetscErrorCode VecPointwiseDivide_Seq(Vec win,Vec xin,Vec yin)
   ierr = VecGetArrayRead(yin,(const PetscScalar**)&yy);CHKERRQ(ierr);
   ierr = VecGetArray(win,&ww);CHKERRQ(ierr);
 
-  for (i=0; i<n; i++) ww[i] = xx[i] / yy[i];
+  for (i=0; i<n; i++) {
+    if (yy[i] != 0.0) ww[i] = xx[i] / yy[i];
+    else ww[i] = 0.0;
+  }
 
   ierr = PetscLogFlops(n);CHKERRQ(ierr);
   ierr = VecRestoreArrayRead(xin,(const PetscScalar**)&xx);CHKERRQ(ierr);
@@ -131,8 +126,6 @@ PetscErrorCode VecPointwiseDivide_Seq(Vec win,Vec xin,Vec yin)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetRandom_Seq"
 PetscErrorCode VecSetRandom_Seq(Vec xin,PetscRandom r)
 {
   PetscErrorCode ierr;
@@ -146,8 +139,6 @@ PetscErrorCode VecSetRandom_Seq(Vec xin,PetscRandom r)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetSize_Seq"
 PetscErrorCode VecGetSize_Seq(Vec vin,PetscInt *size)
 {
   PetscFunctionBegin;
@@ -157,8 +148,6 @@ PetscErrorCode VecGetSize_Seq(Vec vin,PetscInt *size)
 
 
 
-#undef __FUNCT__
-#define __FUNCT__ "VecConjugate_Seq"
 PetscErrorCode VecConjugate_Seq(Vec xin)
 {
   PetscScalar    *x;
@@ -175,8 +164,6 @@ PetscErrorCode VecConjugate_Seq(Vec xin)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecResetArray_Seq"
 PetscErrorCode VecResetArray_Seq(Vec vin)
 {
   Vec_Seq *v = (Vec_Seq*)vin->data;
@@ -187,8 +174,6 @@ PetscErrorCode VecResetArray_Seq(Vec vin)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecCopy_Seq"
 PetscErrorCode VecCopy_Seq(Vec xin,Vec yin)
 {
   PetscScalar       *ya;
@@ -206,8 +191,6 @@ PetscErrorCode VecCopy_Seq(Vec xin,Vec yin)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSwap_Seq"
 PetscErrorCode VecSwap_Seq(Vec xin,Vec yin)
 {
   PetscScalar    *ya, *xa;
@@ -228,8 +211,6 @@ PetscErrorCode VecSwap_Seq(Vec xin,Vec yin)
 
 #include <../src/vec/vec/impls/seq/ftn-kernels/fnorm.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "VecNorm_Seq"
 PetscErrorCode VecNorm_Seq(Vec xin,NormType type,PetscReal *z)
 {
   const PetscScalar *xx;
@@ -241,8 +222,12 @@ PetscErrorCode VecNorm_Seq(Vec xin,NormType type,PetscReal *z)
   ierr = PetscBLASIntCast(n,&bn);CHKERRQ(ierr);
   if (type == NORM_2 || type == NORM_FROBENIUS) {
     ierr = VecGetArrayRead(xin,&xx);CHKERRQ(ierr);
+#if defined(PETSC_USE_REAL___FP16)
+    *z   = BLASnrm2_(&bn,xx,&one);
+#else
     *z   = PetscRealPart(BLASdot_(&bn,xx,&one,xx,&one));
     *z   = PetscSqrtReal(*z);
+#endif
     ierr = VecRestoreArrayRead(xin,&xx);CHKERRQ(ierr);
     ierr = PetscLogFlops(PetscMax(2.0*n-1,0.0));CHKERRQ(ierr);
   } else if (type == NORM_INFINITY) {
@@ -259,8 +244,20 @@ PetscErrorCode VecNorm_Seq(Vec xin,NormType type,PetscReal *z)
     ierr = VecRestoreArrayRead(xin,&xx);CHKERRQ(ierr);
     *z   = max;
   } else if (type == NORM_1) {
+#if defined(PETSC_USE_COMPLEX)
+    PetscReal tmp = 0.0;
+    PetscInt    i;
+#endif
     ierr = VecGetArrayRead(xin,&xx);CHKERRQ(ierr);
+#if defined(PETSC_USE_COMPLEX)
+    /* BLASasum() returns the nonstandard 1 norm of the 1 norm of the complex entries so we provide a custom loop instead */
+    for (i=0; i<n; i++) {
+      tmp += PetscAbsScalar(xx[i]);
+    }
+    *z = tmp;
+#else
     PetscStackCallBLAS("BLASasum",*z   = BLASasum_(&bn,xx,&one));
+#endif
     ierr = VecRestoreArrayRead(xin,&xx);CHKERRQ(ierr);
     ierr = PetscLogFlops(PetscMax(n-1.0,0.0));CHKERRQ(ierr);
   } else if (type == NORM_1_AND_2) {
@@ -270,8 +267,6 @@ PetscErrorCode VecNorm_Seq(Vec xin,NormType type,PetscReal *z)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecView_Seq_ASCII"
 PetscErrorCode VecView_Seq_ASCII(Vec xin,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -420,6 +415,38 @@ PetscErrorCode VecView_Seq_ASCII(Vec xin,PetscViewer viewer)
       }
       ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
     }
+  } else if (format == PETSC_VIEWER_ASCII_GLVIS) {
+    /* GLVis ASCII visualization/dump: this function mimicks mfem::GridFunction::Save() */
+    const PetscScalar       *array;
+    PetscInt                i,n,vdim, ordering = 1; /* mfem::FiniteElementSpace::Ordering::byVDIM */
+    PetscContainer          glvis_container;
+    PetscViewerGLVisVecInfo glvis_vec_info;
+    PetscViewerGLVisInfo    glvis_info;
+    PetscErrorCode          ierr;
+
+    /* mfem::FiniteElementSpace::Save() */
+    ierr = VecGetBlockSize(xin,&vdim);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"FiniteElementSpace\n");CHKERRQ(ierr);
+    ierr = PetscObjectQuery((PetscObject)xin,"_glvis_info_container",(PetscObject*)&glvis_container);CHKERRQ(ierr);
+    if (!glvis_container) SETERRQ(PetscObjectComm((PetscObject)xin),PETSC_ERR_PLIB,"Missing GLVis container");
+    ierr = PetscContainerGetPointer(glvis_container,(void**)&glvis_vec_info);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"%s\n",glvis_vec_info->fec_type);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"VDim: %d\n",vdim);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"Ordering: %d\n",ordering);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
+    /* mfem::Vector::Print() */
+    ierr = PetscObjectQuery((PetscObject)viewer,"_glvis_info_container",(PetscObject*)&glvis_container);CHKERRQ(ierr);
+    if (!glvis_container) SETERRQ(PetscObjectComm((PetscObject)viewer),PETSC_ERR_PLIB,"Missing GLVis container");
+    ierr = PetscContainerGetPointer(glvis_container,(void**)&glvis_info);CHKERRQ(ierr);
+    if (glvis_info->enabled) {
+      ierr = VecGetLocalSize(xin,&n);CHKERRQ(ierr);
+      ierr = VecGetArrayRead(xin,&array);CHKERRQ(ierr);
+      for (i=0;i<n;i++) {
+        ierr = PetscViewerASCIIPrintf(viewer,glvis_info->fmt,(double)PetscRealPart(array[i]));CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
+      }
+      ierr = VecRestoreArrayRead(xin,&array);CHKERRQ(ierr);
+    }
   } else if (format == PETSC_VIEWER_ASCII_INFO || format == PETSC_VIEWER_ASCII_INFO_DETAIL) {
     /* No info */
   } else {
@@ -446,69 +473,58 @@ PetscErrorCode VecView_Seq_ASCII(Vec xin,PetscViewer viewer)
 }
 
 #include <petscdraw.h>
-#undef __FUNCT__
-#define __FUNCT__ "VecView_Seq_Draw_LG"
 PetscErrorCode VecView_Seq_Draw_LG(Vec xin,PetscViewer v)
 {
+  PetscDraw         draw;
+  PetscBool         isnull;
+  PetscDrawLG       lg;
   PetscErrorCode    ierr;
   PetscInt          i,c,bs = PetscAbs(xin->map->bs),n = xin->map->n/bs;
-  PetscDraw         win;
-  PetscReal         *xx;
-  PetscDrawLG       lg;
   const PetscScalar *xv;
-  PetscReal         *yy;
+  PetscReal         *xx,*yy;
+  int               colors[] = {PETSC_DRAW_RED};
 
   PetscFunctionBegin;
-  ierr = PetscMalloc1(n,&xx);CHKERRQ(ierr);
-  ierr = PetscMalloc1(n,&yy);CHKERRQ(ierr);
+  ierr = PetscViewerDrawGetDraw(v,0,&draw);CHKERRQ(ierr);
+  ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr);
+  if (isnull) PetscFunctionReturn(0);
+
+  ierr = PetscMalloc2(n,&xx,n,&yy);CHKERRQ(ierr);
   ierr = VecGetArrayRead(xin,&xv);CHKERRQ(ierr);
   for (c=0; c<bs; c++) {
     ierr = PetscViewerDrawGetDrawLG(v,c,&lg);CHKERRQ(ierr);
-    ierr = PetscDrawLGGetDraw(lg,&win);CHKERRQ(ierr);
-    ierr = PetscDrawCheckResizedWindow(win);CHKERRQ(ierr);
     ierr = PetscDrawLGReset(lg);CHKERRQ(ierr);
+    ierr = PetscDrawLGSetDimension(lg,1);CHKERRQ(ierr);
+    ierr = PetscDrawLGSetColors(lg,colors);CHKERRQ(ierr);
     for (i=0; i<n; i++) {
-      xx[i] = (PetscReal) i;
+      xx[i] = (PetscReal)i;
       yy[i] = PetscRealPart(xv[c + i*bs]);
     }
     ierr = PetscDrawLGAddPoints(lg,n,&xx,&yy);CHKERRQ(ierr);
     ierr = PetscDrawLGDraw(lg);CHKERRQ(ierr);
-    ierr = PetscDrawSynchronizedFlush(win);CHKERRQ(ierr);
+    ierr = PetscDrawLGSave(lg);CHKERRQ(ierr);
   }
   ierr = VecRestoreArrayRead(xin,&xv);CHKERRQ(ierr);
-  ierr = PetscFree(yy);CHKERRQ(ierr);
-  ierr = PetscFree(xx);CHKERRQ(ierr);
+  ierr = PetscFree2(xx,yy);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecView_Seq_Draw"
 PetscErrorCode VecView_Seq_Draw(Vec xin,PetscViewer v)
 {
   PetscErrorCode    ierr;
   PetscDraw         draw;
   PetscBool         isnull;
-  PetscViewerFormat format;
 
   PetscFunctionBegin;
   ierr = PetscViewerDrawGetDraw(v,0,&draw);CHKERRQ(ierr);
-  ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr); if (isnull) PetscFunctionReturn(0);
-
-  ierr = PetscViewerGetFormat(v,&format);CHKERRQ(ierr);
-  /*
-     Currently it only supports drawing to a line graph */
-  if (format != PETSC_VIEWER_DRAW_LG) {
-    ierr = PetscViewerPushFormat(v,PETSC_VIEWER_DRAW_LG);CHKERRQ(ierr);
-  }
+  ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr);
+  if (isnull) PetscFunctionReturn(0);
+  ierr = PetscViewerPushFormat(v,PETSC_VIEWER_DRAW_LG);CHKERRQ(ierr);
   ierr = VecView_Seq_Draw_LG(xin,v);CHKERRQ(ierr);
-  if (format != PETSC_VIEWER_DRAW_LG) {
-    ierr = PetscViewerPopFormat(v);CHKERRQ(ierr);
-  }
+  ierr = PetscViewerPopFormat(v);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecView_Seq_Binary"
 PetscErrorCode VecView_Seq_Binary(Vec xin,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -583,8 +599,6 @@ PetscErrorCode VecView_Seq_Binary(Vec xin,PetscViewer viewer)
 #if defined(PETSC_HAVE_MATLAB_ENGINE)
 #include <petscmatlab.h>
 #include <mat.h>   /* MATLAB include file */
-#undef __FUNCT__
-#define __FUNCT__ "VecView_Seq_Matlab"
 PetscErrorCode VecView_Seq_Matlab(Vec vec,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -601,9 +615,7 @@ PetscErrorCode VecView_Seq_Matlab(Vec vec,PetscViewer viewer)
 }
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "VecView_Seq"
-PetscErrorCode VecView_Seq(Vec xin,PetscViewer viewer)
+PETSC_EXTERN PetscErrorCode VecView_Seq(Vec xin,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   PetscBool      isdraw,iascii,issocket,isbinary;
@@ -616,6 +628,13 @@ PetscErrorCode VecView_Seq(Vec xin,PetscViewer viewer)
 #if defined(PETSC_HAVE_HDF5)
   PetscBool      ishdf5;
 #endif
+  PetscBool      isglvis;
+#if defined(PETSC_HAVE_ADIOS)
+  PetscBool      isadios;
+#endif
+#if defined(PETSC_HAVE_ADIOS2)
+  PetscBool      isadios2;
+#endif
 
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
@@ -631,6 +650,13 @@ PetscErrorCode VecView_Seq(Vec xin,PetscViewer viewer)
 #if defined(PETSC_HAVE_MATLAB_ENGINE)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERMATLAB,&ismatlab);CHKERRQ(ierr);
 #endif
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERGLVIS,&isglvis);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_ADIOS)
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERADIOS,&isadios);CHKERRQ(ierr);
+#endif
+#if defined(PETSC_HAVE_ADIOS2)
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERADIOS2,&isadios2);CHKERRQ(ierr);
+#endif
 
   if (isdraw) {
     ierr = VecView_Seq_Draw(xin,viewer);CHKERRQ(ierr);
@@ -646,16 +672,24 @@ PetscErrorCode VecView_Seq(Vec xin,PetscViewer viewer)
   } else if (ishdf5) {
     ierr = VecView_MPI_HDF5(xin,viewer);CHKERRQ(ierr); /* Reusing VecView_MPI_HDF5 ... don't want code duplication*/
 #endif
+#if defined(PETSC_HAVE_ADIOS)
+  } else if (isadios) {
+    ierr = VecView_MPI_ADIOS(xin,viewer);CHKERRQ(ierr); /* Reusing VecView_MPI_ADIOS ... don't want code duplication*/
+#endif
+#if defined(PETSC_HAVE_ADIOS2)
+  } else if (isadios2) {
+    ierr = VecView_MPI_ADIOS2(xin,viewer);CHKERRQ(ierr); /* Reusing VecView_MPI_ADIOS2 ... don't want code duplication*/
+#endif
 #if defined(PETSC_HAVE_MATLAB_ENGINE)
   } else if (ismatlab) {
     ierr = VecView_Seq_Matlab(xin,viewer);CHKERRQ(ierr);
 #endif
+  } else if (isglvis) {
+    ierr = VecView_GLVis(xin,viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetValues_Seq"
 PetscErrorCode VecGetValues_Seq(Vec xin,PetscInt ni,const PetscInt ix[],PetscScalar y[])
 {
   const PetscScalar *xx;
@@ -676,8 +710,6 @@ PetscErrorCode VecGetValues_Seq(Vec xin,PetscInt ni,const PetscInt ix[],PetscSca
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetValues_Seq"
 PetscErrorCode VecSetValues_Seq(Vec xin,PetscInt ni,const PetscInt ix[],const PetscScalar y[],InsertMode m)
 {
   PetscScalar    *xx;
@@ -709,8 +741,6 @@ PetscErrorCode VecSetValues_Seq(Vec xin,PetscInt ni,const PetscInt ix[],const Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetValuesBlocked_Seq"
 PetscErrorCode VecSetValuesBlocked_Seq(Vec xin,PetscInt ni,const PetscInt ix[],const PetscScalar yin[],InsertMode m)
 {
   PetscScalar    *xx,*y = (PetscScalar*)yin;
@@ -724,32 +754,28 @@ PetscErrorCode VecSetValuesBlocked_Seq(Vec xin,PetscInt ni,const PetscInt ix[],c
   ierr = VecGetBlockSize(xin,&bs);CHKERRQ(ierr);
   ierr = VecGetArray(xin,&xx);CHKERRQ(ierr);
   if (m == INSERT_VALUES) {
-    for (i=0; i<ni; i++) {
+    for (i=0; i<ni; i++, y+=bs) {
       start = bs*ix[i];
       if (start < 0) continue;
 #if defined(PETSC_USE_DEBUG)
       if (start >= xin->map->n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Out of range index value %D maximum %D",start,xin->map->n);
 #endif
       for (j=0; j<bs; j++) xx[start+j] = y[j];
-      y += bs;
     }
   } else {
-    for (i=0; i<ni; i++) {
+    for (i=0; i<ni; i++, y+=bs) {
       start = bs*ix[i];
       if (start < 0) continue;
 #if defined(PETSC_USE_DEBUG)
       if (start >= xin->map->n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Out of range index value %D maximum %D",start,xin->map->n);
 #endif
       for (j=0; j<bs; j++) xx[start+j] += y[j];
-      y += bs;
     }
   }
   ierr = VecRestoreArray(xin,&xx);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDestroy_Seq"
 PetscErrorCode VecDestroy_Seq(Vec v)
 {
   Vec_Seq        *vs = (Vec_Seq*)v->data;
@@ -764,8 +790,6 @@ PetscErrorCode VecDestroy_Seq(Vec v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetOption_Seq"
 PetscErrorCode VecSetOption_Seq(Vec v,VecOption op,PetscBool flag)
 {
   PetscFunctionBegin;
@@ -773,15 +797,12 @@ PetscErrorCode VecSetOption_Seq(Vec v,VecOption op,PetscBool flag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDuplicate_Seq"
 PetscErrorCode VecDuplicate_Seq(Vec win,Vec *V)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   ierr = VecCreate(PetscObjectComm((PetscObject)win),V);CHKERRQ(ierr);
-  ierr = PetscObjectSetPrecision((PetscObject)*V,((PetscObject)win)->precision);CHKERRQ(ierr);
   ierr = VecSetSizes(*V,win->map->n,win->map->n);CHKERRQ(ierr);
   ierr = VecSetType(*V,((PetscObject)win)->type_name);CHKERRQ(ierr);
   ierr = PetscLayoutReference(win->map,&(*V)->map);CHKERRQ(ierr);
@@ -868,8 +889,6 @@ static struct _VecOps DvOps = {VecDuplicate_Seq, /* 1 */
 /*
       This is called by VecCreate_Seq() (i.e. VecCreateSeq()) and VecCreateSeqWithArray()
 */
-#undef __FUNCT__
-#define __FUNCT__ "VecCreate_Seq_Private"
 PetscErrorCode VecCreate_Seq_Private(Vec v,const PetscScalar array[])
 {
   Vec_Seq        *s;
@@ -889,15 +908,10 @@ PetscErrorCode VecCreate_Seq_Private(Vec v,const PetscScalar array[])
 #if defined(PETSC_HAVE_MATLAB_ENGINE)
   ierr = PetscObjectComposeFunction((PetscObject)v,"PetscMatlabEnginePut_C",VecMatlabEnginePut_Default);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)v,"PetscMatlabEngineGet_C",VecMatlabEngineGet_Default);CHKERRQ(ierr);
-#endif
-#if defined(PETSC_USE_MIXED_PRECISION)
-  ((PetscObject)v)->precision = (PetscPrecision)sizeof(PetscReal);
 #endif
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecCreateSeqWithArray"
 /*@C
    VecCreateSeqWithArray - Creates a standard,sequential array-style vector,
    where the user provides the array space to store the vector values.
@@ -944,9 +958,3 @@ PetscErrorCode  VecCreateSeqWithArray(MPI_Comm comm,PetscInt bs,PetscInt n,const
   ierr = VecCreate_Seq_Private(*V,array);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-
-
-
-
-
-
