@@ -33,8 +33,8 @@ PetscErrorCode  ISFinalizePackage(void)
 
 /*@C
       ISInitializePackage - This function initializes everything in the IS package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to ISCreateXXXX()
-  when using static libraries.
+  from PetscDLLibraryRegister_petscvec() when using dynamic libraries, and on the first call to ISCreateXXXX()
+  when using shared or static libraries.
 
   Level: developer
 
@@ -138,8 +138,8 @@ static PetscBool  VecPackageInitialized = PETSC_FALSE;
 
 /*@C
   VecInitializePackage - This function initializes everything in the Vec package. It is called
-  from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to VecCreate()
-  when using static libraries.
+  from PetscDLLibraryRegister_petscvec() when using dynamic libraries, and on the first call to VecCreate()
+  when using shared or static libraries.
 
   Level: developer
 
@@ -197,7 +197,7 @@ PetscErrorCode  VecInitializePackage(void)
   ierr = PetscLogEventRegister("VecViennaCLCopyTo",   VEC_CLASSID,&VEC_ViennaCLCopyToGPU);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("VecViennaCLCopyFrom", VEC_CLASSID,&VEC_ViennaCLCopyFromGPU);CHKERRQ(ierr);
 #endif
-#if defined(PETSC_HAVE_VECCUDA)
+#if defined(PETSC_HAVE_CUDA)
   ierr = PetscLogEventRegister("VecCUDACopyTo",     VEC_CLASSID,&VEC_CUDACopyToGPU);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("VecCUDACopyFrom",   VEC_CLASSID,&VEC_CUDACopyFromGPU);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("VecCopyToSome",     VEC_CLASSID,&VEC_CUDACopyToGPUSome);CHKERRQ(ierr);
@@ -210,7 +210,7 @@ PetscErrorCode  VecInitializePackage(void)
   ierr = PetscLogEventSetCollective(VEC_ViennaCLCopyToGPU,   PETSC_FALSE);CHKERRQ(ierr);
   ierr = PetscLogEventSetCollective(VEC_ViennaCLCopyFromGPU, PETSC_FALSE);CHKERRQ(ierr);
 #endif
-#if defined(PETSC_HAVE_VECCUDA)
+#if defined(PETSC_HAVE_CUDA)
   ierr = PetscLogEventSetCollective(VEC_CUDACopyToGPU,       PETSC_FALSE);CHKERRQ(ierr);
   ierr = PetscLogEventSetCollective(VEC_CUDACopyFromGPU,     PETSC_FALSE);CHKERRQ(ierr);
   ierr = PetscLogEventSetCollective(VEC_CUDACopyToGPUSome,   PETSC_FALSE);CHKERRQ(ierr);
