@@ -583,13 +583,13 @@ PetscErrorCode  VecView(Vec vec,PetscViewer viewer)
       ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     }
   }
-  ierr = VecLockPush(vec);CHKERRQ(ierr);
+  ierr = VecLockReadPush(vec);CHKERRQ(ierr);
   if ((format == PETSC_VIEWER_NATIVE || format == PETSC_VIEWER_LOAD_BALANCE) && vec->ops->viewnative) {
     ierr = (*vec->ops->viewnative)(vec,viewer);CHKERRQ(ierr);
   } else {
     ierr = (*vec->ops->view)(vec,viewer);CHKERRQ(ierr);
   }
-  ierr = VecLockPop(vec);CHKERRQ(ierr);
+  ierr = VecLockReadPop(vec);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(VEC_View,vec,viewer,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -693,7 +693,7 @@ PetscErrorCode  VecGetLocalSize(Vec x,PetscInt *size)
    Note:
    The high argument is one more than the last element stored locally.
 
-   Fortran: NULL_INTEGER should be used instead of NULL
+   Fortran: PETSC_NULL_INTEGER should be used instead of NULL
 
    Level: beginner
 
@@ -772,9 +772,11 @@ PetscErrorCode  VecGetOwnershipRanges(Vec x,const PetscInt *ranges[])
           ignored.
 -     VEC_SUBSET_OFF_PROC_ENTRIES, which causes VecAssemblyBegin() to assume that the off-process
           entries will always be a subset (possibly equal) of the off-process entries set on the
-          first assembly.  This reuses the communication pattern, thus avoiding a global reduction.
-          Subsequent assemblies setting off-process values should use the same InsertMode as the
-          first assembly.
+          first assembly which had a true VEC_SUBSET_OFF_PROC_ENTRIES and the vector has not
+          changed this flag afterwards. If this assembly is not such first assembly, then this
+          assembly can reuse the communication pattern setup in that first assembly, thus avoiding
+          a global reduction. Subsequent assemblies setting off-process values should use the same
+          InsertMode as the first assembly.
 
    Developer Note:
    The InsertMode restriction could be removed by packing the stash messages out of place.
@@ -983,8 +985,8 @@ PetscErrorCode  VecReciprocal(Vec vec)
 
     Usage:
 $      PetscErrorCode userview(Vec,PetscViewer);
-$      ierr = VecCreateMPI(comm,m,M,&x);
-$      ierr = VecSetOperation(x,VECOP_VIEW,(void(*)(void))userview);
+$      ierr = VecCreateMPI(comm,m,M,&x);CHKERRQ(ierr);
+$      ierr = VecSetOperation(x,VECOP_VIEW,(void(*)(void))userview);CHKERRQ(ierr);
 
     Notes:
     See the file include/petscvec.h for a complete list of matrix
@@ -1569,7 +1571,7 @@ PetscErrorCode  VecCopy(Vec x,Vec y)
   if (x == y) PetscFunctionReturn(0);
   VecCheckSameLocalSize(x,1,y,2);
   if (x->stash.insertmode != NOT_SET_VALUES) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Not for unassembled vector");
-  VecLocked(y,2);
+  ierr = VecSetErrorIfLocked(y,2);CHKERRQ(ierr);
 
 #if !defined(PETSC_USE_MIXED_PRECISION)
   for (i=0; i<4; i++) {
@@ -1702,7 +1704,7 @@ PetscErrorCode VecStashViewFromOptions(Vec obj,PetscObject bobj,const char optio
 
   PetscFunctionBegin;
   prefix = bobj ? bobj->prefix : ((PetscObject)obj)->prefix;
-  ierr   = PetscOptionsGetViewer(PetscObjectComm((PetscObject)obj),prefix,optionname,&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr   = PetscOptionsGetViewer(PetscObjectComm((PetscObject)obj),((PetscObject)obj)->options,prefix,optionname,&viewer,&format,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerPushFormat(viewer,format);CHKERRQ(ierr);
     ierr = VecStashView(obj,viewer);CHKERRQ(ierr);
