@@ -40,8 +40,8 @@ PetscErrorCode  VecStrideSet(Vec v,PetscInt start,PetscScalar s)
   PetscValidHeaderSpecific(v,VEC_CLASSID,1);
   PetscValidLogicalCollectiveInt(v,start,2);
   PetscValidLogicalCollectiveScalar(v,s,3);
-  VecLocked(v,1);
-  
+  ierr = VecSetErrorIfLocked(v,1);CHKERRQ(ierr);
+
   ierr = VecGetLocalSize(v,&n);CHKERRQ(ierr);
   ierr = VecGetArray(v,&x);CHKERRQ(ierr);
   ierr = VecGetBlockSize(v,&bs);CHKERRQ(ierr);
@@ -90,8 +90,8 @@ PetscErrorCode  VecStrideScale(Vec v,PetscInt start,PetscScalar scale)
   PetscValidHeaderSpecific(v,VEC_CLASSID,1);
   PetscValidLogicalCollectiveInt(v,start,2);
   PetscValidLogicalCollectiveScalar(v,scale,3);
-  VecLocked(v,1);
-  
+  ierr = VecSetErrorIfLocked(v,1);CHKERRQ(ierr);
+
   ierr = VecGetLocalSize(v,&n);CHKERRQ(ierr);
   ierr = VecGetArray(v,&x);CHKERRQ(ierr);
   ierr = VecGetBlockSize(v,&bs);CHKERRQ(ierr);
@@ -377,8 +377,8 @@ PetscErrorCode  VecStrideScaleAll(Vec v,const PetscScalar *scales)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(v,VEC_CLASSID,1);
   PetscValidScalarPointer(scales,2);
-  VecLocked(v,1);
-  
+  ierr = VecSetErrorIfLocked(v,1);CHKERRQ(ierr);
+
   ierr = VecGetLocalSize(v,&n);CHKERRQ(ierr);
   ierr = VecGetArray(v,&x);CHKERRQ(ierr);
   ierr = VecGetBlockSize(v,&bs);CHKERRQ(ierr);
@@ -1340,7 +1340,7 @@ PetscErrorCode  VecDotNorm2(Vec s,Vec t,PetscScalar *dp, PetscReal *nm)
   ierr = PetscLogEventBegin(VEC_DotNorm2,s,t,0,0);CHKERRQ(ierr);
   if (s->ops->dotnorm2) {
     ierr = (*s->ops->dotnorm2)(s,t,dp,&dpx);CHKERRQ(ierr);
-    *nm  = PetscRealPart(dpx);CHKERRQ(ierr);
+    *nm  = PetscRealPart(dpx);
   } else {
     ierr = VecGetLocalSize(s, &n);CHKERRQ(ierr);
     ierr = VecGetArrayRead(s, &sx);CHKERRQ(ierr);
@@ -1400,6 +1400,60 @@ PetscErrorCode  VecSum(Vec v,PetscScalar *sum)
   PetscFunctionReturn(0);
 }
 
+/*@
+   VecImaginaryPart - Replaces a complex vector with its imginary part
+
+   Collective on Vec
+
+   Input Parameter:
+.  v - the vector
+
+   Level: beginner
+
+.seealso: VecNorm(), VecRealPart()
+@*/
+PetscErrorCode  VecImaginaryPart(Vec v)
+{
+  PetscErrorCode    ierr;
+  PetscInt          i,n;
+  PetscScalar       *x;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(v,VEC_CLASSID,1);
+  ierr = VecGetLocalSize(v,&n);CHKERRQ(ierr);
+  ierr = VecGetArray(v,&x);CHKERRQ(ierr);
+  for (i=0; i<n; i++) x[i] = PetscImaginaryPart(x[i]);
+  ierr = VecRestoreArray(v,&x);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   VecRealPart - Replaces a complex vector with its real part
+
+   Collective on Vec
+
+   Input Parameter:
+.  v - the vector
+
+   Level: beginner
+
+.seealso: VecNorm(), VecImaginaryPart()
+@*/
+PetscErrorCode  VecRealPart(Vec v)
+{
+  PetscErrorCode    ierr;
+  PetscInt          i,n;
+  PetscScalar       *x;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(v,VEC_CLASSID,1);
+  ierr = VecGetLocalSize(v,&n);CHKERRQ(ierr);
+  ierr = VecGetArray(v,&x);CHKERRQ(ierr);
+  for (i=0; i<n; i++) x[i] = PetscRealPart(x[i]);
+  ierr = VecRestoreArray(v,&x);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    VecShift - Shifts all of the components of a vector by computing
    x[i] = x[i] + shift.
@@ -1427,7 +1481,7 @@ PetscErrorCode  VecShift(Vec v,PetscScalar shift)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(v,VEC_CLASSID,1);
   PetscValidLogicalCollectiveScalar(v,shift,2);
-  VecLocked(v,1);
+  ierr = VecSetErrorIfLocked(v,1);CHKERRQ(ierr);
 
   if (v->ops->shift) {
     ierr = (*v->ops->shift)(v,shift);CHKERRQ(ierr);
@@ -1461,8 +1515,8 @@ PetscErrorCode  VecAbs(Vec v)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(v,VEC_CLASSID,1);
-  VecLocked(v,1);
-  
+  ierr = VecSetErrorIfLocked(v,1);CHKERRQ(ierr);
+
   if (v->ops->abs) {
     ierr = (*v->ops->abs)(v);CHKERRQ(ierr);
   } else {
@@ -1497,8 +1551,8 @@ PetscErrorCode  VecPermute(Vec x, IS row, PetscBool inv)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  VecLocked(x,1);
-  
+  ierr = VecSetErrorIfLocked(x,1);CHKERRQ(ierr);
+
   ierr = VecGetOwnershipRange(x,&rstart,&rend);CHKERRQ(ierr);
   ierr = ISGetIndices(row, &idx);CHKERRQ(ierr);
   ierr = VecGetArray(x, &array);CHKERRQ(ierr);
